////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.MKB;

import java.util.*;

public class CMatrix{

private static final boolean DEBUG = false;

  public MKB MainMKB;
  public int MatrixSize=0;
  public Vector RelationList=new Vector();
  public Vector[][] PCMatrix;
  
  public CMatrix(MKB theMKB){
    MainMKB=theMKB;
    addRelations();
    initMatrix();
    addPCs();
    IterMatrix(1);
  }

  public Vector GetDerivedPC(){
    Vector RetList=new Vector();
    PCConstraint TempPC;

    for(int i=0;i<MatrixSize;i++)
      for(int j=0;j<MatrixSize;j++)
	for(int k=0;k<PCMatrix[i][j].size();k++){
	  TempPC=(PCConstraint)(PCMatrix[i][j].elementAt(k));
	  if(TempPC.IterationNum>0)
	    RetList.addElement(TempPC);
	}

    return RetList;

  }



	    
	  
    
  private void initMatrix(){
    MatrixSize=RelationList.size();
    PCMatrix = new Vector[MatrixSize][MatrixSize];
    for(int i=0;i<MatrixSize;i++)
      for(int j=0;j<MatrixSize;j++)
	PCMatrix[i][j]=new Vector();

  }


  private void IterMatrix(int IterNum){
    int i,j,k,l;
    for(i =0;i<MatrixSize;i++)
      for(j=0;j<MatrixSize;j++)
	for(k=0;k<MatrixSize;k++)
	  Multi(i,k,k,j,IterNum); 
    
  }

  private void Multi(int x1,int y1,int x2,int y2,int IterNum){
    Vector V1=PCMatrix[x1][y1];
    Vector V2=PCMatrix[x2][y2];
    PCConstraint PC1,PC2,FinalPC;

    if(V1.size()==0 || V2.size()==0) return;
    
    for(int i=0;i<V1.size();i++)
      for(int j=0;j<V2.size();j++){
	PC1=(PCConstraint)V1.elementAt(i);
	PC2=(PCConstraint)V2.elementAt(j);
	
	if(PC1.IterationNum >= IterNum ||
	   PC2.IterationNum >= IterNum) continue;
	
	FinalPC=PCMulti(PC1,PC2);
	if(FinalPC==null) continue;

	FinalPC.IterationNum=IterNum;
	PCMatrix[x2][y1].addElement(FinalPC);

	
      }
  }	
    
	
	
  private void addPCs(){
    int x,y;
    PCConstraint P;

    for(int i=0;i<MainMKB.PC_List.size();i++){
      P=(PCConstraint)MainMKB.PC_List.elementAt(i);

      x=RelationList.indexOf(P.getR1());
      y=RelationList.indexOf(P.getR2());

      if(x==-1 || y == -1) continue; //Not found, do nothing

      if (DEBUG) System.out.println("Add PC X="+x+" Y= "+y);
      P.displayAll();

      if(P.getExtent().charAt(0)=='<') PCMatrix[x][y].addElement(P);
      else if(P.getExtent().charAt(0)=='>') PCMatrix[y][x].addElement(P);
      else {
	PCMatrix[x][y].addElement(P);
	PCMatrix[y][x].addElement(P);
      }
    }
  }

  private void addRelations(){
    PCConstraint P;

    for(int i=0;i<MainMKB.PC_List.size();i++){
      P=(PCConstraint)MainMKB.PC_List.elementAt(i);
      if(RelationList.indexOf(P.getR1())==-1)
	RelationList.addElement(P.getR1());
      if(RelationList.indexOf(P.getR2())==-1)
	RelationList.addElement(P.getR2());
    }
	
    }

  private PCConstraint PCMulti(PCConstraint PC1, PCConstraint PC2){

    Vector PairList;    //Index of common attributes
    PCConstraint PC1P,PC2P; //Intermedia PCs
    String Ext1,Ext2,newExt; //Intermedia exts

    if(PC1.getR2()!=PC2.getR1()) return null;

    PairList=FindPair(PC1,PC2);
    if(PairList.size()==0) return null; //No common attr
    
    if(PairList.size()==PC1.getPCPairs().size())
      Ext1=PC1.getExtent();
    else
      Ext1=GetSubExt(PC1.getExtent());

    if(PairList.size()==PC2.getPCPairs().size())
      Ext2=PC2.getExtent();
    else
      Ext2=GetSubExt(PC2.getExtent());
    if (DEBUG) System.out.println("Exts are "+Ext1+" "+Ext2);

    newExt=MergeExt(Ext1,Ext2);
    if (DEBUG) System.out.println("After merge "+newExt);

    if(newExt==null)return null;

    PCConstraint retPC=new PCConstraint(PC1.getR1(),PC2.getR2(),newExt);
    Vector newPCPair=new Vector();
    Attribute A1,A2;
    for(int i=0;i<PairList.size();i++){

      A1=((PCPair)(PC1.getPCPairs().
	elementAt(((IntPair)(PairList.elementAt(i))).Index1))).getA1();

      A2=((PCPair)(PC2.getPCPairs().
	elementAt(((IntPair)(PairList.elementAt(i))).Index2))).getA2();

      newPCPair.addElement(new PCPair(A1,A2));
    }

    retPC.addPair(newPCPair);

    return retPC;
  }

  static String MergeExt(String Ext1,String Ext2){
    if(Ext1.equals("="))return Ext2;
    else if(Ext2.equals("=")) return Ext1;

    else if(Ext1.equals("<=") && Ext2.equals("<=")) return "<=";
    else if(Ext1.equals("<=") && Ext2.equals("<")) return "<";
    else if(Ext1.equals("<") && Ext2.equals("<")) return "<";
    else if(Ext1.equals("<") && Ext2.equals("<=")) return "<";
    
    return null;
  }
	    
  private String GetSubExt(String oldExt){
    if(oldExt.equals("="))return "=";
    else if(oldExt.equals("<="))return "<=";
    else if(oldExt.equals("<"))return "<=";
    else if(oldExt.equals("<="))return "<=";
    else if(oldExt.equals(">="))return ">=";
    else if(oldExt.equals(">"))return ">=";
    return "<>";
  }

	    
    
  
  private Vector FindPair(PCConstraint PC1, PCConstraint PC2){
    Vector PairList=new Vector();
    for(int i=0;i<PC1.getPCPairs().size();i++)
      for(int j=0;j<PC2.getPCPairs().size();j++)
	if(((PCPair)PC1.getPCPairs().elementAt(i)).getA2() ==
	   ((PCPair)PC2.getPCPairs().elementAt(j)).getA1())
	  PairList.addElement(new IntPair(i,j));
    return PairList;
  }
    
}



class IntPair{
  int Index1;
  int Index2;
  
  public IntPair(int In1,int In2){
    Index1=In1;
    Index2=In2;
  }

}
