////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.MW;

import java.util.*;
import java.sql.*;
import java.io.*;
import DSRG.MKB.*;

public class DBMAccess extends DBManager implements Serializable{
  // this is the alternative set ot def-Strings for Access. <BR>
  private DBMAccess instance = null;
  private String url = "jdbc:odbc:EVE";
  private String user = "eve";
  private String passwd = "eve";
  private String className = "sun.jdbc.odbc.JdbcOdbcDriver";
  private String DB = "Access";
  // Access doesn't seem to have any metatables.  We need TAB and a subset 
  // of COLS, so we have to create them ourselves and maintain them. Fun.
  

  /**
   * Default Constructor
   */
  public DBMAccess(){
    url = new String("jdbc:odbc:EVE");
    user = new String("eve");
    passwd = new String("passwd");
    className = new String("sun.jdbc.odbc.JdbcOdbcDriver");
    try{
      driver = Class.forName(className);
      connection = java.sql.DriverManager.getConnection(url, user, passwd);
      System.out.println("Connected");
      System.out.println();
    }
    catch (Exception exc) {
      System.out.println("DB Exception Access 1");
      return;
    }
    connected = true;
  }

  /**
   * Constructor with name/password
   */
  public DBMAccess(String myISName, String name, String password, String URL){
      ISName = myISName;
    url = URL;  //new String("jdbc:odbc:EVE");
    user = new String(name);
    passwd = new String(password);
    className = new String("sun.jdbc.odbc.JdbcOdbcDriver");
    try{
      driver = Class.forName(className);
      connection = java.sql.DriverManager.getConnection(url, user, passwd);
      System.out.println("Connected");
      System.out.println();
    }
    catch (Exception exc) {
      System.out.println("DB Exception Access 2\n"+exc.toString());
      return;
    }
    connected = true;
  }

  /**
   * Returns the handle to this instance of the DBManager or creates a new
   * instance. <BR>
   */
  public void getDBManager(){
    if(instance == null)
      instance = new DBMAccess();
  }


  /**
   * addRelation. <BR> creates only the schema, does not insert data
   * or anything <BR> The types have to be like "VARCHAR(.."
   * otherwise they're overwritten by "VARCHAR(31)
   *
   * 10/9/97 koeller */
  public boolean addRelation (String relName,
                              Vector attNames,
                              Vector attTypes) {
      
      java.sql.PreparedStatement stmt = null;
      boolean result = false;
      String schema=new String("("); // first parenthese
      
      // I assume that the two Vectors have the same number of elements
      // create schema
      Enumeration nameEnum = attNames.elements();
      Enumeration typeEnum = attTypes.elements();
      String tmpName, tmpType;
      // if the type is not of the form varchar2(..., reject and use
      // varchar2(31)
      
      while (nameEnum.hasMoreElements()) {
          tmpName = (String)nameEnum.nextElement();
          tmpType = (String)typeEnum.nextElement();
          // try to find the "varchar("-substring
          if (tmpType.toUpperCase().indexOf("VARCHAR(")==-1)
              tmpType = "VARCHAR(31)";
          schema += (tmpName
                     + " "
                     + tmpType
                     + ", ");
      }
      
      schema = schema.substring(0,schema.length()-2)+")"; 
      // replace the comma at the end with a )
      
      // no preparedStatement possible, since Oracle doesn't understand 
      // identifier names as parameters...
      // Access is the same
      
      try {
          
          stmt=connection.prepareStatement(   "create table "
                                              + relName
                                              + " "
                                              + schema);
          
          stmt.executeUpdate();
          
          
          stmt.close ();
          result = true;
          
    }
      catch (java.sql.SQLException exc) {
          System.out.println("SQL Exception in addRelation: " + exc.toString());
      }
      
      if (result && DB.equals("Access")) addRelationAcc(relName, schema);
      // maintain Meta Tables for Access
      
      
      return result;
  }
    


 /**
   * deleteRelation. <BR>
   *
   *
   * 10/6/97 koeller
   */


  public boolean deleteRelation (String name, boolean silent) {

    java.sql.PreparedStatement stmt = null;

    boolean result = false;

    // no preparedStatement possible, since Oracle doesn't understand identifier names
    // as parameters...
    // same thing with Access

    try {

      
      stmt = 
	connection.prepareStatement (   "drop table "
                                      + name);
                                   
      stmt.executeUpdate ();
      

      stmt.close ();
      result = true;
    }
    catch (java.sql.SQLException exc) {
       if (!silent) System.out.println("SQL Exception in deleteRelation: " + exc.toString());
    }


    if (result) deleteRelationAcc(name);
    // maintain Meta Tables for Access

    return result;
  }





  /**
   * renameAttribute. <BR>
   *
   *
   * 10/6/97 koeller
   */


  public boolean renameAttribute (String RelationName, String oldName, String newName) {

    java.sql.PreparedStatement stmt = null;

    boolean result = true;

    // no preparedStatement possible, since Oracle doesn't understand 
    // identifier names as parameters...

    /* Algo for Oracle. We'll do the same with Access, to get the column 
       order right.

       1. get Column names
       2. replace Attribute name; for Access, these are two steps (it doesn't 
          do "create as")
       3. create new table with replaced attribute name
       4. drop old table
       5. recreate old table from new table
       6. drop new table

       5.+6. is a renameRelation
     */


    String origString   = getSchema(RelationName);
    String selectString = makeSelectClause(origString);
    String createString = makeSelectClause(renameAttributeInSchema(origString, oldName, newName));
    String AccessTempSchema = renameAttributeInSchema(origString, oldName, newName);


    String makeTable =   "create table "
                 + "temp ("
                 + createString
                 + ") as select "
                 + selectString
                 + " from "
                 + RelationName;


    // 1. drop temp table, just in case

   try {

      stmt=connection.prepareStatement("drop table temp");
      stmt.executeUpdate();
    }
    catch (java.sql.SQLException exc) {
    } // ignore SQL errors


   try {
    // 2. make new table "temp"
     stmt = connection.prepareStatement(   "create table "
					   + "temp "
					   + AccessTempSchema);

     stmt.executeUpdate();

     addRelationAcc("temp", AccessTempSchema);
     // we have to keep track in tab/cols in order to rename the table later, what a mess

     stmt = connection.prepareStatement(   "insert into temp select "
					   + selectString
					   + " from "
					   + RelationName );
     
     stmt.executeUpdate();  // I love Access


     // 3. drop old table

     stmt=connection.prepareStatement("drop table "+RelationName);
     stmt.executeUpdate();

     deleteRelationAcc(RelationName);

     // 4. rename temp table to old name

     if (!renameRelation("temp", RelationName))
       result = false;

     stmt.close ();
   }
   catch (java.sql.SQLException exc) {
     result = false;
     System.out.println("SQL Exception in renameAttribute: " + exc.toString());
   }


   if (result) renameAttributeAcc(RelationName, oldName, newName);
   // maintain Meta Tables for Access


   return result;
  }



  /**
   * addAttribute. <BR>
   * Extents are being ignored, not needed/possible for the demo <BR>
   *
   *
   * 10/6/97 koeller <BR>
   * patched 07/13/98, Error when creating "NUMBER" data type
   */


  public boolean addAttribute (String RelationName, String AttributeName,
                               String type, int size) {
    java.sql.PreparedStatement stmt = null;

    boolean result = false;


    try {
      if (!type.toUpperCase().equals("NUMBER"))
        stmt=connection.prepareStatement(   "alter table "
                                          + RelationName
                                          + " add column "
                                          + AttributeName
                                          + " "
                                          + type
                                          + "("
                                          + (new Integer(size)).toString()
                                          + ")");
      else
        stmt=connection.prepareStatement(   "alter table "
                                          + RelationName
                                          + " add column "
                                          + AttributeName
                                          + " "
                                          + "NUMBER");
      
         
      stmt.executeUpdate();

      stmt.close ();
      result = true;
      
    }
    catch (java.sql.SQLException exc) {
       System.out.println("SQL Exception in addAttribute: " + exc.toString());
    }



    if (result) addAttributeAcc(RelationName, AttributeName, type);
    // maintain Meta Tables for Access

    return result;
  }




  /**
   * deleteAttribute. <BR>
   *
   *
   * 10/6/97 koeller
   */


  public boolean deleteAttribute (String RelationName, String attributeName) {

     // the only thing that is easier in Access, compared to Oracle

    java.sql.PreparedStatement stmt = null;

    boolean result = true;
    try {

      stmt=connection.prepareStatement(   "alter table "
                                          + RelationName
                                          + " drop column "
                                          + attributeName);
      stmt.executeUpdate();
    }
    catch (java.sql.SQLException exc) {
      result = false;
      System.out.println("SQL Exception in deleteAttribute: " + exc.toString());
    } 

    if (result) deleteAttributeAcc(RelationName, attributeName);
    // maintain Meta Tables for Access

    return result;
  }




/**
  *
  * function necessary since Access doesn't provide metadata info in the SQL interface
  * (...)
  *
  */
public boolean addRelationAcc(String RelationName, String schema) {

   //  This is REALLY bad programming right now, but I don't want to create another class
   //  just for this.
   //  I'll just copy the source code from
   //    private String makeSelectClause(String schema) {}
   //  to parse my schema, and then push it into the DB
   
    java.sql.PreparedStatement stmt = null;
      boolean result=true;

      String attName=null;
      String attType=null;

      try {

        stmt=connection.prepareStatement("insert into tab values(?,?)");
        stmt.setString(1,RelationName);
        stmt.setString(2,"TABLE");
        
        stmt.executeUpdate();

        stmt=connection.prepareStatement(    "insert into cols "
                                          +  "(table_name, column_name, "
                                          +  "data_type, data_length) "
                                          +  "values(?,?,?,?)");

        int position=0;
        int position2=0;

        for (position=0; schema.charAt(position)<'A'; position++); // find first real character

        while (position!=0) { // while there are still commas in the string

        // it works only since there is at least on parenthese in the beginning in the string !!!


          position2 = schema.indexOf(' ',position+1); // end of identifier

          if (position2==-1) // only one attribute, special case
             position2=schema.length()-1;

          attName = schema.substring(position, position2).trim();

          position = schema.indexOf(',', position2)+1; // start of next identifier

          if (position == 0)  // last attribute
             attType = schema.substring(position2,schema.length()-1);
          else
             attType = schema.substring(position2,position-1);

          stmt.setString(1,RelationName.toUpperCase().trim());
          stmt.setString(2,attName.toUpperCase().trim());
          stmt.setString(3,getNameFromType(attType).toUpperCase().trim());
          stmt.setInt(4,getLengthFromType(attType));
        
          stmt.executeUpdate();
          

       }



      }
      catch (java.sql.SQLException exc) {
        result = false;
        System.out.println("SQL Exception in addRelationAcc: " + exc.toString());
      } 

      return result;


}



/**
  *
  * function necessary since Access doesn't provide metadata info in the SQL interface
  * (...)
  *
  */
private boolean deleteRelationAcc(String RelationName) {

    java.sql.PreparedStatement stmt = null;


      boolean result=true;

      try {

        stmt=connection.prepareStatement("delete from tab where tname = ?");
        stmt.setString(1,RelationName);

        stmt.executeUpdate();

        stmt=connection.prepareStatement("delete from cols where table_name = ?");
        stmt.setString(1,RelationName);


        stmt.executeUpdate();
      }
      catch (java.sql.SQLException exc) {
        result = false;
        System.out.println("SQL Exception in deleteRelationAcc: " + exc.toString());
      } 

      return result;

}


/**
  *
  * function necessary since Access doesn't provide metadata info in the SQL interface
  * (...)
  *
  */
private boolean renameAttributeAcc(String relationName, String oldName, String newName) {

    java.sql.PreparedStatement stmt = null;

      boolean result=true;

      try {

        stmt=connection.prepareStatement(   "update cols set column_name = ? "
                                          + "where table_name = ? and "
                                          + "column_name = ?");
        stmt.setString(1, newName.trim());
        stmt.setString(2, relationName);
        stmt.setString(3, oldName);
        
        stmt.executeUpdate();

      }
      catch (java.sql.SQLException exc) {
        result = false;
        System.out.println("SQL Exception in renameRelationAcc: " + exc.toString());
      } 

      return result;

}




/**
  *
  * function necessary since Access doesn't provide metadata info in the SQL interface
  * (...)
  *
  */
private boolean renameRelationAcc(String oldName, String newName) {

    java.sql.PreparedStatement stmt = null;

      boolean result=true;

      try {

        stmt=connection.prepareStatement(   "update tab set tname = ? "
                                          + "where tname = ?");
        stmt.setString(1, newName.trim());
        stmt.setString(2, oldName);
        
        stmt.executeUpdate();

        stmt=connection.prepareStatement(   "update cols set table_name = ? "
                                          + "where table_name = ?");
        stmt.setString(1, newName.trim());
        stmt.setString(2, oldName);
        
        stmt.executeUpdate();

      }
      catch (java.sql.SQLException exc) {
        result = false;
        System.out.println("SQL Exception in renameRelationAcc: " + exc.toString());
      } 

      return result;

}




/**
  *
  * function necessary since Access doesn't provide metadata info in the SQL interface
  * (...)
  *
  */
private boolean addAttributeAcc(String relationName, String attributeName, String attributeType) {

       // attributeType will be a String of the form
       //        name[(<number>)]
       // i.e., I have to parse for parentheses, then extract the number.
       // the name goes to data_type, the number goes to data_precision
       // the rest is table_name and column_name

    java.sql.PreparedStatement stmt = null;

      boolean result=true;

      try {

        stmt=connection.prepareStatement(   "insert into cols (table_name,"
                                          + "column_name, data_type, data_length) "
                                          + "values(?,?,?,?)");
        stmt.setString(1,relationName.trim());
        stmt.setString(2,attributeName.trim());
        stmt.setString(3,getNameFromType(attributeType));
        int param = getLengthFromType(attributeType);

        
        stmt.setInt(4,param);
        
        stmt.executeUpdate();
      }
      catch (java.sql.SQLException exc) {
        result = false;
        System.out.println("SQL Exception in addAttributeAcc: " + exc.toString());
      } 

      return result;

}







/**
  *
  * function necessary since Access doesn't provide metadata info in the SQL interface
  * (...)
  *
  */
private boolean deleteAttributeAcc(String relationName, String attributeName) {

    java.sql.PreparedStatement stmt = null;


      boolean result=true;

      try {

        stmt=connection.prepareStatement(   "delete from cols where table_name = ? "
                                          + " and column_name = ?");
        stmt.setString(1,relationName);
        stmt.setString(2,attributeName);
        
        stmt.executeUpdate();
      }
      catch (java.sql.SQLException exc) {
        result = false;
        System.out.println("SQL Exception in deleteAttributeAcc: " + exc.toString());
      } 

      return result;

}

/**
  *
  * stores a Table object in a database. <BR>
  * Creates a schema and stores data in there.
  * Uses the relation's name and attribute names and lengths
  * everything is created as "VARCHAR2" !!
  * This is for view recomputation and should only be used on LocalIS
  *
  * @author 04/07/1999 koeller
  *
  */


    public boolean storeRelation(Table table) {

        //System.out.println("Storing Relation: "+ relation.getName());

        Integer aSize = null;
        String schema = new String();
        String tuple = new String();
        Vector column = new Vector();
        boolean result;
        java.sql.PreparedStatement stmt = null;
        String relationName = table.getRelName();

        if (DEBUG) System.out.println("DBMAccess.newStoreRelation: Trying to delete Relation"+relationName);
        deleteRelation(relationName,true);  
        // silent delete

        Enumeration attributeSizes = 
            table.getAttributeSizes().elements();
        Vector attTypes = new Vector();
        while (attributeSizes.hasMoreElements()) {
            aSize = (Integer)attributeSizes.nextElement();
            if (aSize.intValue()==0) aSize = new Integer(29); 
            // preventive maintenance...
            attTypes.addElement("VARCHAR2("+aSize.toString()+")");
        }
        result = addRelation(relationName, table.getAttributeNames(),
                             attTypes); 
     
        if (result) {
            Vector rev = table.getTuples();
            if (rev == null) return true;
            if (rev.size()==0) return true;

            // insert tuples
            try {
                Enumeration re = table.getTuples().elements();
                while (re.hasMoreElements()) {
                    tuple = ((Tuple)re.nextElement()).toSQLString();
                    
                    stmt =
                        connection.prepareStatement (   "insert into "
                                                        +  relationName
                                                        + " values "
                                                        + "("
                                                        + tuple
                                                        + ")" );
                    if (DEBUG) System.out.println("DEBUG: inserting: "+   relationName
                                                  + " values "
                                                  + "("
                                                  + tuple
                                                  + ")");
                    
                    stmt.executeUpdate ();
                    stmt.close ();
                }
	   
            }
            catch (java.sql.SQLException exc) {
                System.out.println("SQL Exception in storeRelation: "
                                   + exc.toString());
                result = false;
            }
        }
        return result;
    }


//====================================================================================
//====================================================================================
//====================================================================================
//===============           More String Acrobatics for Access            =============
//====================================================================================
//====================================================================================
//====================================================================================

  /**
   * Helper function. <BR>
   * extracts a name from a (simple) SQL-type
   *
   */
  private   String getNameFromType(String type) {
     int index = type.indexOf('(');
     if (index>=1)
        return type.substring(0, index).toUpperCase();
     else return type.toUpperCase();
  }


  /**
   * Helper function. <BR>
   * extracts a data length from a (simple) SQL-type
   *
   */
  private   int getLengthFromType(String type) {
     int index = type.indexOf('(');
     int result;
     if (index>=1)
        result = Integer.parseInt(type.substring(index+1, type.indexOf(')')));
     else result = -1;

     if (result==-1 && getNameFromType(type).toUpperCase().trim().equals(date_datatype))
           result = 7;   // default value for DATE
        else
        if (result==-1 && getNameFromType(type).toUpperCase().trim().equals(numeric_datatype))
           result = 22;  // default value for NUMBER
              //  Access doesn't accept lengths for numbers, therefore we have more patches
              // up there
     return result;

  }


  /**
    *
    * renames a table by dropping the old one and creating the new one.
    *
    */


  public boolean renameRelation (String oldName, String newName) {
    java.sql.PreparedStatement stmt = null;
    boolean result = false;

    // first, create a table like the old one
    try{
      stmt = 
	connection.prepareStatement (   "create table "
					+  newName
					+ " "
					+ getSchema(oldName));

      stmt.executeUpdate ();

      
      stmt = 
	connection.prepareStatement (   "insert into "
					+  newName
					+ " select "
					+ makeSelectClause(getSchema(oldName))
					+ " from "
					+ oldName); 

      stmt.executeUpdate ();

      stmt = 
	connection.prepareStatement (   "drop table "
					+ oldName);
      
      stmt.executeUpdate ();
      

      stmt.close ();
      result = true;
    }
    catch (java.sql.SQLException exc) {
       System.out.println("SQL Exception in renameRelation: " 
			  + exc.toString());
    }


    if (result) renameRelationAcc(oldName, newName);
    // maintain Meta Tables for Access

    return result;
  }



 /**
   *  Helper function. <BR>
   *  This will read the Schema of a given table in the IS, concatenating it to
   *  a string of the form
   *  <pre>([</lt>columnname</gt> </lt>type</gt>,] </lt>columnname</gt> </lt>type</gt>)</pre>
   *
   */

   public String getSchema(String relationName) {

    java.sql.PreparedStatement stmt = null;
    java.sql.ResultSet rs = null;
    java.sql.ResultSetMetaData rsmt = null;


    String schema = new String("("); // open parenthese
    String result=null;

    String column_name = null;
    String data_type = null;
    String data_length = null;

     try {
      stmt = 
	connection.prepareStatement (   "select column_name, data_type, data_length "
                                      + "from cols "
                                      + "where table_name =?");

      stmt.setString(1, relationName);
	
     // right now, this takes no fixed-precision numbers (like NUMBER(10,2))!!!
     // it only supports simple tables with no indices, foreign-key-constraints,
     // keys etc. just simple datatypes and lengths


	
      rs = stmt.executeQuery ();
      rsmt = rs.getMetaData ();

      while (rs.next ()) {

        column_name = rs.getString(1).trim();
        data_type = rs.getString(2).trim();
        data_length = rs.getString(3).trim();


        schema= schema + column_name + " " + data_type;

          // Another patch for Access, this is a big mess!!!

        if (!data_type.equals(numeric_datatype))
           schema = schema + "("
                           + data_length
                           + ")";

        schema = schema + ",";  
      }

      result= new String(schema.substring(0,schema.length()-1)+")");
      // replace last comma with a ")"

      rs.close ();
      stmt.close ();
    }
    catch (java.sql.SQLException exc) {
       System.out.println("SQL Exception in getAttributeExtent: " + exc.toString());
    }
    
    return result;
   }

  /**
   * Added by Xinz and koeller ;-)
   * Reset the database
   */
  public void resetDB(){
    try{
      // Open the reset.sql file
      File f  = new File("reset_access.sql");
      BufferedReader in = new BufferedReader( new FileReader(f));
      Statement stmt = connection.createStatement();
      String sql = null;
      do{
	try{
	  sql = in.readLine();
	  if(sql == null)
	    break;
	  stmt.execute(sql);
	  System.out.println("SQL: " + sql + " executed!");
	}catch(Exception e){
	  System.out.println("Exception: " + e.getMessage());
	}
      }while(sql!=null);
    }catch(Exception e){
      System.out.println("Exception: " + e.getMessage());
    }
  }


  /**
   * Added by Xinz<BR>
   * Reset the database
   */
    void resetDB(String filename){
        try {
            // Open the reset.sql file
            File f  = new File(filename);
            BufferedReader in = new BufferedReader( new FileReader(f));
            Statement stmt = connection.createStatement();
            String sql = null;
            do{
                try{
                    sql = in.readLine();
                    if(sql == null)
	    break;
                    stmt.execute(sql);
                    if (DEBUG) System.out.println("DBMAccess.resetDB() " + sql + " executed!");
                }catch(Exception e){
                    if (DEBUG) System.out.println("DBMAccess.resetDB() " + e.getMessage());
                }
            }while(sql!=null);
            stmt.close();
        }catch(Exception e){
      if (DEBUG) System.out.println("DBMAccess.resetDB() " + e.getMessage());
        }
    }

}

