////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.MW;

import EVEDemo.MW.*;
import DSRG.VKB.*;
import DSRG.MKB.*;
import java.util.*;

public class QueryEngine {
    /**
     * a handle to the Main class
     */
    private EVEDemo.MW.Main main;


    /** the debug flag */
    private static final boolean DEBUG = false;

    /**
     * the constructor takes the handle to the main class as
     * argument. This is the only connection to the outside world that
     * the query engine has */
    
    public QueryEngine(EVEDemo.MW.Main myMain) {
        main = myMain;
    }
    
    
    /**
     * This method computes the extent of a view by using the
     * queryBreakdown()  method.
     * @return Table for given View .  */

    public Table computeExtent(View v){
        // This method will have to call methods to connect to and query
        // each IS and then it will have to send intermediate relations
        // to middleware database for final computations.
        Vector isQueries;  // Vector of queries for IS's
        //    Vector InfoSources = new Vector();
        Table intermediateR;
        String reassemblyQuery = new String();
        int index = 0,
            i = 0,
            j = 0;

        //otherwise, compute view extent
        isQueries = breakdown_query(v);  // vector now full of ISQuery()'s
        
        
        // next we want to send these separate queries off
        // to their respective information sources.
        //System.err.println("HERE's What we get\n");
        
        
        for(i = 0; i <= isQueries.size()-1; i++){
            // query the database...
            if(DEBUG) {
                System.out.println("DEBUG: We send:"+ ((ISQuery)isQueries.elementAt(i)).getSQLStatement());
                System.out.println("DEBUG: To     :"+ ((ISQuery)isQueries.elementAt(i)).getName());
                
                // this requires deciding which element of the ISs vector we want.
                System.out.println("DEBUG: We have " + 
                                   main.getDBManagers().size() + 
                                   " DB Managers");
            }
            
            DBManager tmpDBM;

            for(j = 0; j < main.getDBManagers().size(); j++){
                tmpDBM = (DBManager)main.getDBManagers().elementAt(j);
                if (DEBUG) {
                    System.out.println("DEBUG: ISQuery - IS : "+((ISQuery)isQueries.elementAt(i)).getName());
                    System.out.println("DEBUG: ISs - IS : "+tmpDBM.getISName());
                }
                if (tmpDBM.getISName().equals(((ISQuery)isQueries.elementAt(i)).getName())){
                    index = j;
                    if (DEBUG) 
                        System.out.println("DEBUG: Query sent to IS No."+index);
                    //send query here
                    
                    intermediateR = (tmpDBM.executeQuery(((ISQuery)isQueries.elementAt(i)).getSQLStatement()));
                    intermediateR.setRelName("Rel_"+i);

                    // a complete intermediate relation is created in
                    // memory and then stored in a different IS. A
                    // black hole for performance...  no too useful to
                    // run performance experiments on this one...

                    if (!main.getLocalDBManager().storeRelation(intermediateR))
                        System.out.println("Intermediate relation " + i 
                                           + " could not be stored");  
                    // intermediate rel.names
                    // a la "Rel_2"
                    break;
                }
            }
            
        }
        

        if (DEBUG) {
            ISQuery isq;
            int ii=0;
            for (Enumeration e = isQueries.elements() ; e.hasMoreElements() ;) {
                isq = (ISQuery)(e.nextElement());
                System.out.println("SQL: "+isq.getSQLStatement());
            }
            System.out.println("=========================================");
        }
        
        
        // Now we reconstruct the intermediate tables into one final reliation
        // and we return this relation.
        
        reassemblyQuery = reassemble_query(v.getName());
        
        if (DEBUG) {
            System.out.println("DEBUG: reassemblyQuery: " + reassemblyQuery);
            System.out.println("=========================================");
        }
        
        
        //System.out.println("Debug:View.java end of Compute Extent!");
        intermediateR = main.getLocalDBManager().executeViewQuery( reassemblyQuery );
        intermediateR.setRelName(v.getName());
        return intermediateR;
        //     return null;
        
    }
    
            
    /**
   * This method creates the queries that are sent to each IS. <BR>
   * koeller 032999, we're replacing iterateClause with VKB-methods
   * (View_Definition-Object).<BR>
   * @return A Vector of ISQuery()'s.  */
    private Vector breakdown_query(View v){

        Vector ISQueryVector = new Vector();
        ISQuery tmpquery = null;
    
        String squery = new String();// Select
        String fquery = new String();// From
        String wquery = new String();// Where
    
        String wherestr    = new String();
        String left_attr   = new String();
        String right_attr  = new String();
        String isname      = new String();
        String prev_isname = new String();
        String relname     = new String();
        int i, j, old=0;

        // new algorithm 032999
        View_Definition        vd = main.getVKB().getVD(v.getName());
        Enumeration            newmf,newmf2,newms,newmw;
        Attribute_Component    tmpAttC;
        Relation_Component     tmpRelC,tmpRelC2;
        Condition_Component    tmpCondC;
        Attribute              tmpAttrL,tmpAttrR;
        Primitive_Value        tmpPV;

        /*
         * First we must cycle through the relations in the from clause.
         * For each relation, we create an ISQuery().
         * Exception: a relation belongs to the same IS as the previous one
         * (koeller, 6/2/98) <BR>
         * Bug (08/10/98): An IS that has been referenced in one relation
         * cannot be used again (i.e., ISs have to be sorted in the from-clause<BR>
         * Not sure if that one is fixed (koeller 300399)
         */

        newmf = vd.getClause(DSRG.VKB.View_Definition.FROM).elements();
        squery = ""; fquery = ""; wquery = "";
        prev_isname = "";
        
        while(newmf.hasMoreElements()){ // foreach IS.R in FROM
            tmpRelC = (Relation_Component)newmf.nextElement(); // read in the first IS.R



            // now we just want the ISName - statement of form ISName.RName
            
            isname = tmpRelC.getISName();
            relname = tmpRelC.getName();
            

            if (!isname.equals(prev_isname)) {
                // if not the same "IS" as the previous one
                // create isquery for PREVIOUS expression
                if (!fquery.equals("")) {
                    // System.out.println("DEBUG. View::queryBreakdown() squery= "+squery);
                    ISQueryVector.addElement(new ISQuery(prev_isname , squery,fquery,wquery));
                }
                squery = "";
                fquery = "";
                wquery = "";
            }
            
            // start over parsing the select clause
            // SELECT = ALL SELECT FOR IS
            newms = vd.getClause(DSRG.VKB.View_Definition.SELECT).elements();
            while(newms.hasMoreElements()){
                tmpAttC = (Attribute_Component)newms.nextElement(); // read in a select-attribute
                if(tmpAttC.getAttribute().getRelation() == tmpRelC.getRelation()) { // same Relation object (i.e. same IS!!)
                    if(squery.equals(""))
                        squery += tmpAttC.getAttribute().getNoIS();
                    else
                        squery += ",\n"+tmpAttC.getAttribute().getNoIS();
                }
            }

            // FROM = ALL FROM FOR THIS IS (isname)
            newmf2 = vd.getClause(DSRG.VKB.View_Definition.FROM).elements();
            while(newmf2.hasMoreElements()){ // foreach IS.R in FROM
                tmpRelC2 = (Relation_Component)newmf2.nextElement(); // read in the first IS.R
                
                
                if(tmpRelC2.getIS()==tmpRelC.getIS()
                   && (fquery.indexOf(tmpRelC2.getName()) == -1) ) {
                    
                    // if isname.relname found
                    // add relname to fquery, with comma
                    
                    if(fquery.equals(""))
                        fquery += tmpRelC2.getName();
                    else
                        fquery += ",\n"+tmpRelC2.getName();
                }
            }
            
            // where-clauses only once per IS (for the first relation that we find in an IS)

            if (!isname.equals(prev_isname)) {
                // Foreach element in WHERE
                newmw = vd.getClause(DSRG.VKB.View_Definition.WHERE).elements();

                while(newmw.hasMoreElements()){
                    tmpCondC = (Condition_Component)newmw.nextElement();
                    //System.out.println("WHERESTR: "+ wherestr);
                    //System.out.println("ISNAME: "+ isname);
                    //System.out.println(clause_type(wherestr));

                    tmpAttrL = tmpCondC.getCondition().getLeft();
                    tmpAttrR = null; tmpPV = null;

                    // we use (tmpPV==null) as test for local vs .join conditions
                    if (tmpCondC.getCondition() instanceof Join_Condition)
                        tmpAttrR = ((Join_Condition)tmpCondC.getCondition()).getRight();
                    else
                        tmpPV = ((Local_Condition)tmpCondC.getCondition()).getRight();
                    
                    // if the right-hand-side is a constant:
                    if( (tmpPV!=null) &&
                        (tmpAttrL.getRelation().getIS()==tmpRelC.getIS())){
                        if(wquery.equals("")){
                            wquery = ((Local_Condition)tmpCondC.getCondition()).getNoIS(); // where-condition without IS name
                        }else {
                            wquery += " AND \n"; // +wherestr.substring(0, pos)+"_"; A.: no
                            wquery += ((Local_Condition)tmpCondC.getCondition()).getNoIS();
                        }

                    }else if (tmpPV==null) { // then we have SOME join-condition
                        // if we have two relations on IS isname
                        // we have to remove the isname on both sides.
                        if (tmpAttrL.getIS()==tmpAttrR.getIS() &&
                            tmpAttrL.getIS()==tmpRelC.getIS()) {
                            // WHERE = ALL WHERE FOR IS
                            wherestr = ((Join_Condition)tmpCondC.getCondition()).getNoIS();
                            if(wquery.equals(""))
                                wquery += wherestr;
                            else {
                                wquery += " AND\n";
                                wquery += wherestr;
                            }
                        }
                        
                    }

                    if (tmpPV==null) {
                        if ( tmpAttrL.getIS()!=tmpAttrR.getIS()){
                            
                            // (((Join_Condition)tmpCondC.getCondition()).contains(tmpRelC.getIS()) )  
                            // )
                            // different ISs, but one of them is isname
                            // two relations on different ISs ==> include atts in select-clause
                            // but without ISname, and only if they don't already exist.
                            
                            left_attr = tmpAttrL.getNoIS();
                            if ( tmpAttrL.getIS()==tmpRelC.getIS() &&
                                  (squery.indexOf(left_attr) == -1) ) {
                                if (squery.equals(""))
                                    squery = left_attr;
                                else
                                    squery += ",\n"+left_attr;
                            }
                            right_attr = tmpAttrR.getNoIS();
                                if ( tmpAttrR.getIS()==tmpRelC.getIS()  &&
                                     (squery.indexOf(right_attr) == -1) ) {
                                    if (squery.equals(""))
                                        squery = right_attr;
                                    else
                                        squery += ",\n"+right_attr;
                                }
                        }
                    }
                    
                    
                }
            }
            
            // remember ISname for next loop through the from-clause
            prev_isname = isname;


        } // no more IS's
        // create last ISQuery
        ISQueryVector.addElement(new ISQuery(isname, squery,fquery,wquery));
        ISQueryVector.trimToSize(); // Let's not waste memory :)

        return(ISQueryVector);
    }



    /**
     * This method creates the reassembly query for the intermediate
     * DB from which this view is computed. <BR> Changed to use MKB
     * objects instead of iterateClause, koeller 300399<BR> we have a
     * bug: the reassembly-query has where-clauses without the
     * relation names.  so if we have a join between attributes with
     * the same name in relations with the same name, the query won't
     * work
     * @return A String with a SQL statement (standard SQL with SELECT/FROM/WHERE..)
     * @author koeller */
  public String reassemble_query(String viewName){
    String query = new String();

    String strbuf = new String();

    String squery = new String();// Select
    String fquery = new String();// From
    String wquery = new String();// Where

    String wherestr    = new String();
    String isname      = new String();
    String prev_isname = new String();
    String relname     = new String();
    int i, j, old=0;
    int pos;
    
    View_Definition vd = main.getVKB().getVD(viewName);
    Enumeration newmf, newmf2, newms, newmw;
    Attribute_Component tmpAttC;
    Relation_Component tmpRelC, tmpRelC2;
    Condition_Component tmpCondC;
    Attribute tmpAttrL, tmpAttrR;
    Primitive_Value tmpPV;
    String condOp; // the operator in a where-condition

    int relationIndex = 0;
    String relationName = new String();
    String compareRelation = new String();

    Vector selectAttrs = new Vector(); // to attach the names of
    // intermediate relations to attributes, needed since order of
    // attributes in the query is important


    /*
     * Same as before: First we must cycle through the relations in
     * the from clause.  For each relation, we have a new intermediate
     * relation name (contiguous numbers from 0) Exception: a relation
     * belongs to the same IS as the previous one (koeller, 6/3/98) */
    
    newmf = vd.getClause(DSRG.VKB.View_Definition.FROM).elements();
    squery = ""; fquery = ""; wquery = "";
    prev_isname = "";
    
    while(newmf.hasMoreElements()){ // foreach IS.R in FROM
        tmpRelC = (Relation_Component)newmf.nextElement();
        
        // now we just want the ISName - statement of form
        // ISName.RName.AttrName
        
        isname = tmpRelC.getISName();
        relname = tmpRelC.getName();
        
        if (!isname.equals(prev_isname)) {
            // if not the same "IS" as the previous one increment
            // relation counter (start new intermediate relation)
            relationName = "Rel_"+relationIndex;
            // relationName is the name of the current relation in the
            // intermediate (local) IS
            relationIndex++;      // next intermediate relation
        }
        
        // start over parsing the select clause
        // SELECT = ALL SELECT FOR IS
        newms = vd.getClause(DSRG.VKB.View_Definition.SELECT).elements();
        while (newms.hasMoreElements()) {
            
            tmpAttC = (Attribute_Component)newms.nextElement(); // read in a SELECT-Attribute
            if(tmpAttC.getAttribute().getRelation() 
               == tmpRelC.getRelation()) { 
                // Attribute is in current Relation (and therefore same
                // IS) temporarily collect attributes in
                // selectAttrs. their order may not be correct, so just
                // remember them and sort them later according to the
                // select-clause
              selectAttrs.addElement(new String(relationName + "." + 
                                                tmpAttC.getAttribute().getRelation().getName() + "_" +
                                                tmpAttC.getName())); // Format: Rel0.MASSBRANCH_STREET
            }
        }
        
        // from-and where-clauses only once per IS
        
        if (!isname.equals(prev_isname)) {
            
            // FROM = ALL FROM FOR THIS IS (isname)
            
            
            if(fquery.equals(""))
                fquery += relationName;
            else
                fquery += ","+relationName;
        
            
            // WHERE clause
            // Foreach element in WHERE
            newmw = vd.getClause(DSRG.VKB.View_Definition.WHERE).elements();
            
            // note: in the Condition-Class, where-conditions are always sorted to have
            // <= or < operator.
            while(newmw.hasMoreElements()){
                tmpCondC = (Condition_Component)newmw.nextElement();
                condOp = tmpCondC.getCondition().getOperator().toString();
                tmpAttrL = tmpCondC.getCondition().getLeft();
                tmpAttrR = null; tmpPV = null;
                
                if (tmpCondC.getCondition() instanceof Join_Condition)
                    tmpAttrR = ((Join_Condition)tmpCondC.getCondition()).getRight();
                else tmpPV = ((Local_Condition)tmpCondC.getCondition()).getRight();
                
                // if we have two relations on different ISs..
                if( tmpPV==null && // join condition
                    tmpAttrL.getIS() != tmpAttrR.getIS() && // external Join
                    (tmpAttrL.getIS()==tmpRelC.getIS() || tmpAttrR.getIS()==tmpRelC.getIS()) ) 
                    { // we are in the right IS
                        // wherestr is the where-condition, with
                        // "Rel_Attr" in the where-clause. BUG: we don't
                        // have the names of the intermediate relations
                        // here, so we may have problems when we have two
                    // tables with the same name, joined on the same
                        // attribute, in two different ISs
                        wherestr = tmpAttrL.getRelation().getName()+"_"+tmpAttrL.getName()
                            + condOp
                            + tmpAttrR.getRelation().getName()+"_"+tmpAttrR.getName();
                        if (wquery.indexOf(wherestr.trim())==-1) {
                            // if this where clause doesn't exist yet
                            if(wquery.equals(""))
                                wquery = wherestr;
                            else
                                wquery += " AND\n"+wherestr;
                        }
                    }
            }
        }
        
      // if the NEXT isname is different from the current one, create a ISQuery
        prev_isname = isname;
        
        
    } // no more IS's
    // create last ISQuery
    
    
    // start over parsing the select clause
    // we need to preserve the order from the view definition
    
    Enumeration selectEnum = null;
    String tmpAttCString;
    newms = vd.getClause(DSRG.VKB.View_Definition.SELECT).elements();
    while (newms.hasMoreElements()) {
        tmpAttC = (Attribute_Component)newms.nextElement(); // read in a SELECT-Attribute
        tmpAttCString = tmpAttC.getAttribute().getRelation().getName()+"_"+tmpAttC.getName();
        selectEnum = selectAttrs.elements();
        // now I AM re-parsing again :) Sorry...  I'm still trying to
        // sort the attributes. The selectEnum vector is sorted
        // according to the Select-Clause of the query
        while (selectEnum.hasMoreElements()) {
            strbuf = ((String)selectEnum.nextElement());
            pos = strbuf.indexOf("."); // the stuff after the local
                                       // Relationname (MASSBRANCH_STREET)
            if(strbuf.substring(pos+1).equals(tmpAttCString)) { 
                // we found the attribute that contains RelName_AttrName
                if(squery.equals(""))
                    squery += strbuf;  // . ==> _
                else
                    squery += ",\n" + strbuf;
            }
        }
    }
    
    
    query = "SELECT " + squery + "\n" +
        " FROM " + fquery + "\n";
    if (!wquery.equals("")) query  += " WHERE " + wquery + "\n";
    if (EVEDemo.MW.Main.DEBUG) 
        System.out.println("DEBUG. View::reassemble_query. Combined Query: "
                           +query);
    return(query);
  }

  //================ end of View extent recomputation ==========

}
