////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.VKB;

import DSRG.MKB.Relation;
import DSRG.MKB.Attribute;

/**
* Represents the Attribute component in the View Definition. It's a part of
* SELECT clause.
*/
public class Attribute_Component extends View_Component implements java.io.Serializable, Cloneable
{
  /**
  * Reference of this Attribute. It stored in MKB.
  */
  Attribute attr;

  /**
  * get Name.
  * It should return "A"
  */
  public String getName(){
    return attr.getName();
  }

  /**
  * Get Reference of Attribute stored in MKB.
  */
  public Attribute getAttribute(){
    return attr;
  }

  /**
  * Set Attribute of this component.
  */
  public void setAttribute(Attribute ref){
    attr = ref;
  }

  /**
  * Display this attribute component in following format:
  * <attribute> ["(" ("AD" = "TRUE" [ "AR" = "TRUE" ] | "AR" = "TRUE" ) ")"]
  */
  public String toString(){
    return attr.getFullName() + displayEvolvingParameter("AD", "AR");
  }
   
    /**
     * Clone the Attribute_Component.
     */
    public Object clone(){
	Attribute_Component ac = new Attribute_Component();
	ac.attr = attr;
	ac.Dispensable = Dispensable;
	ac.Replaceable = Replaceable;

	if(EVEDemo.MW.Main.DEBUG)System.out.println("Attribute_Component.clone()");
	return ac;
    }

}




