////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.VKB;

import DSRG.MKB.Attribute;
import DSRG.MKB.Relation;

import java.io.Serializable;

/**
* Abstract class of Local_Condition and Join_Conditon.
* <p>
* It has the common part: Operator, and left attribute. It follows the rules:
* <br>
* It means Left one is attribute, right one could be a number of another attribute
* If that is another attribute, it sort in a way that operator is "<" or "<=". 
* If the operator is "==" or "!=", we sorted in the alphabet order.
*/
public abstract class Condition implements java.io.Serializable, Cloneable
{
  /**
  * Operator of condition
  */
  Operator op;

  /**
  * First operand
  */
  Attribute left;

  /**
  * Set operator
  */
  public void setOperator(Operator oper){
    op = oper;
  }

  /**
  * Get operator
  */
  public Operator getOperator(){
    return op;
  }

  /**
  * Set Attribute
  */
  public void setAttribute(Attribute attr){
    left = attr;
  }

  /**
  * Get Left Attribute Name in format "IS.R.A"
  */
  public String getLeftName(){
    return left.getName();
  }

  /**
  * Return left attribute.
  */
  public Attribute getLeft(){
    return left;
  }

  /**
  * Abstract function to set second operand
  */
  public abstract void setOperand(Object right);

  /**
  * Abstract function return String
  */
  public abstract String toString();

  /**
  * Abstract function to check if this condition contains specific attribute
  */
  public abstract boolean contains(String attr_name);

  /**
  * Abstract function to check if this condition contains specific relation
  */
  public abstract boolean contains(Relation rel);

  /**
  * Abstract function to get String of Condition with out IS info in it.
  */
  public abstract String getNoIS();

    /**
     * Abstract function to clone() a condition.
     */
    public abstract Object clone();
}

