////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.VKB;

import DSRG.MKB.Attribute;

import java.util.Vector;

/**
* Condition View component. It's a part of WHERE clause.
* All the condition here, should be sorted in one direction.
* <p>
* It means Left one is attribute, right one could be a number of another attribute
* If that is another attribute, it sort in a way that operator is "<" or "<=".
* If the operator is "==" or "!=", we sorted in the alphabet order.
*/
public class Condition_Component extends View_Component implements java.io.Serializable, Cloneable
{
  Condition cond;

  /**
  * Default Construction
  */
  public Condition_Component(){
    super();
    cond = null;
  }

  /**
  * Construct a Condition Component from an condition.
  * The default parameters are all true.
  * @param cd The condition of this component.
  */
  public Condition_Component(Condition cd){
    this();
    cond = cd;
    Replaceable = true;
    // make is indispensable for the reason of QC stuff.
    Dispensable = false;
  }

  /**
  * Set Condition. Could be Local_Condition or Join_Condition
  */
  public void setCondition(Condition c){
    cond = c;
  }

  /**
  * get Condition
  */
  public Condition getCondition(){
    return cond;
  }

  /**
  * Display this condition component
  */
  public String toString(){
    return cond + displayEvolvingParameter("CD", "CR");
  }

  /**
  * Get Relations of this condition components
  * @return A vector of relations, at most two.
  */
  public Vector getRelations(){
    Vector result = new Vector();

    result.addElement(cond.getLeft().getRelation());

    if(cond instanceof Join_Condition)
      result.addElement(((Join_Condition)cond).getRight().getRelation());

    return result;
  }

    /**
     * Clone the Condition_Component.
     */
    public Object clone(){
	Condition_Component cc = new Condition_Component();
	cc.cond = (Condition)cond.clone();
	cc.Dispensable = Dispensable;
	cc.Replaceable = Replaceable;

	if(EVEDemo.MW.Main.DEBUG)System.out.println("Condition_Component.clone()");
	return cc;
    }

}

