////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.VKB;

import java.util.Vector;
import java.util.Enumeration;

import DSRG.MKB.Attribute;
import DSRG.MKB.Relation;

import java.lang.reflect.*;
/**
* A vector stores all the conditions.
*/
public class Condition_index extends Vector implements java.io.Serializable
{
  /**
  * Default constructor of Condition_index.
  */
  public Condition_index(){
    super();
  }

  /**
  * Construct a Condition_index from Vector
  */
  public Condition_index(Vector v){
    Enumeration enum = v.elements();
    while(enum.hasMoreElements()){
      addElement(enum.nextElement());
    }
  }

  /**
  * Get all Condition Components of one Relation
  * @param rel Reference of the relation.
  * @return Vector of Condition Components of such Relation or empty Vector
  */
  public Vector getCondComponents(Relation rel){
    Vector result = new Vector();

    Enumeration enum = elements();

    while(enum.hasMoreElements()){
      Condition_Component cc = (Condition_Component)enum.nextElement();

      Condition c = cc.getCondition();

      if(c.contains(rel))
        result.addElement(cc);
    }

    return result;
  }

  /**
  * Get Condition Components affected by specific Attribute.
  * @param attr_name attribute name in format "IS.R.A"
  * @return if there is any, return a vector of c-comp; otherwise return null.
  */
  public Vector getCondComponents(String attr_name){
    Vector conds = new Vector();

    Enumeration enum = elements();

    while(enum.hasMoreElements()){
      Condition_Component cc = (Condition_Component)enum.nextElement();

      Condition c = cc.getCondition();

      if(c.contains(attr_name))
        conds.addElement(cc);
    }

    return conds;

  }

  /**
  * Get Conditions affected by specific Attribute.
  * @param attr_name attribute name in format "IS.R.A"
  * @return if there is any, return a vector; otherwise return null.
  */
  public Vector getConditions(String attr_name){
    Vector conds = new Vector();

    Enumeration enum = elements();

    while(enum.hasMoreElements()){
      Condition_Component cc = (Condition_Component)enum.nextElement();

      Condition c = cc.getCondition();

      if(c.contains(attr_name))
        conds.addElement(c);
    }
    if(conds.size()>0)
      return conds;
    else
      return null;
  }

  /**
  * Get Join Conditions between two specific Relations.
  * @param rel1 name of one of two relations.
  * @param rel2 name of the other relations.
  * @return if there is any, return a vector; otherwise return an empty vector.
  */
  public Vector getJoinConditions(Relation rel1, Relation rel2){
    Vector jconds = new Vector();

    Enumeration enum = elements();

    while(enum.hasMoreElements()){
      Condition c = ((Condition_Component)enum.nextElement()).getCondition();

      if(c instanceof Join_Condition){
        Join_Condition jc = (Join_Condition)c;
        if(jc.contains(rel1) && jc.contains(rel2))
          jconds.addElement(jc);
      }
    }
    return jconds;
  }

  /**
   * Test if the Condition_Index contains a specific Join_Condition.
   * Here, we don't compare the reference, we will compare the content.
   * We will use the equals function of Join_Condition to test if two
   * JCs are same. ( current, the equal function cannot test the order. )
   */
  public boolean contains(Join_Condition jc){
    // 1. we get all the conditions out of the where clause
    Enumeration enum = elements();
    while(enum.hasMoreElements()){
      Condition_Component cc = (Condition_Component)enum.nextElement();
      Condition c = cc.getCondition();
      // 2. if is join condition
      if(c instanceof Join_Condition){
	// 3. if there is one eqauls
	if(jc.equals((Join_Condition)c))
	  // 4. return true
	  return true;
      }
    }
    // 5. otherwise, return false.
    return false;
  }

  /**
  * DeepClone a Condition_index
  */
    public Condition_index deepClone(){

	Condition_index result = new Condition_index();

	for(int i = 0; i< size(); i++){
	    Object obj = elementAt(i);
	    if(obj instanceof Vector)
		result.addElement(DSRG.VSA.VSA.deepClone((Vector)obj));
	    else
		try{
		    result.addElement(obj.getClass().getDeclaredMethod("clone",null).invoke(obj, null));
		}catch(NoSuchMethodException nsme){
		    result.addElement(obj);
		    //System.out.println("VSA:deepClone one element cannot be cloned: " + nsme.getMessage());
		}catch(IllegalAccessException iae){
		    result.addElement(obj);
		    //System.out.println("VSA:deepClone one element cannot be clones: " + iae.getMessage());
		}catch(InvocationTargetException ite){
		    result.addElement(obj);
		    //System.out.println("VSA:deepClone one element cannot be clones: " + ite.getMessage());
		}
	}

	return result;
    }

}
