////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
/* Generated By:JJTree: Do not edit this line. SimpleNode.java */

package DSRG.VKB.E_SQL;

import DSRG.MKB.MKB;
import DSRG.MKB.Attribute;
import DSRG.MKB.Relation;

import DSRG.VKB.VKB;
import DSRG.VKB.Primitive_Value;
import DSRG.VKB.View_Definition;
import DSRG.VKB.Condition;
import DSRG.VKB.Local_Condition;
import DSRG.VKB.Join_Condition;
import DSRG.VKB.Condition_index;
import DSRG.VKB.Attribute_Component;
import DSRG.VKB.Relation_Component;
import DSRG.VKB.Condition_Component;
import DSRG.VKB.Operator;

import java.util.Vector;
import java.util.Hashtable;
import java.io.StringBufferInputStream;

/**
* By browsing the syntax tree of SimpleNode, we can create two objects:
* <ul>
*   <li> An stub MKB which will store the IS info. of this ESQL
*   <li> An VKB/View_Definition which represents this ESQL
* </ul>
*/
public class SimpleNode implements Node {
  protected Node parent;
  public Node[] children;
  protected int id;
  protected E_SQL parser;

  /**
  * Name of this Node
  */
  public String name;
  /**
  * Boolean value of this Node
  */
  public boolean bool_value;

  /**
  * Information Source Name of (Relation_name, Attribute_name) node
  */
  public String IS_name;
  /**
  * Relation Name of (Relation_name, Attribute_name) node
  */
  public String rel_name;
  /**
  * Attribute Name of (Attribute_name) node
  */
  public String attr_name;

  /**
  * Type of Value node
  */
  public int type;
  public static final int STRING = 0;
  public static final int DOUBLE = 1;
  public static final int LONG = 2;
  public static final int BOOLEAN = 3;

  /**
  * has_dispensable of (Attribute, Relation, Condition) component
  */
  public boolean has_dispensable;
  /**
  * has_replaceable of (Attribute, Relation, Condition) component
  */
  public boolean has_replaceable;
  /**
  * has_condition for WHERE clause of view definition. Because WHERE
  * clause of one view definition is optional.
  */
  public boolean has_condition;
  /**
  * Has VExtent is used to show whether a VD has VE parameter or not.
  */
  public boolean hasVExtent;

  public SimpleNode(int i) {
    id = i;
    name = null;
    has_dispensable = false;
    has_replaceable = false;
    has_condition = false;
    bool_value = false;
    hasVExtent = false;
  }

  public SimpleNode(E_SQL p, int i) {
    this(i);
    parser = p;
  }

  public void jjtOpen() {
  }

  public void jjtClose() {
  }

  public void jjtSetParent(Node n) { parent = n; }
  public Node jjtGetParent() { return parent; }

  public void jjtAddChild(Node n, int i) {
    if (children == null) {
      children = new Node[i + 1];
    } else if (i >= children.length) {
      Node c[] = new Node[i + 1];
      System.arraycopy(children, 0, c, 0, children.length);
      children = c;
    }
    children[i] = n;
  }

  public Node jjtGetChild(int i) {
    return children[i];
  }

  public int jjtGetNumChildren() {
    return (children == null) ? 0 : children.length;
  }

  /* You can override these two methods in subclasses of SimpleNode to
     customize the way the node appears when the tree is dumped.  If
     your output uses more than one line you should override
     toString(String), otherwise overriding toString() is probably all
     you need to do. */

  public String toString() {
    if(name == null)
      return E_SQLTreeConstants.jjtNodeName[id];
    else
      return name;
  }

  public String toString(String prefix) { return prefix + toString(); }

  /* Override this method if you want to customize how the node dumps
     out its children. */

  public void dump(String prefix) {
    System.out.println(toString(prefix));
    if (children != null) {
      for (int i = 0; i < children.length; ++i) {
        SimpleNode n = (SimpleNode)children[i];
        if (n != null) {
          n.dump(prefix + " ");
        }
      }
    }
  }

  // Following functions are used to create stub MKB class
  /**
  * Create a MKB object from the Parser Tree.
  *<BR> XIN: Why do we have this?
  */
  public void createMKB(MKB mkb){

      System.err.println("createMKB called. Why?????");

    if(mkb==null){
      System.out.println("SimpleNode.createMKB() mkb is null");
      return;
    }

    switch(id){
      case E_SQLTreeConstants.JJTACOMPONENT:
        mkb.addAttribute(((SimpleNode)children[0]).IS_name,
                           ((SimpleNode)children[0]).rel_name,
                           ((SimpleNode)children[0]).attr_name,
                            "no type");
        return;

      case E_SQLTreeConstants.JJTRCOMPONENT:
        mkb.addRelation(((SimpleNode)children[0]).IS_name,
                         ((SimpleNode)children[0]).rel_name,null,null);
        return;

      case E_SQLTreeConstants.JJTPCLAUSE:

        mkb.addAttribute(((SimpleNode)children[0]).IS_name,
                           ((SimpleNode)children[0]).rel_name,
                           ((SimpleNode)children[0]).attr_name,"no type");

        if(((SimpleNode)((SimpleNode)children[2]).children[0]).id == E_SQLTreeConstants.JJTANAME){
          // if It's a Join Condition
          mkb.addAttribute(((SimpleNode)((SimpleNode)children[2]).children[0]).IS_name,
                   ((SimpleNode)((SimpleNode)children[2]).children[0]).rel_name,
                   ((SimpleNode)((SimpleNode)children[2]).children[0]).attr_name, "no type");
        }
        return;
    }

    if(children == null)
      return;

    for( int i = 0; i < children.length; i++){
      ((SimpleNode)children[i]).createMKB(mkb);
    }

    return;
  }

  // Following functions are used to create View_Definition and VKB class

  /**
  * Global MKB of current vkb used in following functions
  */
  static MKB mkb;

  /**
  * Create a VKB object from the Parser Tree.
  * @param vd reference of VKB needed to be created by this parser
  * @param ref reference of MKB which parser based on.
  */
  public void createVKB(VKB vkb, MKB ref){
    if(vkb==null){
      System.out.println("SimpleNode.createVKB() vkb is null");
      return;
    }

    if (id != E_SQLTreeConstants.JJTVDSEQUENCE){
      System.out.println("SimpleNode:createVKB Wrong type of node");
      return;
    }

    mkb = ref;

    for( int i = 0; i < children.length; i++){
      View_Definition vd = new View_Definition();
      ((SimpleNode)children[i]).createViewDefinition(vd, mkb);
      vkb.addViewDefinition(vd);
    }

    return;
  }

  /**
  * Create a View_Definition Object from the Parser Tree.
  * @param vd reference of view defintion needed to be created by this parser
  * @param ref reference of MKB which parser based on.
  */
  public void createViewDefinition(View_Definition vd, MKB ref) {
    if(vd==null){
      System.out.println("SimpleNode.createViewDefinition() vd is null");
      return;
    }

    if (id != E_SQLTreeConstants.JJTVDEFINITION){
      System.out.println("SimpleNode:createViewDefinition Wrong type of node");
      return;
    }

    mkb = ref;

    // Set the View name
    vd.setName(((SimpleNode)children[0]).name);

    int i = 1;// temporary index of children

    if(hasVExtent)
      // Set the View extent
      vd.setVExtent(((SimpleNode)children[i]).createVExtent());
    else
      i = 0; // otherwise, we don't have this children,
             // we set it as default value (should be EQUIVALENT).

    // Set select clause
    vd.setSelect(((SimpleNode)children[i+1]).createSelectClause());

    // Set from clause
    vd.setFrom(((SimpleNode)children[i+2]).createFromClause());

    if(has_condition){
      // Set where clause
      vd.setWhere(((SimpleNode)children[i+3]).createWhereClause());
    }
  }

  /**
  * Create the ViewExtent parameter of View Definition.
  * The parameter could be "SUBSET", "SUPERSET", "EQUIVALENT", and "DONTCARE"
  */
  int createVExtent(){
    if (id != E_SQLTreeConstants.JJTVEXTENT){
      System.out.println("SimpleNode.createVExtent Wrong type of node");
      // I return Dontcare to increase error torlerance.
      return View_Definition.DONTCARE;
    }

    if (name.equals("SUBSET"))
      return View_Definition.SUBSET;
    else if(name.equals("SUPERSET"))
      return View_Definition.SUPERSET;
    else if(name.equals("EQUIVALENT"))
      return View_Definition.EQUIVALENT;
    else if(name.equals("DONTCARE"))
      return View_Definition.DONTCARE;
    else{
      System.out.println("SimpleNode.createVExtent Unknown VE parameter:" + name);
      return View_Definition.DONTCARE;
    }
  }

  /**
  * Create Select Clause object
  */
  Vector createSelectClause(){

    if (id != E_SQLTreeConstants.JJTSCLAUSE){
      System.out.println("SimpleNode.createSelectClause Wrong type of node");
      return null;
    }

    Vector clause = new Vector();

    for( int i = 0; i < children.length; i++){
      Attribute_Component ac = ((SimpleNode)children[i]).createAttributeComponent();
      clause.addElement(ac);
    }

    return clause;
  }

  /**
  * Create Attribute Component object
  */
  public Attribute_Component createAttributeComponent(){

    if (id != E_SQLTreeConstants.JJTACOMPONENT){
      System.out.println("SimpleNode.createAttributeComponent Wrong type of node");
      return null;
    }

    Attribute_Component ac = new Attribute_Component();

    // Get Attribute from mkb Stub Function
    Attribute attr = mkb.findAttribute(((SimpleNode)children[0]).IS_name,
                           ((SimpleNode)children[0]).rel_name,
                           ((SimpleNode)children[0]).attr_name);
if (attr==null)System.out.println(((SimpleNode)children[0]).IS_name+"."+
                           ((SimpleNode)children[0]).rel_name+"."+
                           ((SimpleNode)children[0]).attr_name);
    // Set Attribute reference in the attribute component.
    ac.setAttribute(attr);

    // child index
    int i = 1;

    // set Dispensable
    if(has_dispensable){
      ac.setDispensable(((SimpleNode)((SimpleNode)children[i]).children[0]).bool_value);
      i ++;
    }

    // set Replaceable
    if(has_replaceable)
      ac.setReplaceable(((SimpleNode)((SimpleNode)children[i]).children[0]).bool_value);

    return ac;
  }

  /**
  * Create From Clause object
  */
  Vector createFromClause(){

    if (id != E_SQLTreeConstants.JJTFCLAUSE){
      System.out.println("SimpleNode.createFromClause Wrong type of node");
      return null;
    }

    Vector clause = new Vector();

    for( int i = 0; i < children.length; i++){
      Relation_Component rc = ((SimpleNode)children[i]).createRelationComponent();
      clause.addElement(rc);
    }

    return clause;
  }

  /**
  * Create Relation_Component Object
  */
  public Relation_Component createRelationComponent(){

    if (id != E_SQLTreeConstants.JJTRCOMPONENT){
      System.out.println("SimpleNode.createRelationComponent Wrong type of node");
      return null;
    }

    Relation_Component rc = new Relation_Component();

    // Get Attribute from mkb Stub Function
    Relation rel = mkb.findRelation(((SimpleNode)children[0]).IS_name,
                         ((SimpleNode)children[0]).rel_name);

    // Set Attribute reference in the attribute component.
    rc.setRelation(rel);

    // child index
    int i = 1;

    // set Dispensable
    if(has_dispensable){
      rc.setDispensable(((SimpleNode)((SimpleNode)children[i]).children[0]).bool_value);
      i ++;
    }

    // set Replaceable
    if(has_replaceable)
      rc.setReplaceable(((SimpleNode)((SimpleNode)children[i]).children[0]).bool_value);

    return rc;
  }

  /**
  * Create Where Clause
  */
  Condition_index createWhereClause(){

    if (id != E_SQLTreeConstants.JJTWCLAUSE){
      System.out.println("SimpleNode.createWhereClause Wrong type of node");
      return null;
    }

    Condition_index clause = new Condition_index();

    for( int i = 0; i < children.length; i++){
      Condition_Component cc = ((SimpleNode)children[i]).createConditionComponent();

      Condition cond = cc.getCondition();

      clause.addElement(cc);
    }

    return clause;
  }

  /**
  * Create Condition Component
  */
  public Condition_Component createConditionComponent(){

    if (id != E_SQLTreeConstants.JJTCCOMPONENT){
      System.out.println("SimpleNode.createConditionComponent Wrong type of node");
      return null;
    }

    Condition_Component cc = new Condition_Component();

    cc.setCondition(((SimpleNode)children[0]).createCondition());

    // child index
    int i = 1;

    // set Dispensable
    if(has_dispensable){
      cc.setDispensable(((SimpleNode)((SimpleNode)children[i]).children[0]).bool_value);
      i ++;
    }

    // set Replaceable
    if(has_replaceable)
      cc.setReplaceable(((SimpleNode)((SimpleNode)children[i]).children[0]).bool_value);

    return cc;
  }

  /**
  * Create Condition from Primitive_clause
  */
  public Condition createCondition(){

    if (id != E_SQLTreeConstants.JJTPCLAUSE){
      System.out.println("SimpleNode.createCondition Wrong type of node");
      return null;
    }

    Condition cond;

    if(((SimpleNode)((SimpleNode)children[2]).children[0]).id == E_SQLTreeConstants.JJTANAME)
      // if It's a Join Condition
      cond = new Join_Condition();
    else if(((SimpleNode)((SimpleNode)children[2]).children[0]).id == E_SQLTreeConstants.JJTVALUE)
      // if It's a Local Condition
      cond = new Local_Condition();
    else {
      System.out.println("SimpleNode.createCondition Wrong type of node");
      return null;
    }

    Attribute attr = mkb.findAttribute(((SimpleNode)children[0]).IS_name,
                           ((SimpleNode)children[0]).rel_name,
                           ((SimpleNode)children[0]).attr_name);

    cond.setAttribute(attr);

    cond.setOperator(((SimpleNode)children[1]).createOperator());

    if(cond instanceof Join_Condition){
      // it's join condition
      // second operand must be an attribute
      attr = mkb.findAttribute(((SimpleNode)((SimpleNode)children[2]).children[0]).IS_name,
                   ((SimpleNode)((SimpleNode)children[2]).children[0]).rel_name,
                   ((SimpleNode)((SimpleNode)children[2]).children[0]).attr_name);
      cond.setOperand(attr);
    }
    else{
      // it's local condition
      // second operand must be a value
      cond.setOperand(((SimpleNode)((SimpleNode)children[2]).children[0]).createValue());
    }

    return cond;
  }

  /**
  * Create Operator object
  */
  public Operator createOperator(){

    if (id != E_SQLTreeConstants.JJTOP){
      System.out.println("SimpleNode.createOperator Wrong type of node");
      return null;
    }

    Operator op = new Operator(name);

    return op;
  }

  /**
  * Create Value object.
  */
  Primitive_Value createValue(){
    if(id != E_SQLTreeConstants.JJTVALUE){
      System.out.println("SimpleNode.createValue Wrong type of node");
      return null;
    }

    Primitive_Value value = null;

    switch(type){
      case STRING:
        value = new Primitive_Value(name);
        break;
      case DOUBLE:
        value = new Primitive_Value(new Double(name).doubleValue());
        break;
      case LONG:
        value = new Primitive_Value(new Long(name).longValue());
        break;
      case BOOLEAN:
        value = new Primitive_Value(((SimpleNode)children[0]).bool_value);
        break;
      default:
        System.out.println("SimpleNode.createValue Wrong value type");
        return null;
    }

    return value;
  }
}

