////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.VKB;

import DSRG.MKB.Attribute;
import DSRG.MKB.Relation;
import DSRG.MKB.IS;
/**
* Join Condition of view Definition. It's a part of WHERE clause. It will
* help on query break down.
* <p>
* It will confirm the op is in this direction "<", "<=", if the op is
* "==" and "!=", the two attribute with smaller value on the left.
*/

public class Join_Condition extends Condition implements Cloneable // because Condition implements Serializable.
{
  /**
  * Second Operand
  */
  Attribute right;

  /**
  * Default Constructor.
  */
  public Join_Condition(){
    super();
    right = null;
  }

  /**
  * Constructor of Join_Conditon. Construct a JC like " A = B" from two attributes.
  * @param la left attribute
  * @param ra right attribute.
  */
  public Join_Condition(Attribute la, Attribute ra){
    left = la;
    right = ra;
    op = new Operator("=");
  }

  /**
  * Get right attribute name in format "IS.R.A"
  */
  public String getRightName(){
    return right.getFullName();
  }

  /**
  * Set Second Operand
  * @param attr Object of Attribute;
  */
  public void setOperand(Object attr){
    right = (Attribute)attr;
  }

  /**
  * convert join-condition to string
  */
  public String toString(){
    if(op == null)
      System.out.println("Join_Condition.toString() op is null!");
    if(left == null)
      System.out.println("Join_Condition.toString() left is null!");    
    if(right == null)
      System.out.println("Join_Condition.toString() right is null!");    

    return left.getFullName()+op.toString()+right.getFullName();
  }

  /**
   * Check if this join -conditoin is equivalent with another join 
   * Condition.
   * This compare will not considering the directions of the Join
   * Condition, that means: A=B is not same as B=A
   */
  public boolean equals(Join_Condition jc){
    if(toString().equals(jc.toString()))
      return true;
    else
      return false;
  }

  /**
  * check if specific attribute in this condition
  * @param attr_name in format "IS.R.A"
  * @return true: contains; false: not contains.
  */
  public boolean contains(String attr_name){
    if(attr_name == null)
      return false;

    if(attr_name.equals(left.getFullName()))
      return true;

    if(attr_name.equals(right.getFullName()))
      return true;

    return false;
  }

  /**
  * Check if specific relation in this condition.
  * @param Rel the specific Relation.
  * @return true: contains; false: not contains.
  */
  public boolean contains(Relation rel){
    if(rel == left.getRelation())
      return true;

    if(rel == right.getRelation())
      return true;

    return false;
  }

  /**
  * Check if specific IS in this condition.
  * @param IS the specific IS.
  * @return true: contains; false: not contains.
  */
  public boolean contains(IS is){
    if(is == left.getRelation().getIS())
      return true;

    if(is == right.getRelation().getIS())
        return true;
    
    return false;
  }

    /**
     * Get right attribute.  */
    public Attribute getRight(){
        return right;
    }
    
    /**
     * Set right attribute.
     */
    public void setRight(Attribute attr){
    right = attr;
  }

  /**
  * Function to get String of Condition with out IS info in it.
  */
  public String getNoIS(){
    return left.getNoIS()+op.toString()+right.getNoIS();
  }

  /**
   * Test if the left attribute same as the right attribute.
   * JC will not be valid if the left = right.
   */
  public boolean valid(){
    if(left == right)
      return false;
    else
      return true;
  }

    /**
     * Clone Join Condition.
     */
    public Object clone(){
	Join_Condition jc = new Join_Condition(left, right);
	jc.op = op;
	return jc;
    }
}
