////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.VKB;

import DSRG.MKB.Relation;

/**
* Local Condition appread in the SELECT clause. It will help us do query break 
* down. The attribute is at left.
*/
public class Local_Condition extends Condition implements Cloneable // Because Condition implements Serialized
{
  /**
  * Second Operand
  */
  Primitive_Value right;

  /**
  * Get Primitive_Value
  */
  public Primitive_Value getRight(){
    return right;
  }

  /**
  * Set operand.
  * @param pv Object of Primitive_Value
  */
  public void setOperand(Object pv){
    right = (Primitive_Value)pv;
  }

  /**
  * convert local_condition to string
  */
  public String toString(){
    return left.getFullName() + op.toString() + right.toString();
  }

  /**
  * check if specific attribute in this condition
  * @param attr_name in format "IS.R.A"
  * @return true: contains; flase: not contains.
  */
  public boolean contains(String attr_name){
    if(attr_name == null)
      return false;

    if(attr_name.equals(left.getFullName()))
      return true;

    return false;
  }

  /**
  * Check if specific relation in this condition.
  * @param Rel the specific Relation.
  * @return true: contains; false: not contains.
  */
  public boolean contains(Relation rel){
    if(rel == left.getRelation())
      return true;

    return false;
  }

  /**
  * Function to get Condition with out IS information.
  */
  public String getNoIS(){
    return left.getNoIS() + op.toString() + right.toString();
  }

    /**
     * Clone without deep clone the attributes.
     */
    public Object clone(){
	Local_Condition lc = new Local_Condition();
	lc.left = left;
	lc.op = op;
	lc.right = right;
	return lc;
    }
}
