////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.VKB;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.io.StringBufferInputStream;
import java.io.File;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import DSRG.VKB.E_SQL.E_SQL;
import DSRG.VKB.E_SQL.SimpleNode;
import DSRG.VKB.E_SQL.ParseException;

import DSRG.MKB.MKB;
import DSRG.MKB.Attribute;

import DSRG.VSA.VSA_VKB;

/**
* This VKB class represents the VKB of EVE system.  <br> It stores a
* hashtable of View_index.  <br> We find the view definitiosn by name
* is by traverse <p> List of Functions VKB should provide <ul> <li>
* VKB Overall Functions </li> <ul> <li> Add one View Definition </li>
* <li> remove one View Definition </li> <li> Search View Definition by
* View Name </li> <li> Search View Definitions by IS name </li> <li>
* Search View Definitions by Relation name </li> <li> Search View
* Definitions by Attribute name </li> <li> Modify View Component of
* Specifice View Definition. We should provide all the functions of
* getting and setting the View Component attributes. </li> <li> Bulk
* Loading a VKB from a ESQL/SQL file.  </ul> <li> View Definition
* Overall Functions, please reference View_Definition Document </li>
* </ul> <p> VKB should help VSA in the view synchronization. Now, in
* VSA we can do following schema evolutions: <ul> <li> Delete Relation
* <li> Delete Attribute <li> Change Relation Name <li> Change
* Attribute Name </ul> I put some feature to MKB to let it help us do
* the Change Name functions. I do that by putting a reference of that
* object in MKB at every place where need a name. That's makes the
* changing name functions simple, but that increase the coupling
* between MKB and VKB. The reason why I select this strategy is that
* the Changing Name functions will affect both MKB and VKB
* anyway. That means, If we seperate the storage of name in the MKB
* and VKB, we still need another function to call both parts to change
* name synchronized. That will need a lot of more work. As well,
* because I specify the VKB/MKB interface in "interface" VKB_MKB, we
* can easily know how can VKB access MKB. Sure, if we want to use
* Relation and Attribute classes in MKB, we need to know the knowledge
* of them.  I think that's the only drawback.  <p> I try to do all the
* class design to help us make the implementation extendable.  So, if
* in the future MKB needs to be changed, and that change will affect
* the Relation and Attribute classes in MKB, like MKB change its
* structure to keep the history, we just need to remain the same
* interface between VKB and those classes, then everything should be
* fine. I need to define another two interfaces between VKB and
* Relation and Attribute.
* @see View_Definition */

public class VKB implements java.io.Serializable, VSA_VKB{
    /**
     * Index of view definition on View Name.
     */
    Hashtable View_index;
    
    /**
     * The reference of MKB this VKB based on.
     */
    MKB mkb;
    
    /**
     * Get the MKB of this VKB
     */
    public MKB getMKB(){
	return mkb;
    }
    
    /**
     * Set the MKB of this VKB
     */
    public void setMKB(MKB ref){
	mkb = ref;
    }
    
    /**
     * Default constructor.
     */
    public VKB (){
	View_index = new Hashtable();
    }
    
    /**
     * Parse E-SQL/SQL String and create the VKB object.
     * @param ESQL_file E-SQL/SQL of this VKB/ or a file name
     * @param ref MKB of this VKB depends on
     */
    public VKB(String ESQL_file, MKB ref){
	this();
	
	// input stream (may from file or a string)
	InputStream input;
	
	// try if that is a file or not
	File esql_file = new File(ESQL_file);
	
	if(!esql_file.exists()){
	    // file does not exists
	    input = new StringBufferInputStream(ESQL_file);
	}else{
	    try{
		input = new FileInputStream(esql_file);
	    }catch(FileNotFoundException fnfe){
		System.out.println("VKB(ESQL_file): " + fnfe.getMessage());
		return;
	    }
	}
	
	E_SQL parser = new E_SQL(input);
	
	mkb = ref;
	
    SimpleNode root = null;

    try{
      // parse the E_SQL
      root = parser.VD_sequence();
    }catch(ParseException pe){
      System.out.println("VKB(ESQL_file): parse error: " + pe.getMessage());
      return;
    }

    // create the object from the parser
    root.createVKB(this, mkb);
  }

  /**
  * Replace view V with bestV. This is the place to update the view
  * definition after selection. MAYA */
  public void replaceViewDefinition(View_Definition V, View_Definition bestV){
    if(V == null){
      System.out.println("VKB.replaceView please use addViewDefinition(View_Definition).");
      return;
    }

    if(bestV == null){
      System.out.println("VKB.replaceView please use dropViewDefinition(View_Definition).");
      return;
    }

    if(!V.getName().equals(bestV.getName())){
      System.out.println("VKB.replaceView they do not have same name.");
      return;
    }

    if(View_index.get(V.getName()) == null){
      System.out.println("VKB.replaceView view(" + V.getName() + ") is not in the VKB.");
      return;
    }

    View_index.put(bestV.getName(), bestV);
  }

  /**
  * Set View_Index Hashtable of this VKB
  * @param index New index of view definitions
  */
  public void setViewIndex(Hashtable index){
    View_index = index;
  }

  /**
  * Add one view definition to this VKB
  * <p>
  * It will affect the index structure. We need to re-index the View definition.
  */
  public void addViewDefinition(View_Definition newVD){

    if(newVD == null){

      System.out.println("VKB:addViewDefinition() newVD is null");
      return;

    }

    // check if the VD is already in the VKB
    if(View_index.get(newVD.getName())!=null){

      System.out.println("VKB:addViewDefinition() newVD " + newVD.getName() +
        " is already in VKB");
      return;

    }

    // put it in the Hashtable
    View_index.put(newVD.getName(), newVD);

  };

  /**
  * Remove one view definition from this VKB with specific view name.
  * <p>
  * It will affect the index structure. We need to re-index the View definition.
  */
  public void dropViewDefinition(String viewName){

    // if cannot fount it, just return.
    if(View_index.get(viewName)==null){
      System.out.println("VKB.dropViewDefinition() View " + viewName + " not found");
      return;
    }

    // remove it from the VKB
    View_index.remove(viewName);
  }

  /**
  * Remove one view definition from this VKB.
  * <p>
  * It will affect the index structure. We need to re-index the View definition.
  */
  public void dropViewDefinition(View_Definition view){

    // if cannot fount it, just return.
    if(View_index.get(view.getName())==null){
      System.out.println("VKB.dropViewDefinition() View " + view.getName() + " not found");
      return;
    }

    // remove it from the VKB
    View_index.remove(view.getName());
  }

  /**
  * Drop a vector of view definitions specified by Vector VDs.
  */
  public void dropSetOfVDs(Vector VDs){
    if(VDs == null)
      return;

    Enumeration enum = VDs.elements();
    while(enum.hasMoreElements()){
      dropViewDefinition(((View_Definition)enum.nextElement()).getName());
    }
  }

  /**
  * Get a vector of ViewDefinitions mentioning a specific
  * <B>Information Source (IS)</B> name.
  * @param: IS_name Information Source name.
  * @return null: No View Definition affected by this IS, Vector: View Definitions affected by this IS.  */
  public Vector getViewDefinitions(String IS_name){
    // Vector of affected View Definition
    Vector VDs = new Vector();

    Enumeration enum = View_index.elements();

    while(enum.hasMoreElements()){

      View_Definition vd = (View_Definition)enum.nextElement();

      if(vd.contains(IS_name)){
        VDs.addElement(vd);
      }
    }

    if(VDs.size()>0)
      return VDs;
    else
      return null;
  }

  /**
  * Get a vector of all ViewDefinitions mentioning a <B>Relation</B>
  * of a specific Relation Name.
  * @param IS_name Information Source Name
  * @param Rel_name Relation Name
  * @return null: No View Definition affected, Vector: View Definitions affected by relation */
  public Vector getViewDefinitions(String IS_name, String Rel_name){
    // Vector of affected View Definition
    Vector VDs = new Vector();

    Enumeration enum = View_index.elements();

    while(enum.hasMoreElements()){

      View_Definition vd = (View_Definition)enum.nextElement();

      if(vd.contains(mkb.findRelation(IS_name, Rel_name))){
        VDs.addElement(vd);
      }
    }

    if(VDs.size()>0)
      return VDs;
    else
      return null;
  }

  /**
  * Get a vector of ViewDefinitions mentioning an <B>Attribute</B> of
  * the given name.
  * @param IS_name Information Source Name
  * @param Rel_name Relation Name
  * @param Attr_name Attribute Name
  * @return null: No View Definition affected, Vector: View Definitions affected by attribute */
  public Vector getViewDefinitions(String IS_name, String Rel_name, String Attr_name){
    // Vector of affected View Definition
    Vector VDs = new Vector();

    Enumeration enum = View_index.elements();

    while(enum.hasMoreElements()){

      View_Definition vd = (View_Definition)enum.nextElement();

      if(vd.contains(mkb.findAttribute(IS_name, Rel_name, Attr_name))){
        VDs.addElement(vd);
      }
    }

    if(VDs.size()>0)
      return VDs;
    else
      return null;
  }

    /**
     * returns the E-SQl definition fo the view with the given name
     * @param viewName the name of the view to be queried
     * @return E-SQL view definition or empty String object
     */
  public String getESQL(String viewName) {
    View_Definition vd = (View_Definition)View_index.get(viewName);
    if (vd==null)
     return new String();
    return vd.getESQL();
  }

    /**
     * returns the SQl definition fo the view with the given name
     * @param viewName the name of the view to be queried
     * @return SQL view definition or empty String object
     */
  public String getSQL(String viewName) {
    View_Definition vd = (View_Definition)View_index.get(viewName);
    if (vd==null)
     return new String();
    return vd.getSQL();
  }

  /**
  * Get View Definition for a <B>View</B> of a specific name
  * @param view_name name of view definition
  * @return View_Definition or null.
  */
  public View_Definition getVD(String view_name){
    return (View_Definition)View_index.get(view_name);
  }

  /**
  * This function will call MKB to change the specific attribute.
  * It will change all the view_components in all view definitions in VKB
  * from one attribute name to another name.
  * <p>
  * It's different from make one attribute_component in one view_definition
  * reference to another attribute, which will only change one attribute_component
  * of one view_definition.
  * @param IS_name Information Source Name
  * @param Rel_name Relation Name
  * @param Attr_name Attribute Name
  * @param new_name New attribute Name
  */
  public void changeAttributeName(String IS_name, String Rel_name, String Attr_name, String new_name){
    mkb.changeAttributeName(IS_name, Rel_name, Attr_name, new_name);
  }

  /**
  * This function will call MKB to change the specific relation.
  * It will change all hte view_components in all view definitions in VKB
  * from one relation name to another name.
  * <p>
  * It's means it will also affect the attribute_component and condition_component.
  * @param IS_name Information Source Name
  * @param Rel_name Relation Name
  * @param new_name New Relation Name
  */
  public void changeRelationName(String IS_name, String Rel_name, String new_name){
    mkb.changeRelationName(IS_name, Rel_name, new_name);
  }

  /**
  * Get a Vector of ViewNames in this VKB
  * @return null no view stores; else return a vector of view names.
  */
  public Vector getViewNames(){

    if(View_index == null)
      return null;

    Vector result = new Vector();

    Enumeration enum = View_index.keys();

    while(enum.hasMoreElements()){
      result.addElement(enum.nextElement());
    }

    if(result.size() > 0)
      return result;
    else
      return null;
  }

  /**
  * Display VKB in ESQL mode
  */
  public String toString(){
    return View_index.toString();
  }
}
