////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.VKB;

import DSRG.MKB.Relation;
import DSRG.MKB.Attribute;

import java.util.Vector;
/**
* This is a interface between VKB and MKB. It defines all the methods between
* VKB and MKB.
* <p> They are:
* <ul>
*   <li> Get one IS from MKB by name </li>
*   <li> Get one Relation from MKB by name </li>
*   <li> Get one Attribute from MKB by name </li>
*   <li> Check if one IS exists </li>
*   <li> Check if one Relation exists </li>
*   <li> Check if one Attribute exists </li>
* </ul>
*/
public interface VKB_MKB 
{
  /**
  * Get a specific Relation.
  * @param rel_name relation name "R"
  * @param IS_name Information source name "IS"
  */  
  public abstract Relation getRelation(String IS_name, String rel_name);

  /**
  * Get a specific Attribute
  * @param attr_name attribute name "A"
  * @param rel_name relation name "R"
  * @param IS_name Information source name "IS"
  */
  public abstract Attribute getAttribute(String IS_name, String rel_name, String attr_name);

  /**
  * Change a specific Attribute name. from "IS.R.A" to "IS.R.B"
  * @param attr_name attribute name "A"
  * @param rel_name relation name "R"
  * @param IS_name Information source name "IS"
  * @param new_name new attribute name "B"
  */
  public abstract void changeAttributeName(String IS_name, String rel_name, String attr_name, String new_name);

  /**
  * Change a specific Relation name. from "IS.R" to "IS.S"
  * @param IS_name Information Source Name
  * @param Rel_name Relation Name
  * @param new_name New Relation Name
  */
  public abstract void changeRelationName(String IS_name, String Rel_name, String new_name);

  /**
  * Find POC PC constraint Pi(upperline{B}(S)) phi Pi(upperline{A}(R))
  * @param rel Relation R
  * @param atleast subset of upperline{A}
  * @param ve View Extent Parameter
  * @return Vector of replacement.
  */
  public abstract Vector findPOC(Relation rel, Vector atleast, int ve);
}

