////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.VKB;

import java.io.Serializable;

/**
* It's a abstract class of Attribute, Relation and Condition.
* It's the View Component Class.
*/
public abstract class View_Component implements java.io.Serializable, Cloneable{

  /**
  * Evolution parameter. True: component can be dropped, False: cannot be dropped.
  */
  public boolean Dispensable;

  /**
  * Evolution parameter. True: component can be replaced, False: cannot be replaced.
  */
  public boolean Replaceable;

  /**
  * Default constructor
  */
  public View_Component(){
    Dispensable = false;
    Replaceable = false;
  }

  /**
  * Set the Dispensable-Parameter of this View_Component.
  */
  public void setDispensable(boolean value){
    Dispensable = value;
  }

  /**
  * Return the Dispensable-Parameter of this View_Component.
  */
  public boolean getDispensable(){
    return Dispensable;
  }

  /**
  * Set the Replaceable-Parameter of this View_Component.
  */
  public void setReplaceable(boolean value){
    Replaceable = value;
  }

  /**
  * Return the Replaceable-Parameter of this View_Component.
  */
  public boolean getReplaceable(){
    return Replaceable;
  }

  /**
  * Display evolving parameter.
  * We show all the parameters.
  * @param disp Name of dispensable, e.g. "AD", "CD", "RD"
  * @param repl Name of replaceable, e.g. "AR", "CR", "RR"
  */
  String displayEvolvingParameter(String disp, String repl){

    StringBuffer str = new StringBuffer(" ( ");
    
    if(Dispensable == true){// t,*

      str.append(disp + " = " + "TRUE ");

      if(Replaceable == true)// t,t
        str.append(", " + repl + " = " + "TRUE )");
      else// t,f
	str.append(", " + repl + " = " + "FALSE )");


    }else{// f, *
      str.append(disp + " = " + "FALSE ");

      if(Replaceable == true)// f,t
	str.append(", " + repl + " = " + "TRUE )");      
      else// f,f
	str.append(", " + repl + " = " + "FALSE )");
    }

    return str.toString();
  }

    /**
     * Clone the View_Component
     */
    public  abstract  Object clone();

    /*
      {
      if(this instanceof Attribute_Component)
      return ((Attribute_Component)this).clone();
      else if(this instanceof Relation_Component)
      return ((Relation_Component)this).clone();
      else if(this instanceof Condition_Component)
      return ((Condition_Component)this).clone();
      else{
      // Unknow Objects.
      System.out.println("View_Component.clone() Unknow Subclass Object, don't know how to clone.\n"
      + "Return a  NULL object.");
      return null;
      }
      }
    */
}

