////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.VSA;

/**
* It represents a schema change at the Information Source. It will be generated 
* by GISL(Glocal IS Layer). It will be sent to VSA.
* <p>
* I set all the attributes in this Schema_Change public. I think this class only
* like structor. It will be only used in VSA and  GISL. It will use different
* combinations of public attributes to show different change.
* <table border = 1>
* <caption> Change Status</caption>
*   <tr>
*     <th> SE </th> <th> Attributes </th>
*   </tr>
*   <tr>
*     <td> Add Attribute </td> <td> type, IS_name, Rel_name, Attr_name </td>
*   </tr>
*   <tr>
*     <td> Change Attribute Name </td> <td> type, IS_name, Rel_name, Attr_name, new_name </td>
*   </tr>
*   <tr>
*     <td> Delete Attribute </td> <td> type, IS_name, Rel_name, Attr_name </td>
*   </tr>
*   <tr>
*     <td> Add Relation </td> <td> type, IS_name, Rel_name</td>
*   </tr>
*   <tr>
*     <td> Change Relation Name </td> <td> type, IS_name, Rel_name, new_name </td>
*   </tr>
*   <tr>
*     <td> Delete Attribute </td> <td> type, IS_name, Rel_name </td>
*   </tr>
* </table>
*/
public class Schema_Change implements java.io.Serializable
{
  /**
  * Add Attribute Schema Change.
  */
  public final static int ADD_ATT = 0;
  /**
  * Change Attribute Name Schema Change.
  */
  public final static int CHG_ATT = 1;
  /**
  * Delete Attribute Schema Change.
  */
  public final static int DEL_ATT = 2;
  /**
  * Add Relation Schema Change.
  */
  public final static int ADD_REL = 3;
  /**
  * Change Relation Name Schema Change.
  */
  public final static int CHG_REL = 4;
  /**
  * Delete Relation Schema Change.
  */
  public final static int DEL_REL = 5;

  /**
  * Current Type of this Change
  */
  public int type;

  /**
  * Information source name of this Change
  */
  public String IS_name;

  /**
  * Relation name of this Change
  */
  public String Rel_name;

  /**
  * Attribute name of this Change
  */
  public String Attr_name;

  /**
  * New name of relaiton of attribute
  */
  public String new_name;

  /**
  * Constructor of Add/Del Relation Change
  * @tp type of this change. Only could be: ADD_REL, DEL_REL, CHG_REL.
  * @is information source name
  * @rel relation name
  */
    public Schema_Change(int tp, String is, String rel){
        switch(tp){ 
        case ADD_REL:
        case DEL_REL:
        case CHG_REL:
            type = tp;
            break;
        default:
            System.out.println("Schema_Change: Error type!");
            return;
        }
        
        IS_name = is;
        Rel_name = rel;
    }
    
    /**
     * Constructor of Add/Del Attribute Change
     * @tp type of this change. Only could be: ADD_ATT, DEL_ATT, CHG_ATT.
     * @is information source name
     * @rel relation name
     * @attr attribute name
     */
    public Schema_Change(int tp, String is, String rel, String attr){
        switch(tp){ 
        case ADD_ATT:
        case DEL_ATT:
        case CHG_ATT:
            type = tp;
            break;
        default:
            System.out.println("Schema_Change: Error type!");
            return;
        }
        
        IS_name = is;
        Rel_name = rel;
        Attr_name = attr;
    }
    
  /**
  * Constructor of Change Attribute Change
  * @is information source name
  * @rel relation name
  * @attr attribute name
  * @name new name of attribute
  */
  public Schema_Change(String is, String rel, String attr, String name){
    type = CHG_ATT;
    IS_name = is;
    Rel_name = rel;
    Attr_name = attr;
    new_name = name;
  }

  /**
  * Constructor of Change Relation Change
  * @is information source name
  * @rel relation name
  * @name new name of Relation
  */
  public Schema_Change(String is, String rel, String name){
    type = CHG_REL;
    IS_name = is;
    Rel_name = rel;
    new_name = name;
  }
}

