////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.VSA;

import java.util.Enumeration;
import java.util.Vector;

/**
* Used to generate subsets of one Vector.
* The subsets including empty set and whole set.
*/
public class Subset implements java.io.Serializable
{
  Vector subsets;

  /**
  * Construct Subsets of a set.
  * It will call the recursive function getSubsets.
  */
  public Subset(Vector wholeset){
    if(wholeset != null)
      subsets = createSubsets(wholeset);
    else
      subsets = new Vector();
  }


    Vector createSubsets(Vector set) {
        Vector powerset = new Vector(10,10);
        int i,j; 
        powerset.addElement(new Vector());
        //System.out.println(powerset);
        Vector tempElement;
        for (i=0; i<set.size(); i++ ) {
            Object element = set.elementAt(i);
            int powersetSize=powerset.size();
            //System.out.print(powersetSize + ":");
            for (j=0; j < powersetSize; j++) {
                tempElement = (Vector)powerset.elementAt(j);
                tempElement = (Vector)tempElement.clone();
                tempElement.addElement(element);
                powerset.addElement(tempElement);
            }
            //System.out.println(powerset);
        }
        if (powerset.size()>30)
            System.out.println("Generated " + powerset.size() + " subsets.");
        return powerset;
    }

  /**
  * Find the subsets of one set.
  * @param set A set of elements.
  * @return a Vector of subsets of set.
  */
    /*
  Vector createSubsets(Vector set){
    Vector result = new Vector(); // {}
    result.addElement(new Vector()); // {empty}

    if(set.size() == 0)
      return result; // {empty}

    Vector A = new Vector(); // {}
    A.addElement(set.elementAt(0)); // {a}
    if(set.size() == 1)
      result.addElement(A); // {empty, {a}}
    else{
      set.removeElementAt(0);
      result = createSubsets(set);
      result = VSA.union(result, add(result, A.elementAt(0)));
    }

    return result;
  }
    */

    /**
     * Find the subsets of one Set.
     * @param set A set of elements.
     * @param n length of subsets.
     * @return a Vector of subsets of set.
     */
    /*    Vector createSubsets(Vector set){
        Vector result = new Vector();
        if(set.size() == 0){
            result.addElement(new Vector()); // add {emptyset}
            return result;
        }
        result.addElement(set.clone());// add {all}.
        int size = set.size();
        for(int i = 0; i < size; i++){
            Object elm = set.elementAt(i);
            Vector temp = (Vector)set.clone();
            temp.removeElement(elm);
            result = VSA.union(result, createSubsets(temp));
        }
        return result;
        }*/

 

  
  /**
  * Add one Element in a set of subsets.
  * @param subsets A Vector of subsets.
  * @param element one element
  * @return A Vector of added subsets.
  */
  public static Vector add(Vector subsets, Object element){
    
    Vector result = VSA.deepClone(subsets);

    Enumeration enum = result.elements();

    while(enum.hasMoreElements()){
      Vector subset = (Vector)enum.nextElement();
      subset.addElement(element);
    }

    return result;
  }

  public String toString(){
    return subsets.toString();
  }

  public Vector getSubsets(){
    return subsets;
  }
}

