////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package DSRG.VSA;

import DSRG.MKB.Relation;

import DSRG.VSA.Replacement;

import java.util.Vector;

/**
* Interface between VSA and View_Definition Class.
*/
public interface VSA_View_Definition 
{

  /**
  * Drop a vector of components from one view definition.
  */
  public abstract void dropComponents(Vector s);
  /**
  * Get Dispensable components of one view defintion.
  */
  public abstract Vector getDispensableComponent();

  /**
  * Replace relation at From clause.
  * @param r relation to be replaced.
  * @param s substitute relation.
  */
  public abstract void replaceRelFrom(Relation r, Relation s);
  /**
  * Drop the Attributes in Select clause of specific Relation and with
  * specific ESQL parameters.
  */
  public abstract void dropAttrSelect(Relation r, boolean di, boolean re);
  /**
  * Drop the Conditions in Where clause of specific Relation and with 
  * specific ESQL parameters.
  */
  public abstract void dropCondWhere(Relation r, boolean di, boolean re);

  /**
  * Replace the attributes in Select clause of specific Relation and with
  * specific ESQL parameters.
  */
  public abstract boolean replaceAttrSelect(Relation r, Replacement rep, boolean di, boolean re);
  /**
  * Replace the conditions in Where clause of specific Relation and with
  * specific ESQL parameters.
  */
  public abstract boolean replaceCondWhere(Relation r, Replacement rep, boolean di, boolean re);

  /**
  * Get Attributes in the Where clause, which of specific Relation and 
  * with specific ESQL parameters.
  */
  public abstract Vector attrWhere(Relation r, boolean di, boolean re);
  /**
  * Get Attributes in the Select clause, which of specific Relation and
  * with specific ESQL parameters.
  */
  public abstract Vector attrSelect(Relation r, boolean di, boolean re);
}

