////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package EVEDemo.ISChangeApplet;

import EVEDemo.MW.*;
import DSRG.MW.*;
import DSRG.MKB.*;
import java.util.*;
import java.awt.*;
import java.applet.*;
import com.sun.java.swing.*;
import com.sun.java.swing.tree.*;
import com.sun.java.swing.event.TreeSelectionListener;
import com.sun.java.swing.event.TreeSelectionEvent;
import com.sun.java.swing.tree.TreeSelectionModel;


/**
 * This Frame shows the InformationSource hierarchy and lists options for performing capability changes
 *
 * @author Amber Van Wyk, Summer 1998
 * updated: Andreas Koeller, 8/12/98
 */


public class ISChange extends JFrame
{
  /**
   * local copy of allISObjects, used in building mkb tree
   */
  public Vector allISObjects = new Vector();

  /**
     * scroll pane to contain the mkb
     */
  JScrollPane scrollPaneTree;

  /**
     * Selected Path
     */
  String selPath;

  /**
     * Tree structure for the mkb
     */
  JTree mkbTree;

  /**
     * Tree model for the mkbTree
     */
  DefaultTreeModel treeModel;

  /**
     * Button for adding attributes
     */
  JButton addAtt;

  /**
     * Button for adding relation
     */
  JButton addRel;

  /**
     * Button for renaming attribute
     */
  JButton renAtt;

  /**
     * Button for renaming relation
     */
  JButton renRel;

  /**
     * Button for deleting attribute
     */
  JButton delAtt;

  /**
     * Button for deleting relation
     */
  JButton delRel;

  /**
     * Button for data updates
     */
  JButton dataUpd;

  /**
     * Button for being done making changes
     */
  JButton ok;

  JLabel heading;

    /*
     * label which shows path of selected item in tree
     * Elke vetoed this one
     */
    //JLabel selection;

    /**
     * Panel to contain buttons
     */
  JPanel buttons;

  /**
     * Path of a node in the tree
     */
  TreePath fullPath;

  /**
   * Class Constructor. <BR>
   * Intializes all objects on the Frame
   */
  public ISChange(Vector ISObjects)
    {
      allISObjects = ISObjects;
      
      // Set layout, resizability, and visability
      getContentPane().setLayout(new BorderLayout(50,20));
      setResizable(false);
      setVisible(false);
      setSize(450, 550);
      setTitle("Simulate an IS Capability Change");
      
      // Initialize heading label
      heading = new JLabel("Capability Changes",SwingConstants.CENTER);
      heading.setSize(new Dimension(450, 60));
      heading.setFont(new Font("Dialog", Font.PLAIN, 18));
      heading.setForeground(Color.blue);
      
      // Initialize selection label -- it shows the path for the currently selected node
      //selection = new JLabel("A selection has not been made");
      //selection.setSize(new Dimension(400, 45));
      //selection.setFont(new Font("Dialog", Font.PLAIN, 12));
      //selection.setForeground(Color.blue);
      
      // Initialize scrollpane which will hold the tree
      scrollPaneTree = new JScrollPane();
      scrollPaneTree.setSize(new Dimension(200, 300));
      
      // Initialize buttons
      addAtt = new JButton("Add Attribute");
      addRel = new JButton("Add Relation");
      renAtt = new JButton("Rename Attribute");
      renRel = new JButton("Rename Relation");
      delAtt = new JButton("Delete Attribute");
      delRel = new JButton("Delete Relation");
      dataUpd = new JButton("Data Update");
      ok = new JButton("OK");
      
      // Set layout for button panel and add all buttons to it
      buttons = new JPanel(new GridLayout(5, 2, 15, 15));
      buttons.add(addAtt);
      buttons.add(addRel);
      buttons.add(renAtt);
      buttons.add(renRel);
      buttons.add(delAtt);
      buttons.add(delRel);
      buttons.add(dataUpd);
      buttons.add(ok);
      
      // Build the IS tree
      buildISTree();
      
      // Add all items to the applet
      getContentPane().add("North",heading);
      getContentPane().add("Center",scrollPaneTree);
      getContentPane().add("South",buttons);
      //getContentPane().add(selection);
      
    }
  
  /**
   * Builds the tree structure to show the mkb hierarchy
   */
  public void buildISTree()
    {
      // Create root node for mkbTree
      DefaultMutableTreeNode top = new DefaultMutableTreeNode("Meta Knowledge Base");
      
      Enumeration enumIS = allISObjects.elements();
      Enumeration enumRel = null;
      Enumeration enumAtt = null;
      String ISName = null;
      String relName = null;
      String attName = null;
      IS tempIS = null;
      
      // Create temp nodes for tree
      DefaultMutableTreeNode nodeIS = null;
      DefaultMutableTreeNode nodeRel = null;
      DefaultMutableTreeNode nodeAtt = null;
      
      while(enumIS.hasMoreElements())
        {
	  //System.out.println("Inside IS while");
	  
	  // Get next InformationSource
	  tempIS = (IS)(enumIS.nextElement());
	  
	  // Get name of tempIS
	  ISName = tempIS.getName();
	  
	  // Create a node for tempIS
	  nodeIS = new DefaultMutableTreeNode(ISName);
	  
	  // Get relations for tempIS
	  enumRel = tempIS.getRelations().elements();
	  
	  // Traverse enumRel and add all relations of tempIS to nodeIS
	  while(enumRel.hasMoreElements())
            {
	      //System.out.println("Inside rel while");
	      
	      relName = ((Relation)enumRel.nextElement()).getName();
	      nodeRel = new DefaultMutableTreeNode(relName);
	      
	      // Get all attributes for relName
	      enumAtt = tempIS.getAttributes(relName).elements();
	      
	      // Traverse enumAtt and add all attributes of relName to nodeRel
	      while(enumAtt.hasMoreElements())
                {
		  attName = ((Attribute)enumAtt.nextElement()).getName();
		  nodeAtt = new DefaultMutableTreeNode(attName);
		  
		  nodeRel.add(nodeAtt);
                }
	      
	      // Add the relation to the IS
	      nodeIS.add(nodeRel);
            }
	  
	  // add nodeIS to the tree
	  top.add(nodeIS);
        }
      
      treeModel = new DefaultTreeModel(top);
      
      // Create tree with node top as root and it to the scroll pane
      mkbTree = new JTree(treeModel);
      mkbTree.setCellRenderer(new MyTreeCellRenderer());
      mkbTree.setRootVisible(false);
      mkbTree.setShowsRootHandles(true);
      scrollPaneTree.add(mkbTree);
      scrollPaneTree.setViewportView(mkbTree);
      // Set selection mode to single
      mkbTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
      
      // Listen for when the selection changes.  Wish me Luck!
      mkbTree.addTreeSelectionListener(new TreeSelectionListener() {
	public void valueChanged(TreeSelectionEvent e) {
	  // Get the full path of the node
	  fullPath = e.getPath();
	  
	  // Set selpath to be only the data between {}
	  selPath = new String(fullPath.toString());
	  selPath = selPath.substring(selPath.indexOf('{'), selPath.length());
	  
	  // Set selection label to path of selected node
	  // selection.setText(selPath);
	}
      });
    }

  /**
   * (mostly generated by Visual Cafe)
   * in addition to setting visibility, also sets location of the frame
   */
  public synchronized void setVisible(boolean show)
    {
      if(show)
	{
	  setLocation(50, 50);
	  super.setVisible(true);
	}
      else
	{
	  super.setVisible(false);
	}
    }

  /**
     * (generated by Visual Cafe)
     */
  public void addNotify()
    {
      // Record the size of the window prior to calling parents addNotify.
      Dimension d = getSize();

      super.addNotify();

      if (fComponentsAdjusted)
	return;

      // Adjust components according to the insets
      setSize(getInsets().left + getInsets().right + d.width, getInsets().top + getInsets().bottom + d.height);
      Component components[] = getComponents();
      for (int i = 0; i < components.length; i++)
	{
	  Point p = components[i].getLocation();
	  p.translate(getInsets().left, getInsets().top);
	  components[i].setLocation(p);
	}
      fComponentsAdjusted = true;
    }

  /**
     * Used for addNotify check.
     */
  boolean fComponentsAdjusted = false;

  /**
     * (generated by Visual Cafe)
     */
  class SymWindow extends java.awt.event.WindowAdapter
  {
    public void windowClosing(java.awt.event.WindowEvent event)
      {
	Object object = event.getSource();
	if (object == ISChange.this)
	  Frame1_WindowClosing(event);
      }
  }

  /**
     * (generated by Visual Cafe)
     */
  void Frame1_WindowClosing(java.awt.event.WindowEvent event)
    {
      setVisible(false);		 // hide the Frame
    }
}
