////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package EVEDemo.ISChangeApplet;

import DSRG.MKB.*;
import EVEDemo.MW.*;
import DSRG.MW.*;
import DSRG.VKB.QC;
import DSRG.VKB.Version;
import java.util.*;
import java.awt.*;
import java.applet.*;
import com.sun.java.swing.*;
import com.sun.java.swing.tree.*;

/**
 * This applet sets up a user's ability to simulate capability changes
 * on the information sources. <BR> The first thing that occurs when
 * creating an instance of this class is that it attempts to connect
 * to the server "mwMainServer" <BR> An instance of each needed frame
 * is created here, and all mouse click event handling is done here,
 * as well.  Each of these frames is hid and shown from here also.
 * <P> comment koeller 030799. RMI is not used anymore, so we will
 * slowly remove RMI support. I started with commenting out some of
 * the exceptions
 * @author Amber Van Wyk, Summer 1998 */
public class ISMaster extends JFrame
{


    /**
     * Debug flag
     */
    public static final boolean DEBUG = false;
  
    /**
     * The handle to the server
     */
    private EVEDemo.MW.Main mwMain;// = new EVEDemo.MW.Main();

    /** 
     * a reference to the MKB, replaces the InformationSource Vector
     */
    private MKB theMKB;
    
    /**
     * Vector of all Information Sources
     */
    // private Vector allISObjects = new Vector();
    
    /**
     * Vector of all Views. MAYA: Now including a vector of "version" objects
     */
    private Vector allViews = new Vector();
    
    /**
     * Label on applet
     */
    JLabel lblWelcome;
    
    /**
     * Button on applet
     */
    JButton ISchangeButton;
    
    /**
     * Instance of ISChange for choosing which changes to make
     */
    ISChange isFrame;
    
    /**
     * Instance of delFrame for deleting attributes and relations
     */
    delFrame dFrame;
    
    /**
     * Instance of renFrame for renaming attributes and relations
     */
    renFrame rFrame;
    
    /**
     * Intance of addAttFrame for adding an attribute
     */
    addAttFrame aaFrame;
    
    /**
     * Instance of addRelFrame for adding a relation
     */
    addRelFrame arFrame;
    
    /**
   * Instance of evolveFrame for showing evolution of view definitions
   */
    evolveFrame eFrame;
  
    /**
   * Instance of myMessageBox for displaying messages to the user
   */
    myMessageBox messageTime;
  
    /**
   * the selected path when dFrame was shown
   */
    TreePath delTreePath;
  
    /**
   * the selected path when rFrame was shown
   */
    TreePath renTreePath;
  
    /**
   * the selected path when aaFrame was shown
   */
    TreePath aaTreePath;
  
    /**
   * the selected path when arFrame was shown
   */
    TreePath arTreePath;
  
    /**
   * Used in calling delete, change name, and add methods in simulating changes
   */
    String attName;
  
    /**
   * Used in calling delete, change name, and add methods in simulating changes
   */
    String relName;
  
    /**
   * Used in calling delete, change name, and add methods in simulating changes
   */
    String ISName;
  
    /**
   * Used in calling delete, change name, and add methods in simulating changes
   */
    IS currIS = null;
  
    Alive a;
  

    public ISMaster(Main mn){
	super("IS Schema Change Simulator");
	mwMain = mn;
	init();
	setSize(new Dimension(400,200));
	toFront();
	show();
    }

    /**
   * Connects to RMI server called mwMainServer. <BR>
   * Intializes all objects on the applet. Creates instances of each frame
   */
    public void init()
    {
	/*	try{ // RMI-Exception */
	    // Check the Lock
	    if(mwMain.isISChangeRunning())
		{
		    System.out.println("Cannot use now");

		    lblWelcome = new JLabel("This Demo is Single User Only", JLabel.CENTER);
		    JLabel lbl = new JLabel("Please Try Again Later", JLabel.CENTER);
		    lblWelcome.setFont(new Font("Dialog", Font.PLAIN, 24));
		    lblWelcome.setForeground(java.awt.Color.blue);
		    lbl.setFont(new Font("Dialog", Font.PLAIN, 24));
		    lbl.setForeground(java.awt.Color.blue);
		    getContentPane().setLayout(new GridLayout(3, 1, 2, 2));
		    getContentPane().add(lblWelcome);
		    getContentPane().add(lbl);
		}
	    else
		{
		    // Create and start the Alive Message. To Let Server know 
		    // there is a alive connection.
		    //a = new Alive(mwMain, this);
		    //a.start();

		    // reset middle layer and the database.
		    //mwMain.reset();

		    try{
                        theMKB = mwMain.getMKB();
                        //allISObjects = mwMain.getAllInformationSources();
			allViews = mwMain.getAllViews();
		    }catch(Exception e) {
			System.out.println("got an exception getting all ");
			e.printStackTrace();
		    }
		    if (DEBUG) {

			System.out.println("size of allViews: " + allViews.size());
			for(int j = 0; j < allViews.size(); j++)
			    {
				System.out.println(((View)(allViews.elementAt(j))).getName());
			    }
		    }
                    
		    isFrame = new ISChange(theMKB.getISs());
		    dFrame = new delFrame();
		    rFrame = new renFrame();
		    aaFrame = new addAttFrame();
		    arFrame = new addRelFrame();
		    eFrame = new evolveFrame(mwMain, allViews);
	    
		    // Set layout manager
		    getContentPane().setLayout(new GridLayout(3, 1, 20, 20));
	    
		    // Initialize welcome label
		    lblWelcome = new JLabel("Information Space", JLabel.CENTER);
		    lblWelcome.setFont(new Font("Dialog", Font.PLAIN, 36));
		    lblWelcome.setForeground(java.awt.Color.blue);
	    
		    // Initialize the button
		    ISchangeButton = new JButton();
		    ISchangeButton.setText("IS Clients");
		    ISchangeButton.setFont(new Font("Dialog", Font.PLAIN, 17));
	    
		    // Add all items to the applet
		    getContentPane().add(lblWelcome);
		    getContentPane().add(ISchangeButton);
	    
		    // (generated by Visual Cafe)
		    // add a mouse listener to each button on the ISMaster applet, and other frames
		    SymMouse aSymMouse = new SymMouse();
		    ISchangeButton.addMouseListener(aSymMouse);
		    isFrame.ok.addMouseListener(aSymMouse);
		    isFrame.addAtt.addMouseListener(aSymMouse);
		    isFrame.addRel.addMouseListener(aSymMouse);
		    isFrame.renAtt.addMouseListener(aSymMouse);
		    isFrame.renRel.addMouseListener(aSymMouse);
		    isFrame.delAtt.addMouseListener(aSymMouse);
		    isFrame.delRel.addMouseListener(aSymMouse);
		    // added by Andreas
		    isFrame.dataUpd.addMouseListener(aSymMouse);
		    dFrame.yes.addMouseListener(aSymMouse);
		    dFrame.no.addMouseListener(aSymMouse);
		    rFrame.ok.addMouseListener(aSymMouse);
		    rFrame.cancel.addMouseListener(aSymMouse);
		    aaFrame.ok.addMouseListener(aSymMouse);
		    aaFrame.cancel.addMouseListener(aSymMouse);
		    arFrame.ok.addMouseListener(aSymMouse);
		    arFrame.cancel.addMouseListener(aSymMouse);
		    eFrame.ok.addMouseListener(aSymMouse);
		    eFrame.disExtDif.addMouseListener(aSymMouse);

		}
	    /*	}catch(java.rmi.RemoteException e){
	    e.printStackTrace();
	    }*/
    }

    public void start(){
	//if(a == null){
	//a = new Alive(mwMain, this);
	//a.start();
	//}
    }

    public void stop(){
	//if(a!=null){
	//a.destory();
	//a = null;
	//}
    }

    public void destroy(){
	stop();
	System.out.println("Destroying applet");
    }
  
    /**
   * (mostly generated by Visual Cafe) <BR>
   * listener for mouse clicks
   */
    class SymMouse extends java.awt.event.MouseAdapter
    {
	/**
	 * catches mouse clicks
	 */
	public void mouseClicked(java.awt.event.MouseEvent event)
	{
	    Object object = event.getSource();
	
	    // Check to see if button clicked is on ISMaster
	    if(object == ISchangeButton)
		ISchangeButton_MouseClicked();
	
	    // Check to see if button clicked is on isFrame
	    else if(object == isFrame.ok)
		isFrame_ok_MouseClicked();
	    else if(object == isFrame.addAtt)
		isFrame_addAtt_MouseClicked();
	    else if(object == isFrame.addRel)
		isFrame_addRel_MouseClicked();
	    else if(object == isFrame.renAtt)
		isFrame_renAtt_MouseClicked();
	    else if(object == isFrame.renRel)
		isFrame_renRel_MouseClicked();
	    else if(object == isFrame.delAtt)
		isFrame_delAtt_MouseClicked();
	    else if(object == isFrame.delRel)
		isFrame_delRel_MouseClicked();
	    // added by Andreas
	    else if(object == isFrame.dataUpd)
		isFrame_dataUpd_MouseClicked();
	
	    // Check to see if button clicked is on dFrame
	    else if(object == dFrame.yes)
		dFrame_yes_MouseClicked();
	    else if(object == dFrame.no)
		dFrame_no_MouseClicked();
	
	    // Check to see if button cicked is on rFrame
	    else if(object == rFrame.ok)
		rFrame_ok_MouseClicked();
	    else if(object == rFrame.cancel)
		rFrame_cancel_MouseClicked();
	
	    // Check to see if button clicked is on aaFrame
	    else if(object == aaFrame.ok)
		aaFrame_ok_MouseClicked();
	    else if(object == aaFrame.cancel)
		aaFrame_cancel_MouseClicked();
	
	    // Check to see if button clicked is on arFrame
	    else if(object == arFrame.ok)
		arFrame_ok_MouseClicked();
	    else if(object == arFrame.cancel)
		arFrame_cancel_MouseClicked();
	
	    // Check to see if button clicked is on eFrame
	    else if(object == eFrame.ok)
		eFrame_ok_MouseClicked();
	    else if(object == eFrame.disExtDif)
		eFrame_disExtDif_MouseClicked();
	}
    }
  
    /**
   * Shows isFrame
   */
    public void ISchangeButton_MouseClicked()
    {
	isFrame.setVisible(true);
    }
  
    /**
   * Hides isFrame
   */
    public void isFrame_ok_MouseClicked()
    {
	isFrame.setVisible(false);
    }
  
    /**
   * Shows aaFrame if a relation is selected, else shows error message
   */
    public void isFrame_addAtt_MouseClicked()
    {
	System.out.println(isFrame.selPath);
	if(!relSelected(isFrame.fullPath))
	    {
		messageTime = new myMessageBox("In order to add an attribute, you must select a relation");
		messageTime.setVisible(true);
	    }
	else
	    {
		aaTreePath = isFrame.fullPath;
		aaFrame.setVisible(true);
	    }
    }
  
    /**
   * Shows arFrame if an IS is selected, else shows error message
   */
    public void isFrame_addRel_MouseClicked()
    {
	System.out.println(isFrame.selPath);
	if(!ISSelected(isFrame.fullPath))
	    {
		messageTime = new myMessageBox("In order to add a relation, you must select an IS");
		messageTime.setVisible(true);
	    }
	else
	    {
		arTreePath = isFrame.fullPath;
		arFrame.setVisible(true);
	    }
    }
  
    /**
   * Shows rFrame if an attribute is selected, else shows error message
   */
    public void isFrame_renAtt_MouseClicked()
    {
	System.out.println(isFrame.selPath);
	if(!attSelected(isFrame.fullPath))
	    {
		messageTime = new myMessageBox("In order to rename an attribute, you must select an attribute");
		messageTime.setVisible(true);
	    }
	else
	    {
		// Capture the path of the node who is currently selected
		renTreePath = isFrame.fullPath;
	  
		// Set oldName and show rFrame
		rFrame.setOldName(renTreePath.getLastPathComponent().toString());
		rFrame.setVisible(true);
	    }
    }
  
    /**
   * Shows rFrame if a relation is selected, else shows error message
   */
    public void isFrame_renRel_MouseClicked()
    {
	System.out.println(isFrame.selPath);
	if(!relSelected(isFrame.fullPath))
	    {
		messageTime = new myMessageBox("In order to rename a relation, you must select a relation");
		messageTime.setVisible(true);
	    }
	else
	    {
		// Capture the path of the node who is currently selected
		renTreePath = isFrame.fullPath;
	  
		// Set oldName and show rFrame
		rFrame.setOldName(renTreePath.getLastPathComponent().toString());
		rFrame.setVisible(true);
	    }
    }
  
    /**
   * Shows dFrame if an attribute is selected, else shows error message
   */
    public void isFrame_delAtt_MouseClicked()
    {
	System.out.println(isFrame.selPath);
      
	// If an attribute hasn't be selected, inform the user
	if(!attSelected(isFrame.fullPath))
	    {
		messageTime = new myMessageBox("In order to delete an attribute, you must select an attribute");
		messageTime.setVisible(true);
	    }
	else
	    {
		// Capture the path of the node who is currently selected
		delTreePath = isFrame.fullPath;
	  
		// Get the string form of the selected component to use in delete confirmation
		String thing = new String(delTreePath.getLastPathComponent().toString());
	  
		// Set question and show frame
		dFrame.setQuestion("Are you sure you want to delete " + thing + "?");
		dFrame.setVisible(true);
	    }
    }
  
    /**
   * Shows dFrame if a relation is selected, else shows error message
   */
    public void isFrame_delRel_MouseClicked()
    {
	System.out.println(isFrame.selPath);
	// If a relation hasn't been selected, inform the user
	if(!relSelected(isFrame.fullPath))
	    {
		messageTime = new myMessageBox("In order to delete a relation, you must select a relation");
		messageTime.setVisible(true);
	    }
	else
	    {
		// Capture the path of the node who is currently selected
		delTreePath = isFrame.fullPath;

		// Get the string form of the selected component to use in delete confirmation
		String thing = new String(delTreePath.getLastPathComponent().toString());

		// Set question and show frame
		dFrame.setQuestion("Are you sure you want to delete " + thing + "?");
		dFrame.setVisible(true);
	    }
    }

    /**
     * Sends a data update to database and SWEEP-module.
     */
    public void isFrame_dataUpd_MouseClicked()
    {
	try {
	    mwMain.startUpdate();
	}
      
	catch (Exception e) {
	    e.printStackTrace();
	}
    }

    
    
    /**
     * Deletes selected attribute or relation assuming no error in RMI
     * connection is encountered.  If an error is encountered display
     * an error message. <BR> Hides dFrame.  */

    public synchronized void dFrame_yes_MouseClicked()
    {
	// I need to extract relName and attName from delFramePath as
	// well as get the currIS to be the correct instance of
	// InformationSource --> getISfromName(theName) I think that I
	// should write some functions to get attnName and relName
	// from the path as I'm guessing I'll being doing that again
	// in rename.
	attName = getAttNameFromPath(delTreePath);
	relName = getRelNameFromPath(delTreePath);
	ISName = getISNameFromPath(delTreePath);
	if (DEBUG) System.out.println(ISName+relName+attName+"done");
	boolean success = false;

	// Save previous views
	//saveAllViews();
	try{
	    mwMain.saveViews(mwMain);
	}catch(Exception e){e.printStackTrace(); }

	    // Use delTreePath to determine what was selected
	    // this is low cohesion -- bad Software Engineering
	    if(attSelected(delTreePath))
		{
		    success = mwMain.deleteAttribute(ISName, relName, attName);
		}
	    else if(relSelected(delTreePath))
		{
		    success = mwMain.deleteRelation(ISName, relName);
		}

	allViews = new Vector(); // initialize in case the "try" fails

        if (DEBUG) System.out.println("ISMaster:dFrame_yes_MouseClicked() recompute view");
        allViews = (Vector)mwMain.getAllViews();

	dFrame.setVisible(false);

        versionFrame vf = new versionFrame(allViews,this);
        vf.show();
        
        if (DEBUG) System.out.println("ISMaster:dFrame_yes_MouseClicked()"+
                                      "finished recompute");
        
	if(success) // a successful delete-operation
	    {
		// Reset mkb hierarchy tree structure
		isFrame.allISObjects = theMKB.getISs();
		isFrame.buildISTree();
		isFrame.invalidate();
		isFrame.validate();
		isFrame.repaint();
                
	    }
	else
	    {
		messageTime = new myMessageBox("Error deleting item");
		messageTime.setVisible(true);
	    }

    }

    public void setPickedVersions(int[] myChoices) {
        View tempView;
        for (int i = 0 ; i < myChoices.length ; i++) {
            tempView = (View)allViews.elementAt(i);
            if (tempView.numberOfVersions()<0)
                System.out.println("ERROR: trying to replace version in non-evolved view "+tempView.getName());
            else 
                if (myChoices[i]>=0) {
                    tempView.replaceView_DefinitionBy(myChoices[i]);
                }
            // since we're working with references, this affects allViews!!!
        }
		eFrame.updateInfo(allViews);
		eFrame.setVisible(true);
    }
    
    
    /**
     * Hides dFrame
     */
    public void dFrame_no_MouseClicked()
    {
	dFrame.setVisible(false);
    }

    /**
     * Renames item unless error is encountered
     * Hides rFrame
     */
    public void rFrame_ok_MouseClicked()
    {
	// call methods to make changes in the database
	attName = getAttNameFromPath(renTreePath);
	relName = getRelNameFromPath(renTreePath);
	ISName = getISNameFromPath(renTreePath);
	//System.out.println(ISName+relName+attName+"done");
	boolean success = false;
	String theNewName = rFrame.getNewName();

	//saveAllViews();
	try{
	    mwMain.saveViews(mwMain);
	}catch(Exception e){e.printStackTrace();}

	System.out.println("The new name is" + theNewName + '!');

	/*	try{ // RMI-Exception */
	    // Use delFramePath to determine what was selected
	    if(attSelected(renTreePath) && !theNewName.equals(""))
		{
		    success = mwMain.changeAttributeName(ISName, relName, attName, theNewName);
		}
	    else if(relSelected(renTreePath)&& !theNewName.equals(""))
		{
		    success = mwMain.changeRelationName(ISName, relName, theNewName);
		}
	    /*	}catch(java.rmi.RemoteException e){
	    System.out.println("Remote error renaming: ");
	    e.printStackTrace();
	    }*/

	if(success)
	    {
		allViews = new Vector();
		try {
		    allViews = (Vector)mwMain.getAllViews();
		}catch (Exception e) {e.printStackTrace();}

		// Reset mkb hierarchy tree structure
		isFrame.allISObjects = theMKB.getISs();
		isFrame.buildISTree();
		isFrame.invalidate();
		isFrame.validate();
		isFrame.repaint();

		isFrame.mkbTree.expandPath(renTreePath);

		eFrame.updateInfo(allViews);
		eFrame.setVisible(true);
	    }
	else
	    {
		messageTime = new myMessageBox("Error renaming item");
		messageTime.setVisible(true);
	    }

	rFrame.setVisible(false);
    }

    /**
     * Hides rFrame
     */
    public void rFrame_cancel_MouseClicked()
    {
	rFrame.setVisible(false);
    }

    /**
     * Adds attribute unless error is encountered
     * Hides aaFrame
     */
    public void aaFrame_ok_MouseClicked()
    {
	String newName = aaFrame.getNewName();
	String newType = aaFrame.getNewType();
	boolean success = false;

	//saveAllViews();
	try{
	    mwMain.saveViews(mwMain);
	}catch(Exception e){e.printStackTrace();}

	// Extract names from path and get the currIS
	relName = getRelNameFromPath(aaTreePath);
	ISName = getISNameFromPath(aaTreePath);
	//System.out.println(ISName+relName+attName+"done");
	//currIS = getISfromName(ISName);

	// Add the attributed to the database
	/*	try{ // RMI-Exception */
	    if(!newName.equals(""))
		success = mwMain.addAttribute(ISName, relName, newName, newType);
	    else
		success = false;
	    /*	}catch(java.rmi.RemoteException e){
	    e.printStackTrace();
	    }*/

	if(success)
	    {
		allViews = new Vector();
		try {
		    allViews = (Vector)mwMain.getAllViews();
		}catch (Exception e) {e.printStackTrace();}

		// Reset mkb hierarchy tree structure
		isFrame.allISObjects = theMKB.getISs();
		isFrame.buildISTree();
		isFrame.invalidate();
		isFrame.validate();
		isFrame.repaint();

		// expand tree around recent change
		System.out.println("+++++++++++++++++++++++++++++++++++++");
		System.out.println(aaTreePath.toString());
		isFrame.mkbTree.expandPath(aaTreePath);
		System.out.println("+++++++++++++++++++++++++++++++++++++");

	    }
	else
	    {
		messageTime = new myMessageBox("Error adding attribute");
		messageTime.setVisible(true);
	    }

	aaFrame.setVisible(false);

	eFrame.updateInfo(allViews);
	eFrame.setVisible(true);
    }

    /**
     * Hides aaFrame
     */
    public void aaFrame_cancel_MouseClicked()
    {
	aaFrame.setVisible(false);
    }

    /**
     * Renames relation unless error is encountered
     * Hides arFrame
     */
    public void arFrame_ok_MouseClicked()
    {
	String newName = arFrame.getNewName();
	Vector attNames = arFrame.getAttNames();
	Vector attTypes = arFrame.getAttTypes();
	boolean success = false;

	System.out.println("saving views");
	//saveAllViews();
	try{
	    mwMain.saveViews(mwMain);
	}catch(Exception e){e.printStackTrace();}


	System.out.println("get names from path" + arTreePath.toString());
	// Extract names from path and get the currIS
	ISName = getISNameFromPath(arTreePath);
	//System.out.println(ISName+relName+attName+"done");
	//currIS = getISfromName(ISName);

	System.out.println("BEFORE CALL TO ADD");
	System.out.println("names " + attNames);
	System.out.println("types " + attTypes);
	System.out.println("relName " + newName);

	/*	try{ // RMI-Exception */
	    if(!newName.equals("")){
		success = mwMain.addRelation(ISName, newName, attNames, attTypes);
		System.out.println("Done.  got success value back"); }
	    else
		success = false;
	    /*	}catch(java.rmi.RemoteException e){
	    e.printStackTrace();
	    }*/

	if(success)
	    {
		allViews = new Vector();
		try {
		    allViews = (Vector)mwMain.getAllViews();
		}catch (Exception e) {e.printStackTrace();}

		// Reset mkb hierarchy tree structure
		isFrame.allISObjects = theMKB.getISs();
		isFrame.buildISTree();
		isFrame.invalidate();
		isFrame.validate();
		isFrame.repaint();

		isFrame.mkbTree.expandPath(arTreePath);

		eFrame.updateInfo(allViews);
		eFrame.setVisible(true);
	    }
	else
	    {
		messageTime = new myMessageBox("Error adding relation");
		messageTime.setVisible(true);
	    }

	arFrame.setVisible(false);
    }

    /**
     * Hides arFrame
     */
    public void arFrame_cancel_MouseClicked()
    {
	arFrame.setVisible(false);
    }

    /**
     * Hides eFrame
     */
    public void eFrame_ok_MouseClicked()
    {
	eFrame.setVisible(false);
    }

    /**
     * Prompts eFrame to display the extent differences frame
     */
    public void eFrame_disExtDif_MouseClicked()
    {
	eFrame.showExtentDifs();
    }

    /**
     * Return the InformationSource that belongs to the selected name */
    public IS getISfromName(String theIS)
    {
        return theMKB.findIS(theIS);
    }

    /**
     * Returns if an IS has been selected
     */
    public boolean ISSelected(TreePath path)
    {
	if(path == null)
	    return false;
	else
	    return (path.toString().indexOf(',') == path.toString().lastIndexOf(','));
    }

    /**
	 * Return if a relation has been selected
	 */
    public boolean relSelected(TreePath path)
    {
	if(path == null)
	    return false;
	else
	    return (componentCount(path) == 3);
    }

    /**
	 * Return if an attribute has been selected
	 */
    public boolean attSelected(TreePath path)
    {
	if(path == null)
	    return false;
	else
	    return (componentCount(path) == 4);
    }

    /**
     * Returns the number of components in the representated in the path
     * Since each component is separated by a comma, it just counts the
     * commas in the string representation of the path
     */
    public int componentCount(TreePath path)
    {
	int commas = 0;
	int index;

	String sPath = path.toString();

	// Go through the commas and count until get -1 as index of next comma
	do
	    {
		index = sPath.indexOf(',');
		sPath = sPath.substring(index + 1, sPath.length());
		commas++;
	    }while(index != -1);

	return commas;
    }

    /**
     * Takes path and returns name of attribute if the path contains an attribute
     */
    public String getAttNameFromPath(TreePath path)
    {
	// If there are four components in the path then there is an attribute and it is
	// obviously the last component in the path so . . .
	if(componentCount(path) == 4)
	    {
		// . . . get name and return it
		return path.getLastPathComponent().toString();
	    }
	else
	    return new String("no attribute");
    }

    /**
     * Takes path and returns name of relation if the path contains a relation
     */
    public String getRelNameFromPath(TreePath path)
    {
	// If there are four components then path --> {mkb, is, rel, att}
	// the relation is the parent of the last component so . . .
	if (componentCount(path) == 4)
	    {
		// . . . get name and return it
		return ((MutableTreeNode)(path.getLastPathComponent())).getParent().toString();
	    }
	// if there are three components then path --> {mkb, is, rel}
	// the relation is the last component in the path so . . .
	else if(componentCount(path) == 3)
	    {
		// . . . get name and return it
		return path.getLastPathComponent().toString();
	    }
	else
	    return new String("no relation");
    }

    /**
     * Takes path and returns the name of the InformationSource if the path contains an Information Source
     */
    public String getISNameFromPath(TreePath path)
    {
	// If there are four components then path --> {mkb, is, rel, att}
	// the IS is the parent of the parent of the last component so . . .
	if(componentCount(path) == 4)
	    {
		// . . . watch this trick of getting the name and returning it
		return ((MutableTreeNode)(path.getLastPathComponent())).getParent().getParent().toString();
	    }
	// If there are three components then path --> {mkb, is, rel}
	// the IS is the parent of the last component so . . .
	else if(componentCount(path) == 3)
	    {
		// . . . get name and return it
		return ((MutableTreeNode)(path.getLastPathComponent())).getParent().toString();
	    }
	// If there are two components then path --> {mkb, is}
	// the IS is the last component in the path so . . .
	else if(componentCount(path) == 2)
	    {
		// . . . get name and return it
		return path.getLastPathComponent().toString();
	    }
	else
	    return null;

    }

}

class Alive extends Thread{
    Main mwMain;

    boolean execute = true;

    Applet app;

    int i;

    public Alive(Main theMain, Applet theapp){
	mwMain = theMain;
	app = theapp;
	i=0;
	execute = true;
    }

    public void run(){
	while(execute){
	    try{
		app.showStatus("Running " + i + " seconds");
		mwMain.AliveMsg();
		sleep(1000);
		i++;
	    }catch(Exception e){
		System.out.println("Alive cannot sleep !");
	    }
	}
	System.out.println("Alive is destoried!");    
    }

    public void destory(){
	execute = false;
    }
}
