////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package EVEDemo.ISChangeApplet;

import java.lang.*;
import java.util.*;
import java.awt.*;
import java.applet.*;
import java.awt.event.*;
import java.text.NumberFormat;
import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.border.*;

import DSRG.VKB.QC;

public class QCFrame extends JDialog {

    QC myQC;
    GridBagLayout gridbag = new GridBagLayout();
    JPanel jp = new JPanel(gridbag);

    Color qualityTextColor = Color.magenta.darker();
    Color costTextColor = Color.black;
    Color sumTextColor = Color.blue;

    Color firstLevelBGC = Color.gray.brighter();
    Color secondLevelBGC = Color.lightGray;
    Color thirdLevelBGC = Color.white;

    public QCFrame(ISMaster parent, QC theQC) {
        super(parent,false);
        myQC = theQC;
        init();
        show();
    }

    public void init() {

        SymMouse aSymMouse = new SymMouse();

        GridBagConstraints c = new GridBagConstraints();
        JPanel buttonPanel = new JPanel(new FlowLayout());
        JButton ok = new JButton("OK");
        ok.setBackground(java.awt.Color.lightGray);
        ok.addMouseListener(aSymMouse);
        buttonPanel.add(ok);

        jp.setLayout(gridbag);
        c.fill = GridBagConstraints.BOTH;
        c.weightx = 2;
        c.gridwidth = 1;
        c.gridheight = 2;
        c.weighty = 1;

        // first level
           
        makelabel("Rho Attribute:", gridbag, c, 
                  qualityTextColor, firstLevelBGC,false);
        makelabel(formattedDouble(myQC.getRhoAttr()).toString(), gridbag, c, 
                  costTextColor, firstLevelBGC,true);
        
        makelabel("Rho Extent:", gridbag, c, 
                  qualityTextColor, firstLevelBGC,false);
        makelabel(formattedDouble(myQC.getRhoExt()).toString(), gridbag, c, 
                  costTextColor, firstLevelBGC,true);   


        makelabel("Cost M:", gridbag, c, 
                  sumTextColor, firstLevelBGC,false);
        makelabel(formattedDouble(myQC.getCostM()).toString(), gridbag, c, 
                  costTextColor, firstLevelBGC,true);
  
        makelabel("Cost T:", gridbag, c, 
                  sumTextColor, firstLevelBGC,false);
        makelabel(formattedDouble(myQC.getCostT()).toString(), gridbag, c, 
                  costTextColor, firstLevelBGC,true);

        makelabel("Cost IO:", gridbag, c, 
                  sumTextColor, firstLevelBGC,false);
        c.gridwidth = GridBagConstraints.REMAINDER; 
        makelabel(formattedDouble(myQC.getCostIO()).toString(), gridbag, c, 
                  costTextColor, firstLevelBGC,true);

        c.weightx = 2;
        c.gridwidth = 1;
        c.gridheight = 2;
        c.weighty = 1;
        makelabel("Diverg. Attributes:", gridbag, c, 
                  qualityTextColor, firstLevelBGC,false);
        makelabel(formattedDouble(myQC.getDDattr()).toString(), gridbag, c, 
                  costTextColor, firstLevelBGC,true);
         
        makelabel("Diverg. Extents:", gridbag, c, 
                  qualityTextColor, firstLevelBGC,false);
        makelabel(formattedDouble(myQC.getDDext()).toString(), gridbag, c, 
                  costTextColor, firstLevelBGC,true);

        makelabel("CF M:", gridbag, c, 
                  sumTextColor, firstLevelBGC,false);
        makelabel(formattedDouble(myQC.getCFM()).toString(), gridbag, c, 
                  costTextColor, firstLevelBGC,true);

        makelabel("CF T:", gridbag, c, 
                  sumTextColor, firstLevelBGC,false);
        makelabel(formattedDouble(myQC.getCFT()).toString(), gridbag, c, 
                  costTextColor, firstLevelBGC,true);
 
        makelabel("CF IO:", gridbag, c, 
                  sumTextColor, firstLevelBGC,false);
        c.gridwidth = GridBagConstraints.REMAINDER;
        makelabel(formattedDouble(myQC.getCFIO()).toString(), gridbag, c, 
                  costTextColor, firstLevelBGC,true);

        // second level
 
        c.weightx = 1; 
        c.gridwidth = 2;
        c.gridheight = 1;
        makelabel("Rho Quality:", gridbag, c, 
                  qualityTextColor, secondLevelBGC,false);
        //c.gridwidth = 1;
        makelabel(formattedDouble(myQC.getRhoQuality()).toString(), gridbag, c,
                  costTextColor, secondLevelBGC,true);
  

        c.gridwidth = 3;

        makelabel("Rho Cost:", gridbag, c, 
                  sumTextColor, secondLevelBGC,false);
        c.gridwidth = GridBagConstraints.REMAINDER;

        makelabel(formattedDouble(myQC.getRhoCost()).toString(), gridbag, c, 
                  costTextColor, secondLevelBGC,true);
          
        //        c.weightx = 2;
        c.gridwidth = 2;

        makelabel("Divergence on Quality:", gridbag, c, 
                  qualityTextColor, secondLevelBGC,false);

        makelabel(formattedDouble(myQC.getQuality()).toString(), gridbag, c, 
                  costTextColor, secondLevelBGC,true);
 
        c.gridwidth = 3;
        makelabel("Cost:", gridbag, c, 
                  sumTextColor, secondLevelBGC,false);
        c.gridwidth = GridBagConstraints.REMAINDER;
        //c.gridwidth = 1;
        //c.weighty= 3;
        makelabel(formattedDouble(myQC.getCost()).toString(), gridbag, c, 
                  costTextColor, secondLevelBGC,true);
         
        // third level

        //        c.weightx = 3; 
        c.gridwidth =4;

        makelabel("QC Value:", gridbag, c, 
                  sumTextColor, thirdLevelBGC,false);
        c.gridwidth = GridBagConstraints.REMAINDER;

        makelabel(formattedDouble(myQC.getQCValue()).toString(), gridbag, c, 
                  costTextColor, thirdLevelBGC,true);

        getContentPane().add("Center",jp);
        getContentPane().add("South",buttonPanel); 
    }

    public void setQC(QC newQC) {
        myQC = newQC;
        jp.removeAll();
        init();
        invalidate();validate();repaint();
    }

    String formattedDouble(double d)
    {
        String text = null;
        NumberFormat format = NumberFormat.getInstance();
        format.setMaximumFractionDigits(5);
        format.setMinimumFractionDigits(0);
        format.setMinimumIntegerDigits(1);

        text = format.format(d);

        return text;

    }

    protected void makelabel(String name, GridBagLayout gridbag, 
                             GridBagConstraints c, Color fg, Color bg, boolean isANumber) {
        JLabel myLabel = new JLabel(name,JLabel.RIGHT);
        if (isANumber) myLabel.setHorizontalAlignment(JLabel.LEFT);
        myLabel.setOpaque(true);
        myLabel.setBorder(new EmptyBorder(2,4,2,4));
        //myLabel.setBorder(new LineBorder(Color.green));
        myLabel.setBackground(bg);
        myLabel.setForeground(fg);
        gridbag.setConstraints(myLabel, c);
        myLabel.setFont(new Font("Dialog", Font.PLAIN, 12));
        jp.add(myLabel);
    }

    //Constructor for the class

    class SymMouse extends java.awt.event.MouseAdapter
    {
	public void mouseClicked(java.awt.event.MouseEvent event)
	{
            dispose();
	}
    }
}
