////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package EVEDemo.ISChangeApplet;

import EVEDemo.MW.*;
import DSRG.MW.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;

/**
 * This class creates a frame which asks for information about a new relation and confirms its addition
 * It is shown only when Add Relation is selected from an instance of ISChange.
 *
 * @autor Amber Van Wyk
 */
public class addRelFrame extends JFrame
{
    JLabel name;
    JTextField nameTF;
    JLabel atts;
    JComboBox attInfo;
    JButton ok;
    JButton cancel;
    JPanel nameStuff;
    JPanel attStuff;
    JPanel buttons;

    /**
     * Class Constructor. <BR>
     * Initializes all objects on the frame
     */
	public addRelFrame()
	{
	    // Initialize Frame
        setTitle("Add Relation");
		setResizable(false);
		setSize(450,125);
		getContentPane().setLayout(new GridLayout(3, 1, 10, 10));

        // Initialize objects for name information
        name = new JLabel("Relation Name");
        name.setFont(new Font("Dialog", Font.PLAIN, 14));
        name.setForeground(Color.blue);
        nameTF = new JTextField();
        nameTF.setPreferredSize(new Dimension(315, 25));

        // add name stuff to nameStuff panel
        nameStuff = new JPanel();
        nameStuff.add(name);
        nameStuff.add(nameTF);

        // Initialize objects for attribute information
        atts = new JLabel("      Attributes");
        atts.setFont(new Font("Dialog", Font.PLAIN, 14));
        atts.setForeground(Color.blue);
        attInfo = new JComboBox();
        attInfo.setPreferredSize(new Dimension(315, 25));
        attInfo.addItem("oneAttribute(string)");
        attInfo.addItem("twoAttribute(date)");
        // JComboBox is not editable by default.  If we wish to be able to edit it, we must
        // specify that capability.

        // add attribute stuff to attStuff panel
        attStuff = new JPanel();
        attStuff.add(atts);
        attStuff.add(attInfo);

        // Initialize buttons
        ok = new JButton("OK");
        cancel = new JButton("Cancel");

        // add buttons to buttons panel
        buttons = new JPanel();
        buttons.add(ok);
        buttons.add(cancel);

        // Add panels to frame
        getContentPane().add(nameStuff);
        getContentPane().add(attStuff);
        getContentPane().add(buttons);
	}

    /**
     * (mostly generated by Visual Cafe)
     * in addition to setting visibility, sets location and upon showing resets frame
     */
	public synchronized void setVisible(boolean show)
	{
	    if(show)
	    {
		    setLocation(100, 100);
		    nameTF.setText("");
		    super.setVisible(true);
		    nameTF.requestFocus();
		}
		else
		{
		    super.setVisible(false);
		}
	}

    /**
     * (generated by Visual Cafe)
     */
	public void addNotify()
	{
	    // Record the size of the window prior to calling parents addNotify.
	    Dimension d = getSize();

		super.addNotify();

		if (fComponentsAdjusted)
			return;

		// Adjust components according to the insets
		setSize(getInsets().left + getInsets().right + d.width, getInsets().top + getInsets().bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(getInsets().left, getInsets().top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}

    /**
     * Used for addNotify check.
     */
	boolean fComponentsAdjusted = false;

    /**
     * (generated by Visual Cafe)
     */
	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == addRelFrame.this)
				Frame1_WindowClosing(event);
		}
	}

    /**
     * (generated by Visual Cafe)
     */
	void Frame1_WindowClosing(java.awt.event.WindowEvent event)
	{
		setVisible(false);		 // hide the Frame
	}

    /**
     * Gets name for the new relation
     */
	public String getNewName()
	{
	    return nameTF.getText();
	}

    /**
     * Gets names of the attributes which belong to the new relation
     */
	public Vector getAttNames()
	{
	    Vector names = new Vector();
	    String item = null;

	    for(int g = 0; g < attInfo.getItemCount(); g++)
	    {
	        item = (String)attInfo.getItemAt(g);
	        item = item.substring(0, item.indexOf('(')).trim();
	        System.out.println(item);
	        names.addElement(item);
	    }
        System.out.println("Names: " + names.toString());
	    return names;
	}

    /**
     * Gets types of the attributes which belong to the new relation
     */
	public Vector getAttTypes()
	{
	    Vector types = new Vector();
	    String item = null;

	    for(int g = 0; g < attInfo.getItemCount(); g++)
	    {
	        item = (String)attInfo.getItemAt(g);
	        item = item.substring(item.indexOf('(') + 1, item.indexOf(')')).trim();
	        System.out.println(item);
	        types.addElement(item);
	    }

        System.out.println("Types: " + types.toString());
	    return types;
	}
}
