////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package EVEDemo.ISChangeApplet;

import EVEDemo.MW.*;
import DSRG.MW.*;
import DSRG.MKB.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;

/**
 * This class creates a frame which shows the new and old defintions for each view.  It is shown automatically
 * after a capbility change has been made.
 *
 * @author Amber Van Wyk, Summer 1998
 */
public class evolveFrame extends JFrame implements ListSelectionListener
{

    private static final boolean DEBUG = false;

  /**
   * local copy of allViews
   */
  public Vector allViews = new Vector();
  
  JButton ok;
  JButton disExtDif;
  
  JLabel lblView;
  JList viewList;
  JScrollPane viewsScrollPane;
  Vector viewItems;
  
  MyTableModel viewDefNewModel;
  JTable viewDefNewTable;
  JScrollPane viewDefNewSP;
  JLabel lblViewDefNew;
  Object[][] stuff;
  public Object[][] data;   // this will hold the columns to be sent to extDifFrame
  
  MyTableModel viewDefOldModel;
  JTable viewDefOldTable;
  JScrollPane viewDefOldSP;
  JLabel lblViewDefOld;
  
  public EVEDemo.MW.View tempView;  // this will hold the view to be sent to extDifFrame
  EVEDemo.MW.View tempViewOld;
  
  EVEDemo.MW.Main mwMain;
  
  extDifFrame edFrame;
  
  /**
   * Class constructor. <BR>
   * Intializes all objects on this frame.
   */
  public evolveFrame(EVEDemo.MW.Main mymwMain, Vector theViews)
    {
      allViews = theViews;
      mwMain = mymwMain;
      
      // Initialize frame, set Title, layout, visibility, size, and background color
      setTitle("Results of the View Evolution Process");
      getContentPane().setLayout(new FlowLayout());
      setVisible(false);
      setResizable(false);
      setSize(575,585);
      setBackground(java.awt.Color.lightGray);
      
      // Initialize vkbOK button
      ok = new JButton();
      ok.setText("OK");
      ok.setBackground(java.awt.Color.lightGray);
      
      // Initialize display extent button
      disExtDif = new JButton();
      disExtDif.setText("Display Extent Differences");
      disExtDif.setBackground(Color.lightGray);
      
      // Initialize list of possible views
      initViewList();
      viewsScrollPane.setSize(new Dimension(550, 50));
      
      // Initialize views label
      lblView = new JLabel("Views");
      lblView.setFont(new Font("Dialog", Font.PLAIN, 14));
      lblView.setForeground(Color.blue);
      
      // Initialize new view definition label
      lblViewDefNew = new JLabel("New View Definition");
      lblViewDefNew.setFont(new Font("Dialog", Font.PLAIN, 14));
      lblViewDefNew.setForeground(Color.blue);
      
      // Initialize old view definition label
      lblViewDefOld = new JLabel("Old View Definition");
      lblViewDefOld.setFont(new Font("Dialog", Font.PLAIN, 14));
      lblViewDefOld.setForeground(Color.blue);
      
      viewDefOldSP = new JScrollPane();
      
      // Add all objects to the frame.  flow layout works out great for this! yea!
      getContentPane().add(lblView);
      getContentPane().add(viewsScrollPane);
      getContentPane().add(lblViewDefNew);
      getContentPane().add(viewDefNewSP);
      getContentPane().add(lblViewDefOld);
      getContentPane().add(viewDefOldSP);
      getContentPane().add(ok);
      getContentPane().add(disExtDif);
    }
  
  /**
   * Impletmented for ListSelectionListener.  <BR>
   * Catches the event of the selected item in the list of views changing and then
   * shows the views associated with that item
   */
  public void valueChanged(ListSelectionEvent e)
    {
      if (!e.getValueIsAdjusting())
        {
	  JList theList = (JList)e.getSource();
	  int index = theList.getSelectedIndex();
	  if (DEBUG) System.out.println("Selected: " + (String)viewItems.elementAt(index));
	  
	  String defString = null;
	  
	  String[] columnNamesX = {"SQL", "E-SQL"};
	  
	  // get the view that was selected and its old counterpart
	  tempView = (EVEDemo.MW.View)allViews.elementAt(index);
	  if(tempView.getOldView() != null)
            {
	      tempViewOld = tempView.getOldView();
	      try {
		//defString = new String(mwMain.getCorrectOldDefinition(tempViewOld.getName()));
		defString = tempViewOld.getDefinition();
	      }  catch (Exception e1) {}
	      Object dataOld[][] = setTableValues(defString);
	      viewDefOldModel = null;
	      viewDefOldTable = null;
	      viewDefOldModel = new MyTableModel(columnNamesX, dataOld);
	      viewDefOldTable = new JTable(viewDefOldModel);
	      viewDefOldTable.setPreferredScrollableViewportSize(new Dimension(550, 130));
	      
	      TableColumn col = null;
	      col = viewDefOldTable.getColumnModel().getColumn(0);
	      col.setMinWidth(296);
	      col = viewDefOldTable.getColumnModel().getColumn(1);
	      col.setMinWidth(134);
	      if(viewDefOldSP == null)
                {
		  viewDefOldSP = new JScrollPane(viewDefOldTable);
                }
	      else
                {
		  viewDefOldSP.setViewportView(viewDefOldTable);
                }
	      viewDefOldSP.invalidate();
	      viewDefOldSP.validate();
	      viewDefOldSP.repaint();
            }
	  
	  // Get values for the tables
	  try {
	    //defString = new String(mwMain.getCorrectDefinition(tempView.getName()));
	    defString = tempView.getDefinition();
	  }  catch (Exception e1) {}
	  data = setTableValues(defString);
	  
	  // If there are old values, get rid of them
	  viewDefNewModel = null;
	  viewDefNewTable = null;
	  
	  viewDefNewModel = new MyTableModel(columnNamesX, data);
	  viewDefNewTable = new JTable(viewDefNewModel);
	  viewDefNewTable.setPreferredScrollableViewportSize(new Dimension(550, 130));
	  
	  // Set column widths
	  TableColumn col = null;
	  col = viewDefNewTable.getColumnModel().getColumn(0);
	  col.setMinWidth(296);
	  col = viewDefNewTable.getColumnModel().getColumn(1);
	  col.setMinWidth(134);
	  
	  // Create the scroll pane for the selected view and put the table in it
	  if(viewDefNewSP == null)
            {
	      viewDefNewSP = new JScrollPane(viewDefNewTable);
            }
	  else
            {
	      viewDefNewSP.setViewportView(viewDefNewTable);
            }
	  
	  
	  
	  viewDefNewSP.invalidate();
	  viewDefNewSP.validate();
	  viewDefNewSP.repaint();
	  this.invalidate();
	  this.validate();
	  this.repaint();
        }
    }
  
  /**
   * Initializes the list of views
     */
    public void initViewList()
    {

        // Initialize the vector for the view items
        viewItems = new Vector();

        // Create a temporary view
        EVEDemo.MW.View aView = null;

        // Add each view name to the list of view items
        for (int i=0; i < allViews.size(); i++)
        {
             aView = (EVEDemo.MW.View)allViews.elementAt(i);
             viewItems.addElement( (String)aView.getName() );
        }

        // Use vector of items to create new list
        viewList = new JList(viewItems);

        // Set selection mode to single
        viewList.setSelectionMode(0);

        // Add selection listener
        viewList.addListSelectionListener(this);

        // Set size and put in scroll pane
        viewList.setPreferredSize(new Dimension(550, 50));
        if(viewsScrollPane == null)
        {
            viewsScrollPane = new JScrollPane(viewList);
        }
        else
        {
            viewsScrollPane.setViewportView(viewList);
        }

        if (DEBUG) System.out.println("list size: " + viewList.getModel().getSize());

        //Set selection to first view in list
        viewList.setSelectedIndex(0);
        viewsScrollPane.invalidate();
        viewsScrollPane.validate();
        viewsScrollPane.repaint();
    }

    /**
     * Sets the values for the table which displays the view definition
     */
    public Object[][] setTableValues(String defString)
    {
        // Get view defintion string from theView
        //System.out.println("the defString for " + theView.getName() + "is:" + defString + "end:)");
        //if(defString.equals(""))
        //    System.out.println("defString is emptyness and stuff like that");
        // Replace tab characters w/ spaces (needed until getDefinition function is modified to fit our needs)
        defString = defString.replace('\t', ' ');

        // Create the two vectors for building the table
        Vector sqlVec = new Vector();
        Vector esqlVec = new Vector();

        int nextLP; // next left parentheses
        int nextNL; // next new line
        boolean eos = false;    // end of string

        while(!eos)
        {
            nextLP = defString.indexOf('(');
            nextNL = defString.indexOf('\n');
            /*System.out.println("nextLP: " + nextLP);
            System.out.println("nextNL: " + nextNL);
            System.out.println(defString);*/

            // If there is another left parenthesis
            if(nextLP != -1)
            {
                // and it comes before the next newline
                if(nextLP < nextNL)
                {
                    sqlVec.addElement("      " + defString.substring(0, nextLP - 1).trim());
                    esqlVec.addElement(defString.substring(defString.indexOf('('), nextNL).trim());
                    defString = defString.substring(nextNL, defString.length()).trim();
                }
                // If there is another new line
                else if (nextNL < nextLP)
                {
                    if(nextNL == -1)
                    {
                        sqlVec.addElement("      " + defString.substring(0, nextLP).trim());
                        esqlVec.addElement(defString.substring(defString.indexOf('('), defString.length()).trim());
                        defString = "";
                    }
                    else
                    {
                        sqlVec.addElement(defString.substring(0, nextNL).trim());
                        esqlVec.addElement(" ");
                        defString = defString.substring(nextNL, defString.length()).trim();
                    }
                }
            }
            else if(nextNL != -1)
            {
                sqlVec.addElement(defString.substring(0, nextNL).trim());
                esqlVec.addElement(" ");
                defString = defString.substring(nextNL, defString.length()).trim();
            }
            else
            {
                eos = true;
            }
        }

        stuff = new Object[sqlVec.size()][2];

        for(int j = 0; j < sqlVec.size(); j++)
        {
            stuff[j][0] = sqlVec.elementAt(j);
            stuff[j][1] = esqlVec.elementAt(j);
        }

        return stuff;
    }

    /**
     * Updates the allViews Vector and reinitializes the list of views
     */
    public void updateInfo(Vector theViews)
    {
        allViews = (Vector)theViews;
        initViewList();
    }

    /**
     * Shows the frame to display the extent differences between the new and old view
     */
    public void showExtentDifs()
    {
        //System.out.println("tempView name: " + tempView.getName());
        //System.out.println("tempView old : " + tempView.getOldView().getName());

        if(tempView.getDefinition().equals("") || tempViewOld.getDefinition().equals(""))
        {
            myMessageBox messageTime = new myMessageBox("Extent Differences cannot be compared");
            messageTime.setVisible(true);
        }
        else
	  {
            // data is for the defintion table, extent tables will be built by constructor from tempView
            // tempView is the currently selected view
	    /*
	     * Same trick provided here as for mkbvkb.java
	     */
              //RelationExtent extent = null;
	    // Get the Extent of tempView here.
	    //try{
	      // Try to get the Extent from the server Site.
	      //extent = mwMain.getViewExtent(tempView.getName(),false); // current extent
	      // Assign the Extent to the client site, but disable to renew the extent at the client site.
	      //tempView.setExtent(extent);
                //	      extent = tempView.newGetExtent();
                //Attribute a; //=(Attribute)extent.getAttributesNames().elementAt(0);
              //	      extent = tempViewOld.newGetExtent();
	      //extent = mwMain.getViewExtent(tempView.getName(),true);
	      //tempView.getOldView().setExtent(extent);  //  stupid hack again, trying to update the old view extent
	      // on the client side only
	      //a = (Attribute)(extent.getRelation().getAttributes().elementAt(0));

              //}catch(Exception e){
              // System.out.println("Debug: evolveFrame.showExtentDifs");
              //}

	    edFrame = new extDifFrame(data, tempView);
	    edFrame.setVisible(true);
	  }
    }
  
    /**
     * (mostly generated by Visual Cafe)
     * in addition to setting visibility, sets location
     */
	public synchronized void setVisible(boolean show)
	{
	    if(show)
	    {
		    setLocation(50, 50);
		    super.setVisible(true);
		}
		else
		{
		    super.setVisible(false);
		}
	}

    /**
     * (generated by Visual Cafe)
     */
	public void addNotify()
	{
	    // Record the size of the window prior to calling parents addNotify.
	    Dimension d = getSize();

		super.addNotify();

		if (fComponentsAdjusted)
			return;

		// Adjust components according to the insets
		setSize(getInsets().left + getInsets().right + d.width, getInsets().top + getInsets().bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(getInsets().left, getInsets().top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}

    /**
     * Used for addNotify check.
     */
	boolean fComponentsAdjusted = false;
}
