////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package EVEDemo.ISChangeApplet;

import EVEDemo.MW.*;
import DSRG.MW.*;
import DSRG.MKB.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;

/**
 * This class creates a frame which displays the extents associated
 * with the new and old definition of the view
 *
 * @author Amber Van Wyk, Summer 1998 */
public class extDifFrame extends JFrame
{

private static final boolean DEBUG = false;

  JButton edOK;
  JTable viewDefTable;
  MyTableModel viewDefModel;
  JScrollPane defScrollPane;
  JLabel lblViewDef;
  
  JTable extentTable;
  MyTableModel extentModel;
  JScrollPane extentScrollPane;
  JLabel extentLabel;
  
  JTable extentOldTable;
  MyTableModel extentOldModel;
  JScrollPane extentOldScrollPane;
  JLabel extentOldLabel;
  
  Vector allAttr;
  Object[][] data;
  int numRows;
  int numCols;
  
  /**
   * Class Constructor. <BR>
   * Initializes all objects on the frame */
  public extDifFrame(Object[][] data, EVEDemo.MW.View theView)
    {
      // Initialize frame, set Title, layout, visibility, size, and background color
      setTitle("View Extent Differences");
      getContentPane().setLayout(new FlowLayout());
      setVisible(false);
      setResizable(false);
      setSize(575,450);
      setBackground(java.awt.Color.lightGray);
      
      // Initialize OK button
      edOK = new JButton();
      edOK.setText("OK");
      
      // Initialize View Definition Label
      lblViewDef = new JLabel("New View Definition");
      lblViewDef.setFont(new Font("Dialog", Font.PLAIN, 14));
      lblViewDef.setForeground(Color.blue);
      
      // Initalize extent labels
      extentLabel = new JLabel("New Extent");
      extentLabel.setFont(new Font("Dialog", Font.PLAIN, 14));
      extentLabel.setForeground(Color.blue);
      extentOldLabel = new JLabel("Old Extent");
      extentOldLabel.setFont(new Font("Dialog", Font.PLAIN, 14));
      extentOldLabel.setForeground(Color.blue);
      
      // Initialize Table for view definition
      String[] columnNames = {"SQL", "ESQL"};
      viewDefModel = new MyTableModel(columnNames, data);
      viewDefTable = new JTable(viewDefModel);
      viewDefTable.setPreferredScrollableViewportSize(new Dimension(550, 100));
      
      // Set column widths for view definition table
      TableColumn col = null;
      col = viewDefTable.getColumnModel().getColumn(0);
      col.setMinWidth(296);
      col = viewDefTable.getColumnModel().getColumn(1);
      col.setMinWidth(134);
      
      // Put viewDefTable in its scroll pane
      defScrollPane = new JScrollPane(viewDefTable);
      
      // Initialize New Relation Table
      extentModel = buildExtent(theView);
      extentTable = new JTable(extentModel);

      // extentTable.setPreferredSize(new Dimension(numCols * 100, numRows * 15));
      // extentTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
      /*
	for(int i = 0; i < numCols; i++)
        {
	col = extentTable.getColumnModel().getColumn(i);
	col.setMinWidth(100);
        }
      */

      extentScrollPane = new JScrollPane(extentTable);
      extentScrollPane.setPreferredSize(new Dimension(570, 100));
      
      // Initialize Old Relation Table
      extentOldModel = buildExtent(theView.getOldView());
      extentOldTable = new JTable(extentOldModel);
      // extentOldTable.setPreferredSize(new Dimension(numCols * 100, numRows * 15));
      // extentOldTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
      
      /*
	for(int j = 0; j < numCols; j++)
        {
	col = extentOldTable.getColumnModel().getColumn(j);
	col.setMinWidth(100);
        }
      */

      extentOldScrollPane = new JScrollPane(extentOldTable);
      extentOldScrollPane.setPreferredSize(new Dimension(570, 100));
      
      // Add objects
      getContentPane().add(lblViewDef);
      getContentPane().add(defScrollPane);
      getContentPane().add(extentLabel);
      getContentPane().add(extentScrollPane);
      getContentPane().add(extentOldLabel);
      getContentPane().add(extentOldScrollPane);
      getContentPane().add(edOK);
      
      System.out.println(theView.getName());
      
      SymMouse aSymMouse = new SymMouse();
      edOK.addMouseListener(aSymMouse);
    }
  
  /**
   * Builds the TableModel for the extent table
   */
  private MyTableModel buildExtent(EVEDemo.MW.View theView)
    {

        Table extent = theView.getExtent();

        if (DEBUG) System.out.println("NEWEXTENT:\n"+extent);

        // we need this for the attribute names.
        allAttr = extent.getAttributeNames();
        int numCols = allAttr.size();
        int numRows = extent.getSize();
        
        if (DEBUG) System.out.println("DEBUG. extDifFrame::buildExtent called. Size:"+numRows);
        
        String[] colNames = new String[numCols];
        
        // Get all Column Names
        for (int y = 0; y < numCols; y++)
            {
                colNames[y] = (String)allAttr.elementAt(y);
                if (DEBUG) System.out.println("DEBUG.  extDifFrame::buildExtent() ColNames:" + colNames[y]);
            }
        
        // Declare 2D object array for building table
        Object[][] data = new Object[numRows][numCols];
        
        // Fill data! This should be interesting.
        Tuple t;
        Enumeration te;
        int x,y;
        
        // Loop through each row in the extent
        for(y = 0; y < numRows; y++)
            {
                
                //  fetch a tuple
                t = extent.getTuple(y);
                if (t.getSize() != numCols)
                    System.err.println("extDifFrame.buildExtent: Tuple size mismatches Number of Attributes");
                te = t.getEnumeration();
                x = 0;
                while (te.hasMoreElements()) {
                    data[y][x] = (String)te.nextElement();
                    x++;
                }
            }
      
      MyTableModel theModel = new MyTableModel(colNames, data);
	
      return theModel;
	
	
    }
  
  /**
   * (generated by Visual Cafe) <BR>
   * in addition to setting visibility, sets location of the frame
   */
  public synchronized void setVisible(boolean show)
    {
      if(show)
	{
	  setLocation(75, 75);
	  super.setVisible(true);
	}
      else
	{
	  super.setVisible(false);
	}
    }


  /**
   * (generated by Visual Cafe)
   */
  public void addNotify()
    {
      // Record the size of the window prior to calling parents addNotify.
      Dimension d = getSize();

      super.addNotify();

      if (fComponentsAdjusted)
	return;

      // Adjust components according to the insets
      setSize(getInsets().left + getInsets().right + d.width, getInsets().top + getInsets().bottom + d.height);
      Component components[] = getComponents();
      for (int i = 0; i < components.length; i++)
	{
	  Point p = components[i].getLocation();
	  p.translate(getInsets().left, getInsets().top);
	  components[i].setLocation(p);
	}
      fComponentsAdjusted = true;
    }

  /**
   * Used for addNotify check.
   */
  boolean fComponentsAdjusted = false;

  /**
   * (generated by Visual Cafe)
   */
  void Frame1_WindowClosing(java.awt.event.WindowEvent event)
    {
      setVisible(false);		 // hide the Frame
    }

  /**
   * (mostly generated by Visual Cafe)
   * listener to catch mouse clicks
   */
  class SymMouse extends java.awt.event.MouseAdapter
  {
    public void mouseClicked(java.awt.event.MouseEvent event)
      {
	Object object = event.getSource();

	// Check to see if button clicked is on ISMaster
	if(object == edOK)
	  edOK_MouseClick();
      }
  }

  /**
   * Hides the frame
   */
  void edOK_MouseClick()
    {
      setVisible(false);
    }

}
