////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package EVEDemo.ISChangeApplet;

import EVEDemo.MW.*;
import DSRG.MW.*;
import DSRG.VKB.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;

/**
 * This class creates a frame which shows the different versions
   that are available for a particular viewname. This also shows
   the QC values are generated for the different versions. This
   also shows the view definition of the selected view version.
 */

public class versionFrame extends JDialog
    implements ListSelectionListener
{
    private static final boolean DEBUG = false;

    public versionFrame myself = this;

    public Vector allViews = null;
    public Vector versions = new Vector();

    //JFrame frame;
    JPanel myPane;

    JButton ok;
    JButton QC;

    JLabel lblView;
    JLabel lblVersion;
    JList viewList;
    JList versionList;
    JScrollPane viewScrollPane;
    JScrollPane versionScrollPane;
    MyTableModel versionTableModel;
    JTable versionTable;

    Vector viewItems;
    Vector Versions;

    MyTableModel viewDefNewModel;
    // JTable viewDefNewTable;
    JScrollPane viewDefNewSP;
    JLabel lblViewDefNew;
    JTextField ViewDefNewText;
    JTextArea textArea;

    Object[][] stuff;

    int[] pickedVersions;

    ISMaster myParent;
    QC currentQC = null;
    SymMouse aSymMouse = null;

    QCFrame qcFrame = null;

    /**
     * Class constructor. <BR>
     * Intializes all objects on this frame.
     */
    public versionFrame(Vector theViews,ISMaster parent)  {
    
        // create a modal JDialog 
        super(parent,false);

        myParent = (ISMaster)parent;
    
        allViews = theViews;
        pickedVersions = new int[allViews.size()];
        // init picked versions with -1 for error checking
        for (int i = 0 ; i < pickedVersions.length; i++)
            pickedVersions[i]=-1;
    

    
        aSymMouse = new SymMouse();
    
        // Initialize frame, set Title, layout, visibility, size, and
        // background color
        this.setTitle("Select Version for each Affected View");
        this.setSize(790,410);
        //        this.setResizable(false); // crashes the Linux-XServer
        getContentPane().setBackground(java.awt.Color.lightGray);
        getContentPane().setLayout(null);

        // Initialize views label
        lblView = new JLabel("Views");
        lblView.setFont(new Font("Dialog", Font.PLAIN, 14));
        lblView.setForeground(Color.blue);

        // Initialize list of possible views
        initViewList();

        // Initialize versions label
        lblVersion = new JLabel("List of Versions");
        lblVersion.setFont(new Font("Dialog", Font.PLAIN, 14));
        lblVersion.setForeground(Color.blue);

        // Initialize list of possible versions
        //versionScrollPane = new JScrollPane();

        // Initialize new view definition label
        lblViewDefNew = new JLabel("New View Definition");
        lblViewDefNew.setFont(new Font("Dialog", Font.PLAIN, 14));
        lblViewDefNew.setForeground(Color.blue);

        // initDefNew(); ????
        textArea = new JTextArea();
        textArea.setEditable(false);
        viewDefNewSP = new JScrollPane(textArea);


        // Initialize vkbOK button
        ok = new JButton();
        ok.setText("OK");
        ok.setBackground(java.awt.Color.lightGray);
        ok.addMouseListener(aSymMouse);
        // Initialize QC Value button
        QC = new JButton();
        QC.setText("QC");
        QC.setBackground(Color.lightGray);

        QC.addMouseListener(aSymMouse);

        // bounds for the UI elements

        lblView          .setBounds( 168 ,  12 ,  60 ,  24 ); 
        viewScrollPane   .setBounds(  20 ,  36 , 356 , 113 );
        lblVersion       .setBounds( 144 , 158 , 108 ,  27 );
        versionScrollPane.setBounds(  20 , 194 , 356 , 113 );
        lblViewDefNew    .setBounds( 498 ,  12 , 160 ,  26 );
        viewDefNewSP     .setBounds( 400 ,  36 , 356 , 318 );
        ok               .setBounds(  74 , 320 ,  70 ,  30 );
        QC               .setBounds( 252 , 320 ,  70 ,  30 );

        // Add all objects to the frame.
        getContentPane().add(lblView);
        getContentPane().add(viewScrollPane);
        getContentPane().add(lblVersion);
        getContentPane().add(versionScrollPane);
        getContentPane().add(lblViewDefNew);
        getContentPane().add(viewDefNewSP);
        getContentPane().add(ok);
        getContentPane().add(QC);
    }

    /**
     * Catches the event of the selected item in the list of views
     * changing and then shows the versions and the QC values
     * associated with that view item */

    public void valueChanged(ListSelectionEvent e)
    {
        if (!e.getValueIsAdjusting())
            {
                int index = viewList.getSelectedIndex();
                if (DEBUG) System.out.println("Selected: " +
                      (String)viewItems.elementAt(index)+"("+index+")");
                
                //get the list of versions and the respective QC
                //values for the view that was selected
                
                View tempView;
                // !!! List counting starts with 1, the vector starts with 0
                tempView = (EVEDemo.MW.View)allViews.elementAt(index);
                
                if (DEBUG) System.out.println("tempView: " + tempView + " "
                                              + tempView.numberOfVersions());

                String[][] data;

                if (tempView.numberOfVersions()!=-1) {
                    data = new String[tempView.numberOfVersions()][2];

                    String versionDefinition;
                    QC versionQC;

                    for (int i = 0 ; i < tempView.numberOfVersions() ; i++) {
                        //versionDefinition = tempView.getVersion(i);
                        versionQC = tempView.getQC(i);

                        //data[i][0]=versionDefinition.replace('\n',' ');
                        data[i][0]= new String("Version "+i);
                        data[i][1]=new Double(versionQC.getQCValue()).toString();

                    }
                }

                else  {
                    data = new String[0][0];
                    if (textArea != null) textArea.setText("");
                }

                if (versionTable == null) {
                    String[] columnNamesX = {"Versions", "QC Value"};
                    versionTableModel = new MyTableModel(columnNamesX, data);
                    versionTable = new JTable(versionTableModel);
                    versionTable.setPreferredScrollableViewportSize
                        (new Dimension(550, 130));
                    versionTable.addMouseListener(aSymMouse);
                }
                else versionTableModel.updateData(data);

                
                
                TableColumn col = null;
                col = versionTable.getColumnModel().getColumn(0);
                col.setMinWidth(200);
                col = versionTable.getColumnModel().getColumn(1);
                col.setMinWidth( 50);
                
                if (index>=0) 
                    if (pickedVersions[index] != -1) {
                        versionTable.setEditingRow(pickedVersions[index]);
                        versionNewDefinition();
                    }
                    else if (textArea != null) textArea.setText("");
                
                // Create the scroll pane for the selected view and
                // put the table in it
                
                if(versionScrollPane == null) {
                    if (DEBUG) System.out.println("Creating new ScrollPane");
                    versionScrollPane = new JScrollPane(versionTable);
                }
                else versionScrollPane.setViewportView(versionTable);
                
                versionScrollPane.invalidate();
                versionScrollPane.validate();
                versionScrollPane.repaint();
                versionTable.invalidate();
                versionTable.validate();
                versionTable.repaint();

            }
    }

    /**
     * Initializes the list of views
     */
    public void initViewList()
    {
        // Initialize the vector for the view items
        viewItems = new Vector();

        // Create a temporary view
        EVEDemo.MW.View aView = null;

        // Add each view name to the list of view items
        for (int i=0; i < allViews.size(); i++)
            {
                aView = (EVEDemo.MW.View)allViews.elementAt(i);
                viewItems.addElement( (String)aView.getName() );
            }

        // Use vector of items to create new list
        viewList = new JList(viewItems);

        // Set selection mode to single
        viewList.setSelectionMode(0);

        // Add selection listener
        viewList.addListSelectionListener(this);

        // Set size and put in scroll pane
        //viewList.setPreferredSize(new Dimension(550, 50));
        if(viewScrollPane == null)
            {
                viewScrollPane = new JScrollPane(viewList);
            }
        else
            {
                viewScrollPane.setViewportView(viewList);
            }

        //Set selection to first view in list
        viewList.setSelectedIndex(0);
    }


    /**
     * Get the version that was selected and retrieve the view definition for
     * the version
     */
    public void versionNewDefinition()    {
        int index = viewList.getSelectedIndex();
        
        //get the version definition for the version that was selected
        
        View tempView;

        tempView = (EVEDemo.MW.View)allViews.elementAt(index);
        
        if ((tempView.numberOfVersions()!=-1)) {
            
	    int versionNumber = versionTable.getSelectedRow();
            currentQC = tempView.getQC(versionNumber);
	    pickedVersions[index] = versionNumber;
            
            if (qcFrame != null)
                if (qcFrame.isVisible()) qcFrame.setQC(currentQC);

	    String mydef = tempView.getVersion(versionNumber);
	    textArea.setText(mydef);
        }
        
        //        textArea.invalidate();
    }
    
    
    class SymMouse extends java.awt.event.MouseAdapter
    {
	/**
	 * catches mouse clicks
	 */
	public void mouseClicked(java.awt.event.MouseEvent event)
	{
	    Object object = event.getSource();
            
            if (versionTable != null)
                if (object == versionTable) {
                    myself.versionNewDefinition();
                    textArea.setCaretPosition(0);
                }
            if (object == ok) {
                // first check if all versions are picked.
                View tempView;
                boolean all_picked = true;
                for (int i = 0 ; i < allViews.size(); i++) {
                    tempView = (View)allViews.elementAt(i);
                    if (tempView.numberOfVersions()!=-1)
                        if (pickedVersions[i]==-1) {
                            all_picked = false;
                            System.out.println("Please pick a version for view "+
                                               tempView.getName());
                        }
                    // (if a view that has versions has no picked
                    // version, we're not done)
                }
                if (all_picked) {
                    myself.myParent.setPickedVersions(pickedVersions);
                    myself.dispose();
                }
                else Toolkit.getDefaultToolkit().beep();
	    }
            if (object == QC) {
                if (currentQC!=null) {
                    qcFrame = new QCFrame(myParent, currentQC);
                    qcFrame.setTitle("QC Value Derivation for the Current Version");
                    qcFrame.setSize(700, 475);
                    //qcFrame.setResizable(false);
                    
                }
                
            }
        }
    }
}
