////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package EVEDemo.MKBVKBApplet;

/*
 *  The tree cell renderer for having my own icons in an IS tree.
 *
 */

import com.sun.java.swing.Icon;
import com.sun.java.swing.ImageIcon;
import com.sun.java.swing.JLabel;
import com.sun.java.swing.JTree;
import com.sun.java.swing.tree.TreeCellRenderer;
import com.sun.java.swing.tree.DefaultMutableTreeNode;
import com.sun.java.swing.tree.TreePath;
import java.awt.Component;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;

public class MyTreeCellRenderer extends JLabel implements TreeCellRenderer, java.io.Serializable {

  /** Icon to use when the item describes an IS. */
  static protected ImageIcon        ISIcon;
  /** Icon to use when the item describes a relation. */
  static protected ImageIcon        relationIcon;

  /** Whether or not the item that was last configured is selected. */
  protected boolean            selected;

  public void init(EVEDemo.MW.Main mwMain){
    initRenderer(mwMain);
  }
  
  static void initRenderer(EVEDemo.MW.Main mwMain) {
    try {
      ISIcon = mwMain.getMyIcon("IS.gif");
      relationIcon = mwMain.getMyIcon("relation.gif");
    } catch (Exception e) {
      System.out.println("Couldn't load images: " + e);
    }
    
  }

  /**
   * This is messaged from JTree whenever it needs to get the size
   * of the component or it wants to draw it.
   * This attempts to set the font based on value, which will be
   * a TreeNode.
   */
  public Component getTreeCellRendererComponent(JTree tree, Object value,
						boolean selected, boolean expanded,
						boolean leaf, int row,
						boolean hasFocus) {
    
    String  stringValue = tree.convertValueToText(value, selected,
						  expanded, leaf, row, hasFocus);
    
    /* Set the text. */
    setText(stringValue);
    /* Tooltips used by the tree. */
    //setToolTipText(stringValue);
    
    /* Set the image. */

    TreePath myPath = tree.getPathForRow(row);
    
    if (myPath != null) {
      int count = myPath.getPathCount();
      switch (count) {
      case 2 : setIcon(ISIcon); break;
      case 3 : setIcon(relationIcon); break;
      default : setIcon(null);
      }
      //System.out.println("count is:" + count);
    }
    
    /* Set the color and the font. */
    if (selected)
      setForeground(java.awt.Color.white);
    else 
      setForeground(java.awt.Color.black);
    setFont(new Font("Dialog", 0, 12));
    

    /* Update the selected flag for the next paint. */
    this.selected = selected;
    
    return this;
  }

  /**
   * paint is subclassed to draw the background correctly.  JLabel
   * currently does not allow backgrounds other than white, and it
   * will also fill behind the icon.  Something that isn't desirable.
   */

  

  public void paint(Graphics g) {
    Color            bColor;
    Icon             currentI = getIcon();

    if(selected) {
      bColor = java.awt.Color.blue;
    }
    else if(getParent() != null) {
      bColor = getParent().getBackground();
    }
    else {
      bColor = getBackground();
    }
    g.setColor(bColor);
    if(currentI != null && getText() != null) {
      int          offset = (currentI.getIconWidth() + getIconTextGap());
	  
      g.fillRect(offset, 0, getWidth() - 1 - offset,
		 getHeight() - 1);
    }
    else
      g.fillRect(0, 0, getWidth()-1, getHeight()-1);
    super.paint(g);
  }

  
}

