////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package EVEDemo.MKBVKBApplet;

import EVEDemo.MW.*;
import DSRG.MW.*;
import DSRG.MKB.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
import com.sun.java.swing.*;
import com.sun.java.swing.tree.*;
import com.sun.java.swing.event.TreeSelectionListener;
import com.sun.java.swing.event.TreeSelectionEvent;
//import com.sun.java.swing.tree.TreeSelectionModel;
import com.sun.java.swing.table.TableColumn;


/**
 * This class creates a frame which displays the mkb hierarchy and constrainsts <BR>
 * The hierachy is a directory style tree and the constraints are displayed in tables
 * When a relation is selected, the tables are updated.  Currently, if an IS is selected,
 * the information for the most recently selected table remains.  However, this will
 * hopefully be changing soon.
 *
 * @author Amber Van Wyk, Summer 1998
 */
public class mkbFrame extends JFrame
{

  /**
   * Added by Andreas Koeller. The reference of mwMain
   */
  private Main mwMain;


  private static final boolean  DEBUG = false;
  
  Panel panLeft;
  Panel panRight;
  Label lblType;
  Label lblJoin;
  Label lblPC;
  JScrollPane jspT;
  JScrollPane jspJ;
  JScrollPane jspP;
  JTable tableT;
  JTable tableJ;
  JTable tableP;
  
  JoinConstraint thisJC;
  Condition thisCond;
  Vector condVec = new Vector();
  PCConstraint thisPC;
  PCPair thisPCPair;
  Vector pcpairVec = new Vector();
  
  JScrollPane scrollPaneTree;
  com.sun.java.swing.JTree mkbTree;
  JButton mkbOK;
  JButton mkbShowExtent;
  java.awt.Label lblSelectedNode;
  java.awt.Panel panType;
  java.awt.Panel panJoin;
  java.awt.Panel panPC;

  // for displaying relation extents
  EVEDemo.MW.View tempView;
  IS tempIS = null; // we will use this to represent the current IS
                     // (for showing extents)
  String selectedRelation; // some variables that are global now... (bad coding... I'm hacking)
  
  /**
   * Class Constructor, receives all ISObjects from the applet
   */
  public mkbFrame(Main m)
    {

      mwMain = m;
      // not cloning here - we may need to do that again when RMI comes back in
      Vector ISObjects = mwMain.getMKB().getISs(); 
      
      // Initialize frame, set Title, layout, visibility, size, and background color
      setTitle("Meta Knowledge Base");
      GridBagLayout allLayout = new GridBagLayout();
      getContentPane().setLayout(allLayout);
      setVisible(false);
      setResizable(false);
      setSize(650,450);
      setBackground(java.awt.Color.lightGray);
      
      // Initialize left panel which will hold scrollPaneTree and mkbOK
      panLeft = new Panel();
      panLeft.setLayout(new BorderLayout());
      panLeft.setSize(300,450);
      getContentPane().add( panLeft);
      
      // Initialize right panel which will hold all tables
      panRight = new Panel();
      GridBagLayout myLayout = new GridBagLayout();
      panRight.setLayout(myLayout);
      panRight.setSize(350, 450);
      getContentPane().add( panRight);
      
      // Define GridBagConstraints for panLeft and pan Right
      GridBagConstraints c = new GridBagConstraints();
      c.ipadx = 15;
      c.ipady = 15;
      c.insets = new Insets(20, 20, 20, 20);
      c.gridx = 0;
      c.gridy = 0;
      c.weightx = 0.8;
      c.weighty = 0.8;
      c.fill = GridBagConstraints.BOTH;
      allLayout.setConstraints(panLeft, c);
      c.gridx = 1;
      c.gridwidth = GridBagConstraints.REMAINDER;
      c.weightx = 0.8;
      c.weighty = 0.8;
      c.fill = GridBagConstraints.NONE;
      allLayout.setConstraints(panRight, c);
      
      // Initialize scrollPaneTree (it will house the tree) and add it to panLeft
      scrollPaneTree = new JScrollPane();
      scrollPaneTree.setSize(new Dimension(200, 325));
      panLeft.add("Center", scrollPaneTree);
      
      // Initialize the tree, we have to do this later, somewhere down there

      // Initialize mkbOK button and add it to panLeft
      mkbOK = new JButton();
      mkbOK.setActionCommand("button");
      mkbOK.setText("OK");
      mkbOK.setBackground(java.awt.Color.lightGray);
      panLeft.add("South", mkbOK);
      
      mkbShowExtent = new JButton();
      mkbShowExtent.setVisible(false);
      mkbShowExtent.setActionCommand("button");
      mkbShowExtent.setText("Show Extent");
      // mkbShowExtent.setBackground(java.awt.Color.lightGray);
      
      // Initialize label for the selected node
      lblSelectedNode = new Label("To begin, please select an item at left", Label.CENTER);
      lblSelectedNode.setFont(new Font("Dialog", Font.PLAIN, 14));
      lblSelectedNode.setForeground(new Color(128, 0, 128));
      
      // Initialize label for type constraints table
      lblType = new Label ("Type Constraints");
      lblType.setVisible(false);
      lblType.setFont(new Font("Dialog", Font.PLAIN, 14));
      lblType.setForeground(new Color(128, 0, 128));
      
      // Initialize label for join constraints table
      lblJoin = new Label ("Join Constraints");
      lblJoin.setVisible(false);
      lblJoin.setFont(new Font("Dialog", Font.PLAIN, 14));
      lblJoin.setForeground(new Color(128, 0, 128));

      // Initialize label for pc constraints table
      lblPC = new Label ("PC Constraints");
      lblPC.setVisible(false);
      lblPC.setFont(new Font("Dialog", Font.PLAIN, 14));
      lblPC.setForeground(new Color(128, 0, 128));
      
      // Initialize panel which will hold the type constraints table
      panType = new java.awt.Panel();
      panType.setSize(300, 100);
      panType.setLayout(new BorderLayout());
      
      // Initialize panel which will hold the join constraints table
      panJoin = new java.awt.Panel();
      panJoin.setSize(300, 100);
      panJoin.setLayout(new BorderLayout());
      
      // Initialize panel which will hold the pc constraints table
      panPC = new java.awt.Panel();
      panPC.setSize(300, 100);
      panPC.setLayout(new BorderLayout());
      
      // Add all items to panRight
      panRight.add(lblSelectedNode);
      panRight.add(lblType);
      panRight.add(panType);
      panRight.add(lblJoin);
      panRight.add(panJoin);
      panRight.add(lblPC);
      panRight.add(panPC);
      panRight.add(mkbShowExtent);
      
      // Define GridBagConstraints for the components in panRight
      GridBagConstraints myC = new GridBagConstraints();
      
      // Constraints for lblSelectedNode
      myC.gridx = 0;
      myC.gridy = 0;
      myC.gridwidth = GridBagConstraints.REMAINDER;	// all components are the last one in there row
      myC.weighty = 0.4;
      myLayout.setConstraints(lblSelectedNode, myC);
      
      
      // Constraints for lblType
      myC.gridy = 1;
      myLayout.setConstraints(lblType, myC);
      
      // Constraints for panType
      myC.gridy = 2;
      myC.gridheight = 2;
      myC.weighty = 0.3;
      myLayout.setConstraints(panType, myC);
      
      // Constraints for lblJoin
      myC.gridy = 4;
      myC.gridheight = 1;
      myC.weighty = 0.0;
      myLayout.setConstraints(lblJoin, myC);
      
      // Constraints for panJoin
      myC.gridy = 5;
      myC.gridheight = 2;
      myC.weighty = 0.3;
      myLayout.setConstraints(panJoin, myC);
      
      // Constraints for lblPC
      myC.gridy = 7;
      myC.gridheight = 1;
      myC.weighty = 0.0;
      myLayout.setConstraints(lblPC, myC);
      
      // Constraints for panPC
      myC.gridy = 8;
      myC.gridheight = 2;
      myC.weighty = 0.3;
      myLayout.setConstraints(panPC, myC);
      

      // Constraints for mkbShowExtent
      myC.gridy = 10;
      myC.gridheight = 1;
      myC.weighty = 0.3;
      myLayout.setConstraints(mkbShowExtent, myC);

      buildMkbTree(ISObjects);
      
      // (generated by Visual Cafe)
      SymWindow aSymWindow = new SymWindow();
      this.addWindowListener(aSymWindow);
      SymMouse aSymMouse = new SymMouse();
      mkbOK.addMouseListener(aSymMouse);
      mkbShowExtent.addMouseListener(aSymMouse);
    }
  
  /**
   * Resets constraint scrollpanes and lables to not visible
   */
  public void resetConstraintArea()
    {
      if(jspT != null) jspT.setVisible(false);
      if(jspJ != null) jspJ.setVisible(false);
      if(jspP != null) jspP.setVisible(false);

      lblType.setVisible(false);
      lblJoin.setVisible(false);
      lblPC.setVisible(false);
      mkbShowExtent.setVisible(false);
    }
  
  /**
   * Builds the tree by adding all of the leaf nodes to a root node
   */
    public void buildMkbTree(final Vector ISObjects)
    {
        // Create root node for mkbTree
        DefaultMutableTreeNode top = new DefaultMutableTreeNode("Meta Knowledge Base");

        Enumeration enumIS = ISObjects.elements();
        Enumeration enumRel = null;
        String ISName = null;
        String relName = null;
        IS tempIS = null;

        // Create temp nodes for tree
        DefaultMutableTreeNode nodeIS = null;
        DefaultMutableTreeNode nodeRel = null;

        while(enumIS.hasMoreElements())
        {
            // Get next InformationSource
            tempIS = (IS)(enumIS.nextElement());

            // Get name of tempIS
            ISName = tempIS.getName();

            // Create a node for tempIS
            nodeIS = new DefaultMutableTreeNode(ISName);

            // Get relations for tempIS
            enumRel = tempIS.getRelations().elements();

            // Traverse enumRel and all relations of tempIS to nodeIS
            while(enumRel.hasMoreElements())
            {
                relName = ((Relation)enumRel.nextElement()).getName();
                nodeRel = new DefaultMutableTreeNode(relName);
                nodeIS.add(nodeRel);
            }

            // add nodeIS to the tree
            top.add(nodeIS);
        }
	
        // Create tree with node top as root and it to the scroll pane
        mkbTree = new JTree(top);
	mkbTree.setBackground(java.awt.Color.white);
	// works now
	MyTreeCellRenderer mtcr = new MyTreeCellRenderer();
	mtcr.init(mwMain);
	mkbTree.setCellRenderer(mtcr);
	mkbTree.setRootVisible(true);

	scrollPaneTree.setPreferredSize(new Dimension(200, 325));
        scrollPaneTree.getViewport().add(mkbTree);

        // Set selection mode to single
        mkbTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

        // Listen for when the selection changes.  Wish me Luck!
        // When a node is selected set lblSelectedNode to the name of the node
        // This is where the code will need to be added for updating tables when a new node is selected
        mkbTree.addTreeSelectionListener(new TreeSelectionListener() {
	  public void valueChanged(TreeSelectionEvent e) {
	    TreePath fullPath = e.getPath();
	    String path = new String(fullPath.toString());
	    
	    int comma1;
	    int comma2;
	    
	    // If a relation is selected, change the label and constraint tables
	    if (relSelected(path))
	      {
		lblSelectedNode.setText(path.substring(path.indexOf('{'), path.length()));
		comma1 = path.indexOf(",");
		comma2 = path.indexOf(",", comma1 + 1);
		selectedRelation = path.substring(comma2 + 2, path.length() - 1);
		updateTables(ISObjects, path.substring(comma1 + 2, comma2), selectedRelation );
	      }
	    else if(ISSelected(path))
	      {
		if (jspT!=null) jspT.setVisible(false);
		if (jspJ!=null) jspJ.setVisible(false);
		if (jspP!=null) jspP.setVisible(false);
		lblType.setVisible(false);
		lblJoin.setVisible(false);
		lblPC.setVisible(false);
		mkbShowExtent.setVisible(false);
	      }
	  }
        });
    }
  
  /**
   * Returns if a relation has been selected by examining the nodePath
   */
  private boolean relSelected(String nodePath)
    {
      // If there are two commas in the path then a relation has been selected
      return (nodePath.indexOf(',') != nodePath.lastIndexOf(','));
    }
  
  /**
   * Returns if a relation has been selected by examing the nodePath
   */
  private boolean ISSelected(String nodePath)
    {
      return (nodePath.indexOf(',') == nodePath.lastIndexOf(','));
    }
  
  /**
   * Updates the tables to reflect the constraints of the most recently selected relation
   */
  private void updateTables(Vector ISObjects, String currIS, String currRel)
    {
      if (DEBUG) System.out.println("Begin update tables");
      boolean pcNo = false;
      boolean jcNo = false;
      
      try
        {
	  // Declare and initialized String arrays for the column names
	  String[] columnNamesType = {"Attribute",  "Type"};
	  // These two should maybe have something for column names, but what is appropriate????
	  String[] columnNamesJoin = {"Attribute", "Relationship", "Attribute"};
	  String[] columnNamesPC = {" ", " ", " "};
	  
	  // Declare variables
	  Enumeration enumIS = ISObjects.elements();
	  Vector vecTC = new Vector();
	  Vector vecJC = new Vector();
	  Vector vecPC = new Vector();
	  String ISName = null;
	  boolean done = false;
	  
	  // This loop finds the IS whose name is currIS and places that IS in tempIS
	  // Then it gets the constraints for currRel
	  while(!done)
            {
	      // If there are more ISs to look at
	      if(enumIS.hasMoreElements())
                {
		  // get the name of the next IS in the enumeration
		  tempIS = (IS)(enumIS.nextElement());
		  ISName = tempIS.getName();
		  
		  // if the IS in the enumeration is the currIS, get its constraints
		  if(ISName.equals(currIS))
                    {
                        Relation tmpRel = tempIS.findRelation(currRel);
		      // Get constraints for currIS
		      vecTC = tmpRel.getTCs();
		      vecJC = tmpRel.getJCs();
		      vecPC = tmpRel.getPCs();
		      
		      // Set done to true to end the while loop
		      done = true;
                    }
                }
	      else
                {
		  System.out.println(currIS + " does not exist in MKB");
		  done = true;
                }
            }
	  
	  // Create 2D object arrays, using vector size to determine size of arrays
	  Object[][] dataT = new Object[vecTC.size()][2];
	  Object[][] dataJ = new Object[JCRowCount(vecJC)][3];
	  Object[][] dataP = new Object[PCRowCount(vecPC)][3];
	  
	  // Get information about type constraints from vectors and place in dataT
	  for(int i = 0; i < vecTC.size(); i++)
            {
	      //System.out.println("tc: " + vecTC.elementAt(i).getClass());
	      dataT[i][0] = (Object)((TypeConstraint)vecTC.elementAt(i)).getName();
	      dataT[i][1] = (Object)((TypeConstraint)vecTC.elementAt(i)).getType();
            }
	  
	  int x = 0;
	  // Get information about join constraints from vectors and place in dataJ
	  for(int i = 0; i < vecJC.size(); i++)
            {
	      //System.out.println(  vecJC.elementAt(i).getClass()    );
	      thisJC = (JoinConstraint)vecJC.elementAt(i);
	      
	      // Put in relation names
	      dataJ[x][0] = (Object)((Relation)thisJC.getR1()).getFullName();
	      dataJ[x][1] = " ";
	      dataJ[x][2] = (Object)((Relation)thisJC.getR2()).getFullName();
	      
	      x++;
	      
	      // Get all conditions
	      condVec = thisJC.getConditions();
	      
	      // Put in attributes
	      for(int j = 0; j < condVec.size(); j++)
                {
		  thisCond = (Condition)condVec.elementAt(j);
		  
		  dataJ[x][0] = (Object)(((Attribute)(thisCond.getA1())).getName());
		  dataJ[x][1] = (Object)thisCond.getExtent();
		  dataJ[x][2] = (Object)(((Attribute)(thisCond.getA2())).getName());
		  x++;
                }
	      
	      dataJ[x][0] = " ";
	      dataJ[x][1] = " ";
	      dataJ[x][2] = " ";
	      x++;
            }
	  
	  x = 0;
	  // Get information about pc constraints from vectors and place in dataP
	  for (int i = 0; i < vecPC.size(); i++)
            {
	      thisPC = (PCConstraint)vecPC.elementAt(i);
	      
	      dataP[x][0] = (Object)((Relation)(thisPC.getR1())).getFullName();
	      dataP[x][1] = (Object)thisPC.getExtent();
	      dataP[x][2] = (Object)((Relation)(thisPC.getR2())).getFullName();
	      
	      x++;
	      
                pcpairVec = thisPC.getPCPairs();

                for(int j = 0; j < pcpairVec.size(); j++)
                {
                    thisPCPair = (PCPair)pcpairVec.elementAt(j);
                    dataP[x][0] = (Object)((Attribute)(thisPCPair.getA1())).getName();
                    dataP[x][1] = " ";
                    dataP[x][2] = (Object)((Attribute)(thisPCPair.getA2())).getName();
                    x++;
                }

                dataP[x][0] = " ";
                dataP[x][1] = " ";
                dataP[x][2] = " ";

                x++;
            }

            // Get rid of old information in tables
            tableT = null;
            tableJ = null;
            tableP = null;

            MyTableModel TModel = new MyTableModel(columnNamesType, dataT);
            MyTableModel JModel = new MyTableModel(columnNamesJoin, dataJ);
            MyTableModel PModel = new MyTableModel(columnNamesPC, dataP);

            // Create new tables
            tableT = new JTable(TModel);
            tableJ = new JTable(JModel);
            tableP = new JTable(PModel);

            // Set size dimension of tables
            tableT.setPreferredScrollableViewportSize(new Dimension(250, 75));
            tableJ.setPreferredScrollableViewportSize(new Dimension(250, 75));
            tableP.setPreferredScrollableViewportSize(new Dimension(250, 75));

            // Set size of each column
            TableColumn column = null;
            // column widths for type constraints
            for(int i = 0; i < 2; i++)
            {
                column = tableT.getColumnModel().getColumn(i);
                column.setMinWidth(100);
            }
            // column widths for join constraints
            for(int i = 0; i < 3; i++)
            {
                column = tableJ.getColumnModel().getColumn(i);
                if(i == 1)  // Equality column
                    column.setMinWidth(20);
                else
                    column.setMinWidth(80);
            }
            // column widths for pc constraints
            for(int i = 0; i < 3; i++)
            {
                column = tableP.getColumnModel().getColumn(i);
                if (i == 1)  // Equality column
                    column.setMinWidth(20);
                else
                    column.setMinWidth(80);
            }

            // Update Type Constraint scroll pane
            if(jspT == null)
            {
                jspT = new JScrollPane(tableT);
            }
            else
            {
                jspT.setViewportView(tableT);
            }

            // Update Join Constraint scroll pane
            if(jspJ == null)
            {
                if(JCRowCount(vecJC) == 0)
                    jcNo = true;
                else
                {
                    jspJ = new JScrollPane(tableJ);
                    jcNo = false;
                }
            }
            else
            {
                if(JCRowCount(vecJC) == 0)
                    jcNo = true;
                else
                {
                    jspJ.setViewportView(tableJ);
                    jcNo = false;
                }
            }

            // Update PC Constraint scroll pane
            if(jspP == null)
            {
                if(PCRowCount(vecPC) == 0)
                    pcNo = true;
                else
                {
                    jspP = new JScrollPane(tableP);
                    pcNo = false;
                }
            }
            else
            {
                if(PCRowCount(vecPC) == 0)
                    pcNo = true;
                else
                {
                    jspP.setViewportView(tableP);
                    pcNo = false;
                }
            }

            // Show labels and scroll panes
            if(pcNo)
            {
                lblPC.setVisible(false);
                if(jspP != null)
                    jspP.setVisible(false);
            }
            else
            {
                lblPC.setVisible(true);
                jspP.setVisible(true);
            }

            if(jcNo)
            {
                lblJoin.setVisible(false);
                if(jspJ != null)
                    jspJ.setVisible(false);
            }
            else
            {
                lblJoin.setVisible(true);
                jspJ.setVisible(true);
            }

            lblType.setVisible(true);
            jspT.setVisible(true);
	    mkbShowExtent.setVisible(true);

            // Get rid of old information in panels and add new scrollpanes
            panType.removeAll();
            panType.add(jspT);

            panJoin.removeAll();
            if(jspJ != null)
                panJoin.add(jspJ);

            panPC.removeAll();
            if(jspP != null)
                panPC.add(jspP);
	    
            // Repaint the frame so the panels will actually show

	    
	    panRight.setBounds(new Rectangle(300,0,350, 450)); // hacking... one of these days, 
	    // I'll figure out how layoutmanagers work
	    
	    panRight.invalidate();
	    panRight.validate();
	    panRight.repaint();
	 
        }
      
      catch (Exception e)
        {
	  //System.out.println(e);
	  e.printStackTrace();
        }
      
    }

    /**
     * Returns the number of rows needed in the Join Constraint table
     */
    private int JCRowCount(Vector vecJC)
    {
        int hi = 0;

        for(int i = 0; i < vecJC.size(); i++)
        {
            hi++;  // there is another jc

            hi += ((JoinConstraint)(vecJC.elementAt(i))).getConditions().size();  // the number of conditions

            hi++; // for the blank line after it
        }

        //System.out.println("hi is: " + hi);
        return hi;
    }

    /**
     * Returns the number of rows needed in the PC Constraint Table
     */
    private int PCRowCount(Vector vecPC)
    {
        int hi = 0;

        for(int i = 0; i < vecPC.size(); i++)
        {
            hi++; // there is another jc

            hi += ((PCConstraint)(vecPC.elementAt(i))).getPCPairs().size();

            hi++; // for the blank line after it
        }

        //System.out.println("hi is: " + hi);
        return hi;
    }

    /**
     * (Mostly generated by Visual Cafe) <BR>
     * Besides setting visibility, it has been modified to setLocation
     */
	public synchronized void setVisible(boolean show)
	{
	    if(show)
	    {
		    setLocation(25, 25);
		    super.setVisible(true);
		}
		else
		{
		    super.setVisible(false);
		}
	}

    /**
     * (generated by Visual Cafe)
     */
	public void addNotify()
	{
	    // Record the size of the window prior to calling parents addNotify.
	    Dimension d = getSize();

		super.addNotify();

		if (fComponentsAdjusted)
			return;

		// Adjust components according to the insets
		setSize(getInsets().left + getInsets().right + d.width, getInsets().top + getInsets().bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(getInsets().left, getInsets().top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}

    /**
     * (generated by Visual Cafe) <BR>
     * Used for addNotify check.
     */
	boolean fComponentsAdjusted = false;

    /**
     * (generated by Visual Cafe)
     */
	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == mkbFrame.this)
				Frame1_WindowClosing(event);
		}
	}

    /**
     * (generated by Visual Cafe)
     */
	void Frame1_WindowClosing(java.awt.event.WindowEvent event)
	{
		setVisible(false);		 // hide the Frame
	}

    /**
     * (generated by Visual Cafe) <BR>
     * Sets up a mouse click listener
     */
	class SymMouse extends java.awt.event.MouseAdapter
	{
        /**
         * Catches mouse clicks
         */
		public void mouseClicked(java.awt.event.MouseEvent event)
		{
			Object object = event.getSource();
			if (object == mkbOK)
				mkbOK_MouseClick(event);
			else if(object == mkbShowExtent)
			        mkbShowExtent_MouseClick(event);

		}
	}

    /**
     * Called by mouseClick when mkbOK button is clicked.
     * Hides this frame
     */
	void mkbOK_MouseClick(java.awt.event.MouseEvent event)
	{
        setVisible(false);
	}
  
  /**
   * Called by mouseClick when mkbOK button is clicked.
   * Shows a relation extent.
   */
  void mkbShowExtent_MouseClick(java.awt.event.MouseEvent event)
    {
      Table extent = null;
      relExtentFrame extentFrame;
      // Get the Extent of a relation here.
      try{
	// Try to get the Extent from the server Site.
	// use the currently selected IS
	extent = mwMain.getTable(tempIS.getName(),"select * from " + selectedRelation);
      }catch(Exception e){
	System.out.println("Debug: mkbFrame.disExtButton_MouseClick");
      }
      // Assign the Extent to the client site, but disable to renew the extent at the client site.
      //tempView.setExtent(extent);
      extentFrame = new relExtentFrame(extent, selectedRelation);
      extentFrame.setVisible(true);
    } 
}
