////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package EVEDemo.MKBVKBApplet;

import EVEDemo.MW.*;
import DSRG.MW.*;
import DSRG.MKB.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;

/**
 * This class creates a frame which displays the extent of the selected view
 *
 * @author Amber Van Wyk, Summer 1998
 */
public class relExtentFrame extends JFrame
{
  private static final boolean DEBUG = false;
    
  JButton veOK;
  JTable extentTable;
  MyTableModel extentModel;
  JScrollPane extentScrollPane;
  Vector allAttr;
  EVEDemo.MW.View myView;
  EVEDemo.MW.Main myMain;
  
  /**
   * Class Constrctor. <BR>
   * Intializes all objects on the frame
   * theView is the view whose extent needs to be displayed
   */
  public relExtentFrame(Table extent, String relName)
    {

      // Initialize frame, set Title, layout, visibility, size, and background color
      setTitle("Relation Extent ("+relName+")");
      getContentPane().setLayout(new FlowLayout());
      setVisible(false);
      setResizable(false);
      setSize(575,325);
      setBackground(java.awt.Color.lightGray);
      
      // Initialize OK button
      veOK = new JButton();
      veOK.setText("OK");
      
      

      // Initialize Table

      //extentTable = new JTable();
      //extentTable.setPreferredSize(new Dimension(400,300));
      // extentTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

      // Initialize Scrollpane
      //extentScrollPane = new JScrollPane(extentTable);
      extentScrollPane = new JScrollPane();

      // Initialize Relation Table
      buildExtent(extent);


      
        // Add objects
      getContentPane().add(extentScrollPane);
      getContentPane().add(veOK);
       
      // System.out.println(myView.getName());
      
      // (generated by Visual Cafe)
      SymWindow aSymWindow = new SymWindow();
      this.addWindowListener(aSymWindow);
      SymMouse aSymMouse = new SymMouse();
      veOK.addMouseListener(aSymMouse);
    }
  
  /**
   * Sets up the table which displays the extent for the
   * theView. Rewritten to use the new Table object.  */
    protected void buildExtent(Table extent)
    {
        
        if (DEBUG) System.out.println("NEWEXTENT:\n"+extent);

        // we need this for the attribute names.
        allAttr = extent.getAttributeNames();
        int numCols = allAttr.size();
        int numRows = extent.getSize();
        
        if (DEBUG) System.out.println("DEBUG. relExtentFrame::buildExtent called. Size:"+numRows);
        
        String[] colNames = new String[numCols];
        
        // Get all Column Names
        for (int y = 0; y < numCols; y++)
            {
                colNames[y] = (String)allAttr.elementAt(y);
                if (DEBUG) System.out.println("DEBUG.  relExtentFrame::buildExtent() ColNames:" + colNames[y]);
            }
        
        // Declare 2D object array for building table
        Object[][] data = new Object[numRows][numCols];
        
        // Fill data! This should be interesting.
        Tuple t;
        Enumeration te;
        int x,y;
        
        // Loop through each row in the extent
        for(y = 0; y < numRows; y++)
            {
                
                //  fetch a tuple
                t = extent.getTuple(y);
                if (t.getSize() != numCols)
                    System.err.println("relExtentFrame.buildExtent: Tuple size mismatches Number of Attributes");
                te = t.getEnumeration();
                x = 0;
                while (te.hasMoreElements()) {
                    data[y][x] = (String)te.nextElement();
                    x++;
                }
            }
        
        extentModel = new MyTableModel(colNames, data);
        //        extentTable.setModel(extentModel);
        extentTable = new JTable(extentModel);
        // extentTable.setPreferredSize(new Dimension(numCols * 100, numRows * 25));
        // extentTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        
        TableColumn col = null;
        int width;
        if (DEBUG) System.out.println("Setting col widths");
        
        for(int i = 0; i < numCols; i++)
            {
                if (DEBUG) System.out.println("col: " + i);
                col = extentTable.getColumnModel().getColumn(i);
                width = colNames[i].length();
                if (DEBUG) System.out.println("colwidth: " + width);
                //col.setMinWidth(100);
            }
        
        extentScrollPane = new JScrollPane(extentTable);
        //        extentScrollPane.setViewportView(extentTable);
        extentScrollPane.setPreferredSize(new Dimension(570, 275));
        
        extentTable.invalidate();
        extentTable.validate();
        extentTable.repaint();
        extentScrollPane.invalidate();
        extentScrollPane.validate();
        extentScrollPane.repaint();

        if (DEBUG) System.out.println("Done Building Extent Table!");
    }
    
    /**
     * (mostly generated by Visual Cafe) <BR>
     * in addition to setting visibility, sets the location of the frame
     */
	public synchronized void setVisible(boolean show)
	{
	    if(show)
	    {
		    setLocation(75, 75);
		    super.setVisible(true);
		}
		else
		{
		    super.setVisible(false);
		}
	}

    /**
     * (generated by Visual Cafe)
     */
	public void addNotify()
	{
	    // Record the size of the window prior to calling parents addNotify.
	    Dimension d = getSize();

		super.addNotify();

		if (fComponentsAdjusted)
			return;

		// Adjust components according to the insets
		setSize(getInsets().left + getInsets().right + d.width, getInsets().top + getInsets().bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
                    {
			Point p = components[i].getLocation();
			p.translate(getInsets().left, getInsets().top);
			components[i].setLocation(p);
                    }
		fComponentsAdjusted = true;
	}
    
    /**
     * Used for addNotify check.
     */
    boolean fComponentsAdjusted = false;
    
    /**
     * (generated by Visual Cafe)
     */
    class SymWindow extends java.awt.event.WindowAdapter
    {
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == relExtentFrame.this)
				Frame1_WindowClosing(event);
		}
	}

    /**
     * (generated by Visual Cafe)
     */
	void Frame1_WindowClosing(java.awt.event.WindowEvent event)
	{
		setVisible(false);		 // hide the Frame
	}

    /**
     * (mostly generated by Visual Cafe) <BR>
     * sets up listener for mouse clicks
     */
  class SymMouse extends java.awt.event.MouseAdapter
  {
    /**
     * catches mouse click
     */
    public void mouseClicked(java.awt.event.MouseEvent event)
      {
	Object object = event.getSource();
	if(object == veOK)
	  {
	    veOK_MouseClick(event);
	  }
      }
  }
  
  /**
   * called bye mouseCliced when veOK button is clicked <BR>
   * sets visibility of frame to false
   */
  void veOK_MouseClick(java.awt.event.MouseEvent event)
    {
      setVisible(false);
    }
}
