////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package EVEDemo.MKBVKBApplet;

import EVEDemo.MW.*;
import DSRG.MW.*;
import DSRG.MKB.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;

/**
 * This class creates a frame which displays the extent of the selected view
 *
 * @author Amber Van Wyk, Summer 1998
 */
public class viewExtentFrame extends JFrame
{

    private static final boolean DEBUG = false;


  JButton veOK;
  JButton veRefresh;
  JTable viewDefTable;
  JTable extentTable;
  MyTableModel viewDefModel;
  MyTableModel extentModel;
  JScrollPane defScrollPane;
  JScrollPane extentScrollPane;
  JLabel lblViewDef;
  JLabel space;
  Vector allAttr;
    //Table table = null;
  EVEDemo.MW.View myView;
  EVEDemo.MW.Main myMain;
  
  /**
   * Class Constrctor. <BR>
   * Intializes all objects on the frame
   * theView is the view whose extent needs to be displayed
   */
  public viewExtentFrame(Object[][] data, EVEDemo.MW.View theView, 
			 EVEDemo.MW.Main theMain)
    {

      myView = theView;
      myMain = theMain;

      // Initialize frame, set Title, layout, visibility, size, and background color
      setTitle("View Extent");
      getContentPane().setLayout(new FlowLayout());
      setVisible(false);
      setResizable(false);
      setSize(575,450);
      setBackground(java.awt.Color.lightGray);
      
      // Initialize OK button
      veOK = new JButton();
      veOK.setText("OK");
      
      // Initialize Refresh Button
      veRefresh = new JButton();
      veRefresh.setText("Refresh");

      // Initialize View Definition Label
      lblViewDef = new JLabel("View Definition");
      lblViewDef.setFont(new Font("Dialog", Font.PLAIN, 14));
      lblViewDef.setForeground(new Color(128, 0, 128));
      
      space = new JLabel(" ");
      
      // Initialize Table
      String[] columnNames = {"SQL", "ESQL"};
      
      viewDefModel = new MyTableModel(columnNames, data);
      viewDefTable = new JTable(viewDefModel);
      viewDefTable.setPreferredScrollableViewportSize(new Dimension(550, 175));
      
      // Set column widths
      TableColumn col = null;
      col = viewDefTable.getColumnModel().getColumn(0);
      col.setMinWidth(296);
      col = viewDefTable.getColumnModel().getColumn(1);
      col.setMinWidth(134);

      extentTable = new JTable();
      
      defScrollPane = new JScrollPane(viewDefTable);
      extentScrollPane = new JScrollPane(extentTable);
      
      // Initialize Relation Table
      buildExtent(myView.getExtent());


      
        // Add objects
      getContentPane().add(lblViewDef);
      getContentPane().add(defScrollPane);
      getContentPane().add(space);
      getContentPane().add(extentScrollPane);
      getContentPane().add(veOK);
      getContentPane().add(veRefresh);
      
      // System.out.println(myView.getName());
      
      // (generated by Visual Cafe)
      SymWindow aSymWindow = new SymWindow();
      this.addWindowListener(aSymWindow);
      SymMouse aSymMouse = new SymMouse();
      veOK.addMouseListener(aSymMouse);
      veRefresh.addMouseListener(aSymMouse);

    }
  
  /**
   * Copy any object inside the java by using arraycopy.
   */
  private Object copy(Object obj){
    Object oldObj[] = new Object[1];
    oldObj[0] = obj;
    Object newObj[] = new Object[1]; 
    System.arraycopy(oldObj, 0, newObj, 0, 1);
    return newObj[0];
  }

  /** attempts to refresh the whole frame, not just the first window
   */
  public void refresh(JScrollPane defSP) {
    Table extent = null;

    defScrollPane = (JScrollPane)copy(defSP);

    try{
      // Try to get the Extent from the server Site.
      //extent = myMain.getViewExtentWithoutRecomputation(myView.getName());
      extent = myView.getExtent();
    }catch(Exception e){
      System.out.println("viewExtentFrame.veRefresh_MouseClick ERROR!");
    }
    // Assign the Extent to the client site, but disable to renew the extent at the client site.
    //myView.setExtent(extent);
    buildExtent(extent);
    this.invalidate();
    this.validate();
    this.repaint();
}


  /**
   * Sets up the table which displays the extent for the theView
   */


    protected void buildExtent(Table extent)
    {
        if (DEBUG) System.out.println("NEWEXTENT:\n"+extent);

        // we need this for the attribute names.
        allAttr = extent.getAttributeNames();
        int numCols = allAttr.size();
        int numRows = extent.getSize();
        
        if (DEBUG) System.out.println("DEBUG. viewExtentFrame::buildExtent called. Size:"+numRows);
        
        String[] colNames = new String[numCols];
        
        // Get all Column Names
        for (int y = 0; y < numCols; y++)
            {
                colNames[y] = (String)allAttr.elementAt(y);
                if (DEBUG) System.out.println("DEBUG.  viewExtentFrame::buildExtent() ColNames:" + colNames[y]);
            }
        
        // Declare 2D object array for building table
        Object[][] data = new Object[numRows][numCols];
        
        // Fill data! This should be interesting.
        Tuple t;
        Enumeration te;
        int x,y;
        
        // Loop through each row in the extent
        for(y = 0; y < numRows; y++)
            {
                
                //  fetch a tuple
                t = extent.getTuple(y);
                if (t.getSize() != numCols)
                    System.err.println("viewExtentFrame.buildExtent: Tuple size mismatches Number of Attributes");
                te = t.getEnumeration();
                x = 0;
                while (te.hasMoreElements()) {
                    data[y][x] = (String)te.nextElement();
                    x++;
                }
            }
      
      extentModel = new MyTableModel(colNames, data);
      extentTable.setModel(extentModel);
      // extentTable.setPreferredSize(new Dimension(numCols * 100, numRows * 25));
      // extentTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
      
      extentScrollPane.setPreferredSize(new Dimension(570, 150));
      extentScrollPane.setViewportView(extentTable);
      System.out.println("Done Building Extent Table!");
    }
  
    /**
     * (mostly generated by Visual Cafe) <BR>
     * in addition to setting visibility, sets the location of the frame
     */
	public synchronized void setVisible(boolean show)
	{
	    if(show)
	    {
		    setLocation(75, 75);
		    super.setVisible(true);
		}
		else
		{
		    super.setVisible(false);
		}
	}

    /**
     * (generated by Visual Cafe)
     */
	public void addNotify()
	{
	    // Record the size of the window prior to calling parents addNotify.
	    Dimension d = getSize();

		super.addNotify();

		if (fComponentsAdjusted)
			return;

		// Adjust components according to the insets
		setSize(getInsets().left + getInsets().right + d.width, getInsets().top + getInsets().bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(getInsets().left, getInsets().top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}

    /**
     * Used for addNotify check.
     */
	boolean fComponentsAdjusted = false;

    /**
     * (generated by Visual Cafe)
     */
	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == viewExtentFrame.this)
				Frame1_WindowClosing(event);
		}
	}

    /**
     * (generated by Visual Cafe)
     */
	void Frame1_WindowClosing(java.awt.event.WindowEvent event)
	{
		setVisible(false);		 // hide the Frame
	}

    /**
     * (mostly generated by Visual Cafe) <BR>
     * sets up listener for mouse clicks
     */
  class SymMouse extends java.awt.event.MouseAdapter
  {
    /**
     * catches mouse click
     */
    public void mouseClicked(java.awt.event.MouseEvent event)
      {
	Object object = event.getSource();
	if(object == veOK)
	  {
	    veOK_MouseClick(event);
	  }
	else if (object == veRefresh) {
			  veRefresh_MouseClick(event);
	}
	
	
      }
  }
  
  /**
   * called bye mouseCliced when veOK button is clicked <BR>
   * sets visibility of frame to false
   */
  void veOK_MouseClick(java.awt.event.MouseEvent event)
    {
      setVisible(false);
    }
 
  /**
   * called bye mouseCliced when veRefresh button is clicked <BR>
   * refreshes Screen
   */
  void veRefresh_MouseClick(java.awt.event.MouseEvent event)
    {
        //      Table extent = null;
        //try{
	// Try to get the Extent from the server Site.
	//extent = myMain.getViewExtentWithoutRecomputation(myView.getName());
        //	extent = myView.getExtent();
        //}catch(Exception e){
        //	System.out.println("viewExtentFrame.veRefresh_MouseClick ERROR!");
        //}
      // Assign the Extent to the client site, but disable to renew the extent at the client site.
      //myView.setExtent(extent);
      buildExtent(myView.getExtent());
      this.invalidate();
      this.validate();
      this.repaint();
    }

}

