////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package EVEDemo.MKBVKBApplet;

import EVEDemo.MW.*;
import DSRG.MW.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
import com.sun.java.swing.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.event.*;

/**
 * This class creates a frame which displays the views and there defintions.
 * Upon selecting a view from the list of views the table will change to reflect
 * the defintion for that view.  The view defintion is split into two columns
 * in the table so that the esql can be separated from the sql.  This frame
 * contains an button to show the extent of a selected view.
 *
 * @author Amber Van Wyk, Summer 1998
 */
public class vkbFrame extends JFrame implements ListSelectionListener
{

  /**
   * Added by Xin Zhang. The reference of mwMain
   */
  private Main mwMain;

  /**
   * A local copy of allViews
   */
  private Vector allViews;
  
  JButton vkbOK;
  
  /**
   * Used to trigger showing of extent of currently selected view.
   */
  JButton disExtButton;
  JList viewList;
  MyTableModel viewDefModel;
  JTable viewDefTable;
  JTable viewDefTableSQL;
  JTable viewDefTableESQL;
  JScrollPane viewsScrollPane;
  JScrollPane viewDefSP;
  JLabel lblView;
  JLabel lblViewDef;
  Vector viewItems;
  Object[][] stuff;
  EVEDemo.MW.View tempView;
  viewExtentFrame extentFrame;
  
  
  
  /**
   * Class Constuctor.  Intializes all objects on the frame
   * Modified by Xin: Only add one parameter: mwMain
   */
  public vkbFrame(Vector theViews, Main m)
    {
      mwMain = m;
      allViews = theViews;
      
      // Initialize frame, set Title, layout, visibility, size, and background color
      setTitle("View Knowledge Base");
      getContentPane().setLayout(new FlowLayout());
      setVisible(false);
      setResizable(false);
      setSize(575,450);
      setBackground(java.awt.Color.lightGray);
      
      // Initialize vkbOK button
      vkbOK = new JButton();
      vkbOK.setText("OK");
      vkbOK.setBackground(java.awt.Color.lightGray);
      
      // Initialize display extent button
      disExtButton = new JButton();
      disExtButton.setText("Display Extent");
      disExtButton.setBackground(Color.lightGray);
      
      // Initialize list of possible views
      initViewList(allViews);
      
      // Intialize views label
      lblView = new JLabel("Views");
      lblView.setFont(new Font("Dialog", Font.PLAIN, 14));
      lblView.setForeground(new Color(128, 0, 128));
      
      // Initlaize view definition label
      lblViewDef = new JLabel("View Definition");
      lblViewDef.setFont(new Font("Dialog", Font.PLAIN, 14));
      lblViewDef.setForeground(new Color(128, 0, 128));
      
      // Add all objects to the frame.  flow layout works out great for this! yea!
      getContentPane().add(lblView);
      getContentPane().add(viewsScrollPane);
      getContentPane().add(lblViewDef);
      getContentPane().add(viewDefSP);
      getContentPane().add(vkbOK);
      getContentPane().add(disExtButton);
      
      // (generated by Visual Cafe)
      SymWindow aSymWindow = new SymWindow();
      this.addWindowListener(aSymWindow);
      SymMouse aSymMouse = new SymMouse();
      vkbOK.addMouseListener(aSymMouse);
      disExtButton.addMouseListener(aSymMouse);
    }
  
  /**
   * Impletmented for ListSelectionListener.  <BR>
   * Catches the event of the selected item in the list of views changing and then
   * shows the view associated with that item
   */
  public void valueChanged(ListSelectionEvent e)
    {
      if (!e.getValueIsAdjusting())
        {
	  JList theList = (JList)e.getSource();
	  int index = theList.getSelectedIndex();
	  // System.out.println("Selected: " + (String)viewItems.elementAt(index));
	  
	  // System.out.println("Debug: vkbFrame.valueChanged index:" + index);
	  // get the view that was selected
	  tempView = (EVEDemo.MW.View)allViews.elementAt(index);
	  
	  JTable tempTable = new JTable();
	  /* Edit by Xin Zhang.
	   * I just directly pass the View definition to the setTableValues
	   * Function. Because RMI cannot receive COMPLEX object.
	   * I have added mwMain reference in this class.
	   */  
	  tempTable = setTableValues(tempView);
	  
	  //try{
	  //  tempTable = setTableValues(mwMain.getAView(index));
	  //}catch(java.rmi.RemoteException re){
	  //  System.out.println("debug: vkbFrame.valueChanged Bug!?");
	  //}
	  
	  // Create the table for the selected view
	  if(viewDefSP == null)
            {
	      viewDefSP = new JScrollPane(tempTable);
            }
	  else
            {
	      viewDefSP.setViewportView(tempTable);
            }
	  
	  viewDefSP.invalidate();
	  viewDefSP.validate();
	  viewDefSP.repaint();
	  this.invalidate();
	  this.validate();
	  this.repaint();
        }
    }
  
    /**
     * Initializes the list of views
     */
  public void initViewList(Vector views)
    {
      allViews = views;
      
      // Initialize the vector for the view items
      viewItems = new Vector();
      
      // Create a temporary view
      tempView = null;
      
      // Add each view name to the list of view items
      for (int i=0; i < allViews.size(); i++)
        {
	  tempView = (EVEDemo.MW.View)allViews.elementAt(i);
	  // Create the table for the first view
	  if(viewDefSP == null)
	    {
	      JTable tempTable = setTableValues(tempView);
	      viewDefSP = new JScrollPane(tempTable);
	    }
	  viewItems.addElement( (String)tempView.getName() );
        }
      
      // Use vector of items to create new list
      viewList = new JList(viewItems);
      
      // Set selection mode to single
      viewList.setSelectionMode(0);
      
      // Add selection listener
      viewList.addListSelectionListener(this);
      
      // Set size and put in scroll pane
      viewList.setPreferredSize(new Dimension(550, 10));
      if(viewsScrollPane == null)
        {
	  viewsScrollPane = new JScrollPane(viewList);
	  viewList.setSelectedIndex(0);
        }
      else
        {
	  viewsScrollPane.setViewportView(viewList);
        }

      // update viewExtentFrame is on screen
      if(extentFrame != null){
	extentFrame.setVisible(false);
      }
      /*
	if (extentFrame != null) {
	EVEDemo.Relation extent = null;
	int index = viewList.getSelectedIndex();
        if (index == -1) index = 0;
	
	tempView = (EVEDemo.MW.View)allViews.elementAt(index);
	// Get the Extent of tempView here.
	try{
	// Try to get the Extent from the server Site.
	//extent = mwMain.getViewExtent(tempView.getName(),false); // not the old extent, but the current one
	extent=tempView.getExtent();
	}catch(Exception e){
	System.out.println("Debug: vkbFrame.initViewList");
	}
	// Assign the Extent to the client site, but disable to renew the extent at the client site.
	//tempView.setExtent(extent);
       	//extentFrame.buildExtent(tempView);
	
	// Reset the view defintion scroll pane also.
	extentFrame.refresh(viewDefSP);
	//	extentFrame.invalidate();
	//extentFrame.validate();
	//extentFrame.repaint();
	//extentFrame.setVisible(true);
	}
      */

      
      //Set selection to first view in list

      viewsScrollPane.invalidate();
      viewsScrollPane.validate();
      viewsScrollPane.repaint();
      
    }
  
    /**
     * Sets the values for the table which displays the view definition
     */
    public JTable setTableValues(EVEDemo.MW.View theView)
    {
        // Get view defintion string from theView
        String defString = new String(theView.getDefinition());
        // System.out.println(defString);
        // Replace tab characters w/ spaces (needed until getDefinition function is modified to fit our needs)
        defString = defString.replace('\t', ' ');

        // Create the table and scroll pane to display the view definition
        String[] columnNamesX = {"SQL", "E-SQL"};

        // Create the two vectors for building the table
        Vector sqlVec = new Vector();
        Vector esqlVec = new Vector();

        int nextLP; // next left parentheses
        int nextNL; // next new line
        boolean eos = false;    // end of string

        while(!eos)
        {
            nextLP = defString.indexOf('(');
            nextNL = defString.indexOf('\n');
            /*System.out.println("nextLP: " + nextLP);
            System.out.println("nextNL: " + nextNL);
            System.out.println(defString);*/

            // If there is another left parenthesis
            if(nextLP != -1)
            {
                // and it comes before the next newline
                if(nextLP < nextNL)
                {
                    sqlVec.addElement("      " + defString.substring(0, nextLP).trim());
                    esqlVec.addElement(defString.substring(defString.indexOf('('), nextNL).trim());
                    defString = defString.substring(nextNL, defString.length()).trim();
                }
                // If there is another new line
                else if (nextNL < nextLP)
                {
                    if(nextNL == -1)
                    {
                        sqlVec.addElement("      " + defString.substring(0, nextLP).trim());
                        esqlVec.addElement(defString.substring(defString.indexOf('('), defString.length()).trim());
                        defString = "";
                    }
                    else
                    {
                        sqlVec.addElement(defString.substring(0, nextNL).trim());
                        esqlVec.addElement(" ");
                        defString = defString.substring(nextNL, defString.length()).trim();
                    }
                }
            }
            else if(nextNL != -1)
            {
                sqlVec.addElement(defString.substring(0, nextNL).trim());
                esqlVec.addElement(" ");
                defString = defString.substring(nextNL, defString.length()).trim();
            }
            else
            {
                eos = true;
            }

        }

        stuff = new Object[sqlVec.size()][2];

        for(int j = 0; j < sqlVec.size(); j++)
        {
            stuff[j][0] = sqlVec.elementAt(j);
            stuff[j][1] = esqlVec.elementAt(j);
        }

        // If there are old values, get rid of them
        viewDefModel = null;
        viewDefTable = null;

        viewDefModel = new MyTableModel(columnNamesX, stuff);
        viewDefTable = new JTable(viewDefModel);
        viewDefTable.setPreferredScrollableViewportSize(new Dimension(550, 175));

        // Set column widths
        TableColumn col = null;
        col = viewDefTable.getColumnModel().getColumn(0);
        col.setMinWidth(296);
        col = viewDefTable.getColumnModel().getColumn(1);
        col.setMinWidth(134);

        return viewDefTable;

    }
    /**
     * Sets the values for the table which displays the view definition
     */
    public JTable setTableValues(String defString)
    {

        // System.out.println(defString);
        // Replace tab characters w/ spaces (needed until getDefinition function is modified to fit our needs)
        defString = defString.replace('\t', ' ');

        // Create the table and scroll pane to display the view definition
        String[] columnNamesX = {"SQL", "E-SQL"};

        // Create the two vectors for building the table
        Vector sqlVec = new Vector();
        Vector esqlVec = new Vector();

        int nextLP; // next left parentheses
        int nextNL; // next new line
        boolean eos = false;    // end of string

        while(!eos)
        {
            nextLP = defString.indexOf('(');
            nextNL = defString.indexOf('\n');
            /*System.out.println("nextLP: " + nextLP);
            System.out.println("nextNL: " + nextNL);
            System.out.println(defString);*/

            // If there is another left parenthesis
            if(nextLP != -1)
            {
                // and it comes before the next newline
                if(nextLP < nextNL)
                {
                    sqlVec.addElement("      " + defString.substring(0, nextLP - 1).trim());
                    esqlVec.addElement(defString.substring(defString.indexOf('('), nextNL).trim());
                    defString = defString.substring(nextNL, defString.length()).trim();
                }
                // If there is another new line
                else if (nextNL < nextLP)
                {
                    if(nextNL == -1)
                    {
                        sqlVec.addElement("      " + defString.substring(0, nextLP).trim());
                        esqlVec.addElement(defString.substring(defString.indexOf('('), defString.length()).trim());
                        defString = "";
                    }
                    else
                    {
                        sqlVec.addElement(defString.substring(0, nextNL).trim());
                        esqlVec.addElement(" ");
                        defString = defString.substring(nextNL, defString.length()).trim();
                    }
                }
            }
            else if(nextNL != -1)
            {
                sqlVec.addElement(defString.substring(0, nextNL).trim());
                esqlVec.addElement(" ");
                defString = defString.substring(nextNL, defString.length()).trim();
            }
            else
            {
                eos = true;
            }

        }

        stuff = new Object[sqlVec.size()][2];

        for(int j = 0; j < sqlVec.size(); j++)
        {
            stuff[j][0] = sqlVec.elementAt(j);
            stuff[j][1] = esqlVec.elementAt(j);
        }

        // If there are old values, get rid of them
        viewDefModel = null;
        viewDefTable = null;

        viewDefModel = new MyTableModel(columnNamesX, stuff);
        viewDefTable = new JTable(viewDefModel);
        viewDefTable.setPreferredScrollableViewportSize(new Dimension(550, 175));

        // Set column widths
        TableColumn col = null;
        col = viewDefTable.getColumnModel().getColumn(0);
        col.setMinWidth(296);
        col = viewDefTable.getColumnModel().getColumn(1);
        col.setMinWidth(134);

        return viewDefTable;

    }

    /**
     * (mostly generated by Visual Cafe) <BR>
     * in addition to setting visibility, sets location of the frame
     */
	public synchronized void setVisible(boolean show)
	{
	    if(show)
	    {
		    setLocation(50, 50);
		    super.setVisible(true);
		}
		else
		{
		    super.setVisible(false);
		}
	}

    /**
     * (generated by Visual Cafe)
     */
    public void addNotify()
	{
	    // Record the size of the window prior to calling parents addNotify.
	    Dimension d = getSize();

		super.addNotify();

		if (fComponentsAdjusted)
			return;

		// Adjust components according to the insets
		setSize(getInsets().left + getInsets().right + d.width, getInsets().top + getInsets().bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(getInsets().left, getInsets().top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}

    /**
     * Used for addNotify check.
     */
	boolean fComponentsAdjusted = false;

    /**
     * (generated by Visual Cafe)
     */
	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == vkbFrame.this)
				Frame1_WindowClosing(event);
		}
	}

    /**
     * (generated by Visual Cafe)
     */
	void Frame1_WindowClosing(java.awt.event.WindowEvent event)
	{
		setVisible(false);		 // hide the Frame
	}

    /**
     * (mostly generated by Visual Cafe) <BR>
     * sets up listener to catch mouse clicks
     */
	class SymMouse extends java.awt.event.MouseAdapter
	{
		/**
		 * catches mouse click
		 */
		public void mouseClicked(java.awt.event.MouseEvent event)
		{
			Object object = event.getSource();
			if (object == vkbOK)
	        {
				vkbOK_MouseClick(event);
			}
			else if (object == disExtButton)
			{
			    disExtButton_MouseClick(event);
			}
		}
	}

    /**
     * called by mouseClicked when vkbOK button is clicked
     */
	void vkbOK_MouseClick(java.awt.event.MouseEvent event)
	{
        setVisible(false);
	}

    /**
     * called by mouseClicked when disExtbutton is clicked
     */
	void disExtButton_MouseClick(java.awt.event.MouseEvent event)
	{
            //	  RelationExtent extent = null;
	  // Get the Extent of tempView here.
	  try{
	    // Try to get the Extent from the server Site.
	    //extent = mwMain.getViewExtent(tempView.getName(),false); // not the old extent, but the current one
              // extent=tempView.newGetExtent();
	  }catch(Exception e){
	    System.out.println("Debug: vkbFrame.disExtButton_MouseClick");
	  }
	  // Assign the Extent to the client site, but disable to renew the extent at the client site.
	  //tempView.setExtent(extent);
	  extentFrame = new viewExtentFrame(stuff, tempView, mwMain);
	  extentFrame.setVisible(true);
	}
}
