////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package EVEDemo.MW;

import java.util.*;
//import java.rmi.*;
import java.io.*;
import com.sun.java.swing.ImageIcon;

import DSRG.MKB.*;

import DSRG.VSA.VSA;

import EVEDemo.ISChangeApplet.*;
import DSRG.MW.*;

import DSRG.VSA.Schema_Change;
/**
 *
 * The main class in the "middleware" layer. <BR>
 * Here, we initialize the object structure by loading all the views from
 * the VKB, the ISs from the MKB. Instantiating an object of the IS class
 * reads the appropriate information from the DB <BR>
 * The RMI-support has been discontinued.
 *
 * @author koeller, kajal, xinz, avanwyk
 */

public  class Main {
  
    /**
     * Debug flag
     */
    public static final boolean DEBUG = false;
    
    /**
     * XIN: Please explain or remove.<BR>
     * count total remote queries.
     */
    public static int rcount = 0;
    
    /**
     * XIN: Please explain or remove.<BR>
     * increase the rcount
     */
    public synchronized static void incRcount(){
    	rcount ++;
    }
  
    /**
     * XIN: Please explain or remove<BR>
     * count total local queries.
     */
    public static int lcount = 0;
  
    /**
     * XIN: Please explain or remove <BR>
     * increase the lcount
     */
    public synchronized static void incLcount(){
    	lcount ++;
    }

    /**
     *
     * the vector of View objects
     *
     */
    private Vector Views=new Vector();

    /**
     *
     * the local InformationSource
     *
     */
    //private IS localIS;

    /**
     * a vector of DBManager objects to connect to physical databases 
     */
    Vector DBManagers = new Vector();
  
    /**
     * a handle (type DBManager) for the local DB-Manager
     */
    DBManager localDBManager;

    /**
     *
     *    the MKB object
     *
     */
  
    private MKB mainMKB;
  
    /**
     *
     *    the VKB object
     *
     */
  
    private DSRG.VKB.VKB mainVKB;
  
    /**
     *
     *    the VSA object
     *
     */
  
    private DSRG.VSA.VSA mainVSA;


    /** a handle to the Query Engine
     */
    private DSRG.MW.QueryEngine queryEngine;

  
    /**
     * has there been a change made?
     */
    private boolean changed = false;
  
  
    /**
     * handle to eve.properties
     */
    Properties props;
    
  
    /**
     * The Global variable to terminate the process. 
     * it's only used for batch mode of data update
     * detection. It will stores the time spend on a specific
     * experiments.
     * @author xinz
     */
    public static double expTime = 0;
    
    /**
     * The Flag which will be changed by the alived user.
     * @author xinz
     */
    static private int AliveCounter = 0;

  
    /**
     * Constructed for create Main object in the Start class.
     */
    public Main(){
        
	try{
            
	    props = new Properties(System.getProperties());
	    props.load(new BufferedInputStream(new FileInputStream("eve.properties")));
	    System.setProperties(props);
	}catch(Exception e){
	    e.printStackTrace();
	}

        // initialize the middleware layer to a defined state.
	reset();
        
	//**************************************************************************
	//******************************** Only for Exps of PSWEEP******************
	// start the data update automatically
	//      try{
	//        startUpdate();
	//      }catch(RemoteException re){
	//        System.out.println("Main:<INIT> Got Exception for startUpdate(): " + re.getMessage());
	//      }
    
	//      try{
	//        PrintWriter presult = new PrintWriter(new BufferedWriter(new FileWriter("expResult", true)));
      
      
	//        // wait until the update fully handled.
	//        while(expTime == 0){
	//          Thread.sleep(1000);
	//        }
	//        presult.println("This test is completed in " + expTime + " seconds");
	//        presult.println("This test has send local join queries: " + lcount );
	//        presult.println("This test has send remote join queries: " + rcount);
	//        presult.close();
	//      }catch(Exception e){
	//        System.out.println("Dam, what happened?: " + e.getMessage());
	//      }
	//      // terminate the program
	//      System.exit(0);
	//************************** you can delete it ******************************
	//******************************* after exps ********************************
    }


    /** returns a handle to the Main MKB */
    public MKB getMKB () {
        return mainMKB;
    }
    
    /**
     *
     * Get all view names from the VKB and create View objects for
     * each name. <BR>
     * We construct a view object for each view name that we find in
     * the VKB and then instantiate it with default parameters (the
     * handles to InformationSources, ISs, MKB, VKB)
     * @author koeller
     */
  
    private Vector getAllViewsFromVKB ( )
    {
        
	// first, get the names of all the views that should exist
	Vector views = new Vector();
	try{
	    views = mainVKB.getViewNames();
	}catch(Exception e){
	    if(mainVKB == null)
		System.out.println("Debug:Main.getAllViewsFromVKB mainVKB is null!");
	    e.printStackTrace();
	}
        
	// Lets create a View object for each of these names
        
	int length = views.size();
        
	Views = new Vector(); // we'll add elements later
        
	for ( int i = 0; i < length; i++ ) {
            String viewName = (String)views.elementAt(i);
            if (viewName.length()!=0) {
                View v = null;
                try {  
                    if (DEBUG) 
                        System.out.println(
                                           "Debug: Main.getAllViewsFromVKB: Create View:"
                                           + i + " MKB is: " + mainMKB + " VKB is: " 
                                           + mainVKB);
                    
                    v = new View ( this, viewName, mainMKB, mainVKB );
                }catch(Exception e) {
                    
                    System.out.println(
                                       "Debug: Main.getAllViewsFromVKB: Exception:" 
                                       + e.getMessage());
                    e.printStackTrace();
                }
                
                // Add this view to the Views
                if (v!=null) Views.addElement ( v );
            }
        }
	// The View Collection should have all the views at this point.
	return Views;
    }
    



    //================================================================
    //================================================================
    //================================================================
    //================================================================
    //=============== Schema Changes, the highest level ==============
    // every schema change consists of three steps: 
    // do the physical change, evolve the MKB, evolve the VKB
    // so three calls are made for each of the changes below.
    //================================================================
    //================================================================
    //================================================================
    //================================================================

    /**
     * Deletes an attribute by calling the appropriate method in the
     * MKB. 
     *
     * @param isName  name of the IS the attribute is in
     * @param relName  name of the relation the attribute is in
     * @param attName name of the attribute to delete */
    
    public boolean deleteAttribute(String isName, String relName, 
				   String attName)
                                   /* throws RemoteException */ {
	if (DEBUG) System.out.println("Called deleteAttribute");
	changed = true;
        mainVSA.viewEvolution(new Schema_Change(Schema_Change.DEL_ATT, 
                                                isName, relName, 
                                                attName)); 
        DBManager dbmgr;
        if ((dbmgr=getDBManagerFromISName(isName))!=null) {
            if (dbmgr.deleteAttribute(relName,attName))
                return mainMKB.deleteAttribute(isName,relName, attName);
        }
        return false;
    }
    
    /*
     * Adds an attribute by calling the MKB. <BR> <B>Note that the
     * attribute type is hardcoded right now (VARCHAR(37)) in the MKB
     * class as well as here!!!</B>
     *
     * @param isName  name of the IS the attribute is in
     * @param relName  name of the relation the attribute is in
     * @param attName  name of the attribute to add
     * @param attType type of the new attribute */

    public boolean addAttribute(String isName, String relName, 
				String attName, String attType)
                                /* throws RemoteException */    
    {
	if (DEBUG) System.out.println("Called addAttribute");
	changed = true;
        DBManager dbmgr;
        if ((dbmgr=getDBManagerFromISName(isName))!=null) {
            if (dbmgr.addAttribute(relName,attName,"varchar",37))
                return mainMKB.addAttribute(isName,relName, attName, "varchar");
        }
        return false;
    }
    
    /**
     * Renames an attribute by calling the appropriate method in the
     * MKB
     *
     * @param isName  name of the IS the attribute is in
     * @param relName  name of the relation the attribute is in
     * @param attName  name of the attribute to rename
     * @param newName new name of the attribute */

    public boolean changeAttributeName(String isName, String relName, 
				       String attName, String newName)
                                       /* throws RemoteException */    
    {
	if (DEBUG) System.out.println("Called changeAttributeName");
	changed = true;
        mainVSA.viewEvolution(new Schema_Change(Schema_Change.CHG_ATT, 
                                                isName, relName, 
                                                attName)); 

        DBManager dbmgr;
        if ((dbmgr=getDBManagerFromISName(isName))!=null) {
            if (dbmgr.renameAttribute(relName,attName,newName))
                return mainMKB.changeAttributeName(isName,relName, 
                                                   attName, newName);
        }
        return false;
    }
    
    /**
     * Adds a relation by calling the appropriate method in the
     * MKB
     *
     * @param isName  name of the IS the relation is in
     * @param relName  name of the relation to add
     * @param attNames  names of the attribute to add to relName
     * @param attTypes types of the attributes */
    
    public boolean addRelation(String isName, String relName, 
			       Vector attNames, Vector attTypes)
                               /* throws RemoteException */    
    {
	if (DEBUG) System.out.println("Called addRelation");
	changed = true;
        DBManager dbmgr;
        if ((dbmgr=getDBManagerFromISName(isName))!=null) {
            if (dbmgr.addRelation(relName,attNames,attTypes))
                return mainMKB.addRelation(isName, relName, 
                                           attNames, attTypes);
        }
        return false;
        
    }
  
    /**
     * Deletes a relation by calling the appropriate method in the
     * MKB
     *
     * @param isName  name of the IS the relation is in
     * @param relName name of the relation to delete */

    public boolean deleteRelation(String isName, String relName)
    {
	if (DEBUG) System.out.println("Called deleteRelation");
	changed = true;
        mainVSA.viewEvolution(new Schema_Change(Schema_Change.DEL_REL, 
                                                isName, relName));
        
        DBManager dbmgr;
        if ((dbmgr=getDBManagerFromISName(isName))!=null) {
            if (dbmgr.deleteRelation(relName, false)) 
                // not silent ==> error msg
                return mainMKB.deleteRelation(isName, relName);
        }
        return false;
    }
    
    /**
     * Renames a relation by calling the appropriate method in the
     * MKB
     *
     * @param isName  name of the IS the relation is in
     * @param relName  name of the relation to rename
     * @param newName new name of the relation */

    public boolean changeRelationName(String isName, String relName, 
                                      String newName)
                                      /* throws RemoteException */    
    {
	if (DEBUG) System.out.println("Called changeRelationName");
	changed = true;
        mainVSA.viewEvolution(new Schema_Change(Schema_Change.CHG_REL, 
                                                isName, relName));
        DBManager dbmgr;
        if ((dbmgr=getDBManagerFromISName(isName))!=null) {
            if (dbmgr.renameRelation(relName, newName)) 
                return mainMKB.changeRelationName(isName, relName, newName);
        }
        return false;
    }


    //===================================================================
    //===================================================================
    //===================================================================
    //===================================================================
    //===================================================================
    //===================================================================
    //===================================================================
    //===================================================================
    //===================================================================



    /**
     * searches for an IS  object by name 
     * @return null if IS with this name not found */

    public IS getISfromName(String theIS)
    /* throws RemoteException */
    {
        return mainMKB.findIS(theIS);
    }
    /**
     * Returns if there has been a capability change and resets
     "changed" bit to false.
    */
    public boolean getChangeStatus() 
    /* throws RemoteException */
    {
        
	boolean viewChanged = false;
	Enumeration e = Views.elements();
	while (e.hasMoreElements()) {
	    viewChanged |= ((View)e.nextElement()).getChangeStatus();
	}
        
	if(changed || viewChanged) {
            changed = false;
            return true;
        }
	else
	    return false;
    }
    
    /**
     * Returns if a user is currently able to make capability changes
     */
    public boolean isISChangeRunning() 
    /* throws RemoteException */
    {
	int OldCounter = AliveCounter;
        
	try{
	    // try to sleep 1.5 second and get the second number.
	    Thread.sleep(1500);
	}catch(Exception e){
	    System.out.println("DEBUG: Main.isISChangeRunnning() Thread exception");
	}
        
	if(AliveCounter == OldCounter){// the current user is dead, or no current user
	    return false;
	}
        
	return true;
    }
    
    /**
     * Reset Middle Layer. <BR> To do: change static trade-off
     * parameters to something more dynamic. The database is not being
     * reset here anymore, this is done with the separate class
     * "resetDB" from a shell script. */

    public void reset() {


        queryEngine = new QueryEngine(this);

	String mkbName = System.getProperty("mkbFileLocation");
	mainMKB=new DSRG.MKB.MKB(mkbName);
	if (mainMKB == null)
	    System.err.println("MKB not initialized");
    
	String vkbName = System.getProperty("vkbFileLocation");
	mainVKB=new DSRG.VKB.VKB(vkbName, mainMKB);
        
	if (mainVKB == null)
	    System.err.println("VKB not initialized");

        mainVSA = new DSRG.VSA.VSA(mainMKB, mainVKB);

	if (mainVSA == null)
	    System.err.println("VSA not initialized");
    

        // after the merge of the EVEDemo.* package and the EVEDemo.MKB.* 
        // package, the MKB needs a reference to the VSA
        mainMKB.setVSA(mainVSA);
        
        // we "abuse" the MKB.IS class to represent the local data
        // warehouse.  that gives us access to the "relation"
        // functionality in the MKB class.
        //	localIS = new IS("LOCAL_IS");
        //localIS = mainMKB.getLocalIS();
        //if (localIS==null) System.out.println("No local IS available");

        Enumeration ISs = mainMKB.getISs().elements();
        IS oneIS;

        // create one DBManager for each IS
        while (ISs.hasMoreElements()) {
            oneIS = (IS)ISs.nextElement();
            if (!oneIS.getName().equals("LOCAL_IS")) {
                if (DEBUG) System.err.println("Creating DBManager for "+oneIS.ISName);
                DBManagers.addElement( createDBManager(oneIS.ISName,
                                                       oneIS.DBType,
                                                       oneIS.DBURL,
                                                       oneIS.username,
                                                       oneIS.password));
            }
        }

        // and the DBManager for the local IS.

        oneIS = mainMKB.getLocalIS();
        localDBManager = createDBManager(oneIS.ISName,
                                         oneIS.DBType,
                                         oneIS.DBURL,
                                         oneIS.username,
                                         oneIS.password);
    

	try{
	    Views=new Vector();
	    Views = getAllViews();
	}catch(Exception e){
	    System.err.println("Main.reset(): " + e.getMessage());
	}

        
	MyTreeCellRenderer.initRenderer(this); // this is for showing
        // little icons in the tree views in the applets. Had to be
        // here in the RMI version, so we'll leave it here.
        
	changed = true;
    }
      
    /**
     * The current connect user will use this function to increase the counter.
     */
    public void AliveMsg() /* throws RemoteException */{
	if (DEBUG) System.out.print("AliveCounter: " + AliveCounter + "\r");
	AliveCounter ++;
    }
  
    /**
     * An attempt at fixing some RMI problem. Not needed anymore?
     * (koeller 030799)
     * @author avanwyk */

    public String getCorrectDefinitionx(String theViewName)
    /* throws RemoteException */
    {
	for (int i=0; i < Views.size(); i++) {
	    if (theViewName.equals(((View)Views.elementAt(i)).getName())) {
		return ((View)Views.elementAt(i)).getDefinition();
	    }
	}
	return new String("View not found\n");
    }
  
    /**
     * An attempt at fixing some RMI problem. Not needed anymore?
     * (koeller 030799)
     * @author avanwyk */
    
    public String getCorrectOldDefinitionx(String theViewName) 
    /* throws RemoteException */{
	for (int i=0; i < Views.size(); i++) {
	    if (theViewName.equals(((View)Views.elementAt(i)).getName())) {
		return ((View)Views.elementAt(i)).getOldView().getDefinition();
	    }
	}
	return new String("View not found\n");
    }


    /**
     * return the collection of views. <BR>
     * if it doesn't exist yet, get it through getAllViewsFromVKB()
     * If it does exist, refresh the views.
     * <P> koeller 030799: Added support for the QC-Module. This is a
     * new function aimed at selecting one of several view version
     * suggested by the QC-Module. The vector Views contains
     * views like before, but here the new "Versions" attribute (a
     * vector of objects of class "version") is initialized.  <BR>
     * @author: koeller */
    
    public synchronized Vector getAllViews()  /* throws RemoteException */
    {
	// Returns all views defined in the system
	if ( Views.size() == 0)
	    {
		if (DEBUG) System.out.println
                               ("Debug:Main.getAllViews() rmi Exp. here2?");
		getAllViewsFromVKB ();
	    }else {
		if (DEBUG) System.out.println("main.getAllViews() refresh");
		for(int i = 0; i < Views.size(); i++){
		    View v = (View)Views.elementAt(i);
		    if (DEBUG) System.out.println("refreshed view #"+i);
		    v.refresh();
		}
		if (DEBUG) System.out.println("main.getAllViews() refresh complete");
	    }
        
	if (DEBUG) 
	    System.out.println("Debug:Main.getAllViews() the First View is:" 
                               + ((View)Views.elementAt(0)).getDefinition());
        
	return Views;
    }

    /**
     * Save all Views
     */

    public void saveViews(EVEDemo.MW.Main something) /* throws RemoteException */ {
	/*Vector tempViews = something.getAllViews();
	  for (int i = 0; i < tempViews.size(); i++)
	  ((EVEDemo.MW.View)tempViews.elementAt(i)).saveView();*/
	for(int j = 0; j < Views.size(); j++)
	    ((EVEDemo.MW.View)Views.elementAt(j)).saveView();
    }
  
    /**
     * Get Extent of a specific view. Calls the QueryEngine class.
     * @param viewName Name of the View.
     */

    public Table getViewExtent(String viewName, boolean old_view_extent) 
    /* throws RemoteException */{
	// Get the View Object from the Views by viewName
	
	Enumeration enum = Views.elements();
	EVEDemo.MW.View v=null;
	while(enum.hasMoreElements()){
	    v = (EVEDemo.MW.View)enum.nextElement();
	    if(v.getName().equals(viewName))
		break;
	    else
		v = null;
	}
	if(v == null)
	    return null;
	if (old_view_extent)
	    return queryEngine.computeExtent(v.getOldView());
	else
	    return queryEngine.computeExtent(v);
    }
  
    /**
     * Get extent of a single relation as a Table
     * object. this is for displaying a relation extent in the applet
     * @param isName, SQL-query
     * */
    
    public Table getTable(String isName, String SQL) {
        
        DBManager dbmgr;
        if ((dbmgr=getDBManagerFromISName(isName))!=null) 
            return dbmgr.executeViewQuery(SQL);
        else return null;
    }
    

    /** this method is used to demonstrate incremental view
        maintenance and is currently not used. Will have to be fixed
        later. It's disabled now. */

    public void startUpdate() {
        System.err.println("called main::startUpdate, but this is disabled");
    }

    /*
    public void startUpdate()  {
	System.out.println("Starting Update");
    
	Enumeration e = null;
	IS is = null;
	boolean success=false;
    
	String oneUpdate = null ;
	String updISName, updRelationName, updUpdate;
	int updCount;
	String updateFileName = System.getProperty("updateFileName");
	BufferedReader br=null;
	StringTokenizer st;
	StringBuffer str;
	long startPerformance;
    
	try {
	    br = new BufferedReader (
				     new InputStreamReader (
							    new FileInputStream (updateFileName )));
	}
	catch (FileNotFoundException fnfExc) {
	    System.out.println(fnfExc.toString());
	}
    
    
    
	System.out.println("============================================");
	System.out.println("============================================");
	startPerformance = System.currentTimeMillis();
	System.out.println("UPDATE. TIME before: " + startPerformance);
	System.out.println("============================================");
	System.out.println("============================================");
    
	e = Views.elements();
	while (e.hasMoreElements()) {
	    ((View)e.nextElement()).startPerformance = startPerformance;
	}
    
    
	try {
      
	    int dus = 0;
      
	    while ((oneUpdate = br.readLine())!=null) {
        
		dus ++;
        
		System.out.println(dus + " updates from File.");
        
		st = new StringTokenizer(oneUpdate, "%");
        
		str = new StringBuffer();
        
		updISName       = st.nextToken();
		updRelationName = st.nextToken();
		updCount        = (new Integer(st.nextToken())).intValue();
		updUpdate       = st.nextToken();
        
		e = getMKB().getISs().elements();
		while (e.hasMoreElements()) {
		    is = (IS)e.nextElement();
		    if (is.getName().equals(updISName)) { 
			//  success = is.getIS().sendUpdate("MINI_CUSTOMER",1,
			    "31|'Name'|'Address'|123|'8273824'|12345.89|'SEGMENT'|'This is a comment'");
			
			success = is.sendUpdate(updRelationName, updCount, updUpdate);
		    }
		}
        
		if (success) {
		    e = Views.elements();
		    while (e.hasMoreElements()) {
			//System.out.println("Main.startUpdate() Here!");
			//((View)e.nextElement()).sendUpdate("IS1","MINI_CUSTOMER",1,
			  "31|'Name'|'Address'|123|'8273824'|12345.89|'SEGMENT'|'This is a comment'");
			
			((View)e.nextElement()).sendUpdate(updISName, updRelationName, updCount, updUpdate.replace('\'',' '));
		    }	  
		}
	    }
	    // because this kind of data update notification violated
	    // the FIFO assumption, we block the PSWEEP transaction
	    // first, then we let it go by another special method.
	    // need to wait for a while, and then start, let's say, wait for key pressed
	    synchronized(System.out){
		System.out.println("***PRESS ANY KEY IF THE SYSTEM QUIET***");
		//System.in.read();
		System.out.println("***Auto Start in 3 seconds***");
		try{
		    Thread.sleep(3000);
		}catch(Exception ee){
		    System.out.println("cannot wait? exception:" + ee.getMessage());
		}
	    }
	    EVEDemo.PSWEEP.Timer.start();
	    e = Views.elements();
	    while (e.hasMoreElements()) {
		((View)e.nextElement()).startVM();
	    }
	}
	catch (IOException ioe) {
	    ioe.printStackTrace();
	}
    
	System.out.println("Done With Update");
    }
*/

    public ImageIcon getMyIcon(String filename) /* throws RemoteException */ {
	ImageIcon temp;
	try {
	    return new ImageIcon("images/"+filename);
	} catch (Exception e) {
	    System.out.println("Couldn't load images at Server: " + e);
	    return null;
	}
        
    }

    
    /**
     * Constructed used by MKBParser. It used to create an IS object with
     * associated DBManager.
     * @author Xin Zhang.
     */

    public DBManager createDBManager (String ISName,  String sDBType, 
                                      String sDBURL, String sUsername, 
                                      String sPassword) {

        DBManager dbmgr=null;
      
        if(sDBType.equals("ORACLE")){
            dbmgr = new DBMOracle(ISName, sUsername, sPassword, sDBURL);
        }
        else if(sDBType.equals("MS_ACCESS")){
            dbmgr = new DBMAccess(ISName, sUsername, sPassword, sDBURL);
        }
        else System.out.println("UNKNOWN TYPE OF DBManager. PLEASE CHECK THE MKB-File!!!");
    return dbmgr;
    }
    


    /**
     * searches for a DBManager from an IS name
     * @return null if IS with this name not found
     */

    public DBManager getDBManagerFromISName(String theIS) {
        Enumeration e = DBManagers.elements();
        DBManager dbmgr;
        while (e.hasMoreElements()) {
            dbmgr = (DBManager)e.nextElement();
            if (dbmgr.ISName.equals(theIS))
                return dbmgr;
        }
        System.err.println("No DBManager found for IS "+theIS);
        return null;
    }

    /** this vector contains objects of class DBManager */
    public Vector getDBManagers() {
        return DBManagers;
    }
    
    public DBManager getLocalDBManager() {
        return localDBManager;
    }
    
    public DSRG.VKB.VKB getVKB() {
        return mainVKB;
    }

    public QueryEngine getQueryEngine() {
        return queryEngine;
    }

    }
