////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package PSWEEP;

import SWEEP.UpdateData;
import SWEEP.StampedElement;

/**
 * This class implement a set of data updates with a specific size. It
 * represents the Parallel Processing Update Set.  
 */

public class PPUSet extends UpdateDataSet implements
java.io.Serializable {

  /**
  * The size of this set.
  */
  int SizeOfSet;

  /**
   * Construct a set with size i.
   * @param i the size of set.
   */
  public PPUSet(int i){
    super();
    SizeOfSet = i;
  }

  /**
   * Append one element to this set.
   * If the set still has space, we add it, otherwise, we return false.
   * @param elm to be added element
   * @return true means element added, otherwise false.
   */
   public synchronized boolean append(Object elm){
    if(!isFull())
      super.addElement(elm);
    else
      return false;
    
    return true;
  }
  
  /**
   * Check whether the set is full.
   * @return true means the set is full, false means the set is not full.
   */
  public synchronized boolean isFull(){
    if(size() == SizeOfSet)
      return true;
    else if(size() < SizeOfSet)
      return false;
    else
      System.out.println("SizedSet:isFull() What's going on here?");

    return false;
  }

  /**
   * Check if the specific data update is in the set.
   * It will check all the elements in this set, if one of them is earlier than
   * the specific data update, then return true, else return false.
   * @param du the specific data update need to be checked.
   * @return true if related, otherwise false.
   */
  public synchronized boolean isRelated(UpdateData du){
    for(int i = 0; i < size(); i++){
      StampedElement se = (StampedElement)elementAt(i);
      if(!se.OlderThan(du))
      	return true;
    }
    
    return false;
  }
  
}
