////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
// All the classes in the "//" means they are duplicate with the type in PSWEEP
// but used in the PSWEEP.

package PSWEEP;

import DSRG.VKB.View_Definition;
import DSRG.VKB.Relation_Component;
import DSRG.VKB.Attribute_Component;
import DSRG.VKB.Local_Condition;
import DSRG.VKB.Join_Condition;
import DSRG.VKB.Primitive_Value;
//DSRG.VKB.Type

import DSRG.VKB.E_SQL.*;

import DSRG.MKB.MKB;
//DSRG.MKB.Relation
//DSRG.MKB.Attribute

import SWEEP.DataUpdateReceiver;
import SWEEP.MultiQueue;
import SWEEP.ISProcess;
import SWEEP.Relation;
import SWEEP.RelationSchema;
import SWEEP.Type;
import SWEEP.Attribute;
import SWEEP.ProjectList;
import SWEEP.SelectList;
import SWEEP.Operator;
import SWEEP.PrimitiveValue;
import SWEEP.LocalCondition;
import SWEEP.JoinList;
import SWEEP.JoinCondition;
import SWEEP.ViewDefinition;
import SWEEP.JCAttrPair;
import SWEEP.ProcessQuery;

import EVEDemo.MW.View;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;
import java.io.StringBufferInputStream;

/**
* PSWEEP object will maintain one view in it. It will provide a data update
* receiver to outside, where you can send data update to the PSWEEP object.
* It requires:
*    1. View Definition either in ESQL format, or an object of ViewDefinition in VKB.
*    2. Data update which will affect this view.
* It will provice:
*    1. a DataUpdateReceiver to accept the data update.
*
* It will DO:
*    Maintain the real extend of that view in the database.
*/
public class PSWEEP implements java.io.Serializable
{

private final static boolean DEBUG = false;
    /**
     * The sleep timer for all the threads.
     */
    public final static int step = 5;
    /**
     * DataUpdateReceiver of this PSWEEP Object. The object will be used by
     * Andreas from the middle layer to send the DU down.
     */
    DataUpdateReceiver dur;
    
    /**
     * FifoChannel used to create communication between ISs and DW.
     */
    MultiQueue FifoChannel;
    
    /**
     * Data warehouse process of this view definition.
     * It should be the PSWEEP.DWProcess here. :)
     */
    DWProcess dw;
    
    /**
     * Array of ISProcess
     */
    ISProcess[] isps;
    
    /**
     * Number of ISs.
     */
    int ISnum;
    
    /**
     * Get Receiver from PSWEEP. Where you can use it to send data update to PSWEEP
     */
    public DataUpdateReceiver getReceiver(){
	return dur;
    }

    /**
     * Start the UpdateView of PSWEEP DWProcess.
     */
    public void start(){
	dw.startUV();
    }

    /**
     * Create a PSWEEP object from View object in EVEDemo.MW package.
     */
    public PSWEEP(View v, MKB mkb) throws Exception{

      
	//System.out.println("debug:sweep<init> v def is: \r\n" + v.getDefinition());

	if(mkb == null)
	    System.out.println("Debug:sweep<init> mkb is null!");


	View_Definition vd = new View_Definition(v.getDefinition(), mkb);

	//System.out.println("debug: PSWEEP<init> vd got from vkb is: " + vd);

	init(vd);

    if(EVEDemo.MW.Main.DEBUG)
	System.out.println("debug: PSWEEP<init> init successful!");

	if(dw == null)
	    System.out.println("PSWEEP<init> dw is null.");
	else
	    /*
	     * For EVEDemo Integration
	     */
	    dw.currentUpdateView.setView(v);

	// System.out.println("Debug: PSWEEP Object Constructed Successful!");

    }

  /**
   * Create a PSWEEP object from View_definition class in EVEDemo.VKB package.
   */
  public PSWEEP(View_Definition vd) throws Exception{
    init(vd);
  }

  /**
   * Init the PSWEEP object.
   */
    void init(View_Definition vd) throws Exception{

	// Creat the Relations modules in PSWEEP of relations in from clause of view.
	// 1. get the Relation reference from the vd
	// 2. for each reference get the schema out
	// 3. follow the example in FlightSubSystem create the relations.
	// System.out.println ("debug: PSWEEP.init create Relations");

	Vector from = vd.getClause(View_Definition.FROM);

	// Get the number of ISs(relations).
	ISnum = from.size();

	Relation[] relations = new Relation[ISnum];
	for(int i = 0; i< ISnum; i++){
	    DSRG.MKB.Relation rel = ((Relation_Component)from.elementAt(i)).getRelation();
	    relations[i] = new Relation(rel.getName());
	    // Get the attribute vector of this Relation.
	    Vector attrs = rel.getAttributes();

	    int attnum = attrs.size();

	    String schema[] = new String[attnum];
	    for( int j = 0; j < attnum; j++){
		schema[j] = ((DSRG.MKB.Attribute)attrs.elementAt(j)).getName();
	    }

	    relations[i].setSchema(new RelationSchema(schema));
	}

	// create the DW view definition
	// We assume MKB know the type of the attribute.
	// 1. Get the Project list from view.
	// 2. select the attribute name out as the View schema.
	// 3. create the attribute object, then put in AttrList.
	// 4. create project list, and create select list, join list next.
	// 5. create the view definition.

	// System.out.println("Debug:PSWEEP.init: create DW View definition");

	// 1. Create the view schema
	Vector project = vd.getPSJList(View_Definition.PROJECT);
	int snum = project.size();
	String vschema[] = new String[snum];
	for( int i = 0; i< snum; i++){
	    DSRG.MKB.Attribute att = ((Attribute_Component)project.elementAt(i)).getAttribute();
	    vschema[i] = att.getRelation().getName()+"_"+att.getName();
	}
	RelationSchema DWrs = new RelationSchema(vschema);

	// 2. Create Project List

	// attribute list of sweep attribute.
	Vector attrList = new Vector();

	// 2.1 for each relation, find the project attributes of it.
	for( int i = 0; i< ISnum; i++){
	    // Get the attributes belongs to the relaion.
	    Vector attrs = getAttrs(project, relations[i]);
	    // for each attribute, create the attribute object
	    int anum = attrs.size();
	    for(int j = 0; j< anum; j++){
		DSRG.MKB.Attribute att = (DSRG.MKB.Attribute)attrs.elementAt(j);

		// here we assume the MKB stores the type of attribute.
		Type t = new Type(Type.STRING);
		/*
		  switch(att.getType()){
		  default: t = new Type(Type.STRING);
		  }
		*/

		attrList.addElement(new Attribute(att.getName(), relations[i], t, 1));
	    }
	}

	Attribute[] attrs = new Attribute[attrList.size()];
	attrList.copyInto(attrs);
	ProjectList PL = new ProjectList(attrs);

	// 3. create select list
	// We are going to create the select list from view.
	// a. we need get the select list from view
	// b. create the local conditions for each LC in select list.
	// c. create the select list.
	SelectList SL = new SelectList();
	Vector select = vd.getPSJList(View_Definition.SELECT);

	// Need to find the attributes that already created in the previous projectlist
	// Otherwise, create the new attribute for it.
	int ssize = select.size();
	for(int i =0 ; i<ssize; i++){
	    Local_Condition lc = (Local_Condition)select.elementAt(i);

	    DSRG.MKB.Attribute lcattr = lc.getLeft();

	    // check if the Attr is already in the list.
	    Attribute attr = checkAttrList(attrList, lcattr.getName());

	    if(attr == null){// not found
		/// create the new attribute.

		// Get the PSWEEP Relation object for this Attribute.
		Relation r = getRelation(relations, lcattr.getRelation().getName());

		Type t = new Type(Type.STRING);
		/*
		  switch(lcattr.getType()){
		  default: new Type(Type.STRING);
		  }
		*/

		attr = new Attribute(lcattr.getName(), r, t, 1);
		attrList.addElement(attr);
	    }

	    // Get the operator from the local condition in VKB
	    String op = lc.getOperator().toString();

	    Operator oprt = new Operator(op);

	    // Get the Prmitive Value from Local condition.
	    Primitive_Value pv = lc.getRight();

	    PrimitiveValue priv;

	    switch(pv.getType()){
	    case DSRG.VKB.Type.LONG:
		priv = new PrimitiveValue(pv.getLong());
		break;
	    case DSRG.VKB.Type.DOUBLE:
		priv = new PrimitiveValue(pv.getDouble());
		break;
	    case DSRG.VKB.Type.BOOLEAN:
		priv = new PrimitiveValue(pv.getBoolean());
		break;
	    case DSRG.VKB.Type.STRING:
		priv = new PrimitiveValue(pv.getString());
		break;
	    default:
		priv = null;
		// System.out.println("Invalide type of primitive type.");
	    }

	    SL.addLocalCondition(new LocalCondition(attr, oprt, priv));
	}

	// 4. Create Join List
	JoinList JL = new JoinList();

	//System.out.println("debug: PSWEEP:init: JoinList is: \r\n" + JL);

	// 4.1 Get Join List from View.
	Vector join = vd.getPSJList(View_Definition.JOIN);

	// 4.2 Get Relations from the Join List.
	Vector joinrels = new Vector();
	Enumeration enum = join.elements();
	while(enum.hasMoreElements()){
	    Join_Condition jc = (Join_Condition)enum.nextElement();
	    if(!joinrels.contains(jc.getLeft().getRelation()))
		joinrels.addElement(jc.getLeft().getRelation());
	    if(!joinrels.contains(jc.getRight().getRelation()))
		joinrels.addElement(jc.getRight().getRelation());
	}

	// 4.3 Convert the DSRG.MKB.Relation to PSWEEP Relation and add to join list
	enum = joinrels.elements();
	while(enum.hasMoreElements()){
	    DSRG.MKB.Relation r = (DSRG.MKB.Relation)enum.nextElement();
	    JL.addRelation(getRelation(relations, r.getName()));
	}

	// 4.4 add Join conditions to the Join List.
	enum = join.elements();
	while(enum.hasMoreElements()){
	    Join_Condition jc = (Join_Condition)enum.nextElement();

	    Attribute attr1 = getAttribute(attrList, jc.getLeft().getName());

	    if(attr1 == null){// not found
		/// create the new attribute.

		// Get the PSWEEP Relation object for this Attribute.
		Relation r = getRelation(relations, jc.getLeft().getRelation().getName());

		Type t = new Type(Type.STRING);
		/*
		  switch(jc.getLeft().getType()){
		  default: new Type(Type.STRING);
		  }
		*/

		attr1 = new Attribute(jc.getLeft().getName(), r, t, 1);
		attrList.addElement(attr1);
	    }


	    Attribute attr2 = getAttribute(attrList, jc.getRight().getName());

	    if(attr2 == null){// not found
		/// create the new attribute.

		// Get the PSWEEP Relation object for this Attribute.
		Relation r = getRelation(relations, jc.getRight().getRelation().getName());

		Type t = new Type(Type.STRING);
		/*
		  switch(jc.getLeft().getType()){
		  default: new Type(Type.STRING);
		  }
		*/

		attr2 = new Attribute(jc.getRight().getName(), r, t, 1);
		attrList.addElement(attr2);
	    }

	    String op = jc.getOperator().toString();

	    Operator oprt = new Operator(op);

	    JoinCondition joinC = new JoinCondition(attr1, oprt, attr2);
	    JL.addJC(joinC);
	}

	// create the View definition.
	ViewDefinition view = new ViewDefinition(vd.getName(), DWrs, PL, SL, JL);

	// Initilize our system.
	FifoChannel = new MultiQueue(ISnum);

	dw = new DWProcess(FifoChannel, view);

	dw.currentUpdateView.setn(ISnum);

	isps = new ISProcess[ISnum];

	Hashtable[] jcpairsLC = new Hashtable[ISnum]; // join attr pairs for local compensation.

	for(int i = 0; i< ISnum; i++){
    isps[i] = new ISProcess(FifoChannel, relations[i], i+1);
    // Need to set the join condition in Processery Query.
    // Here we need one name, we assume only one relation is connected to this
    // relation. One name is Local name, one name is remote name.
    // 1. find the vector of  join conditions of this relation.
    // 2. create a hashtable of JCAttrPair, and index on name of the remote 
    // relation.
    // 3. assign the name.
    Vector  jcs = getJCs(JL, relations[i]);
    Hashtable jcpairs = new Hashtable();
    jcpairsLC[i] = new Hashtable();
    for(int j = 0; j< jcs.size() ; j++){
  		JoinCondition jc = (JoinCondition) jcs.elementAt(j);
	  	String relname = getRemoteRelName(jc, relations[i]);
		  String lat = getLocalAttrName(jc, relations[i]);
  		String rat = getRemoteAttrName(jc, relations[i]);
	  	jcpairs.put(relname, new JCAttrPair(lat, rat));

      lat = relations[i].getName() + "_" + lat;
      // switch the lat and rat here.
  		jcpairsLC[i].put(relname, new JCAttrPair(rat, lat));
	  	if (DEBUG) System.out.println("JCAttrPair: Relation: " + relname + " lat: " + rat + " rat: " + lat);
	  }
	  isps[i].currentQuery.JCAttrs = jcpairs;
	    
	}
	// put the jcpairs info into the DW process.
	dw.currentUpdateView.JCAttrs = jcpairsLC;

	// 5. create the createUpdateFile to run the program.
	// In order to disable the update detection by PSWEEP, we comment next block.
    
	/*
	  CreateUpdateFile update[] = new CreateUpdateFile[ISnum];
      
	  for(int i = 0; i< update.length; i++){
	  update[i] = new CreateUpdateFile(relations[i].getName(), i+1);
	  isps[i].currentUpdates.setCUF(update[i]);
	  }
	*/
	// Instead of CreateUpdateFile, we create another class, which
	// will be called by Andreas, and get the data update from the
	// middle layer

	// System.out.println("Debug: PSWEEP.init Create Receiver.");
	dur = new DataUpdateReceiver(relations);

	// System.out.println("Debug: PSWEEP.init Receiver created successful.");

	for(int i = 0; i< ISnum; i++){
	    isps[i].currentUpdates.setReceiver(dur);
	}
    
    }

    /**
  * Get a list of join conditions related to a specific relation from 
  * the current viewdefinition of this PSWEEP object.
  * @param JL join list
  * @param rel reference of relation.
  * @return return empty vector is not found.
  */
    Vector getJCs(JoinList JL, Relation rel){

	Vector result = new Vector();

	Vector jcl = JL.getJCList().getJCList();

	Enumeration enum = jcl.elements();
	while(enum.hasMoreElements()){
	    JoinCondition jc = (JoinCondition)enum.nextElement();
	    try{
		if(jc.refers(rel))
		    result.addElement(jc);
	    }catch(Exception e){
		System.out.println("PSWEEP:getJC " + e.getMessage());
		return result;
	    }
	}

	return result;
    }

    /**
  * Get Local Attribute Name of this join condition of specific relation name.
  * @param jc Join Conditoin.
  * @param rel relation's name.
  */
    String getLocalAttrName(JoinCondition jc, Relation rel){
	Attribute opd1 = jc.getOpd1();
	Attribute opd2 = jc.getOpd2();

	if(opd1.getRelation() == rel)
	    return opd1.getName();
	if(opd2.getRelation() == rel)
	    return opd2.getName();

	return null;
    }
  

    /**
   * Get Remote Relation Name of this Join Condition of Specific Relation name.
   * @param jc Join Condition.
   * @param rel Relation's Name.
   * @return "R"
   */
    String getRemoteRelName(JoinCondition jc, Relation rel){
	Attribute opd1 = jc.getOpd1();
	Attribute opd2 = jc.getOpd2();

	if(opd1.getRelation() == rel)
	    return opd2.getRelation().getName();
	if(opd2.getRelation() == rel)
	    return opd1.getRelation().getName();
    
	return null;
    }

    /**
  * Get Remote Attribute Name of this join condition of specific relation name.
  * @param jc Join Condition.
  * @param rel relation's name.
  * @return "A"
  */
    String getRemoteAttrName(JoinCondition jc, Relation rel){
	Attribute opd1 = jc.getOpd1();
	Attribute opd2 = jc.getOpd2();

	if(opd1.getRelation() == rel)
	    return opd2.getRelation().getName() + "_" + opd2.getName();
	if(opd2.getRelation() == rel)
	    return opd1.getRelation().getName() + "_" + opd1.getName();

	return null;
    }

    /**
  * check if a specific attribute is already in an vector of attribute
  * @param attrlist vector of attributes
  * @param aname attribute name "R.A".
  * @return not found return null.
  */
    Attribute  checkAttrList(Vector attrlist, String aname){
	Enumeration enum = attrlist.elements();
	while(enum.hasMoreElements()){
	    Attribute attr = (Attribute)enum.nextElement();

	    if(attr.getName().equals(aname))
		return attr;
	}

	return null;
    }

    /**
  * Get the PSWEEP Relation object from a list of relations.
  * @param rels array of relations.
  * @param String name of relation.
  * @return not found return null;
  */
    Relation getRelation(Relation[] rels, String Rname){
	for(int i = 0; i< rels.length; i++)
	    if(rels[i].getName().equals(Rname))
		return rels[i];

	return null;
    }

    /**
  * Get the Attributes of one relation in the project list.
  * @param project (Vector) Project list composed of attribute_component
  * @param rel specific relation object in sweep package.
  * @return a vector of objects of DSRG.MKB.Attributes.
  */
    Vector getAttrs(Vector project, Relation rel){
	Vector result = new Vector();

	Enumeration enum = project.elements();
	while(enum.hasMoreElements()){
	    Attribute_Component ac = (Attribute_Component)enum.nextElement();
	    if(ac.getAttribute().getRelation().getName().equals(rel.getName()))
		result.addElement(ac.getAttribute());
	}

	return result;
    }

    /**
  * Get Attribute object of PSWEEP from the AttrList (Vector) of PSWEEP Attribute
  * of has the specific name in format R.A
  * @param AttrList vector of PSWEEP Attributes.
  * @param String Attribute Name in format "R.A"
  */
    Attribute getAttribute(Vector AttrList, String attrName){
	Enumeration enum = AttrList.elements();
	while(enum.hasMoreElements()){
	    Attribute att = (Attribute)enum.nextElement();

	    if(att.getSQL().equals(attrName))
		return att;
	}

	return null;
    }
}
