////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package PSWEEP;

import java.util.Date;

import SWEEP.UpdateData;
import SWEEP.StampedElement;

/**
 * Class of Related Processed Update. It will store all the processed
 * data update but still need for concurrent DU detection.
 */

public class RPUSet extends UpdateDataSet implements
java.io.Serializable{ 

  /**
   * Construct a RPUSet.
   */
  public RPUSet(){
    super();
  }

  /**
   * Clean up the useless Data updates in the RPU. The useless data updates
   * are those updates elder than all the data updates in the PPU.
   * @param ppu the Parallel Processing Update set.
   */
  public synchronized void cleanUp(PPUSet ppu){
    StampedElement oldest = null;

    // get the latest timestamp in PPU.
    
    for(int i = 0;  i < ppu.size() ;i++){
      UpdateData du = (UpdateData)ppu.elementAt(i);
      if(oldest != null){
	if(!oldest.OlderThan(du))
	  oldest = du;
      }else{
	oldest = du;
      }
    }

    // Compare it to the RPU, and clean all the DU before that oldest
    // timestamp.
    for(int i = 0; i < size(); i++){
      UpdateData du = (UpdateData)elementAt(i);
      if(!oldest.OlderThan(du))
	removeElement(du);
    }
  }

}
