////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package PSWEEP;

import java.util.Vector;
import java.util.Enumeration;

import SWEEP.UpdateData;
import SWEEP.Relation;

/**
 * This class implement a set of data updates with a specific size. It
 * represents the Parallel Processing Update Set.  
 */

public class UpdateDataSet implements java.io.Serializable{

  /**
  * In order to synchronize, I am going to use the Vector.
  */
  Vector Set;

  /**
   * Construct a UpdateDataSet.
   */
  public UpdateDataSet(){
    Set = new Vector();
  }

  public synchronized void addElement(Object obj){
    Set.addElement(obj);
  }

  public synchronized Object elementAt(int index){
    return Set.elementAt(index);
  }

  public synchronized int size(){
    return Set.size();
  }

  public synchronized boolean removeElement(Object obj){
    return Set.removeElement(obj);
  }

  Enumeration enum = null;

  /**
   * Get the data update from a specific information sources.
   */
  public synchronized Relation peek(int theIndex) throws Exception{

    if(theIndex < 0)
      throw new Exception("Index less than Zero");

    enum = Set.elements();

    UpdateData ud = null;
    
    while(enum.hasMoreElements()){
	
    	ud = (UpdateData)enum.nextElement();
	
    	if(ud != null){
	      if(theIndex == ud.getSource()){
          //System.out.println("UpdateDataSet: Got one Potential Concurrent DU.");
          // reset the temp stamp.
          Relation r = ud.getDeltaRelation();
          r.setTimeStamp(ud.getTimeStamp());
          return r;
        }
      }
    }

    //throw new Exception("Cannot peek specifice Update in MessageQueue");
    return null;
  }

    /**
     * Get the next data update from the specific information sources.
     * It should be used with the peek() function.
     */
  public synchronized Relation peekNext(int theIndex) throws Exception{

    if(enum == null)
      return peek(theIndex);

    if(theIndex < 0)
      throw new Exception("Index less than Zero");

    UpdateData ud = null;
    
    while(enum.hasMoreElements()){
	
    	ud = (UpdateData)enum.nextElement();
	
    	if(ud != null)
  	    if(theIndex == ud.getSource()){
            // reset the temp stamp.
            Relation r = ud.getDeltaRelation();
            r.setTimeStamp(ud.getTimeStamp());
            return r;
        }
    }

    //throw new Exception("Cannot peek specifice Update in MessageQueue");
    enum = null;
    return null;
  }
    

  /**
   * Check whether a data update from IS j is stored in the UpdateDataSet.
   * @param j the index of the IS
   * @return true if there is DUj in the set, otherwise false.
   */
  public synchronized boolean exists(int j){
    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("UpdateDataSet.exists() Freeze here before loop?");
    for(int i = 0; i<size(); i++){
      UpdateData du = (UpdateData)elementAt(i);
      if(EVEDemo.MW.Main.DEBUG)
        System.out.println("UpdateDataSet.exists() Freeze at item " + i );
      if(du.getSource() == j)
      	return true;
    }
    return false;
  }
}
