////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package PSWEEP;

import EVEDemo.MW.View;

import java.util.Hashtable;

import SWEEP.MultiQueue;
import SWEEP.Queue;
import SWEEP.ViewDefinition;
import SWEEP.UpdateData;
import SWEEP.DatabaseInterface;

public class UpdateView extends SWEEP.UpdateView implements java.io.Serializable{
  
  PPUSet PPU;
  RPUSet RPU;    
  

 /**
   * Actually, we have three kinds of transactions between UMQ, PPU, and RPU.
   * 1. move UMQ to PPU.
   * 2. move PPU to RPU.
   * 3. take snapeshot of UMQ and PPU and RPU.
   * They are all locked each other.
   * in this case, I created one global lock called UPR(UMQ, PPU, RPU) lock.
   */
  public Lock UPRLock;


  /**
  * "1" like SWEEP, if ">1" then it is PSWEEP
  */
  static final int PPUsize = 6;
  
  /**
  * The Join Condition for the Local Compensation
  */
  public Hashtable[] JCAttrs;
  
  /**
  * The counter of threads of view changes
  */
  int threads = 0;

  /**
  * The Maximum Threads can run on this machine. Say 10000.
  */
  static int maxThreads = 10000;
  
  public UpdateView(MultiQueue theFIFOChannel, Queue theMessageQueue, ViewDefinition theView){
    
    super(theFIFOChannel, theMessageQueue, theView);
    
    UPRLock = new Lock();

    // initialize the PPU and RPU
    PPU = new PPUSet(PPUsize);
    RPU = new RPUSet();
    
  }
  
  public void run(){
    
    while(!finished){
      
      if(PPU!=null && RPU!=null){
        if(!PPU.isFull()){
          
          UpdateData theUpdate  = null;
          do{
            theUpdate = (UpdateData)UpdateMessageQueue.peekNext();
            if(theUpdate == null){
              try{
                sleep(PSWEEP.step);
              }catch(InterruptedException ie){
                System.out.println("UpdateView:run cannot sleep!");
              }
            }
          }while(theUpdate == null);

          System.out.println("UpdateView:run() going to lock DU: " + theUpdate.getTimeStamp() + " and UMQ, PPU ");

          {
            UPRLock.lock();
            System.out.println("UpdateView: run() UMQ - > PPU get lock.");

            if(EVEDemo.MW.Main.DEBUG)
              System.out.println("UpdateView:run() Testing Update Message Queue");
      
            // Remove (Delta R, i, t) FROM UpdateMessageQueue; BLOCK if queue empty 
            theUpdate = (UpdateData)UpdateMessageQueue.remove();
            // Put (DeltaR, i, t) To ParallelProcessingUpdate
            PPU.append(theUpdate);

            UPRLock.unlock();
            System.out.println("UpdateView: run() UMQ - > PPU release lock.");
          }


          System.out.println("UpdateView: UMQ:" + UpdateMessageQueue.length() + " PPU: " + PPU.size() + " RPU: " + RPU.size());        
          
          boolean startSuccessful = false;
          int waitUntil = 0;
          while(!startSuccessful && threads < maxThreads){
            try{          
              new ViewChange(theUpdate, View, EVEview, publicFIFOChannel, UpdateMessageQueue, PPU, RPU, n, this);
              threads++;
              startSuccessful = true;
            }catch(OutOfMemoryError oome){
              maxThreads = (int)(threads *0.9);
              System.out.println("Sorry, I cannot start more threads, wait previous threads done." + 
                                "\nso far we have: " + threads + " threads." + 
                                "\nwe set maxThreads to: " + maxThreads );
              waitUntil = threads/2;
              while(threads > waitUntil){
                try{
                  sleep(PSWEEP.step);
                }catch(InterruptedException ie){
                   System.out.println("System in too low memory to sleep?, try again!" +
                          "\n" + ie.getMessage());
                }
              }
            } 
          }// end while startSuccessful
        }// end if PPU is full
      }// end if PPU is null
      try{
        sleep(PSWEEP.step);
      }catch(InterruptedException ie){
        System.out.println("UpdateView:run() " + ie.getMessage());
      }
    }
  }
  
  /**
  * Callback function for ViewChange. When the ViewChange
  * terminated, it will decrease a counter when the counter is 0
  * and UMQ is empty, all the data updates are handled and system
  * in queiscence state. then, we can stop the timer.
  */
  public synchronized void finishDU(){
    threads --;

    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("UpdateView:finishDU(): still have " + threads + " thread.");

    if(threads == 0 && UpdateMessageQueue.isEmpty()){
      double t = Timer.end();
      System.out.println("*******************************");
      System.out.println(" Time spend: " + t + " seconds");
      System.out.println(" The max threads can support by this machine is:" + maxThreads);
      System.out.println("*******************************");
      // Only for Experiments It will access the global variable in the Main.java
      EVEDemo.MW.Main.expTime = t;
    }
    
  }
  
}
