////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package PSWEEP;

import SWEEP.UpdateData;
import SWEEP.Relation;
import SWEEP.MessageQueue;
import SWEEP.MultiQueue;
import SWEEP.DatabaseInterface;
import SWEEP.ViewDefinition;

import EVEDemo.MW.View;

import java.util.*;
/**
* New thread created by PSWEEP.
* One ViewChange thread will handle one data updates.
*/
public class ViewChange extends Thread{
  
/**
* The instance ID + ":" + COUNT of the ViewChange. To distinguish different threads.
  */
  static int count = 0;
  int id;
  
  /**
  * count how many concurrent DU for this du.
  */
  int clcnt = 0;

  Relation DeltaView;
  int theIndex;
  UpdateData du;
  PPUSet PPU;
  RPUSet RPU;
  MessageQueue UMQ;
  MultiQueue publicFIFOChannel;
  ViewDefinition View;
  View EVEview;
  
  /**
  * the UpdateView reference
  */
  UpdateView uv;
  
  /**
  * Total number of ISs.
  */
  int n;
  
  /**
  * increase the count.
  */
  static synchronized void incCount(){
    count++;
  }
  
  public ViewChange(UpdateData theUpdate, ViewDefinition theView,
    View theEVEview, MultiQueue theFIFO, 
    MessageQueue theMessageQueue, PPUSet pu, 
    RPUSet ru, int totalISs, UpdateView theUV){
    
    
    
    uv = theUV;
    
    incCount();
    id = count;
    
    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("ViewChange" + id + ":" + count + ":<init> " + theUpdate);
    
    //keep du to be removed from PPU and put in RPU
    du = theUpdate;
    
    EVEview = theEVEview;
    View = theView;
    
    //keep FIFO channel
    publicFIFOChannel = theFIFO;
    
    //keep umq, ppu and rpu
    PPU = pu;
    RPU = ru;
    UMQ = theMessageQueue;
    
    // keep totalIS Num.
    n = totalISs;
    
    // Delta V = Delta R
    DeltaView = theUpdate.getDeltaRelation();
    theIndex = theUpdate.getSource();
    
    start();
  }
  
  boolean isConcurrent(int j){

    boolean result = false;

    uv.UPRLock.lock();
    System.out.println("ViewChange " + id + ":" + count + " isConcurrent() get lock.");

    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("ViewChange.isConcurrent(): for j= " + j );

    if(UMQ.exists(j))
      result = true;

    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("ViewChange.isConcurrent(): no deadlock UMQ? ");

    if(PPU.exists(j))
      result = true;

    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("ViewChange.isConcurrent(): no deadlock PPU? ");

    if(RPU.exists(j))
      result = true;

    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("ViewChange.isConcurrent(): no deadlock RPU? ");

    uv.UPRLock.unlock();
    System.out.println("ViewChange " + id + ":" + count + " isConcurrent() release lock.");
    return result;
  }
  
  Relation processSubQuery(Relation Delta, int j) throws Exception{
    
    Relation TempView;
    
    // if DeltaView is empty, we return null.
    if(Delta.getExtent().size() == 0)
      return null;
    
    TempView = (Relation)Delta.clone();
    
    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("ViewChange" + id + ":" + count + ":processSubQuery Stp1: Send Delta V to IS["+j+"]");
    
    // send Delta V to Data source j
    publicFIFOChannel.sendQuery(Delta, j);
    
    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("ViewChange" + id + ":" + count + ":processSubQuery Stp2: Receive Delta V' from IS["+j+"]");        
    
    // receive Delta V from Data Source j
    // I think here, we need block to waiting
    Delta = publicFIFOChannel.receiveQueryResult(j);

    if(Delta == null){
      // error on the remote site. return null to tolerent the error.
      System.out.println("ViewChange" + id + ":" + count + ":processSubQuery ERROR: Remote Error in ProcessQuery Class. Cancel VM");
      return null;
    }
          

    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("ViewChange" + id + ":" + count + ":processSubQuery Stp2.5: the Delta V' is: \r\n" + Delta );
    
    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("ViewChange" + id + ":" + count + ":processSubQuery Stp3: Test Concurrent Update for IS["+j+"]"); 
    
    // remove the error due to concurrent update if any
    // if Exist Delta Rj in UMQ then Delta V = Delta V - 
    // Delta Rj join TempView.
    if(isConcurrent(j)){    

      if(EVEDemo.MW.Main.DEBUG)
        System.out.println("ViewChange" + id + ":" + count + ":processSubQuery Stp4: Concurrent Update detected");
      
      Relation DeltaRj = null;
      
      // get Delta Rj from UMQ without remove it
      //Object[] lock = new Object[3];
      //lock[0] = UMQ;
      //lock[1] = PPU;
      //lock[2] = RPU;

      // pick all the potential DeltaRs.
      Vector DeltaRjs = new Vector();

      //synchronized (lock){
      {
        uv.UPRLock.lock();
        System.out.println("ViewChange" + id + ":" + count + ":processSubQuery SnapShot UPR get lock.");

        DeltaRj = UMQ.peek(j);      
        while(DeltaRj != null){
          System.out.println("DEBUG: Delta (QR) : " + Delta.getTimeStamp() + " DeltaRj(DUj) : " + DeltaRj.getTimeStamp() + " du(DUi) : " + du.getTimeStamp() );
          DeltaRjs.addElement(DeltaRj);
          DeltaRj = UMQ.peekNext(j);
        }

        DeltaRj = PPU.peek(j);
        while(DeltaRj != null){
          System.out.println("DEBUG: Delta (QR) : " + Delta.getTimeStamp() + " DeltaRj(DUj) : " + DeltaRj.getTimeStamp() + " du(DUi) : " + du.getTimeStamp() );
          DeltaRjs.addElement(DeltaRj);
          DeltaRj = PPU.peekNext(j);
        }
          
        DeltaRj = RPU.peek(j);
        while(DeltaRj != null){
          System.out.println("DEBUG: Delta (QR) : " + Delta.getTimeStamp() + " DeltaRj(DUj) : " + DeltaRj.getTimeStamp() + " du(DUi) : " + du.getTimeStamp() );
          DeltaRjs.addElement(DeltaRj);
          DeltaRj = RPU.peekNext(j);
        }

        uv.UPRLock.unlock();
        System.out.println("ViewChange" + id + ":" + count + ":processSubQuery SnapShot UPR release lock.");
      }
      //}
      if(DeltaRjs.size() == 0){
        if(EVEDemo.MW.Main.DEBUG)
           System.out.println("ViewChage" + id + ":" + count + ":processSubQuery Stp4.5: cannot find"+ 
            " the data update from IS" + j + " which should be exists");
      }else{
        if(EVEDemo.MW.Main.DEBUG)
          System.out.println("ViewChange" + id + ":" + count + ": DeltaRjs size is: " + DeltaRjs.size());
        Enumeration enumDeltaRj = DeltaRjs.elements();
        // Here we need to handle more than one concurrent DUs.
       while(enumDeltaRj.hasMoreElements()){

          if(EVEDemo.MW.Main.DEBUG)
            System.out.println("ViewChange" + id + ":" + count + ": Loop enumDeltaRj.hasMoreElements Dead Lock Here?");

          DeltaRj = (Relation) enumDeltaRj.nextElement();        

         if(EVEDemo.MW.Main.DEBUG)
            System.out.println("ViewChange" + id + ":" + count + ":processSubQuery Stp5: Local Correction");
          // Here I check the time stampe of Query Result and Delta R j
          // if the time stamp of Query Result is Older than Delta R j, then
          // there must be a concurrency
          if(EVEDemo.MW.Main.DEBUG)
            System.out.println("DEBUG: ViewChange" + id + ":" + count + ":processSubQuery Stp5.5: DeltaRj is: " + DeltaRj);
        
          // if Delta (QR) received earlier than DelraRj (DUj)
          if(DeltaRj.OlderThan(Delta)&&du.OlderThan(DeltaRj)){
          /**********************************
          ******** Skip the local compensation. ***************
          System.out.println("DEBUG: ViewChange" + id + ":" + count + ":processSubQuery Stp5.7 DeltaRj (duj) is earlier than Delta( query result ), going to erase it");
          // clone and change the DeltaRj's name to avoid conflicts.
          Relation tmp1 = (Relation)TempView.clone();
          tmp1.setName("T1" + System.currentTimeMillis());
          Relation tmp2 = (Relation)DeltaRj.clone();
          tmp2.setName("T2" + System.currentTimeMillis());
          Relation tmp3 = tmp1.joinVirtualOn(tmp2, uv.JCAttrs[j-1]);
          if(tmp3 != null)
    		    Delta = Delta.minus(tmp3);	  
            */
            // but sleep for a while.
            try{
              sleep(10);
            }catch(Exception e){
              System.out.println("ViewChange:" + id + ":" + count + " cannot sleep.");
            }
            EVEDemo.MW.Main.incLcount();
            incClcnt();
            if(EVEDemo.MW.Main.DEBUG)
              System.out.println("ViewChange:" + id + ":" + count + " " + EVEDemo.MW.Main.lcount + " Local Compensation Canceled on PURPOSE");
          }// end if old

        }// end While enumeration
      }// end if any potential concurrent DeltaRjs.
     } 
     else{
     if(EVEDemo.MW.Main.DEBUG)
        
        System.out.println("ViewChange" + id + ":" + count + ":processSubQuery Stp6: No Concurrent Update Detected:)");
    }
    return Delta;
  }

  synchronized void incClcnt(){
    clcnt ++;
  }  

  public void run(){
    
    int j;
    
    try{
      
      if(EVEDemo.MW.Main.DEBUG)
        System.out.println("ViewChange" + id + ":" + count + ":run Stp0: Scan left part of ISs["+1 + ".." + (theIndex -1)+"]");
      
      // init the counter of concurrent DUs of this DU.
      clcnt = 0;

      String alias = DeltaView.getAlias();      

      // compute the left part of the incremental view resulting from Delta R
      for(j = theIndex-1; j>=1; j--){
        DeltaView = processSubQuery(DeltaView, j);
        if(DeltaView == null)
          break;
      }

      DeltaView.setAlias(alias);
      
      if(EVEDemo.MW.Main.DEBUG)
        System.out.println("ViewChange" + id + ":" + count + ":run Stp7: Scan right part of ISs["+(theIndex + 1 )+ ".." + n + "]");
      // compute the right part of the incremental view sesulting from Delta R
      for(j = theIndex+1; j <= n; j++){
        DeltaView = processSubQuery(DeltaView, j);
        if(DeltaView == null)
          break;
      }
  
      System.out.println("ViewChange" + id + ":" + count + " total LC " + clcnt + " concurrent data updats.");
    
      if(EVEDemo.MW.Main.DEBUG)
        System.out.println("ViewChange" + id + ":" + count + ":run Stp14: Commit Delta View");
      
      commitUpdate(DeltaView);
    }catch(Exception e){
      
      e.printStackTrace();
    }
    
    // cleanup  RPU.
    // RPU.cleanUp(PPU);

    System.out.println("ViewChange " + id + ":" + count + " : Step2  PPU: " + PPU.size() + " RPU: " + RPU.size());    

    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("ViewChange" + id + ":" + count + " :run RPU cleaned");

    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("ViewChange" + id + ":" + count + ":run() going to lock DU: " + du.getTimeStamp());
    
    {
      uv.UPRLock.lock();
      System.out.println("ViewChage" + id + ":" + count + " : run() PPU - > RPU get lock DU: " + du.getTimeStamp());
      PPU.removeElement(du);
      //if(PPU.isRelated(du))
      RPU.addElement(du);
      uv.UPRLock.unlock();
      System.out.println("ViewChange" + id + ":" + count + " : run() PPU - > RPU release lock DU: " + du.getTimeStamp());
    }
    
    System.out.println("ViewChange " + id + ":" + count + " : Step3  PPU: " + PPU.size() + " RPU: " + RPU.size());

    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("ViewChange" + id + ":" + count + " :ViewChange finished");    
    
    // call back the UpdateView
    uv.finishDU();
  }
  
  void commitUpdate(Relation deltaView){
    // System.out.println("UpdateView:run() get Update from Update Message Queue" + theUpdate);
    
    // System.out.println("Updateview:run() Add Data Warehouse(" + View.getName() + ") with Delta V:\r\n" + deltaView);
    // It changed for integration: Call DCI(DBI Class) update the View
    if(deltaView != null){
      
      // Here we do the double check.
      //DatabaseInterface dbi = new DatabaseInterface();
      //dbi.UpdateView(View,deltaView);
      //dbi.finalize();
      
      /**
	     * Assume all the signs of each tuple in the EVEview are same: either
       * '+' or '-'.
	     */
      if(EVEview != null){
        try{
          //1. Projection
          deltaView.projectOn(View);
        }catch(Exception e){
          System.out.println("ViewChange" + id + ":" + count + ":commitUpdate error Project:" + e.getMessage());
        }
        
        if(EVEDemo.MW.Main.DEBUG)
          System.out.println("ViewChange" + id + ":" + count + ":commitUpdate Debug: deltaview after projection is:\r\n" + deltaView);
        
        if(deltaView.getExtent().size() > 0){
          synchronized (EVEview){
            if(deltaView.getExtent().getTuple(0).getNumber() > 0)
              EVEview.addTuples(deltaView);
            else
              EVEview.removeTuples(deltaView);
          }
        }
      }
    }
    // Stop the data update performance timer in the mid layer.
    EVEview.stopTimer();
    
    if(EVEDemo.MW.Main.DEBUG)
      System.out.println("Updateview" + id + ":" + count + ":commitUpdate() Data Warehouse(" + View.getName() + ") Update Complete!");
    try{
      // After that Sleep for a while
      sleep(PSWEEP.step);
    }catch(InterruptedException iee){
      System.out.println("UpdateView" + id + ":" + count + ":commitUpdate() sleep Error: " + iee.getMessage());
    }
  }
}
