////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
// CreateUpdateFile.java

// this class will be called by other modules to create the Update file.
// Update file here is very important, because if there is
// no Update file the whole system will not running. So, it is very
// importand. It is just like a trigger of reaction.
// Format of this Update file:
// FileName:  DeltaR_<Index>
//    here <Index> is a integer Number assigned for each Information
//    source. <Index> should be not less than 0, because negative number
//    is not very hard to show in the FileName format.
// Context: It's the Object of Class Relation.
//    if you want to know more detial, please reference "Relation"


// Because this class will only create a Relation Object, So, here, 
// I just make a wrapper of Relation.
// This function will take a snap shot of Relation and store in 
// inside of this class.
// For each Update, It will get the new relation set from the Orcle
// Then do the Minus of Two Relation.
// After that It will Update the snape shot stores inside this class.

// This object must create after the Relation is already in the Oracle

class CreateUpdateFile implements java.io.Serializable{

  /**
  * Snap Shot to the Relation R
  */
  Relation OriginalRelation;  
  /**
  * Update of Relation R
  */
  Relation DeltaR;            
  /**
  * Information Source Index
  */
  int      Index;             

  /** this constructor creates an object for Relation with Specific RelationName.
  * That Relation must appeared in the Oracle Database
  * The second parameter: NewIndex is used to write the UpdateFileName.
  */
  public CreateUpdateFile(String RelationName, int NewIndex){
    Index = NewIndex;

    // From Oracle Database get the Relation of the Information Source
    DatabaseInterface dbi = new DatabaseInterface();
    OriginalRelation = dbi.getRelation(RelationName, Index);
    dbi.finalize();
  }


  /**
  * This method will check the Update of specific Relation by comparing the 
  * current relation with snapshot. Then, it will write the update to an 
  * external file.
  */
  public UpdateData checkUpdate(){

    // get current Relation Extent
    DatabaseInterface dbi = new DatabaseInterface();
    do{
      //System.out.println("debug: CUF:checkUpdate step1");
      DeltaR = dbi.getRelation(OriginalRelation.getName(), Index);
    }while(DeltaR == null);// unexpected error occurs here
      
    try{
      // Minus original Relation and get the Delta R
      DeltaR.minus(OriginalRelation);
    }catch(Exception e){
      System.out.println("CUF:CheckUpdate() Error: " + e.getMessage());
    }

    //System.out.println("debug: CUF:checkUpdate step2");

    try{
      if(DeltaR.getCardinality() > 0){
	// Debug, to see what is the old and new relation
	// System.out.println("debug: CUF:checkUpdate old relation is :\r\n" + dbgrel);
	// System.out.println("debug: CUF:checkUpdate new relation is :\r\n" + OriginalRelation);
	/*
	 * debug: keep the old relation
	 */
	Relation dbgrel = (Relation)OriginalRelation.clone();
	
	do{
	  // Update the snapshot of current Relation
	  OriginalRelation = dbi.getRelation(OriginalRelation.getName(), Index);
	}while(OriginalRelation == null);// unexpected error occurs here
	return new UpdateData(DeltaR, Index);
      }
      else
	return null;
    }catch(Exception e){
      System.out.println("CUF:checkUpdate() error: " + e.getMessage());
      return null;
    }finally{
      //System.out.println("debug: CUF:checkUpdate step3");
      dbi.finalize();
    }
  }
}

