////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;

import java.io.Serializable;


/**
 * This classed is only used for Andreas do Data Update Performance Evaluation.
 * It will received data updates from the middle layer of Andreas code. 
 * Then, it will put the data updates to the corresponding queue for each IS 
 * object. Then, the SWEEP will get those updates.
 */
public class DataUpdateReceiver implements java.io.Serializable{

  /**
   * Data Update Queue Array for each Information Sources
   */
  Queue[] DUQueues;
  /**
   * Use relation as index.
   */
  Relation[] Index;

  /**
   * Construct DataUpdateReceiver for an array of Relations.
   * The order of rels is same as the order of ISProcess. So, they have same
   * index number.
   * @param rels An array of Relations
   */
  public DataUpdateReceiver(Relation[] rels){
    // System.out.println("Debug: DataUpdateReceiver:init(Relation[]) enter");
    DUQueues = new Queue[rels.length];
    for(int i = 0; i < rels.length; i++){
      DUQueues[i] = new Queue();
    }
    Index = new Relation[rels.length];
    for(int i = 0; i < rels.length; i++)
      Index[i] = rels[i];
  }

    /**
     * count how many data updates received.
     */
    int dus=0;

  /**
   * Put data update into this receiver.
   * @param ISName Information Source Name, Indeed, we don't need that.
   * @param RelName Relation Name
   * @param Count Number of tuples, could be negative
   * @param Value String of the tuple, format like: "a|b|c"
   */
  public synchronized void putDataUpdate(String ISName, String RelName, int Count, String Value){

    if(EVEDemo.MW.Main.DEBUG)
    System.out.println("Debug: DataUpdateReceiver.putDataUpdate() IS: " + ISName + " Rel: " + RelName + " Count: " + Count + " Value: " + Value);

    // Find the Index number of this data updates
    for(int i = 0 ; i< Index.length; i++){

      if(Index[i].getName().equals(RelName)){
	// System.out.println("Debug: DataUpdateReceiver.putDataUpdate() IS[" + i + "] is: " + Index[i+1].getName());	
	// Clone the Relation from the index.
	Relation r = (Relation)Index[i].clone();
	// Reset the Extent of that relation.
	r.setExtent(new RelationExtent(Count, Value));

	UpdateData ud = null;
	try{
	  ud = new UpdateData(r, i+1);
	}catch(Exception e){
	  // System.out.println("DataUPdateReceiver.putDataUpdate() error: " + e.getMessage());
	  e.printStackTrace();
	}
	try{
	  // put the UpdateData in the specific queue.
	  DUQueues[i].enqueue(ud);
	    dus ++;
    if(EVEDemo.MW.Main.DEBUG)
	  System.out.println("Debug: DataUpdateReceiver.putDataUpdate() " + dus + " data updates received");
	}catch(Exception e){
	  // System.out.println("DataUpdateReceiver.putDataUpdate() error: " + e.getMessage());
	  e.printStackTrace();
	}
      }
    }
  }

  /**
   * Get data update from this receiver.
   * @param ISIndex index number of IS, start from 1.
   * @return Relation of the update.
   */
  public synchronized UpdateData getDataUpdate(int ISIndex){
    

    try{
      // Get the DataUpdate from the corresponding Queues.
      UpdateData ud = (UpdateData)DUQueues[ISIndex-1].dequeue();
      // System.out.println("DataUpdateReceiver.getDataUpdate() of IS[" + ISIndex + "] Relation: " + Index[ISIndex-1].getName());
      return ud;
    }catch(Exception e){
      //System.out.println("DataUpdateReceiver.getDataUpdate() error: " + e.getMessage());
      //e.printStackTrace();
    }

    return null;
  }

}


