////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
import java.io.*;
import java.sql.*;
import java.util.*;

public class DatabaseInterface implements java.io.Serializable{
  
  static java.sql.Connection con; // SQL Connection
  //  static String url = "jdbc:odbc:EVE"; // odbc(oracle url)
  static String url = "jdbc:oracle:thin:@shiba.wpi.edu:1521:ORCL";
  //  static String driver = "sun.jdbc.odbc.JdbcOdbcDriver"; // jdbc driver
  static String driver = "oracle.jdbc.driver.OracleDriver";
  static String logname = "xinz"; // oracle login name
  static String logpass = "xinz"; // oracle password

  /**
  * The maximum number of statements I can created at the same time.
  */
  static int cursor_open = 40;

  /**
  * The currend cursor-opened.
  */
  static int cur_cursors = 0;

  /**
  * openCursor for this connection.
  */
  synchronized Statement openCursor(){
    if(cur_cursors < cursor_open){
      try{
        Statement stmt=con.createStatement();
        cur_cursors ++;
        if(EVEDemo.MW.Main.DEBUG)
          System.out.println("DBI: openCursor() current opened " + cur_cursors + " cursors.");
        return stmt;
      }catch(SQLException sqle){
        System.out.println("DBI: openCursor() SQLException: " );
        sqle.printStackTrace();
      }
    }else{
      try{
        wait();
        return openCursor();
      }catch(InterruptedException ie){
        System.out.println("DBI: openCursor() InterruptedException: ");
        ie.printStackTrace();

      }
    }
    return null;
  }

  /**
  * closeCursor for this statement.
  */
  synchronized void closeCursor(Statement stmt){
    if(stmt == null){
      System.out.println("DBI:closeCursor() Statement is null!!!");
      return;
    }
    try{
      stmt.close();
      cur_cursors = cur_cursors > 0 ? cur_cursors - 1 : 0;
      if(EVEDemo.MW.Main.DEBUG)
        System.out.println("DBI: closeCursor() currend opened " + cur_cursors + " cursors.");
      notify();
    }catch(SQLException sqle){
      System.out.println("DBI: closeCursor() SQLException: " );
      sqle.printStackTrace();
    }
  }


  public void finalize(){
    /* I do nothing here.  
    try{
      con.close();
    }catch(Exception e){
      System.out.println("DBI:finalize " + e.getMessage());
    }
    */
  }
  
  public DatabaseInterface(){

    if(con != null){
      //System.out.println("DBI:DBI() DBI has already initialized, no need to reset");
      return;
    }

    try{
      
      // load jdbc driver
      Class.forName(driver);
      
      // login to oracle
      con = DriverManager.getConnection(url, logname, logpass);
      
    }catch (SQLException e){
      
      System.out.println("DBI:DBI() SQL Error: " + e.getMessage());
    }catch (ClassNotFoundException e){
      System.out.println("DBI:DBI() Class Not Found: " + e.getMessage());
    }
  }
  
  /**
  * Add Sign Field SWEEPSIGN. Now I need this method can support more than one threads call it. So, I need to 
  * distinguish the statements each time some threads called.
  */
  public void createTable(Relation theRelation, String sign, Statement stmt){
    
    String query = null;
    try{
      
      query = theRelation.createTableSQL(sign);
      
      //if(EVEDemo.MW.Main.DEBUG)
      //System.out.println("DBI::createTable() query is: " + query);

      // create temporary table
      stmt.executeUpdate(query);
      
      // insert tuples into the table
      theRelation.initIterator();

      while(theRelation.moreTuple()){

        StringBuffer Query = new StringBuffer();

        int querynumber = theRelation.insertTupleSQL(Query, sign);

        if(EVEDemo.MW.Main.DEBUG)
          System.out.println("DBI::createTable() query[" + querynumber + "]: " + Query.toString());

        for(int i = 0; i< Math.abs(querynumber); i++){

          try{
            stmt.executeUpdate(Query.toString());
          }catch(SQLException sqle){
            System.out.println("DBI:createTable(): SQL error:" + sqle.getMessage());
            System.out.println("DBI:createTable(): Query is:\r\n" + Query);
            System.out.println("DBI:createTable(): Relation is:\r\n" + theRelation);
          }
        }      
      }
    }catch(Exception e){
      
      System.out.println("DBI::createTable(): Exception: " + e.getMessage() + "\r\n at Query: " + query);
      
    }
  }
  
  /**
  * Add Sign Field SWEEPSIGN. Now I need this method can support more than one threads call it. So, I need to 
  * distinguish the statements each time some threads called.
  */
  public void createTable(Relation theRelation, String sign){

    Statement stmt = null;

    try{

      stmt = openCursor();      

      createTable(theRelation, sign, stmt);

    }catch(Exception e){
      
      System.out.println("DatabaseInterface::createtable: " + e.getMessage());
      
    }finally{
      
        closeCursor(stmt);
    }
  }

  public void createTable(Relation theRelation){
    
    Statement stmt = null;
    try{
      
      stmt = openCursor();
      
      String query = theRelation.createTableSQL();
      
      //System.out.println("DBI::createTable() query is: " + query);

      // create temporary table
      stmt.executeUpdate(query);
      
      // insert tuples into the table
      theRelation.initIterator();
      while(theRelation.moreTuple()){
        StringBuffer Query = new StringBuffer();
        int querynumber = theRelation.insertTupleSQL(Query);

        //System.out.println("DBI::createTable() query[" + querynumber + "]: " + Query.toString());

        for(int i = 0; i< Math.abs(querynumber); i++){

          try{
            stmt.executeUpdate(Query.toString());
          }catch(SQLException sqle){
            System.out.println("DBI:createTable(): SQL error:" + sqle.getMessage());
            System.out.println("DBI:createTable(): Query is:\r\n" + Query);
          }
        }
      }
      
    }catch(Exception e){
      
      System.out.println("DatabaseInterface::createtable: " + e.getMessage());
      
    }finally{
      
        closeCursor(stmt);
    }
  }

  /**
  * Join operation on the remote IS.
  */
  public Relation join(Relation theView, Relation theRelation){
    
    Statement stmt = null;
    try{
      stmt = openCursor();
      
      // get the view's name and relation's name
      String ViewName = theView.getName();
      String RelationName = theRelation.getName();
      
      // because I cannot find the Natural Join between two relations, So, I will find the common attributes between two relations.
      // here we assume they only have one common attribute
      String Common = theView.getCommon(theRelation);

      // System.out.println("DBI::join() : Common Attr of R(" + ViewName + ") & R(" + RelationName + ") is : " + Common);
      // generate query
      String query = "select * from " + ViewName + " r1, " + RelationName +  " r2  where r1." + Common + " = r2." + Common;
      if(EVEDemo.MW.Main.DEBUG)
      System.out.println("DBI::join() : SQL : " + query);

      // create sql to join the two relations and get the result back
      ResultSet rs = stmt.executeQuery(query);
            if(EVEDemo.MW.Main.DEBUG)
      System.out.println("DBI::join() : SQL executed success!");
      // return the join result
      try{

        // Now the Relation will have two same name Column, I need to do delete one column from it.
        Relation r = new Relation(rs, ViewName + RelationName, "SWEEPSIGN");
        // delete one of the common column.
        // debug info
        // System.out.println("DBI:join() r before delete col(" + Common + ")");
        // System.out.println(r);

        r.delCol(Common);

        // debug info
        // System.out.println("DBI:join() r after delete col(" + Common + ")");
        // System.out.println(r);
        
        return r;

      }catch(Exception e){
        System.out.println("DBI:join() error new Relation from Join ResultSet " + e.getMessage());
      }
    }catch (SQLException e){
      
      System.out.println("DatabaseInterface::join() SQL Error: " + e.getMessage());
      
    }finally{
      
     closeCursor(stmt);
        
      
    }
    
    return null;
  }

  /**
  * Join operation on the remote IS. It will join on different named
  * attribute. Att1 and att2.
  * This joinOn function will add the relation name to the attribute names 
  * of  the local relations in the Result.
  */
  public Relation joinOn(Relation theView, Relation theRelation, String att1, String att2){
    
    Statement stmt = null;
    String query = null;
    try{
      stmt = openCursor();

      // clone and change the DeltaRj's name to avoid conflicts.
      Relation tmp1 = (Relation)theView.clone();
      tmp1.setName("TPQ" + System.currentTimeMillis() + nextUnique());
      
      createTable(tmp1, "SWEEPSIGN", stmt);
      
      // get the view's name and relation's name
      String ViewName = tmp1.getName();
      String RelationName = theRelation.getName();

      // generate query
      query = "select * from " + ViewName + " r1, " + RelationName +  " r2  where r1." + att1 + " = r2." + att2;
      //if(EVEDemo.MW.Main.DEBUG)
      //System.out.println("DBI::joinOn() : SQL : " + query);
      //if(EVEDemo.MW.Main.DEBUG)
      //System.out.println("DBI::joinOn() : tmp1 is: " + tmp1);

      // create sql to join the two relations and get the result back
      ResultSet rs = stmt.executeQuery(query);
      
      // System.out.println("DBI::joinOn() : SQL executed success!");

      try{

        // Now the Relation will have two same name Column, I need to do delete one column from it.
        Relation r = new Relation(rs, ViewName + RelationName, theRelation, "SWEEPSIGN");
      	r.setAlias(RelationName);
      	/*
      	 * In the Join Result the local attribute is at the end of the schema.
      	 * So, from the end of the schema we add RelationName on it and then
      	 * we have consistent name. :-)
      	 */
      	RelationSchema rsch = r.getSchema();
      	int end = rsch.size();// size of join result schema
      	int rsSize = theRelation.getSchema().size();// size of local relation schema.
      	for(int i = end-1; i > (end-rsSize-1); i--){
      	  if(rsch.getAttrName(i).indexOf(RelationName)== -1)
      	    rsch.changeAttrName(RelationName + "_" + rsch.getAttrName(i),i);
      	}
        // delete one of the common column.
        if(att2.equals(att1)){
          // debug info
          // System.out.println("DBI:JoinOn() r before delete col(" + att2 + ")");
          // System.out.println(r);

          r.delCol(att2);

          // debug info
          // System.out.println("DBI:JoinOn() r after delete col(" + att2 + ")");
          // System.out.println(r);
        }

        dropTable(tmp1, stmt);
        
        return r;

      }catch(Exception e){
        System.out.println("DBI:joinOn() error new Relation from Join ResultSet " + e.getMessage());
      }
    }catch (SQLException e){
      
      System.out.println("DBI::joinOn() SQL Error: " + e.getMessage() + " \r\n when query is: " + query);
      e.printStackTrace();

    }finally{
      
      closeCursor(stmt);
      
    }
    
    return null;
  }

  /**
  * Join on the Local Datawarehouse.
  */
  public Relation joinLocal(Relation theView, Relation theRelation){
    
    Statement stmt = null;
    try{
      stmt = openCursor();
      
      // get the view's name and relation's name
      String ViewName = theView.getName();
      String RelationName = theRelation.getName();
      
      // because I cannot find the Natural Join between two relations, So, I will find the common attributes between two relations.
      // here we assume they only have one common attribute
      String Common = theView.getCommon(theRelation);

      //System.out.println("DBI::joinLocal() : Common Attr of R(" + ViewName + ") & R(" + RelationName + ") is : " + Common);
      // generate query
      String query = "select * from " + ViewName + " r1, " + RelationName +  " r2  where r1." + Common + " = r2." + Common;
      if(EVEDemo.MW.Main.DEBUG)
      System.out.println("DBI::joinLocal() : SQL : " + query);

      // create sql to join the two relations and get the result back
      ResultSet rs = stmt.executeQuery(query);
            if(EVEDemo.MW.Main.DEBUG)
      System.out.println("DBI::joinLocal() : SQL executed success!");
      // return the join result
      try{

        // Now the Relation will have two same name Column, I need to do delete one column from it.
        Relation r = new Relation(rs, ViewName + RelationName, "SWEEPSIGN1", "SWEEPSIGN2");
        // delete one of the common column.
        // debug info
        //System.out.println("DBI:joinLocal() r before delete col(" + Common + ")");
        //System.out.println(r);

        r.delCol(Common);

        // debug info
        //System.out.println("DBI:joinLocal() r after delete col(" + Common + ")");
        //System.out.println(r);
        
        return r;

      }catch(Exception e){
        System.out.println("DBI:joinLocal() error new Relation from Join ResultSet " + e.getMessage());
      }
    }catch (SQLException e){
      
      System.out.println("DatabaseInterface::joinLocal() SQL Error: " + e.getMessage());
      
    }finally{
              
      closeCursor(stmt);
    }
    
    return null;
  }

  /**
  * Join on the Local Datawarehouse on the specified Join attributes.
  */
  public Relation joinLocalOn(Relation theView, Relation theRelation, String Attr1, String Attr2){

      String func = "DBI::joinLocalOn():";
    
    Statement stmt = null;
    try{
      stmt = openCursor();
      
      // get the view's name and relation's name
      String ViewName = theView.getName();
      String RelationName = theRelation.getName();
      
      //System.out.println(func + " Common Attr of R(" + ViewName + ") & R(" + RelationName + ") is : " + Common);
      // generate query
      String query = "select * from " + ViewName + " r1, " + RelationName +  " r2  where r1." + Attr1 + " = r2." + Attr2;
      if(EVEDemo.MW.Main.DEBUG)
      System.out.println(func + " SQL : " + query);

      // create sql to join the two relations and get the result back
      ResultSet rs = stmt.executeQuery(query);
            if(EVEDemo.MW.Main.DEBUG)
      System.out.println(func + " SQL executed success!");
      // return the join result
      try{

	  // Erase the Sign Field.
        Relation r = new Relation(rs, ViewName + RelationName, "SWEEPSIGN1", "SWEEPSIGN2");
        // Now the Relation will have two same name Column, I need to do delete one column from it.
        // delete one of the common column.
        // debug info
        //System.out.println(func + " r before delete col(" + Common + ")");
        //System.out.println(r);

        // delete one of the common column.
        if(Attr2.equals(Attr1)){
          // debug info
          // System.out.println(func + " r before delete col(" + att2 + ")");
          // System.out.println(r);

          r.delCol(Attr2);

          // debug info
          // System.out.println(func + " r after delete col(" + att2 + ")");
          // System.out.println(r);
        }

        //r.delCol(Common);

        // debug info
        //System.out.println(func + " r after delete col(" + Common + ")");
        //System.out.println(r);
        
        return r;

      }catch(Exception e){
        System.out.println(func + " error new Relation from Join ResultSet " + e.getMessage());
      }
    }catch (SQLException e){
      
      System.out.println(func + " SQL Error: " + e.getMessage());
      
    }finally{
      
        closeCursor(stmt);
        
      
    }
    
    return null;
  }
  
  public void dropTable(Relation theRelation){
    
    Statement stmt = null;
    try{
      
      stmt = openCursor();
      
      String query = "drop table " + theRelation.getName();
      
      stmt.executeUpdate(query);
      
    }catch(Exception e){
      
      System.out.println("DatabaseInterface::droptable: " + e.getMessage());
    }finally{
      
        closeCursor(stmt);
        
    }
    
  }

  public void dropTable(Relation theRelation, Statement stmt){
    
    try{
      
      String query = "drop table " + theRelation.getName();
      
      stmt.executeUpdate(query);
      
    }catch(Exception e){
      
      System.out.println("DatabaseInterface::droptable: " + e.getMessage());
    }    
  }

  // This function will return a Relation Object from the Oracle Database by sending
  // Query to Oracle.
  // The second Parameter will be ignored in this version, because All the Relations
  // come from one Oracle Database.
  // In the future work, we will select corresponsding Database or the Index
  public Relation getRelation(String RelationName, int Index){

    Statement stmt = null;
    String query = null;
    try{

      stmt = openCursor();

      query = "select * from " + RelationName;

      ResultSet rs = stmt.executeQuery(query);

      try{
	Relation r = new Relation(rs, RelationName);
	// Debug
        //System.out.println("Debug: dbi:getRelation the relation is: \r\n" + r);
        return r;
      }catch(Exception e){
	System.out.println("Debug: DBI:getRelation() query is: " + query);
        System.out.println("DBI:getRelation() Relation:" + RelationName + " Error:" + e.getMessage());
	//System.out.println("Debug: DBI:getRelation() ResultSet is: \r\n" + rs);
      }

    }catch(SQLException e){

      System.out.println("DBI:getRelation() SQL Error: of Relation: " + RelationName + " " + e.getMessage());
      System.out.println("debug:DBI:getRelation() the SQL is:" + query);

    }finally{

          closeCursor(stmt);
    }

    System.out.println("DBI:getRelation() Relation is null!");
    return null;
  }
  
  // This function is just a wrapper of executeUpdate() function of SQL Function
  public void executeUpdate(String Query) throws Exception{

    Statement stmt = openCursor();

    try{
      // System.out.println("DBI:executeUpdate() Query is: " + Query);
      stmt.executeUpdate(Query);
    }catch(SQLException e){
      System.out.println("DBI:executeUpdate() SQL Error: " + e.getMessage());
    }
  
    closeCursor(stmt);

  }

  // This function is just a wrapper of executeQuery() function of SQL function
  public Relation executeQuery(String Query) throws Exception{
    Statement stmt = openCursor();

    ResultSet rs = stmt.executeQuery(Query);

    Relation result = new Relation(rs, "OnlyExtent");

    closeCursor(stmt);

    return result;
  }

  /**
  * This Function will update the View stored on the Oracle by using the Update.
  * 1. It will project the Update to View Schema.<p>
  * 2. Convert projected Update to Oracle.<p>
  * <ul>
  *    if the number of Tuple is positive, we insert this tuple in the Oracle.<br>
  *    if the number of Tuple is negative, we delete it from the Oracle.<br>
  *</ul>
  *
  *<p>
  * Problem: How to delete tuple from oracle for only couple of them, e.g.<p>
  *    Original: (5,6)[5], Delete: (5,6)[3], Result: (5,6)[2].
  * Solution:
  *    delete from <table> where <conditions> and rownum =1;
  */
  void UpdateView(ViewDefinition View, Relation theUpdate){

    try{
      //1. Projection
      theUpdate.projectOn(View);
    }catch(Exception e){
      System.out.println("DBI:UpdateView() error Project:" + e.getMessage());
    }

    //2. Do Update
    Statement stmt = null;
    try{

      stmt = openCursor();

      theUpdate.setName(View.getName());

      //System.out.println("Debug: DBI: UpdateView() theUpdate is : " + theUpdate);

      theUpdate.initIterator();

      while(theUpdate.moreTuple()){

        StringBuffer query = new StringBuffer();
        int num= theUpdate.updateTupleSQL(query);
	//System.out.println("Debug: DBI:UpdateView() query is:" + query.toString());

        for(int i=0; i< Math.abs(num); i++)
          stmt.executeUpdate(query.toString());
      }

    }catch(SQLException e){

      System.out.println("DBI:UpdateView() SQL Error:" + e.getMessage());

    }finally{

          closeCursor(stmt);
    }
  }
  static int unique = 0;
  
  synchronized int nextUnique(){
    unique ++;
    return unique;
  }
}
