////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
//
//
// JoinCondition
//
//

/**
* This function will represent the formula Operand1 Op Operand2.
* Here Operands are Attribute, and Op could be '='. Here I remain
* use Operator class is for future that we can add '>', '<',
* '>=', '<=' operators.
*/
public class JoinCondition implements java.io.Serializable {


  Attribute Oprand1;
  Attribute Oprand2;
  Operator  Op;

  public Attribute getOpd1(){
    return Oprand1;
  }

  public Attribute getOpd2(){
    return Oprand2;
  }

  public JoinCondition(Attribute Attr1, Operator newOp, Attribute Attr2) throws Exception{

    if(Attr1 == null || Attr2 == null || newOp == null)
      throw new Exception("JoinCondition:JoinCondition() Input Para has null value.");
    Oprand1 = Attr1;
    Op = newOp;
    Oprand2 = Attr2;
  }

  /**
  * test if the JC is fully dependes on the Relation List.
  * It used for checking if a new JC is defined on the Relation List of Join List of
  * a PSJ View. If used for preventing some inconsistency between Relation List and
  * a Join Condition List.
  */
  public boolean dependsOn(RelationList RList) throws Exception{

    if(RList == null)
      throw new ParaNullException("JoinCondition","dependsOn()","RList");

    // This will find exactly same reference
    if(!RList.find(Oprand1.getRelation()))
      return false;

    if(!RList.find(Oprand2.getRelation()))
      return false;

    return true;
  }

  /**
  * Test if a Relation is involved in this Join Condition.
  */
  public boolean refers(Relation Ref) throws Exception{

    if(Ref == null)
      throw new ParaNullException("JoinCondition", "refers()", "Ref");

    // Compare EXACTLY the reference
    if(Ref == Oprand1.getRelation() || Ref == Oprand2.getRelation())
      return true;
    else
      return false;
  }

  /**
  * Generate a part of SQL.
  * Convert Join Condition to : R.A = S.B
  */
  public String getSQL(){
    return Oprand1.getSQL() + Op.getSQL() + Oprand2.getSQL();
  }
}