////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
import java.util.Vector;
/**
  *
  *
  * JoinConditionList
  *
  *
  */
public class JoinConditionList implements java.io.Serializable
{
  Vector JCList;

  public Vector getJCList(){
    return JCList;
  }

  public JoinConditionList(){
    JCList = new Vector();
  }

  /**
  * This function will drop all the JCs of the specific Relation
  */
  public void drop(Relation ref) throws Exception{
    if(ref == null)
      throw new Exception("JoinConditionList:drop() Input Para \"ref\" is null");

    int size = JCList.size();

    for(int i=0; i< size; i++){
      JoinCondition JC = (JoinCondition)JCList.elementAt(i);
      // if this JC reference the Relation ref
      if(JC.refers(ref))
        JCList.removeElement(JC);
    }
  }

  /**
  * This function will simple drop the specific JC
  */
  public void drop(JoinCondition JC) throws Exception{
    if(JC == null)
      throw new Exception("JoinConditionList:drop() Input Para \"ref\" is null");

    JCList.removeElement(JC);
  }

  /**
  * this function will add one JC to the JC List. 
  * The JC in the JCList must be unique
  */
  public void add(JoinCondition JC) throws Exception{
    if(JC == null)
      throw new Exception("JoinConditionList:add() Input Para \"JC\" is null");

    if(find(JC))
      throw new Exception("JoinConditionList:add() JC is already in this JCList");

    JCList.addElement(JC);
  }

  /**
  * This function will only check the EXACT JC whether in JCList
  */
  public boolean find(JoinCondition JC) throws Exception{
    return JCList.contains(JC);
  }

  /**
  * This function will Generate a Part of SQL back.
  * This part of SQL will join in SQL of View Definition:
  * E.g. ... WHERE B.A = C.D and D.E = F.G, it will returns B.A = C.D and D.E = F.G
  */
  public String getSQL(){
    int size = JCList.size();
    String query = new String();
    for(int i = 0; i< size-1; i++){
      JoinCondition JC = (JoinCondition)JCList.elementAt(i);
      query += JC.getSQL() + " and ";
    }
    if(size > 0)
      query += ((JoinCondition)JCList.elementAt(size-1)).getSQL();

    return query;
  }

  /**
  * Get Size of JoinConditionList.
  */
  public int size(){
    return JCList.size();
  }
}


