////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
import java.util.Vector;

/**
  *
  * JoinList
  *
  *
  *
  *Join List will hold all the Relations involved in this view.
  *All the join conditions (Global conditions) of Join
  *Here, the Join conditions must related to the Relaltion in
  *the join list, otherwise, we cannot insert that conditions in
  *our join list.
  */
public class JoinList extends Vector implements java.io.Serializable
{
  RelationList RList;
  JoinConditionList JCList;

  public JoinConditionList getJCList(){
    return JCList;
  }
  
  public JoinList(){

    RList = new RelationList();
    JCList = new JoinConditionList();
  }

  // Add One Relation References to This Join List
  public void addRelation(Relation Ref) throws Exception{
    if(Ref == null)
      throw new Exception("JoinList:addRelaiton() Input Para \"Ref\" is null");
    // Call RelationList to add the Relation Reference.
    // add  function will check the Ref exist.
    RList.add(Ref);
  }

  // Delete the Relation from the Relation List
  private void dropRelation(Relation Ref) throws Exception{

    if(Ref == null)
      throw new Exception("JoinList:dropRelaiton() Input Para \"Ref\" is null");

    // This will erase the Join Condition remove from the 
    // Join Condition List first
    JCList.drop(Ref);

    // Then remove the Relation from the JCList
    RList.drop(Ref);
  }

  // Add One Join Condition to JCList
  public void addJC(JoinCondition JC) throws Exception{

    if(JC == null)
      throw new Exception("JoinList:addJC Input Para \"JC\" is null");
    // check the JC first, make sure JC is based on the Relations
    // in the Relation List
    if(!JC.dependsOn(RList))
      throw new Exception("JoinList:addJC the JC is not satisfied by RelationList");

    // Add the JC to the JCList, the add() will check the Consistency of JC
    // It will throw Exception is the JC is not consistency with original one.
    JCList.add(JC);  
  }

  // Delete One JC from JCList
  private void dropJC(JoinCondition JC) throws Exception{

    if(JC == null)
      throw new Exception("JoinList:dropJC Input Para \"JC\" is null");

    // simple drop this JC from the JC List
    JCList.drop(JC);
  }

  /**
  * Get SQL for this JoinList.
  * It will return a relation list seperated by ','.
  * It directly called method: <b>RelationList:getSQL()</b>.
  */
  public String getSQL(){
    return RList.getSQL();
  }

  /**
  * Get SQL format of JoinConditions back.
  * E.g. ... WHERE R.A = S.B, S.C = T.D ... , will return R.A = S.B, S.C = T.D
  * It directly called method: <b>JoinConditionList:getSQL()</b>
  */
  public String getConditionSQL(){
    return JCList.getSQL();
  }


}

