////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
//
//
// MessageQueue
// 
// Here is a assumption for this Queue:
// We assume only one Update can be conflict with Query.
// So, in my code, I didn't handle MERGE Updates.

public class MessageQueue extends Queue implements java.io.Serializable{

  public MessageQueue(){

    super();
  }

  ///////////////////////////////////
  // this object has more function like
  // testing if one Index exists
  public boolean exists(int theIndex){

    boolean found = false;

    if(theIndex < 0)
      return found;

    Element e = getFirst();

    if(e != null)
      do{
        UpdateData ud = (UpdateData)e;

        if(ud != null)
          if(theIndex == ud.getSource())
            return true;
        // debug info
        //System.out.println("MessageQueue:exists() here! e is:" + e);
      }while((e = getNext())!=null);

    return false;
  }

  /**
   * Get the data update from a specific information sources.
   * peek and peeknext should be called within one loop.
   */
  public Relation peek(int theIndex) throws Exception{

    if(theIndex < 0)
      throw new Exception("Index less than Zero");

    Element e = getFirst();

    if(e != null)
      do{
        UpdateData ud = (UpdateData)e;

        if(ud != null)
          if(theIndex == ud.getSource()){
            // reset the temp stamp.
            Relation r = ud.getDeltaRelation();
            r.setTimeStamp(ud.getTimeStamp());
            return r;
          }
      }while((e = getNext())!=null);

    //throw new Exception("Cannot peek specifice Update in MessageQueue");
    return null;
  }

    /**
     * Get the next data update from the specific information sources.
     * It should be used with the peek() function.
     */
  public Relation peekNext(int theIndex) throws Exception{

    if(theIndex < 0)
      throw new Exception("Index less than Zero");

    Element e = getNext();

    if(e != null)
      do{
        UpdateData ud = (UpdateData)e;
	
        if(ud != null)
          if(theIndex == ud.getSource()){
            // reset the temp stamp.
            Relation r = ud.getDeltaRelation();
            r.setTimeStamp(ud.getTimeStamp());
            return r;
          }
      }while((e = getNext())!=null);
    
    return null;
  }

  /**
  * Get one Message from the MessageQueue and remove it.
  * If the Queue is empty, it will wait for a while and retry again.
  */
  public synchronized Element remove(){

    Element elm;
    do{
      try{
        elm = Dequeue();
        return elm;
      }catch(Exception e){
        //System.out.println("MessageQueue::remove " + e.getMessage());
        try{
          Thread.sleep(PSWEEP.PSWEEP.step);
        }catch(InterruptedException iee){
          System.out.println("MessageQueue:remove Sleep error:" + iee.getMessage());
        }
      }
    }while(true);
  }

  /**
  * Peek the next element in the queue for future use.
  * e.g. to lock it.
  */
  public Element peekNext(){
    if(!isEmpty())
      return index[head];

    return null;
  }
}



