////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
/**
* Here MultiQueue will simulate the Multi-Channel FIFO channel between DW and ISs.
* There are three kinds of Queues in this Multi-Channel.
* 1. QueryQueues: Send Query from DW to IS, will be shared by DW. Belongs to each IS
* 2. QueryResultQueues: Send Query Resutl from IS to DW, will be shared by DW. Belongs to each IS
* 3. UpdateDataQueue: Send Update from IS to DW, will be shared by each IS. Belongs to each DW
* Now, In my implementation, I NEW all the queues in this MultiQueue system, and distribute the Queues
*   to corresponding ISs. <p>
* In the future, is better to put all the Queues inside corresponding DW or ISs. Then, create
* Another two methods, to insert and delete the Queues reference to the MultiQueue.
*/

public  class MultiQueue implements java.io.Serializable{
  
  Queue[] QueryQueues; // queue storing all queries from DW to IS
  Queue[] QueryResultQueues;// queue storing all query results from IS to DW
  Queue UpdateDataQueue;// queue storing all Update from IS to DW
  int QueueNumber; // record how many queue pair(Query, QueryResult) stores in MultiQueue 
  
  public MultiQueue(int ISNumber){
    
    QueueNumber = ISNumber;
    QueryQueues = new Queue[ISNumber];
    QueryResultQueues = new Queue[ISNumber];
    
    int i;
    for(i = 0; i < ISNumber; i++){
      
      QueryQueues[i] = new Queue();
      QueryResultQueues[i] = new Queue();
    }
    
    UpdateDataQueue = new Queue();
    
  }
  
  public boolean SendUpdate(UpdateData theUpdate) throws Exception{
    
    ////////////////////////////////////////////
    // If I want to make sure, everything in MultiQueue must be FIFO
    // I must assign a unique time stamp on each element.
    // But, now, I just assume everything is fine. I skip the time stamp

    try{
      
      // Enqueue the Update to UpdateDataQueue
      UpdateDataQueue.Enqueue(theUpdate);
      
      //System.out.println("Multiqueue:SendUpdate() " + theUpdate);
      
      return true;
      
    }catch(Exception e){
      
      System.out.println("MultiQueue::SendUpdate: " + e.getMessage());
      
      throw new Exception(" Cannot send Update, maybe the queue is full ");
      
    }
    
  }
  
  ///////////////////////////////////////////////////////
  // theIndex will range from 1 to n
  /**
  * Send Query from DW to ISs.
  */
  public boolean sendQuery(Relation theDeltaR, int theIndex) throws Exception{
    
    if(theIndex < 1 || theIndex > QueueNumber)
      throw new Exception("MultiQueue::sendQuery: theIndex is out of bound");

    try{
      //System.out.println("MultiQueue:sendQuery() Index is: " + theIndex + " Delta R is:\r\n" + theDeltaR);
      QueryQueues[theIndex-1].Enqueue(theDeltaR);
      //System.out.println("MultiQueue:sendQuery() DeltaR sended from DW to IS:\r\n" + theDeltaR);
    }catch(QueueFullException qfe){
      System.out.println("MultiQueue:sendQuery() Enqueue Error! " + qfe.getMessage());
      return false;
    }
    return true;
    
  }

  ///////////////////////////////////////////////////////
  // theIndex will range from 1 to n
  /**
  * Send QueryResult from ISs to DW.
  */
  public boolean sendQueryResult(Relation theDeltaR, int theIndex) throws Exception{
    
    if(theIndex < 1 || theIndex > QueueNumber)
      throw new Exception("MultiQueue::sendQueryResult: theIndex is out of bound");

    try{
      //System.out.println("MultiQueue:sendQueryResult() Index is: " + theIndex + " Delta R is:\r\n" + theDeltaR);
      QueryResultQueues[theIndex-1].Enqueue(theDeltaR);
      //System.out.println("MultiQueue:sendQueryResult() DeltaR sended from DW to IS:\r\n" + theDeltaR);
    }catch(QueueFullException qfe){
      System.out.println("MultiQueue:sendQueryResult() Enqueue Error! " + qfe.getMessage());
      return false;
    }
    return true;
    
  }
  
  /**
  * theIndex ranges from 1 to n.
  * receive the queryresult from information source (theIndex) to DW.
  * Wait until got some reply.
  */
  public Relation receiveQueryResult(int theIndex) throws Exception{
    
    if(theIndex < 1 || theIndex > QueueNumber)
      throw new Exception("MultiQueue::receiveQueryResult: theIndex is out of bound");

    while(true){
      try{
        //System.out.println("MultiQueue:receiveQueryResult() Index:"+theIndex);

        Relation theResult = (Relation)QueryResultQueues[theIndex-1].Dequeue();

        //System.out.println("MultiQueue:receiveQueryResult() QuerResult Received from IS:" + theResult);
        return theResult;
      }
      catch(Exception e){
        Thread.sleep(PSWEEP.PSWEEP.step);
        //System.out.println("MultiQueue:receiveQueryResult() QueryResult Empty : " + e.getMessage());
        
      }
    }
  }
  
  /**
  * theIndex ranges from 1 to n.
  * receive the query from DW to ISs.
  * Wait until got some reply.
  */
  public Relation receiveQuery(int theIndex) throws Exception{
    
    if(theIndex < 1 || theIndex > QueueNumber)
      throw new Exception("MultiQueue::receiveQuery: theIndex is out of bound");

    while(true){
      try{
        //System.out.println("MultiQueue:receiveQuery() Index:"+theIndex);

        Relation theResult = (Relation)QueryQueues[theIndex-1].Dequeue();

        //System.out.println("MultiQueue:receiveQuery() Query Received from DW:" + theResult);
        return theResult;
      }
      catch(Exception e){
        Thread.sleep(PSWEEP.PSWEEP.step);
        //System.out.println("MultiQueue:receiveQuery() Query Empty : " + e.getMessage());
        
      }
    }
  }

  // receive the UpdateData from IS to DW
  public UpdateData receive() throws Exception{
    
    UpdateData theUpdate = (UpdateData)UpdateDataQueue.Dequeue();
    
    return theUpdate;
  }
  
}
