////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;

import java.util.Hashtable;

///////////////////////////////////////////////
// Class ProcessQuery
// is a thread will be running on the IS site
// It will pick one Query from DW by MultiQueue
// then send it to database to get Join Result
// then Send the Result back to DW by MultiQueue
// Assume each IS only has one Relations

public class ProcessQuery extends Thread implements java.io.Serializable{
  
  MultiQueue publicFIFOChannel; // global object for SWEEP
  Relation currentRelation; // Relation of this IS, for our flight system, I treat the local view as relation.
  
  int MyIndex; // Index number for the current IS
  
  
               /**
               * Because there will be more than one Join Condition connects to one
               * Relation, So, we use Hashtable stores all the Join Conditions, and ordered
               * by the Relation.
               * This is the Hashtable of Attribute Pairs. They are indexed on the Relation
               * Name.
  */
  public Hashtable JCAttrs;
  
  public ProcessQuery(MultiQueue FIFOChannel, Relation theRelation, int theIndex){
    
    // keep the FIFO channel
    publicFIFOChannel = FIFOChannel;
    currentRelation = theRelation;
    MyIndex = theIndex;
    this.start();
  }
  
  public void run(){
    
    try{
      
      while(true){
        
        // receive Delta V from Data Warehouse 
        Relation DeltaView= publicFIFOChannel.receiveQuery(MyIndex);
        
        if(EVEDemo.MW.Main.DEBUG)
          System.out.println("ProcessQuery:run() IS[" + MyIndex + "] Received Query: join R(" + DeltaView.getName() +") & R(" + currentRelation.getName() + ")");
        // Need to get the Linked Name of this Relation.
        String linkedName = DeltaView.getAlias();
        
        // Depends on the LinkedName of this DeltaView, select the remote
        // Attribute and the local Attribute.
        JCAttrPair jap = (JCAttrPair)JCAttrs.get(linkedName);
        
        if(EVEDemo.MW.Main.DEBUG)
          System.out.println("ProcessQuery:run() linkedName is: " + linkedName);
        
        EVEDemo.MW.Main.incRcount();
        
        // Delta V = computeJoin(Delta V , R)
        if(jap != null){
          if(EVEDemo.MW.Main.DEBUG)
            System.out.println("\r\n\r\nDeltaView: " + DeltaView + "\r\ncurrentRelation: " + currentRelation + "\r\njap.rat: " + jap.rat + " jap.lat: " + jap.lat + "\r\n\r\n");

          DeltaView = ComputeJoinOn(DeltaView, currentRelation, jap.rat, jap.lat);
        }else{
          System.out.println("\r\n\r\n WARNING!!!!!!!!!!!! \r\n\r\n Join Attributes of DeltaView:" + DeltaView.getName() + 
            "("+ DeltaView.getAlias() + ")" +  " and currentRelation:" + currentRelation.getName() + " is null");
        }
        
        if(EVEDemo.MW.Main.DEBUG)
          System.out.println("ProcessQuery:run() IS[" + MyIndex + "] Send Query Result back\r\n'" + DeltaView +"'");
        // send Delta V to Data Warehouse
        publicFIFOChannel.sendQueryResult(DeltaView, MyIndex);
        
        //check queue per 0.5 seconds
        sleep(PSWEEP.PSWEEP.step);
      }
      
    }catch(Exception e){
      
      System.out.println("ProcessQuery::run:" + e.getMessage());
      e.printStackTrace();
    }
    
  }
  
  public Relation ComputeJoinOn(Relation theView, Relation theRelation, String att1, String att2)
  {
    /////////////////////////////////////////////////////////////////
    // this function will interact with Relation, use JDBC communicate with
    // Database, and do the Query stuff.
    // this function will create a DCI object.
    DatabaseInterface dbi = new DatabaseInterface();
    
    // create a temporary relation at the oracle
    //dbi.createTable(theView, "SWEEPSIGN");
    
    //System.out.println("Debug: ProcessQuery.ComputeJoinOn press any key to continue!");
    
    /*
    try{
    System.in.read();
    }catch(Exception e){
    } 
    */
    
    // send the sql, do join on this view with the relation, and get back the 
    // result, theRelation only store the Relation name.
    Relation result = dbi.joinOn(theView, theRelation, att1, att2);
    
    //System.out.println("Debug: ProcessQuery.ComputeJoinOn press any key to continue!");
    
    /*try{
    System.in.read();
    }catch(Exception e){
    } 
    */
    
    //if(result != null){
      // drop the temporary view
      //dbi.dropTable(theView);
      //dbi.finalize();
    //}else { // error on join, keep the temporary table.
      //System.out.println("ProcessQuery:ComputeJoin() Error on Join keep temporary table: " + theView.getName());
    //}
    
  if(EVEDemo.MW.Main.DEBUG)
    // debug info
    System.out.println("ProcessQuery:ComputeJoin() Query Result is:\r\n" + result);
    
    return result;
  }

}


