////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
/**
 * This class implement a rotate queue
 * I think this class need to consider "Syncronize"
 */
public class Queue implements java.io.Serializable{

  /**
   * default queue size is 10000
   */
  int size; 
  /**
   * index of elements
   */
  Element index[]; 
  /**
   * head of queue
   */
  int head;
  /**
   * tail of queue
   */
  int tail; 

  /**
   * For the queue, not only the method to be synchronized that only one
   * thread can access the method at one time, we also need all the methods
   * need to be synchronized, that means only one modification methods can 
   * access the queue at one time.
   */
  PSWEEP.Lock wl;

  /**
   * Constructor of Queue with default length 10000.
   */
  public Queue(){

    wl = new PSWEEP.Lock();

    size = 10000;// default queue size is 10000

    // new the queue with default length
    index = new Element[size];
    
    // initialize the queue with such size
    Initialize();

  }

  /**
   * Append one UpdateData as end of the Queue. Same as the Enqueue method.
   * @param theUpdata a data update.
   * @return length of the queue
   */
  synchronized public int Append(UpdateData theUpdata) throws Exception
    {
      try{
	
	return Enqueue(theUpdata);

      }catch (Exception e){
	
	System.out.println("Queue::Append " + e.getMessage());
      }
      
      throw new Exception("Queue::Append: Cannot Append"); 
    }

  /**
   * Construct a Queue with specific Length.
   * @param newSize length of the queue.
   */
//      public Queue(int newSize) throws Exception{

//      if ( newSize < 0)
//        throw new Exception("Queue::Queue(int): newSize lessthan 0");

//      // new the queue with specific length
//      size = newSize;
//      index = new Element[size];

//      // initialize the queue with such size
//      Initialize();

//    }

  private synchronized void Initialize(){

    // initialize the queue is empty
    head = tail = 0;

    // here, I assume all Elements in Index will initialized to null by system.
  }


  /**
   * Same as Dequeue function. Correct Naming format of function.
   */
  synchronized public Element dequeue() throws Exception{
    return Dequeue();
  }

  /**
   * this Method will dequeue an Element from the queue
   * if the queue is emtpy, the return Element will be null
   */
  synchronized public Element Dequeue() throws Exception{
    
    Element currentElement;

    if(!isEmpty()){
      
      wl.lock();

      // get Element pointed by head
      currentElement = index[head];

      // increase head point to next Element
      head = head + 1 >= size ? 0 : head + 1;

      wl.unlock();

      return currentElement;

    }else{
      wl.unlock();
      throw new Exception("Queue::Dequeue: queue is empty");
    }
  }

  /**
   * Just for standards the Naming rules. Functions should start with lower case
   * letters.
   */
  synchronized public int enqueue(Element NewElement) throws QueueFullException{
    return Enqueue(NewElement);
  }
  
  synchronized public int Enqueue(Element NewElement) throws QueueFullException{
    
    if(!isFull()){

      wl.lock();

      index[tail] = NewElement;

      // increase tail point to next empty space
      tail = tail + 1 >= size ? 0 : tail + 1;

      //System.out.println("Queue:Enqueue() Here!(Only for Debug)");

      wl.unlock();
      // return current length of queue
      return length();
    }else{
      wl.unlock();
      throw new QueueFullException();
    }
  }

  public boolean isEmpty(){
    
    return length() == 0;
  }

  public boolean isFull(){

    return length() == size;
  }

  /**
   * Return the current length of the queue.
   */
  public int length(){

    wl.lock();

    int result = tail - head;

    wl.unlock();

    if(result >= 0)
      return result;
    else
      return result + size;

  }

  /////////////////////////////////////////
  // following two funtions are iterating functions
  // we use getFirst to get the first Element
  // we use getNext to get the next Element
  int current; // variable to store the current iterator

  protected synchronized Element getFirst(){

    wl.lock();

    current = head;

    if(size != 0){
      current = increase(current);
      wl.unlock();
      return index[head];
    }else{
      wl.unlock();
      return null;
    }
  }

  ////////////////////////////////////////////
  // if the queue still has more element
  // then return the element
  // otherwise return null
  protected synchronized Element getNext(){

    wl.lock();

    if(current != tail && size != 0){

      Element e =index[current];
      current = increase(current);
      wl.unlock();
      return e;

    }else{
      wl.unlock();
      return null;
    }

  }

  // this private method will increase number within size of queue
  private synchronized int increase(int Num){
    return  Num + 1 >= size ? 0 : Num + 1;
  }
}
