////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
import java.sql.*;
import java.io.*;
import java.util.StringTokenizer;
import java.util.Hashtable;
//
//
// Relation
//
//
/**
* This is a class represent a Relational Database Table.
* We can create it from:
*   1. null
*   2. ResultSet from JDBC
*   3. File created by write() method
*
* We can following operations on it:
*   1. Join
*   2. Plus
*   3. Minus
*   4. Projection
*   5. delColumn
*
* Tuple Format: (values)[number]
*   e.g. (5,6)[3] means we have three duplicate (5,6) tuples.
*/
public class Relation extends StampedElement implements java.io.Serializable,Cloneable
{
  String theName;
  /**
   * Alias of this Relation. It used for Join Condition to keep track of
   * Where this Relation comes from.
   */
  String alias;
  RelationSchema theSchema;
  RelationExtent theExtent;

  /**
   * debug variable to trace how many relation object created.
   */
  static int objnum = 0;

  public void finalize(){
    //System.out.println("debug: Relation "+ theName + " of " + objnum + "object  is dead!");
    objnum --;
  }
  
  public Object clone(){
    Relation Rclone = new Relation(theName);
    Rclone.setAlias(alias);

    RelationSchema RSclone = (RelationSchema)theSchema.clone();
    Rclone.setSchema(RSclone);

    RelationExtent REclone = (RelationExtent)theExtent.clone();
    Rclone.setExtent(REclone);

    return Rclone;
  }

  /**
  * This function set the new extent of this relation.
  * It should check the consistency of current schema and current extent,
  * but I haven't implement it.
  */
  public void setExtent(RelationExtent newExtent){
    theExtent = newExtent;
  }

  private Relation(){
    
    super();
    theName = "default";
    alias = new String();
    theSchema = new RelationSchema();
    theExtent = new RelationExtent();
    objnum ++;
  }

  /**
   * this is a constructor of Relation from result set.
   */
  public Relation(ResultSet rs, String newName) throws Exception{
    
    this();
    
    // copy newName to Relation Name
    theName = newName.substring(0);
    
    // if we have tuples in rs, 
    // we need get schema from rs.
    theSchema = new RelationSchema(rs);
    
    // we need get Extent from rs
    try{
      
      theExtent = new RelationExtent(rs, theSchema.size());
      
    }catch(Exception e){
      
      System.out.println("Relation::Relation: " + e.getMessage());
      
      throw new Exception("Relation cannot Created");
      
    }
    
  }
  
  // this is a constructor of Relation from result set.
  /**
  * This constructor will create a Relation from Resultset.
  * Important part here, is that this constructor will search for the "SWEEPSIGN" field.
  * if it is 'p', means positive; if it is 'n', means negative.
  */
  public Relation(ResultSet rs, String newName, String sign) throws Exception{
    
    this();
    
    // copy newName to Relation Name
    theName = newName.substring(0);
    
    // if we have tuples in rs, 
    // we need get schema from rs.
    theSchema = new RelationSchema(rs,sign);
    
    // we need get Extent from rs
    try{
      
      theExtent = new RelationExtent(rs, theSchema.size(), sign);
      
    }catch(Exception e){
      
      System.out.println("Relation::Relation: " + e.getMessage());
      
      throw new Exception("Relation cannot Created");
      
    }
    
  }

  // this is a constructor of Relation from result set.
  /**
  * This constructor will create a Relation from Resultset.
  * Important part here, is that this constructor will search for the "SWEEPSIGN" field.
  * if it is 'p', means positive; if it is 'n', means negative.
  */
  public Relation(ResultSet rs, String newName, Relation localrel, String sign) throws Exception{
    
    this();
    
    // copy newName to Relation Name
    theName = newName.substring(0);
    
    // if we have tuples in rs, 
    // we need get schema from rs.
    theSchema = new RelationSchema(rs, localrel, sign);
    
    // we need get Extent from rs
    try{
      
      theExtent = new RelationExtent(rs, theSchema.size(), sign);
      
    }catch(Exception e){
      
      System.out.println("Relation::Relation: " + e.getMessage());
      
      throw new Exception("Relation cannot Created");
      
    }
    
  }
  /**
  * This constructor will create a Relation from Resultset.
  * Important part here, is that this constructor will search for the "SWEEPSIGN1" field
  * and "SWEEPSIGN2" field.
  * if they are same, means positive; otherwise, means negative.
  */
  public Relation(ResultSet rs, String newName, String sign1, String sign2) throws Exception{

    // First , create a big relation from it.
    this(rs, newName);

    // Then, recalculate the sign for tuples depends on to sign fields
    String tempRS[] = {sign1, sign2};
    RelationSchema tempSchema = new RelationSchema(tempRS);
    int flags[] = theSchema.getFlags(tempSchema); 
    theExtent.calSign(flags);

    // Finally, drop the two columns.
    delCol(sign1);
    delCol(sign2);
    
  }

  public Relation(String newName){
    
    this();
    
    theName = newName.substring(0);
    
  }
  
  // this is a constructor of Relation from a Relation Update File
  public Relation(FileInputStream fin){
    try{
      
      //Create DataInputStream using FileInputStream
      DataInputStream din = new DataInputStream(fin);
      
      //Read Relation Name from din(file)
      theName = din.readUTF();
      
      //Read Relation Schema
      theSchema = new RelationSchema(din);
      
      //Read Relation Extent
      theExtent = new RelationExtent(din);
    }catch(Exception e){
      
      System.out.println("Relation::Relation: " + e.getMessage());
      
    }
  }
  
  public String getAlias(){
    return alias;
  }

  public void setAlias(String newname){
    alias = newname;
  }
  
  public void setSchema(RelationSchema newSchema){
    
    // the newschema will destory the original extent
    theExtent = new RelationExtent();
    
    // store the new Schema
    theSchema = newSchema;
    
  }
  
  // insert one tuple to this relation
  public void insertTuple(Tuple newTuple) throws Exception{
    
    // check the tuple must match the schema
    if(newTuple.getNum() != theSchema.size()){
      //throw new Exception("Relation::insertTuple() The Extent and Tuple are not match");
      System.out.println("Relation::insertTuple() The Extent and Tuple are not match");
      return;
    }
    
    // insert this tuple in the Extent
    theExtent.insertTuple(newTuple);
    
  }
  
  
  // this funciton will generate a create table sql for such relation
  public String createTableSQL(){
    
    StringBuffer Query= new StringBuffer("create table " + theName + " (");
    
    Query.append(theSchema.createSQL());
    
    Query.append(")");
    
    return Query.toString();
    
  }
  
  // this funciton will generate a create table sql for such relation
  /**
  * This function will create a table with sign field.
  * e.g. "col1, col2" will be "SWEEPSIGN, col1, col2"
  */
  public String createTableSQL(String sign){
    
    StringBuffer Query= new StringBuffer("create table " + theName + " (");
    
    Query.append(theSchema.createSQL(sign));
    
    Query.append(")");
    
    return Query.toString();
    
  }

  // this function will initIterator of Relation Extent
  /**
  * Cooperate with moreTuple and InsertTupleSQL, TupleSQL.
  */
  public void initIterator(){
    
    theExtent.initIterator();
    
  }
  
  // this function will test if there is more tuples need to iterate
  /**
  * Cooperate with initIterator and InsertTupleSQL, TupleSQL.
  */
  public boolean moreTuple(){
    
    return theExtent.moreTuple();
    
  }
  
  // this function will return the SQL of insert one turple
  /**
  * Cooperate with moreTuple and initIterator.
  * This function is only used for Create Temporary Table in Oracle.
  * So, this function will ASSUME all the QUERYNUMBER should be POSITIVE.
  * if the QUERYNUMBER is negative, it will genereate "delete" query.
  */
  public int updateTupleSQL(StringBuffer Query){
    
    // number of tuples have same value
    int querynumber; 
    StringBuffer tempQuery = new StringBuffer();

    // append next Extent in SQL format
    querynumber = theExtent.nextExtentSQL(tempQuery);
    
    if(querynumber > 0){
      Query.append("insert into " + theName + " values (");
      Query.append(tempQuery);
      Query.append(")");
    }
    else{
      // delete query:
      // "delete from <relation> where <conditions> and rownum = 1"
      Query.append("delete from " + theName + " where ");
      // if the querynumber is negative, the tempQuery is like "value1,value2,value3"
      try{
        Query.append(makeConditions(tempQuery));
      }catch(Exception e){
        System.out.println("Relation:insertTupleSQL makeCondition Exception:" + e.getMessage());
      }
      Query.append(" and rownum = 1");
    }
    
    return querynumber;
  }

  // this function will return the SQL of insert one turple
  /**
  * Cooperate with moreTuple and initIterator.
  * This function is only used for Create Temporary Table in Oracle.
  * So, this function will ASSUME all the QUERYNUMBER should be POSITIVE.
  * if the QUERYNUMBER is negative, it will genereate "delete" query.
  */
  public int insertTupleSQL(StringBuffer Query){
    
    // number of tuples have same value
    int querynumber; 
    StringBuffer tempQuery = new StringBuffer();

    // append next Extent in SQL format
    querynumber = theExtent.nextExtentSQL(tempQuery);
    
    Query.append("insert into " + theName + " values (");
    Query.append(tempQuery);
    Query.append(")");
    
    return querynumber;
  }

  // this function will return the SQL of insert one turple
  /**
  * Cooperate with moreTuple and initIterator.
  * This function is only used for Create Temporary Table in Oracle.
  * So, this function will ASSUME all the QUERYNUMBER should be POSITIVE.
  * if the QUERYNUMBER is negative, it will genereate "delete" query.
  */
  public int insertTupleSQL(StringBuffer Query, String sign){
    
    // number of tuples have same value
    int querynumber; 
    StringBuffer tempQuery = new StringBuffer();

    // append next Extent in SQL format
    querynumber = theExtent.nextExtentSQL(tempQuery, sign);
    
    Query.append("insert into " + theName + " values (");
    Query.append(tempQuery);
    Query.append(")");
    
    return querynumber;
  }

  /**
  * This function will generate the conditions from Relation Schema and the Tuple
  * This function should be only used by method: insertTupleSQL().
  * The input stringbuffer should fit in such format:
  *   value1,value2,value3
  */

  private String makeConditions(StringBuffer tupleValues) throws Exception{
    if (tupleValues == null)
      throw new ParaNullException("Relation", "makeConditions", "tupleValues");

    StringTokenizer st = new StringTokenizer(tupleValues.toString(), ",");

    int size = theSchema.size();

    if (theSchema.size() != st.countTokens())
      throw new Exception("Column Number does not match Schema Number");
    
    StringBuffer str = new StringBuffer();

    for(int i=0; i < size-1 && st.hasMoreTokens(); i++){
      str.append(theSchema.getColName(i) + "=");
      str.append(st.nextToken() + " and ");
    }

    if(size > 0 && st.hasMoreTokens()){
      str.append(theSchema.getColName(size-1) + "=");
      str.append(st.nextToken());
    }

    return str.toString();

  }
  public String getName(){
    
    return theName;
    
  }
  
  // this will get common attribute(only one) between current relation and new relation
  public String getCommon(Relation theRelation){
    
    if(theRelation == null)
      return null;
    
    // return the common attributes defined in the schema
    return theSchema.getCommon(theRelation.theSchema);
    
  }
  
  public RelationExtent getExtent(){
    
    return theExtent;
  }
  
  
  // this method write relation to a file
  public void write(FileOutputStream fout){
    
    try{
      
      //Create DataOutputStream using FileOutputStream
      DataOutputStream dout = new DataOutputStream(fout);
      
      //write Relation Name to file
      dout.writeUTF(theName);
      
      //write Relation Schema
      theSchema.write(dout);
      
      //write Relation Extent
      theExtent.write(dout);
    }catch(Exception e){
      
      System.out.println("Relation::write: " + e.getMessage());
      
    }
  }
  
  // this method is Minus
  // minus will sub one relation from another relation
  public Relation minus(Relation sndRelation) throws Exception{
    
    // if two Relations Schema are not match
    if(!theSchema.equals(sndRelation.theSchema))
      new Exception("Relation::minus: The schema of two relation are not match");

    // Debug Info
    //System.out.println("Relation:minus() This relation is:");
    //System.out.println(this);

    // Debug Info
    //System.out.println("Relation:minus() sndRelations is:");
    //System.out.println(sndRelation);

    // minus the two Relations Extent
    theExtent = theExtent.minus(sndRelation.theExtent);
    
    // Debug Info
    //System.out.println("Relation:munus() result is:");
    //System.out.println(this);

    // Debug if the minus work with same relation
    if(theExtent.size()>0){
      //System.out.println("debug:relation:minus: first relation is : " + theName);
      //System.out.println("debug:relation:minus: second relation is: " + sndRelation.getName());
    }

    // return the substituted relation
    return this;
  }
  
  // This method will add two Relation
  public Relation plus(Relation sndRelation){
    
    try{
      // if two Relations Schema are not match
      if(!theSchema.equals(sndRelation.theSchema))
        new Exception("Relation::plus: The schema of two relation are not match");
      
      // plus the two Relations Extent
      theExtent = theExtent.plus(sndRelation.theExtent);
      
      // return the substituted relation
      return this;
    }catch(Exception e){
      System.out.println("Relation::plus failed, because exception: " + e.getMessage());
      return this;
    }
  }

  /**
  *This method will join two relations with a specific join condition.
  * and return the result.
  * It is assumed the second Relation is already in the database.
  */
/*  
  public Relation joinOn(Relation sndRelation, JoinCondition jc){
    /////////////////////////////////////////////////////////////////
    // this function will interact with Relation, use JDBC communicate with
    // Database, and do the Query stuff.
    // this function will create a DCI object.
    DatabaseInterface dbi = new DatabaseInterface();
    
    // create a temporary relation at the oracle
    dbi.createTable(this);
    
    // send the sql, do join on this view with second relation, and return the 
    // result.
    Relation result = dbi.joinOn(this, sndRelation, jc);
    
    // drop the temporary view
    dbi.dropTable(this);
    
    return result;
    
  }
*/
  /**
  *This method will join two relations and return the result.
  * It is assumed the second Relation is already in the database.
  */
  public Relation join(Relation sndRelation){
    /////////////////////////////////////////////////////////////////
    // this function will interact with Relation, use JDBC communicate with
    // Database, and do the Query stuff.
    // this function will create a DCI object.
    DatabaseInterface dbi = new DatabaseInterface();
    
    // create a temporary relation at the oracle
    dbi.createTable(this);
    
    // send the sql, do join on this view with the relation, and get back the 
    // result, theRelation only store the Relation name.
    Relation result = dbi.join(this, sndRelation);
    
    // drop the temporary view
    dbi.dropTable(this);
    dbi.finalize();
    
    return result;
    
  }
  
  /**
  *This method will join two relations and return the result.
  * It is assumed both relation are not on the database.
  */
  public Relation joinVirtual(Relation sndRelation){
    /////////////////////////////////////////////////////////////////
    // this function will interact with Relation, use JDBC communicate with
    // Database, and do the Query stuff.
    // this function will create a DBI object.
    DatabaseInterface dbi = new DatabaseInterface();
    
    // create a temporary relation at the oracle
    dbi.createTable(this, "SWEEPSIGN1");

    // Debug Info
    //System.out.println("Relation:joinVirtual() R(" + this.getName() + ") is:");
    //System.out.println(this);          

    dbi.createTable(sndRelation, "SWEEPSIGN2");
    
    // Debug Info
    //System.out.println("Relation:joinVirtual() R(" + sndRelation.getName() + ") is:");
    //System.out.println(sndRelation);

    // send the sql, do join on this view with the relation, and get back the 
    // result, theRelation only store the Relation name.
    Relation result = dbi.joinLocal(this, sndRelation);
    
    // Debug Info
    //System.out.println("Relation:joinVirtual() R(" + result.getName() + ") is:");
    //System.out.println(result);

    // drop the temporary view
    dbi.dropTable(this);
    dbi.dropTable(sndRelation);
    dbi.finalize();
    
    return result;
  }

  /**
  *This method will join two relations and return the result.
  * It is assumed both relation are not on the database.
  */
  public Relation joinVirtualOn(Relation sndRelation, Hashtable JCAttrs){
    /////////////////////////////////////////////////////////////////
    // this function will interact with Relation, use JDBC communicate with
    // Database, and do the Query stuff.
    // this function will create a DBI object.
    DatabaseInterface dbi = new DatabaseInterface();
    
    // create a temporary relation at the oracle
    dbi.createTable(this, "SWEEPSIGN1");

    // Debug Info
    //System.out.println("Relation:joinVirtual() R(" + this.getName() + ") is:");
    //System.out.println(this);          

    dbi.createTable(sndRelation, "SWEEPSIGN2");
    
    // Debug Info
    //System.out.println("Relation:joinVirtual() R(" + sndRelation.getName() + ") is:");
    //System.out.println(sndRelation);

  	String linkedName = sndRelation.getAlias();

	  // Depends on the LinkedName of this DeltaView, select the remote
  	// Attribute and the local Attribute.
	  JCAttrPair jap = (JCAttrPair)JCAttrs.get(linkedName);

  	System.out.println("ProcessQuery:run() linkedName is: " + linkedName);

    Relation result = null;

    if(jap != null){
      // send the sql, do join on this view with the relation, and get back the 
      // result, theRelation only store the Relation name.
      result = dbi.joinLocalOn(this, sndRelation, jap.rat, jap.lat);
    
      // Debug Info
      //System.out.println("Relation:joinVirtual() R(" + result.getName() + ") is:");
      //System.out.println(result);
    }

    // drop the temporary view
    dbi.dropTable(this);
    dbi.dropTable(sndRelation);
    dbi.finalize();
    
    return result;
  }

  // Get the number of DISTINCT tuples in the current Relation
  public int getCardinality(){
    return theExtent.size();
  }
  
  /**
  * Convert Relation to String
  */
  public String toString(){
    
    StringBuffer Output = new StringBuffer();
    
    Output.append("Name: " + theName + " Alias: " + alias + "\r\n");
    
    Output.append(theSchema.toString() + "\r\n");
    
    Output.append(theExtent.toString());
    
    return Output.toString();
    
  }

  /**
  * Give this Relation a new name
  */
  public void rename(String newName){
    theName = newName;
  }

  /**
  * Do Project Operation.
  * Project will project this relation on specific relation's schema.
  * Here the new Relation's schema must be a subset of this relation's schema.
  */
  public void projectOn(Relation Rel) throws Exception{
    if(Rel == null)
      throw new NullPointerException("Relation:ProjectOn Rel is null");
      if(EVEDemo.MW.Main.DEBUG)
    System.out.println("Relation: " + Rel.getName() + " Schema is :" + Rel.getSchema().toString());
      if(EVEDemo.MW.Main.DEBUG)
    System.out.println("Relation: " + theName + " Schema is:" + theSchema.toString());

    if(!Rel.getSchema().isSubsetOf(theSchema))
      throw new Exception("Schema of Relation 'rel' is not a subset of this Relation's schema");

    // Here the flag will show which column will be projected.
    // if this schema is projected, then, the flag is one, otherwise, it is 0.
    int flags[] = theSchema.getFlags(Rel.getSchema());

    // Update the schema, depends on the flags.
    // The reason of why I get the flag first and then do the project of schema is:
    // I need the flag later in my Extent Projection.
    theSchema.doProject(flags);
    
    theExtent.doProject(flags);
  }
   
  /**
  * get Schema of this Relation.
  */
  public RelationSchema getSchema(){
    return theSchema;
  }

  /**
  * Set New name for this relation.
  */
  public void setName(String newName){
    theName = newName;
  }

  /**
  * Delete one column from the Relation
  */
  public void delCol(String colName){
    /**
    * Here, we use the Projection function to implement projections.
    * e.g. if flags[i]]  = 1, then that column will be kept. So, I just set
    * the flag of specific colName is "zero".
    * For Example: we have "col1, col2, col3", we need to delete "col2", then 
    * the Flags are "1,0,1", so, we based on this flags do projection, we will
    * delete "col2".
    */
    int flags[] = theSchema.getFlags(colName);
    try
    {
      theSchema.doProject(flags);
      theExtent.doProject(flags);
    }catch(Exception e){
      System.out.println("Relation:delCol() doProject Exception :" + e.getMessage());
    }
  }
}


