////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
import java.io.*;
import java.util.*;
import java.sql.*;

////////////////////////////////////
// class RelationExtent will store the
// extent of a specific Relation

public class RelationExtent implements java.io.Serializable, Cloneable{
  
  Vector tuples;
  
  /**
   * this enumaretion used for iterator
   */
  Enumeration theIterator;
  
  public RelationExtent(){
    
    tuples = new Vector();
     
  }

  /**
   * Create RelationExtent from String and the Count. Only for integration
   * with Andreas on Data Update Performance Evaluation.
   * @param Count Count of the tuples
   * @param Value Value of tuple, format: "A|B|C"
   */
  public RelationExtent(int Count, String Value){
    this();
    Tuple t = new Tuple(Count, Value);
    tuples.addElement(t);
  }
  

  /**
   * Create RelationExtent from A DataInputStream(file)
   */
  public RelationExtent(DataInputStream din){
    
    this();
    
    try{
      
      tuples = new Vector();
      
      // first get how many tuples we have
      int tNums = din.readInt();
      
      // then get tuples
      for(int i = 0; i < tNums; i++){
        
        addTuple(new Tuple(din));
        
      }
      
    }catch(Exception e){
      
      System.out.println("RelaitonExtent::RelationExtent: " + e.getMessage());
      
    }      
  }
  
  // Create RelationExtent from SQL Result Set
  public RelationExtent(ResultSet rs, int colNum) throws Exception{
    
    this();
    
    int i=0;
    
    try{
      while(rs.next()){
        
        
        // get next tuple from ResultSet
        addTuple(new Tuple(rs, colNum));
       	
        // Increase tuple number
        i ++ ;
      }    
    }catch(Exception e){
      
      System.out.println("RelationExtent::RelationExtent " + e.getMessage());
      
      throw new Exception("RelationExtent create Failed");
    }
  }
  
  // Create RelationExtent from SQL Result Set
  /**
  * should skip sign. and set the Sign bit.
  */
  public RelationExtent(ResultSet rs, int colNum, String sign) throws Exception{
    
    this();
    
    int i=0;
    
    try{
      while(rs.next()){
        
        
        // get next tuple from ResultSet
        Tuple t = new Tuple(rs, colNum, sign);

        addTuple(t);

        // Increase tuple number
        i ++ ;
      }    
    }catch(Exception e){
      
      System.out.println("RelationExtent::RelationExtent " + e.getMessage());
      
      throw new Exception("RelationExtent create Failed");
    }
  }

  public void initIterator(){
    
    theIterator = tuples.elements();
  }

  public boolean moreTuple(){
    
    return theIterator.hasMoreElements();
    
  }
  
  // this method will continue iterate element from the iterator
  /**
  * This function will generate a part of SQL.
  * It will generate a part of "insert" SQL.
  */
  public int nextExtentSQL(StringBuffer Query){
    
    Tuple t = (Tuple)theIterator.nextElement();

    Query.append(t.SQL());
    // return number of querys for such tuple
    return t.getNumber();
  }
  
  // this method will continue iterate element from the iterator
  /**
  * This function will generate a part of SQL.
  * It will generate a part of "insert" SQL.
  * Add SWEEPSIGN field.
  */
  public int nextExtentSQL(StringBuffer Query, String sign){
    
    Tuple t = (Tuple)theIterator.nextElement();

    Query.append(t.SQL(sign));
    // return number of querys for such tuple
    return t.getNumber();
  }

  // minus two relation extent, they have same schema, the schema checking is done 
  // in Relation class.
  // E.g. +(3,4) minus +(3,4) == 0
  //      +(3,4) minus -(3,4) == +(3,4)[2]
  //      -(3,4) minus -(3,4) == 0
  //      -(3,4) minus +(3,4) == -(3,4)[2]
  //      +(3,4) minus +(5,6) == +(3,4), -(5,6)
  //      +(3,4) minus -(5,6) == +(3,4), +(5,6)
  //      -(3,4) minus -(5,6) == -(3,4), +(5,6)
  //      -(3,4) minus +(5,6) == -(3,4), -(5,6)
  // Minus will reverse the sign of second Extent, and then do the plus
  // Here I only need the Parameter sndExtent,
  // it cannot be edited by this function, 
  // I cannot find such kind of function, Maybe there no such kind of syntax
  // in JAVA
  // KEY IDEA:
  //    1. make copy of second Extent
  //    2. reverse the sign of this extent
  //    3. call plus function
  public RelationExtent minus(RelationExtent sndExtent) throws Exception{
    
    RelationExtent tmpExtent;

    try{
      // clone the sndExtent first
      tmpExtent = (RelationExtent)sndExtent.clone();
      
      // reverse the sign of tuple in this Extent
      tmpExtent.reverse();
      
      // call plus() method
      plus(tmpExtent);
      
      return this;
      
    }catch(CloneNotSupportedException e){
      
      System.out.println("RelationExtent::minus " + e.getMessage());
      
      return null;
    }
  }
  
  // plus two relation extent, they have same schema
  // In general, it will put all the elements together, except opposite pairs
  // e.g. +(3,4) plus -(3,4) = 0
  //      -(3,4) plus +(3,4) = 0
  public RelationExtent plus(RelationExtent sndExtent) throws Exception{
    
    RelationExtent tmpExtent;
    
    try{
      // if they are same RelationExtent
      if (sndExtent == this){
        // clone the sndExtent first
        tmpExtent = (RelationExtent)sndExtent.clone();
      }else{
        
        // directly use it
        tmpExtent = sndExtent;
        
      }
      
      int size = tmpExtent.size();
      
      // for each tuple in RelationExtent, insert to this Extent
      for(int i = 0; i < size; i++){
	Tuple t= tmpExtent.getTuple(i);
	if(t != null)
	  addTuple(t);
      }
      
      // purge the reverse pairs like: +(3,4) and -(3,4)
      DeleteOppositeTuples();
      
      return this;
    }catch(CloneNotSupportedException e){
      
      System.out.println("ReltionExtent::plus " + e.getMessage());
      return null;
    }
  }
  
  // return the specifice tuple
  public Tuple getTuple(int i){

    if(tuples.size() == 0)// No tuples
      return null;
    else
      return (Tuple)tuples.elementAt(i);
  }

  // return the size of tuples.
  public int size(){
     return tuples.size();
  }

  // this private method will reverse the sign of tuples in this extent
  private void reverse(){
      // for each tuple in sndExtent
      for(int i = 0; i < tuples.size(); i++){
        
        // get tuple from tempExtent
        Tuple t = (Tuple)tuples.elementAt(i);
        
        // reverse the sign of tuple (+ --> -, - --> +)
        t.reverse();
      }
  }

  // delete opposite tuples like: +(3,4) and -(3,4)
  private void DeleteOppositeTuples(){
    
    // first It's better to sort the tuples, because the positions in java doesn't matter.
    // We can try the recursive QuickSort to do it.

  }
  
  // write Relation Extent to file
  public void write(DataOutputStream dout){
    
    try{
      // get number of tuples in such extent
      int tNums = tuples.size();
      
      // write the number to file
      dout.writeInt(tNums);
      
      // write the tuples to file
      for(int i = 0; i < tNums; i ++){
        
        // get each tuple
        Tuple t = (Tuple)tuples.elementAt(i);
        
        // write the tuple to file
        t.write(dout);
      }
    }catch(Exception e){
      
      System.out.println("RelationExtent::write: " + e.getMessage());
      
    }
  }
  
  public void insertTuple(Tuple newTuple) throws Exception{
    
    // Here I don't need to check the newTuple, because the Relation has checked it
    // And here, I don't check the Integrate Constraints(IC) such as:Primary Key, 
    // Foreign Key. Because, I didn't store such Information.
    
    addTuple(newTuple);
    
  }


  // This function will first search for this tuple
  // if we can find this tuple, then we just add the number field of this two tuples
  // if the number field of this tuple is 0, we delete this tuple from the tuples
  public void addTuple(Tuple newTuple) throws Exception{

    //1. find this tuple in the tuples
    // use scanning, because this is only a demo, the data set will not be very large
    int size = size();
    Tuple t = new Tuple(); // a temporary tuple 
    int i; // for loop variable

    for(i = 0; i< size; i++){
      t = (Tuple)tuples.elementAt(i);
      if(newTuple.equals(t))
        break;
    }


    //2. if we found the tuple
    if(i < size){
      // compute the new number
      int number = t.getNumber() + newTuple.getNumber();
      if (number == 0)
        // delete this tuple from this extent
        tuples.removeElement(t);
      else
        // set the new number of the tuple
        t.setNumber(number);
    }else{// add this new tuple to this extent
      tuples.addElement(newTuple);
    }
  }

    /**
     * Find the corresponding tuple in this relationextent without compare the counter
     * @param Tuple the finding tuple
     * @param the matched tuple, otherwise null
     */
    public Tuple find(Tuple newTuple) throws Exception{
	
	//1. find this tuple in the tuples
	// use scanning, because this is only a demo, the data set will not be very large
	int size = size();
	Tuple t = new Tuple(); // a temporary tuple 
	int i; // for loop variable
	
	for(i = 0; i< size; i++){
	    t = (Tuple)tuples.elementAt(i);
	    if(newTuple.equals(t))
		break;
	}
	
	
	//2. if we found the tuple
	if(i < size){
	    return t;
	}else
	    return null;
    }

    /**
     * clone this RelationExtent
     */
  public Object clone(){
    RelationExtent clone = new RelationExtent();

    try{
      for(int i = 0; i< size(); i++){
	Tuple t = getTuple(i);
	if(t != null)
	  clone.addTuple((Tuple)t.clone());
      }
    }catch(Exception e){
      System.out.println("RelationExtent::clone() " + e.getMessage());
    }
    return clone;
  }

  /**
  * Output the RelationExtent to String.
  */
  public String toString(){
    StringBuffer Output = new StringBuffer();
    int number = tuples.size();

    Tuple t;

    for(int i=0; i< number -1 ; i++){
      t = (Tuple)tuples.elementAt(i);
      Output.append(t.toString() + "\r\n");
    }
    
    if(number > 0){
      t = (Tuple)tuples.elementAt(number -1);
      Output.append(t.toString() + "\r\n");
    }


    return Output.toString();
  }

  /**
  * Do the projection on the extends depends on the flags.
  * e.g. have tuple (1,2,3,4,5), flags(0,3,2,0,1), then the result is (5,2,3).
  */
  public void doProject(int flags[]) throws Exception{
    int size = tuples.size();

    for(int i=0; i<size; i++)
      ((Tuple)tuples.elementAt(i)).doProject(flags);
  }

  /**
  * Recalculate the sign for each tuple, depends on the signs.
  */
  public void calSign(int flags[]) throws NullPointerException{
    if(flags == null)
      throw new NullPointerException("RelatoinExtent:calSign() null in parameter");

    int size = size();
    for(int i = 0; i < size; i++){
      Tuple t = getTuple(i);
      
      if(t != null)
	try{
	  t.calSign(flags);
	}catch(Exception e){
	  System.out.println("RelationExtent:calSign() " + e.getMessage());
	}
    }

  }
}
