////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
import java.util.Vector;

/**
  *
  *
  *RelationList
  *
  *
  *
  *The Relation here must be unique.
  *P.S. If the Relations with different Alias, they are different
*/
public class RelationList implements java.io.Serializable
{
  Vector RList;

  public RelationList(){

    RList = new Vector();

  }

  /** 
  *Add One Relation in the Relation list.
  */
  public void add(Relation R) throws Exception{
    
    if(R == null)
      throw new Exception("RelationList:add() Input Para \"R\" is null");
    // Check if the Relation is exist in the List
    if(find(R))
      throw new Exception("RelationList:add() R(" + R.getName() + "," + R.getAlias() + ") is already in Relation List");
    
    // Add the Relation in the Vector
    RList.addElement(R);

  }

  /**
  *Drop One Relation in the Relation List.
  * This function will only drop the EXACTLY Relation from the Relation List
  * Not by name(, alias). So, it should be another drop(String RName).
  */
  public void drop(Relation R) throws Exception{
    
    if(R == null)
      throw new Exception("RelationList:drop() Input Para\"R\" is null");

    // Check if the Relation is in the List
    if(!find(R))
      throw new Exception("RelationList:drop() R(" + R.getName() + "," + R.getAlias() + ") is not in Relation List");

    // Drop it
    RList.removeElement(R);
  }

  /**
  * This function will find the EXACT Relation.
  */
  public boolean find(Relation R) throws Exception{

    if(R == null)
      throw new Exception("RelationList:find() Input Para\"R\" is null");

    return RList.contains(R);
  }

  /**
  * Return the a part of SQL back.
  * The SQL is used for CreateViewUpdate, to make the view at the first time.
  * E.g If we have view definition like: ... FROM R1, R2 ..., then it return
  * "R1, R2"
  */
  public String getSQL(){
    int size = RList.size();
    String query = new String();

    for(int i = 0; i< size-1; i++){
      Relation R = (Relation)RList.elementAt(i);
      query += R.getName() + ", ";
    }

    if(size > 0)
      query += ((Relation)RList.elementAt(size-1)).getName();

    return query;
  }
}

