////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
import java.sql.*;
import java.io.*;
import java.util.Vector;

/////////////////////////////////////
// Class RelationSchema
// only store the schema of one relation
// including the name of attributes
// For oracle varchar(>1) should be varchar2(>1), but access should 
// be varchar(>1)
/**
* Schema will be empty.
*/
public class RelationSchema implements java.io.Serializable, Cloneable{

  Vector labels;


  /**
   * @param r is the new cloned relation.
   */
  public Object clone(){
    RelationSchema clone = new RelationSchema();
    int size = size();
    for(int i = 0; i< size; i++)
      clone.addCol(getColName(i).substring(0));

    return clone;
  }

  private void addCol(String colName){
    labels.addElement(colName);
  }

  /**
   * This is for View and Clone functions.
   */
  public RelationSchema(){
    labels = new Vector();
  }

  public RelationSchema(String[] newSchema){

    this();

    int numcols = newSchema.length;

    for(int i = 0; i< numcols; i++)
      labels.addElement(newSchema[i]);

  }

  public RelationSchema(DataInputStream din){

    this();

    try{
      //read number of columns from din
      int numcols = din.readInt();
      
      // read all the labels
      for(int i = 0; i < numcols; i++){
        labels.addElement(din.readUTF());
      }
    }catch(Exception e){

      System.out.println("RelationSchema::RelationSchema: " + e.getMessage());

    }
  }

  /**
   * Here we generate each name as "R_A".
   */
  public RelationSchema(ResultSet rs) throws Exception{
    
    this();

    try{

      ResultSetMetaData metadata = rs.getMetaData();
      
      int numcols = metadata.getColumnCount();
      
      for(int i = 0; i< numcols; i++)
        labels.addElement(metadata.getColumnLabel(i + 1));
     
    }catch(Exception e){

      System.out.println("RelationSchema::RelationSchema: " + e.getMessage());

      throw new Exception("RelationSchema cannot Created");

    }
  }

  /**
  * Should erase SWEEPSIGN field from the Schema.
  */
  public RelationSchema(ResultSet rs, String sign) throws Exception{
    
    this();

    try{

      ResultSetMetaData metadata = rs.getMetaData();
      
      int numcols = metadata.getColumnCount();
      
      // Here assume first one is sign field
      // So, we skip the first field.

      for(int i = 1; i< numcols; i++)
        labels.addElement( metadata.getColumnLabel(i + 1));
     
    }catch(Exception e){

      System.out.println("RelationSchema::RelationSchema: " + e.getMessage());

      throw new Exception("RelationSchema cannot Created");

    }
  }

  /**
  * Should erase SWEEPSIGN field from the Schema.
  */
  public RelationSchema(ResultSet rs, Relation rel, String sign) throws Exception{
    
    this();

    try{

      ResultSetMetaData metadata = rs.getMetaData();
      
      int numcols = metadata.getColumnCount();
      
      // Here assume first one is sign field
      // So, we skip the first field.

      for(int i = 1; i< numcols; i++){
	String str = metadata.getColumnLabel(i+1);
	if(str.indexOf("_")>=0)
	  labels.addElement( metadata.getColumnLabel(i + 1));
	else
	  labels.addElement( rel.getName() + "_" + str);
      }
     
    }catch(Exception e){

      System.out.println("RelationSchema::RelationSchema: " + e.getMessage());

      throw new Exception("RelationSchema cannot Created");

    }
  }

  /**
  *this function return the Number of labels in the schema
  */
  public int size(){

    return labels.size();

  }

  /**
  *this function will return SQL, for create table.
  *because I cannot get the type from the ResultSet,
  *so, here, I just assume everything is char(250).
  */
  public String createSQL(){

    String query = new String();

    // append first n labels
    int i;
    int numcols = labels.size();

    for(i=0; i<numcols-1; i++)
      query = query + " " + labels.elementAt(i) + " varchar(250), ";

    // append last label
    if(numcols > 0)
      query = query + " " + labels.elementAt(i) + " varchar(250) ";

    return query;
  }

  /**
  *this function will return SQL, for create table.
  *because I cannot get the type from the ResultSet,
  *so, here, I just assume everything is char(250).
  *It will add a SWEEPSIGN field.
  */
  public String createSQL(String sign){

    String query = new String();

    // append first n labels
    int i;
    int numcols = labels.size();

    // append sign Label.
    query = query + " " + sign + " varchar(1), ";

    for(i=0; i<numcols-1; i++)
      query = query + " " + labels.elementAt(i) + " varchar(250), ";

    if(numcols > 0)
      query = query + " " + labels.elementAt(numcols-1) + " varchar(250) ";

    return query;
  }

  /**
  *get the only one common Attribute of two schemas
  */
  public String getCommon(RelationSchema theSchema){

    int numcols = labels.size();

    for(int i = 0; i < numcols; i++)
      for(int j = 0; j < theSchema.size(); j++){
        String str1 = (String)labels.elementAt(i);
        String str2 = (String)theSchema.labels.elementAt(j);
	      if(str1.equals(str2))
         return str1;
      }

    return null;
  }

  /**
  *write the schema to the file
  */
  public void write(DataOutputStream dout){

    try{

      int numcols = labels.size();

      // write columns to the file
      dout.writeInt(numcols);
      
      // write label names  to the file
      for(int i = 0; i < numcols; i++)
        dout.writeUTF((String)labels.elementAt(i));

    }catch(Exception e){

      System.out.println("RelationSchema::write: " + e.getMessage());
    }
  }

  /**
  * Get SQL from this Schema List. For View Definition
  * E.g. CREATE VIEW V (PRICE, RETIME) ..., it will returns "PRICE, RETIME"
  */
  public String getSQL(){
    int numcols = labels.size();
    String query = new String();

    for(int i = 0; i< numcols-1; i++)
      query += (String)labels.elementAt(i) + ", ";

    if(numcols > 0)
      query += (String)labels.elementAt(numcols-1);

    return query;

  }

  /**
  * Convert to String.
  */
  public String toString(){
    int numcols = labels.size();
    StringBuffer Output = new StringBuffer();

    for(int i = 0; i< numcols-1; i++)
      Output.append((String)labels.elementAt(i) + ", ");

    if(numcols > 0)
      Output.append((String)labels.elementAt(numcols-1));

    return Output.toString();

  }

  /**
  * Return column Name of specific Column Number.
  * E.g. a relation schema is: 'col1', 'col2', 'col3'.
  * if input index is 0, return 'col1'; 1, return 'col2'; 2, return 'col3'.
  */
  public String getColName(int index){
    if(index > size())
      throw new IndexOutOfBoundsException("RelationSchema:getColName: input index");

    return (String)labels.elementAt(index);
  }

  /**
   * Get Index of one labels in the schema.
   * the Index start from 1.
   * @param colname name of label.
   * @return index of the Label; 0 means not found.
   */
  int getIndex(String colName){
    for(int i = 0 ; i< size(); i++){
      if(colName.equals((String)labels.elementAt(i)))
	return i+1;
    }
    return 0;
  }

  /**
  * Return a set of flags to show which column is projected.
  * E.g. the relation shema is 'col1', 'col2', 'col3'. projected schema is
  * "col3, col2", then the flags are "0,2,1".
  */
  public int[] getFlags(RelationSchema schema){
    if(schema == null)
      throw new NullPointerException("RelationSchema:getFlags Para Schema is null");

    int size = size();
    int flags[] = new int[size];

    for(int i = 0; i< size; i++){
      String colName = (String)labels.elementAt(i);

      if(schema.find(colName))	
        flags[i] = schema.getIndex(colName);
      else
        flags[i] = 0;
    }

    return flags;
  }

  /**
  * This will return flags for projection.
  * The flag of the specific colName will be set to zero. At later doProject()
  * it will be deleted from the Relation.
  * E.g. we have schema "col1, col2, col3", we want to delete "col3", the flags
  * are "1,1,0".
  */
  public int[] getFlags(String colName){

    int size = size();
    int flags[] = new int[size];

    for(int i = 0; i< size; i++)
      flags[i] = 1;
    try
    {
      flags[whereIs(colName)] = 0;
    }catch(NotFoundException nfe){
      System.out.println("RelationSchema:getFlags() whereIS NotFoundException:" + nfe.getMessage());
    }
    return flags;
  }

  /**
  * Check if such column name stored in such schema.
  * This is a linear searching.
  */
  private boolean find(String colName){

     int size = size();

     for(int i = 0; i< size; i++)
       if(colName.equals((String)labels.elementAt(i)))
          return true;

     return false;
  }

  /**
  * Find where is such column name stored in this schema.
  * This is a linear searching.
  */
  private int whereIs(String colName) throws NotFoundException{

     int size = size();

     for(int i = 0; i< size; i++)
       if(colName.equals((String)labels.elementAt(i)))
          return i;

     throw new NotFoundException(colName);
  }

  /**
  * Check is current Schema is a subset of other Schema.
  * It is used for the Projection operation of Relation class.
  */
  public boolean isSubsetOf(RelationSchema theSchema){

    int size = size();
    for(int i=0; i< size; i++)
      if(!theSchema.find((String)labels.elementAt(i))){
    if(EVEDemo.MW.Main.DEBUG)
        System.out.println("Debug: RelationSchema:isSubsetOf() " + labels.elementAt(i));
        return false;
      }
    return true;
  }

  /**
  * This function will decrease the Schema based on the flags.
  * E.g. if the original schema is "col1, col2, col3", flags is "2, 0, 1", then
  * the new schema is "col3, col1".
  */
  public void doProject(int flags[]) throws Exception{
    if(flags == null)
      throw new NullPointerException("RelationSchema:doProject() flags is null");

    if(flags.length != size())
      throw new Exception("RelaitonSchema:doProject() Flags lenght does not match size of schema");

    // A string to hold sorted labels.
    String sort[] = new String[flags.length];

    // sort it.
    for(int i = 0; i <flags.length; i++){
      if(flags[i] > 0)
	sort[flags[i]-1]=(String)labels.elementAt(i);
    }

    // create new labels to hold new sorted projected labels.
    labels = new Vector();

    for(int i = 0; i <flags.length; i++){
      if(sort[i]!=null)
	labels.addElement(sort[i]);
    }


  }

  /**
  * Check if two Schemas are quivaluent.
  */
  public boolean equals(RelationSchema rs) throws Exception{
    if(rs == null)
      return false;

    if(rs.size() != size())
      return false;

    int size = size();

    for(int i=0; i<size; i++){
      if(!getColName(i).equals(rs.getColName(i)))
        return false;
    }

    return true;
  }

  /**
   * Change specific Attribute Name in this schema.
   */
  public void changeAttrName(String newName, int index){
    labels.setElementAt(newName, index);
  }

  /**
   * Get one specific Attribute 's name.
   */
  public String getAttrName(int index){
    return (String)labels.elementAt(index);
  }
}

