////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
import java.util.Vector;

//
//
// SelectList
//
//

/**
* Select List is a list of LOCAL Conditions.
* It will include a Vector of LOCAL Conditions.
*/
public class SelectList extends Vector implements java.io.Serializable
{
  Vector LCList;

  public SelectList(){
    LCList = new Vector();
  }

  /**
  * Add Local Condition to this Select List.
  * This will do the Consistency checking first then, if consistent, I will add it
  * in the LocalCondition List. Local Condition must be Unique.
  */
  public void addLocalCondition(LocalCondition LC) throws Exception{
    if(LC == null)
      throw new ParaNullException("SelectList","addLocalCondition()","LC");

    // Find the LC in the LClist first
    if(LCList.contains(LC))
      throw new Exception("SelectList:addLocalCondition() LC is already in it");

    // Check Consistency of LC with original LCList
    if(!isConsistent(LC))
      throw new Exception("SelectList:addLocalCondition() New Local Condition is inconsistent");

    // Add this LC to LCList.
    LCList.addElement(LC);
  }

  /**
  * Check Consistency with LC with LCList.
  */
  private boolean isConsistent(LocalCondition LC) throws Exception{
    if(LC == null)
      throw new ParaNullException("SelectList", "isConsistent", "LC");

    /**
    * Here I skip the Consistency Checking. Considering Moving some code from Yong
    */
    return true;
  }
  /**
  * Drop Local Condition from this Select List.
  * The reference of LC must be in the LCList, It will not be delete from the LCList
  * if they only have same value.
  */
  public void dropLocalCondition(LocalCondition LC) throws Exception{
    if(LC == null)
      throw new ParaNullException("SelectList", "dropLocalCondtion()", "LC");

    // Find the LC in the LClist first
    if(!LCList.contains(LC))
      throw new Exception("SelectList:dropLocalCondition() LC is not in LCList");

    // drop this LC to LCLists
    LCList.removeElement(LC);
  }

  /**
  * Get SQL from SelectList.
  * E.g. WHERE R.A > 5 AND R.B < 10, sql will be: R.A > 5 AND R.B < 10
  */
  public String getSQL(){
    int size = LCList.size();

    String query = new String();

    for(int i=0; i<size-1 ; i++){
      LocalCondition LC = (LocalCondition)LCList.elementAt(i);
      query += LC.getSQL() + " and ";
    }

    if(size > 0)
      query += ((LocalCondition)LCList.elementAt(size-1)).getSQL();

    return query;

  }
}

