////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
/**
* Class SendUpdates 
* is a process running on the IS site
* it will detect the update of IS
* and then send it to DW by MultiQueue
*/

public class SendUpdates extends Thread implements java.io.Serializable{
  
  /**
   * Global Object for ISProcess.
   */
  MultiQueue publicFIFOChannel; 
  /**
   * Global object for ISProcess. describes current IS's number
   */
  int MyIndex; 
  /**
   * used to detect the update.
   */
  CreateUpdateFile cuf; 
  /**
   * The Receiver where SendUpdates will get the Data Updates.
   */
  DataUpdateReceiver Receiver; 

  static int sno = 0; // the serial number of this data update.
  static final int MAXSNO = 10000; // maximum serial number of this data update. It decides maximum threads can run at the same time.
  

  /**
   * Set the Receiver of this SendUpdate Module.
   * It's for Andreas Performance Test.
   */
  public void setReceiver(DataUpdateReceiver dur){
    Receiver = dur;
  }
  
  /**
   * Set the CUF for this sendupdate message.
   */
  public void setCUF(CreateUpdateFile u){
    cuf = u;
  }

  public SendUpdates(MultiQueue FIFOChannel, int ISnumber){
    
    publicFIFOChannel = FIFOChannel;
    MyIndex = ISnumber;
    cuf = null;
    this.start();
  }
  
  public void run(){
    
    UpdateData theUpdate;
    
      while(true){
        try{
	  /*
	    // skip running before the CreateUpdateFile object is created.
	    if(cuf == null)
	    continue;
	    // Receive DeltaR from R
	    // here, theUpdate should include the MyIndex inside the Data structor
	    //System.out.println("debug: SendUpdates:run() throw exception in next statement");
	    theUpdate = cuf.checkUpdate();
	    if(theUpdate == null)
	    continue;
	  */

	  // skip running before the Receiver object is created.
	  if(Receiver == null)
	    continue;
	  // Receive DeltaR from R
	  // here, theUpdate should include the MyIndex inside the Data structor
	  //System.out.println("debug: SendUpdates:run() throw exception in next statement");
	  theUpdate = Receiver.getDataUpdate(MyIndex);
	  if(theUpdate == null)
	    continue;
	  
	  // System.out.println("debug:Sendupdates:run() theUpdate is:\r\n" + theUpdate);
	  
	  // Change the field name of this update.
	  RelationSchema rs = theUpdate.getDeltaRelation().getSchema();
	  int size = rs.size();
          String relName = theUpdate.getDeltaRelation().getName();
	  for(int i = 0; i < size; i++){
	    rs.changeAttrName(relName + "_" + rs.getAttrName(i), i);
	  }
	  
          // Change the Update name to "DeltaR_Index"
	  theUpdate.rename("DeltaR#" + sno + "_" + MyIndex);
	  if(sno < MAXSNO )sno ++;
	  else sno = 0;
	  
	  // System.out.println("debug:SendUpdate:run() relName is: " + relName);

	  // Set the alias of this Update Relation.
	  theUpdate.setAlias(relName);

          // Send (DeltaR, MyIndex) to Datawarehouse
          publicFIFOChannel.SendUpdate(theUpdate);
          
          //System.out.println("SendUpdates:run() Update Sended" + theUpdate);
        }catch(NoUpdateException nue){
          
          //System.out.println("SendUpdates::run:" + nue.getMessage());
        }catch(Exception e){
          System.out.println("SendUpdates:run(): FIFO Channel error: " + e.getMessage());

	}finally{
	  try{
	    sleep(PSWEEP.PSWEEP.step);
          }catch(InterruptedException iee){
	    System.out.println("SendUpdates:run() Sleep error: " + iee.getMessage());
	  }
	}
      }
  }
  
}


