////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
import java.io.*;
import java.util.*;
import java.sql.*;
import java.lang.Object;
import java.lang.Exception;

public class Tuple implements java.io.Serializable, Cloneable{

  Vector fields;/**
                * String Arrays.
                */
  int number; // number of tuples with same value. e.g. (3,4)[3], here number is 3
              // if the number is positive, means insert, otherwise means delete
              // This number could not be ZERO, if it is ZERO, this tuple will be destoried.
              // In any Relation or View, it should be positive, except the middle result of
              // delta V or delta R ( Relation Update )

  /**
  * Convert to String.
  */
  public String toString(){

    int size = fields.size();
    StringBuffer Output = new StringBuffer();

    Output.append("<");

    for(int i = 0; i< size-1; i++)
      Output.append((String)fields.elementAt(i)+ ", ");
    
    if(size > 0)
      Output.append((String)fields.elementAt(size-1));

    Output.append(">");

    Output.append("[" + number + "]");

    return Output.toString();
  }

  public Tuple(){

    fields = new Vector();
    number = 1;

  }

  /**
   * Construct a Tuple from a count number and a String.
   * @param Count number of copies
   * @param Value Fields in format: "A|B|C"
   */     
  public Tuple(int Count, String Value){
    this();
    if(Count == 0)
      System.out.println("debug: Tuple.Tuple(int, string) Count is 0.");
    number = Count;
    StringTokenizer st = new StringTokenizer(Value, "|");
    while(st.hasMoreTokens())
      fields.addElement(st.nextToken().trim());
  }

  /**
   * this constructor will create tuple from a list of strings, default sign is insert.
   */
  public Tuple(String[] newFields){

    this();

    int length = newFields.length;
    
    for(int i = 0; i< length ; i++){

      fields.addElement(newFields[i]);
    }

  }

  // this constructor will create tuple from a list of strings, and specific sign 
  public Tuple(String[] newFields, int newNumber){

    this(newFields);

    number = newNumber;

  }

  public Tuple(DataInputStream din){

    this();

    try{
      // first get how many fields in each tuple
      int fNums = din.readInt();
      
      // Read each fields from the file
      for(int i = 0; i < fNums; i++){
	
	      fields.addElement(din.readUTF());
	
      }

      // Read Number from the file
      number = din.readInt();

    }catch(Exception e){

      System.out.println("Tuple::Tuple: " + e.getMessage());

    }
  }

  // Create a tuple from the SQL Result Set of current position
  // para:
  //    Result rs:  ResultSet of SQL
  //    int colNum: Column Number, e.g. (3,4) has 2 columns
  public Tuple(ResultSet rs, int colNum) throws Exception{

    this();

    fields = new Vector();
    
    // check the Parameter
    if(colNum < 1)
      throw new Exception("Tuple::Tuple: index lower than 1");

    if(rs == null)
      throw new Exception("Tuple::Tuple: result set is NULL");

    // create the tuple of the specific index
    for(int i=0; i< colNum; i++){
      // Here I don't check the rs.next(), because the outside function has
      // checked.
      String data = null;
      try{
	data=rs.getObject(i+1).toString().trim();
      }catch(Exception e){
	// Here, the object in ResultSet is null, so, we create an empty string.
	data=new String();
      }
      //System.out.println("Debug: data is : " + data);
      if(Type.isDate(data))
{
	//System.out.println("Debug: Guess error here, not date type!");
        data = Type.DateTransfer(data);
      }
	      
      fields.addElement(data);
    }

  }

  // Create a tuple from the SQL Result Set of current position
  // para:
  //    Result rs:  ResultSet of SQL
  //    int colNum: Column Number, e.g. (3,4) has 2 columns
  /**
  * Skip Sign field and set the Sign bit.
  */
  public Tuple(ResultSet rs, int colNum, String sign) throws Exception{

    this();

    fields = new Vector();
    
    // check the Parameter
    if(colNum < 1)
      throw new Exception("Tuple::Tuple: index lower than 1");

    if(rs == null)
      throw new Exception("Tuple::Tuple: result set is NULL");

    // create the tuple of the specific index
    for(int i=1; i< colNum+1; i++){
      // Here I don't check the rs.next(), because the outside function has
      // checked.
      String data=rs.getObject(i+1).toString().trim();
      if(Type.isDate(data))
        data = Type.DateTransfer(data);
	      
      fields.addElement(data);
    }

    if(rs.getObject(1).toString().trim().equals("N"))
      number = -number;

  }

  // this function will reverse the sign of number.
  // used for Minus of two relaitons
  // e.g. +3 --> -3
  //      -4 --> +4
  public void reverse(){
    number = -number;    
  }

  // Generate this tuples SQL language
  // 
  public String SQL(){

    StringBuffer Query = new StringBuffer();
    int size = fields.size();

    for ( int i = 0; i < size; i ++ ){

      /**
      * Here, I need to convert Date type from '19yy-mm-dd ...' to 'dd-mmm-yy'.
      * In general, I should do a global Type class ( I have it, but not complete), to do
      * the format transaction.
      */
      String t = (String)fields.elementAt(i);
      if(Type.isDate(t))
        t = Type.DateTransfer(t);

      Query.append( "'" + t + "',");

    }
    
    // here is a debug information line
    //System.out.println("Tuple::SQL tuple is " + Query + " [ " + number + " ] ");
    
    return Query.toString().substring(0, Query.toString().length()-1);

  }

  // Generate this tuples SQL language
  // 
  /**
  * Add SWEEPSIGN field.
  */
  public String SQL(String sign){

    StringBuffer Query = new StringBuffer();
    int size = fields.size();

    if(number > 0)
      Query = new StringBuffer("'P',") ;
    else
      Query = new StringBuffer("'N',") ;

    for ( int i = 0; i < size; i ++ ){

      /**
      * Here, I need to convert Date type from '19yy-mm-dd ...' to 'dd-mmm-yy'.
      * In general, I should do a global Type class ( I have it, but not complete), to do
      * the format transaction.
      */
      String t = (String)fields.elementAt(i);
      if(Type.isDate(t))
        t = Type.DateTransfer(t);

      Query.append( "'" + t + "',");

    }
    
    // here is a debug information line
    //System.out.println("Tuple::SQL tuple is " + Query + " [ " + number + " ] ");
    
    return Query.toString().substring(0, Query.toString().length()-1);

  }

  // this function will return the number of fields in this tuple
  public int getNum(){

    return fields.size();

  }

  // this function write the data output to a file
  public void write(DataOutputStream dout){

    try{
      // get number of fields in this tuple
      int fNums = fields.size();
      
      // write how many fields in each tuple
      dout.writeInt(fNums);
      
      // write all the fields in file
      for(int i = 0; i < fNums; i ++){
	
	      dout.writeUTF((String) fields.elementAt(i));
      }

      // write the number to file
      dout.writeInt(number);

    }catch(Exception e){

      System.out.println("Tuple::write: " + e.getMessage());
    }
  }

  // return the number of values appeared
  public int getNumber(){
    return number;
  }

  // set the number of this tuple
  public void setNumber(int newNumber) throws Exception{
    if(newNumber == 0)
      throw new Exception("Tuple::setNumber newNumber is zero");
    number = newNumber;
  }

  // get specific fields
  public Object getField(int i) throws Exception{
    if(i < 0 || i > size())
      throw new Exception("Tuple::getField input paramenter i out of bound");
    return fields.elementAt(i);
  }

  // check is two tuples equals
  public boolean equals(Tuple t) throws Exception{
    if(t == null)
      return false;
    if(size() != t.size())
      return false;
    int size = size();

    for(int i = 0; i < size; i++){
      if(!getField(i).equals(t.getField(i)))
        return false;
    }

    return true;
  }

  // return number of fields of this tuple
  public int size(){
    return fields.size();
  }

  // add one field to this tuple
  public void addField(Object newField){
    fields.addElement(newField);
  }

  // clone this tuple
  public Object clone(){
    Tuple clone = new Tuple();

    try{
      for(int i = 0; i< size() ; i++)
        clone.addField(getField(i));

      clone.setNumber(number);

    }catch(Exception e){
      System.out.println("Tuple::clone " + e.getMessage());   
    }
    return clone;
  }

  /**
  * Do Project depends on the flags.
  */
  public void doProject(int flags[]) throws Exception{

    if(flags == null)
      throw new NullPointerException("Tuple:doProject() flags is null");

    if(flags.length != size())
      throw new Exception("Tuple:doProject() Flags length does not match Tuples size");

    // A string to hold sorted labels.
    String sort[] = new String[flags.length];

    // sort it.
    for(int i = 0; i <flags.length; i++){
      if(flags[i] > 0)
	sort[flags[i]-1]=(String)fields.elementAt(i);
    }

    // create new labels to hold new sorted projected labels.
    fields = new Vector();

    for(int i = 0; i <flags.length; i++){
      if(sort[i]!=null)
	fields.addElement(sort[i]);
    }

  }

  /**
  * It will recalculate the sign depends on the flags.
  * all the fields corresponding of flags should be same, then the sign is positive;
  * otherwise, it is negative.
  * e.g. tuple (n, 5, n, 6)[-3], flags is (0, 1, 0, 1), then the tuple should be:
  * (n,5,n,6)[3];
  */
  public void calSign(int flags[]) throws NullPointerException, NotMatchException, Exception{
    if(flags == null)
      throw new NullPointerException("Tuple:calSign() null in parameters");

    int size = size();

    if(flags.length != size)
      throw new NotMatchException("Tuple:calSign() flags not Match the Tuple");

    int negNum = 0; // counts of negative sign

    for(int i = 0; i < size; i++){
      if(flags[i] == 0)
        if(((String)getField(i)).equals("N"))
          negNum ++;
    }

    if(negNum%2==0) 
      number = Math.abs(number);
    else
      number = -Math.abs(number);
  }
}
