////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
///////////////////////////////////////
// Class UpdateData will stores 
// schema information and data informatio
// of Delta R from a specific (IS, R)

import java.io.*;

public class UpdateData extends StampedElement implements java.io.Serializable{

  Relation UpdateRelation;// object to store the schema of Update Data
  int ISIndex; // the index of Information Source
  static int sno = 0; // the serial number of this data update.
  static final int MAXSNO = 10000; // maximum serial number of this data update. It decides maximum threads can run at the same time.

  public UpdateData(int theIndex) throws Exception{

    super ();
    if(theIndex < 0)
      throw new Exception("UpdateData::UpdateData: theIndex is less than 0");
    
     // Initilized the Schema and Extent
    UpdateRelation = new Relation("DeltaR#" + sno + "_" + theIndex);
    if(sno < MAXSNO )sno ++;
    else sno = 0;
    ISIndex = theIndex;
  }
  
  public UpdateData(Relation theRelation, int theIndex) throws Exception{

    this(theIndex);

    UpdateRelation = theRelation;
  }

  public UpdateData(FileInputStream fin, int theIndex) throws Exception{

    // initialize the currrent object
    this(theIndex);

    try{
      
      // Read Delta Relation from File Input Stream
      UpdateRelation = new Relation(fin);

    }catch(Exception e){

      System.out.println("UpdateData::UpdataData: " + e.getMessage());

      throw new Exception("Cannot create UpdateData");

    }
  }
    
  // return DeltaRelation stored inside
  public Relation getDeltaRelation(){

    return UpdateRelation;
  }

  // return source index stored inside
  public int getSource(){

    return ISIndex;
  }


  // write the current data to a file
  public void write(FileOutputStream fout) throws Exception{
		UpdateRelation.write(fout);
  }

  /**
  * Output the UpdateData to the Screen.
  */
  public String toString(){
  
    StringBuffer Output = new StringBuffer();
    
    Output.append("\r\nIndex is: " + ISIndex + "\r\n");

    Output.append(UpdateRelation.toString() + "\r\n");

    return Output.toString();
  }

  /**
  * Rename the UpdateData.
  */
  public void rename(String newName){
    UpdateRelation.rename(newName);
  }

  /**
   * Set alias of the Relation
   */
  public void setAlias(String nName){
    UpdateRelation.setAlias(nName);
  }

  /**
   * Get Alias of the Relation
   */
  public String getAlias(){
    return UpdateRelation.getAlias();
  }

}




