////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;

import EVEDemo.MW.View;

public class UpdateView extends Thread implements java.io.Serializable{
  
  protected MessageQueue UpdateMessageQueue; // this is a global variable of DWProcess
  protected MultiQueue publicFIFOChannel; // this is a global variable of SWEEP

  protected int n = 2; // n describe how many ISs. Now, we only have
 // two. make is constant. assume the initail value is 2, it could be
 // changed by function setn()

  protected ViewDefinition View; // This is the View in the DW, it is a global variable of DWProcess
  protected boolean finished = false; // flag to terminate while loop
  protected View EVEview; // The EVEDemo View object.

  /**
  * Set the number of ISs.
  */
  public void setn(int ISnum){
    n = ISnum;
  }

  /**
   * Set the View object.
   */
  public void setView(View v){
    EVEview = v;
  }

  public UpdateView(MultiQueue theFIFOChannel, Queue theMessageQueue, ViewDefinition theView){
    
    super();
    
    try{
      
      UpdateMessageQueue = (MessageQueue)theMessageQueue;
      publicFIFOChannel = theFIFOChannel;
      View = theView;
      EVEview = null;
      //this.start();
    }catch(Exception e){
      
      // System.out.println("UpdateView::Updateview: " + e.getMessage());
    }
  }
  
  public void run(){
    
    while(!finished){
      
      System.out.println("UpdateView:run() Testing Update Message Queue");
      // Remove (Delta R, i) FROM UpdateMessageQueue; BLOCK if queue empty 
      UpdateData theUpdate = (UpdateData)UpdateMessageQueue.remove();
      
      // System.out.println("UpdateView:run() get Update from Update Message Queue" + theUpdate);

      Relation deltaView = ViewChange(theUpdate);

      // System.out.println("Updateview:run() Add Data Warehouse(" + View.getName() + ") with Delta V:\r\n" + deltaView);
      // It changed for integration: Call DCI(DBI Class) update the View
      if(deltaView != null){

	// Here we do the double check.
	DatabaseInterface dbi = new DatabaseInterface();
	//dbi.UpdateView(View,deltaView);
	dbi.finalize();
	
	/**
	 * Assume all the signs of each tuple in the EVEview are same: either
	 * '+' or '-'.
	 */
	if(EVEview != null){
	  try{
	    //1. Projection
	    deltaView.projectOn(View);
	  }catch(Exception e){
	    System.out.println("DBI:UpdateView() error Project:" + e.getMessage());
	  }
	  
    if(EVEDemo.MW.Main.DEBUG)
	  System.out.println("Debug: deltaview after projection is:\r\n" + deltaView);

	  if(deltaView.getExtent().size() > 0){
	    
	    if(deltaView.getExtent().getTuple(0).getNumber() > 0)
	      EVEview.addTuples(deltaView);
	    else
	      EVEview.removeTuples(deltaView);
	  }
	}
      }
      // Stop the data update performance timer in the middle layer.
      EVEview.stopTimer();

      System.out.println("Updateview.run() Data Warehouse(" + View.getName() + ") Update Complete!");
      try{
        // After that Sleep for a while
        sleep(PSWEEP.PSWEEP.step);
      }catch(InterruptedException iee){
        System.out.println("UpdateView:run() sleep Error: " + iee.getMessage());
      }
    }
  }
  
  public Relation ViewChange(UpdateData theUpdate){
    
    Relation DeltaView, TempView;
    int j;
    
    try{
      
      System.out.println("UpdateView:ViewChange " + theUpdate);

      // Delta V = Delta R
      DeltaView = theUpdate.getDeltaRelation();


      int theIndex = theUpdate.getSource();
      
      System.out.println("UpdateView:ViewChange Stp0: Scan left part of ISs["+1 + ".." + (theIndex -1)+"]");

      // compute the left part of the incremental view resulting from Delta R
      for(j = theIndex-1; j>=1; j--){
        
	// if DeltaView is empty, we return null.
	if(DeltaView.getExtent().size() == 0)
	  return null;
        
        TempView = (Relation)DeltaView.clone();

        System.out.println("UpdateVeiw:ViewChange Stp1: Send Delta V to IS["+j+"]");
        
        // send Delta V to Data source j
        publicFIFOChannel.sendQuery(DeltaView, j);

        System.out.println("UpdateVeiw:ViewChange Stp2: Receive Delta V' from IS["+j+"]");        

        // receive Delta V from Data Source j
        // I think here, we need block to waiting
        DeltaView = publicFIFOChannel.receiveQueryResult(j);

	System.out.println("Updateview:ViewChange Stp2.5: the Delta V' is: \r\n" + DeltaView );

        System.out.println("UpdateView:ViewChange Stp3: Test Concurrent Update for IS["+j+"]"); 

        // remove the error due to concurrent update if any
        // if Exist Delta Rj in UpdateMessageQueue then Delta V = Delta V - 
        // Delta Rj join TempView.
        if(UpdateMessageQueue.exists(j)){

          System.out.println("UpdateView:ViewChange Stp4: Concurrent Update detected");
          
          // get Delta Rj from UpdateMessageQueue without remove it
          Relation DeltaRj = UpdateMessageQueue.peek(j);
	  if(DeltaRj == null)
	      throw new Exception("Cannot peek specifice Update in MessageQueue");
          
          System.out.println("UpdateView:ViewChange Stp5: Local Correction");
          // Here I check the time stampe of Query Result and Delta R j
          // if the time stamp of Query Result is Older than Delta R j, then
          // there must be a concurrency
          if(DeltaView.OlderThan(DeltaRj)){
            DeltaView = DeltaView.minus(TempView.joinVirtual(DeltaRj));
            
          }
        }
        else{
          System.out.println("Updateview:ViewChange Stp6: No Concurrent Update Detected:)");
        }
      }

      System.out.println("UpdateView:ViewChange Stp7: Scan right part of ISs["+(theIndex + 1 )+ ".." + n + "]");
      // compute the right part of the incremental view sesulting from Delta R
      for(j = theIndex+1; j <= n; j++){

	// if DeltaView is empty, we return null.
	if(DeltaView.getExtent().size() == 0)
	  return null;

        TempView = (Relation)DeltaView.clone();

        System.out.println("UpdateVeiw:ViewChange Stp8: Send Delta V to IS["+j+"]");
        // send Delta V to Data Source j
        publicFIFOChannel.sendQuery(DeltaView, j);
        
	System.out.println("Updateview:ViewChange Stp8.5: the Delta V' is: \r\n" + DeltaView );

        System.out.println("UpdateVeiw:ViewChange Stp9: Receive Delta V' from IS["+j+"]");        
        // receive Delta V from Data Source j
        // I think here, we need block to waiting
        DeltaView = publicFIFOChannel.receiveQueryResult(j);
        
        System.out.println("UpdateView:ViewChange Stp10: Test Concurrent Update for IS["+j+"]"); 
        // remove the error due to concurrent update if any
        // if Exist Delta Rj in UpdateMessageQueue then Delta V = Delta V - Delta Rj join TempView.
        if(UpdateMessageQueue.exists(j)){
          
          System.out.println("UpdateView:ViewChange Stp11: Concurrent Update detected");

          // get Delta Rj from UpdateMessageQueue without remove it
          Relation DeltaRj = UpdateMessageQueue.peek(j);
          if(DeltaRj == null)
	      throw new Exception("Cannot peek specifice Update in MessageQueue");

          System.out.println("UpdateView:ViewChange Stp12: Local Correction");
          // Here I check the time stampe of Query Result and Delta R j
          // if the time stamp of Query Result is Older than Delta R j, then
          // there must be a concurrency
          if(DeltaView.OlderThan(DeltaRj)){
            DeltaView = DeltaView.minus(TempView.joinVirtual(DeltaRj));
          }
        }
        else{
          System.out.println("Updateview:ViewChange Stp13: No Concurrent Update Detected:)");
        }
        
      }
      System.out.println("UpdateView:ViewChange Stp14: Return Delta View");

      return DeltaView;
    }catch(Exception e){
      
      System.out.println("UpdateView::ViewChange: " + e.getMessage());
    }
    
    return null;
  }
}



