////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  EVE 2.0 - Evolvable View Environment
//
//  Copyright 1997-2000, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Andreas Koeller, Xin Zhang, Elke Rundensteiner
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/dsrg/EVE
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  EVE Development Team: <rundenst@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //
package SWEEP;
/** This ViewDefinition Class will define the PSJ formal.
* PSJ is Project Select Join.
* <br>
* Project: A Vector of Attribute.
* <br>
* Select: A Vector of Condition.
* <br>
* Join: A Vector of Join Constraint.
* <br>
* This class will not do any kind of consistency checking.<br>
* There should be three levels of consistency checking: <br>
* (Like EVE View Definition Consistency Checking)<br>
*  1. Check the consistency within Select clause; and consistency within Join clause<br>
*  2. Check the consistency between Select and Join clause<br>
*  3. Check the consistency between ViewDefinition and Involved Relations<br>
*/

/** This view is the subclass of Relation, so, it has schema, name and extent.<br>
* Here the view will be materialized, so, both "schema", "extent" are required.
*/
public class ViewDefinition extends Relation implements java.io.Serializable{

  // Alias of each column e.g. CREATE VIEW V(A1, A2, A3) is same as Schema
  ProjectList Project; // SELECT Clause of View.
  SelectList Select; // Local Conditions in the WHERE Clause of View.
  JoinList Join; // FROM Clause of View and Join Conditions in the WHERE Clause of View

  public ViewDefinition(String newName, RelationSchema newSchema, ProjectList newProject, SelectList newSelect, JoinList newJoin){
    super(newName);

    // Set the schema and P.S.J.
    theSchema = newSchema;
    Project = newProject;
    Select = newSelect;
    Join = newJoin;

    // Here we don't need to initialized the extent.
    /*
      // Init the View Extent with a very simple algorithm
      init();
    */

  }

  /**
   * This method will call DatabaseInterface to interact with Oracle and 
   * get the View Extent from the Oracle.
   * This function will be called by VD Constructor.
   */
  private void init(){
    String Query;

    DatabaseInterface dbi = new DatabaseInterface();

    try{
      // 0. Drop the View in Oracle first
      Query = "drop table " + theName;
      dbi.executeUpdate(Query);
      
      // 1. Need to create a SQL and create a View on the Oracle
      Query = getCreateQuery();
      
      dbi.executeUpdate(Query);
      // 2. Send Query SQL and get the View Extent from the Oracle
      Query = "select * from tempView";
      Relation r = dbi.executeQuery(Query);
      // store the extent to this view
      theExtent = r.getExtent();

      // 3. Create a table corresponding this View on Oracle
      Query = "create table " + theName + " as select * from tempView";

      dbi.executeUpdate(Query);
      // 4. Send Query and delete the View Definition from the Oracle
      Query = "drop view tempView";
      dbi.executeUpdate(Query);
    }catch(Exception e){
      System.out.println("VD:init() Error: " + e.getMessage());
    }
    dbi.finalize();

  }

  // this funciton will return a SQL that can create the View in the Oracle
  private String getCreateQuery(){
    // CREATE VIEW V 
    StringBuffer query = new StringBuffer("CREATE VIEW tempView ");
    //  ... (col1, col2, col3...) AS
    if(theSchema.size()>0){
      query.append(" ( ");
      query.append(theSchema.getSQL());
      query.append(" ) ");
    }
    // SELECT R1.A1, R2.A2
    query.append(" AS SELECT ");
    query.append(Project.getSQL()); // Attributes List with Relation Reference.
    // FROM R1, R2
    query.append(" FROM ");
    query.append(Join.getSQL()); // Relation list
    // WHERE 
    if(Select.size()>0 || Join.getJCList().size()>0){
      query.append(" WHERE "); 
      String temp = query.toString();
      query.append(Select.getSQL()); // Local Constraints e.g. R.C > 5
      //System.out.println("Debug: ViewDefinition:getCreateQuery: temp is \r\n'" + temp + "' \r\nquery is \r\n'" + query.toString() + "'");
      if(!temp.equals(query.toString())) // Add "AND" between LOCAL and GLOBAL Conditions
        query.append(" AND ");

      temp = query.toString();
      query.append(Join.getConditionSQL()); // Global Constraitns e.g. R.A = S.B
      if(temp.equals(query.toString())) // if only has local conditions, remove and.
        query.setLength(query.length() - 5);
    }
    return query.toString();
  }
}
